/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/
package com.ibm.faces.component;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import javax.faces.component.EditableValueHolder;
import javax.faces.component.NamingContainer;
import javax.faces.component.UIComponent;
import javax.faces.component.UIComponentBase;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;
import javax.faces.event.AbortProcessingException;
import javax.faces.event.FacesEvent;
import javax.faces.model.DataModel;
import com.ibm.faces.event.DataIndexEvent;
import com.ibm.faces.util.DataModelUtil;


/**
 * <p>UIIterator - iterates over a DataModel providing data to its child components.</p>
 * @author pandrews
 */
public class UIDataIterator extends UIComponentBase implements NamingContainer{
	
	private Object value;
	private String var;
	private int index;
	private DataModel model;
	private HashMap savedValues;
	private int first = 0;
	private Boolean firstSet = Boolean.FALSE;
	private int rows = 0;
	private Boolean rowsSet = Boolean.FALSE;

	
	public static final String COMPONENT_TYPE = "com.ibm.faces.DataIterator"; //$NON-NLS-1$
	public static final String COMPONENT_FAMILY = "com.ibm.faces.DataIterator"; //$NON-NLS-1$
	
	
	public UIDataIterator(){
		super();
	}
	
	public String getFamily(){
		return COMPONENT_FAMILY;
	}
	
	
	
	/**
	 * @return Returns the index of the first row to be process (first row is zero), default is 0.
	 */
	public int getFirst() {
		if(this.firstSet.booleanValue()){
			return this.first;
		}
		ValueBinding vb = getValueBinding("first");
		if(vb != null){
			Integer v = (Integer)vb.getValue(FacesContext.getCurrentInstance());
			if(v != null){
				return v.intValue();
			}
		}
		return this.first;
	}
	/**
	 * @param first Set the index of the first row to be processed, if set to value less than zero it will be set to zero.
	 */
	public void setFirst(int first) {
		if(first < 0){
			first = 0;
		}
		this.first = first;
		firstSet = Boolean.TRUE;
	}
	/**
	 * @return Returns the number of rows to be process starting from the index.
	 */
	public int getRows() {
		if(rowsSet.booleanValue()){
			return this.rows;
		}
		ValueBinding vb = getValueBinding("rows");
		if(vb != null){
			Integer v = (Integer)vb.getValue(FacesContext.getCurrentInstance());
			if (v != null){
				return v.intValue();
			}
		}
		return this.rows;
	}
	
	/**
	 * @param rows Sets the number of rows to be process starting from the index.
	 */
	public void setRows(int rows) {
		this.rows = rows;
		rowsSet = Boolean.TRUE;
	}
	
	public Object getValue(){
		if(this.value != null){
			return this.value;
		}
		ValueBinding vb = getValueBinding("value");
		if(vb != null){
			return vb.getValue(FacesContext.getCurrentInstance());
		}
		return null;
	}
	
	public void setValue(Object o){
		this.value = o;
		//reset model
		this.model = null;
	}
	
	
	/**
	 * @return Returns the rowIndex.
	 */
	public int getIndex() {
		return index;
	}
	/**
	 * @param rowIndex The rowIndex to set.
	 */
	public void setIndex(int index) {
		
		//save the state for the previous row
		Iterator children;
		children = getChildren().iterator();
		while(children.hasNext()){
			saveValueHolderState(FacesContext.getCurrentInstance(),(UIComponent)children.next());
		}
		
		//set to new row index
        this.index = index;
        getDataModel().setRowIndex(index);
		
        //store the var rowdata on the request
        if (var != null) {
            if (isRowAvailable()) {
            	getFacesContext().getExternalContext().getRequestMap().put(var, getDataModel().getRowData());
            } else {
            	getFacesContext().getExternalContext().getRequestMap().remove(var);
            }
        }
        
		//reset for this row
		children = getChildren().iterator();
		while(children.hasNext()){
			restoreValueHolderState(FacesContext.getCurrentInstance(),(UIComponent)children.next());
		}
	}
	/**
	 * @return Returns the var.
	 */
	public String getVar() {
		return var;
	}
	/**
	 * @param var The var to set.
	 */
	public void setVar(String var) {
		this.var = var;
	}
	
	/* (non-Javadoc)
	 * @see javax.faces.component.UIComponent#getRendersChildren()
	 */
	public boolean getRendersChildren() {
		return true;
	}
	
	
    protected DataModel getDataModel() {

        if (this.model != null) {
            return (this.model);
        }

        Object v = getValue();
        this.model = DataModelUtil.getDataModel(v);
        
        return this.model;
        
    }
	
    protected void saveValueHolderState(FacesContext context, UIComponent component){
        
    	if (component instanceof EditableValueHolder) {
            EditableValueHolder valueHolder = (EditableValueHolder) component;
            String clientId = component.getClientId(context);
            Object[] valueState = null; 
            if(savedValues != null)
            	valueState = (Object[]) savedValues.get(clientId);
            if (valueState == null) {
                valueState = new Object[4];
                savedValues.put(clientId, valueState);
            }
            
            valueState[0] = valueHolder.getLocalValue();
            valueState[1] = valueHolder.getSubmittedValue();
            valueState[2] = valueHolder.isValid() ? Boolean.TRUE : Boolean.FALSE;
            valueState[3] = valueHolder.isLocalValueSet()? Boolean.TRUE : Boolean.FALSE;
            
        }

        // Look after this components children's state
        Iterator children = component.getChildren().iterator();
        while (children.hasNext()) {
            saveValueHolderState(context, (UIComponent) children.next());
        }
        
    }
    
    protected void restoreValueHolderState(FacesContext context, UIComponent component){
    	
        //reset id, this will make the clientid regenerate, required so the row index can be pushed into the clientid
        String id = component.getId();
        component.setId(id);

        if (component instanceof EditableValueHolder) {
            EditableValueHolder valueHolder = (EditableValueHolder) component;
            String clientId = component.getClientId(context);
            Object[] valueState = null;
            if(savedValues != null)
            	valueState = (Object[]) savedValues.get(clientId);
            
            if(valueState != null){
            	valueHolder.setValue(valueState[0]);
            	valueHolder.setSubmittedValue(valueState[1]);
            	valueHolder.setValid(((Boolean)valueState[2]).booleanValue());
            	valueHolder.setLocalValueSet(((Boolean)valueState[3]).booleanValue());
            }
        }

        // look after restoring this components children
        Iterator children = component.getChildren().iterator();
        while (children.hasNext()) {
            restoreValueHolderState(context, (UIComponent) children.next());
        }

    }

    
	/* (non-Javadoc)
	 * @see javax.faces.component.UIComponent#getClientId(javax.faces.context.FacesContext)
	 */
	public String getClientId(FacesContext context) {
		
		String clientid = super.getClientId(context);
		
		if(index > -1 ){
			clientid = clientid + NamingContainer.SEPARATOR_CHAR + index;
		}
		return clientid;
	}
	
	
	
	
	/* (non-Javadoc)
	 * @see javax.faces.component.UIComponent#processDecodes(javax.faces.context.FacesContext)
	 */
	public void processDecodes(FacesContext context) {
		
		if(!isRendered()){
			return;
		}
		//reset model
		model = null;
		 
		if(savedValues == null || !isNestedInUIIterator()){
			//reset savedValues
			savedValues = new HashMap();
		}
	    	
		setIndex(getFirst()); 
		while (getDataModel().isRowAvailable() && isNotEndRow()) {		
			Iterator children = getChildren().iterator();
			while (children.hasNext()) {
				UIComponent child = (UIComponent) children.next();
				
				if (!child.isRendered()) {
					continue;
				}
				child.processDecodes(context);
			}
			setIndex(++index);
		}
		setIndex(-1);
		
		decode(context);
	}
	
	
	/* (non-Javadoc)
	 * @see javax.faces.component.UIComponent#processUpdates(javax.faces.context.FacesContext)
	 */
	public void processUpdates(FacesContext context) {
		
		if (!isRendered()) {
	            return;
	    }
		if (isNestedInUIIterator()) {
		    model = null;
		}
		
		setIndex(getFirst()); 
		while (getDataModel().isRowAvailable() && isNotEndRow()) {	
			Iterator children = getChildren().iterator();
			while (children.hasNext()) {
				UIComponent child = (UIComponent) children.next();
				
				if (!child.isRendered()) {
					continue;
				}
				child.processUpdates(context);
			}
			setIndex(++index);
		}
		setIndex(-1);
	}
	
	/* (non-Javadoc)
	 * @see javax.faces.component.UIComponent#processValidators(javax.faces.context.FacesContext)
	 */
	public void processValidators(FacesContext context) {
		if (!isRendered()) {
			return;
		}
		if (isNestedInUIIterator()) {
			model = null;
		}
		
		setIndex(getFirst()); 
		while (getDataModel().isRowAvailable() && isNotEndRow()) {	
			Iterator children = getChildren().iterator();
			while (children.hasNext()) {
				UIComponent child = (UIComponent) children.next();
				
				if (!child.isRendered()) {
					continue;
				}
				child.processValidators(context);
			}
			setIndex(++index);
		}
		setIndex(-1);
	}
	
	protected boolean isNestedInUIIterator(){
		UIComponent parent = getParent();
		while(parent != null){
			if(parent instanceof UIDataIterator){
				return true;
			}
			parent = parent.getParent();
		}
		return false;
	}
	
	
	/* (non-Javadoc)
	 * @see javax.faces.component.UIComponent#encodeBegin(javax.faces.context.FacesContext)
	 */
	public void encodeBegin(FacesContext context) throws IOException {
		//reset model
		model = null;
		if(!isNestedInUIIterator()){
			savedValues = new HashMap();
		}
		super.encodeBegin(context);
	}
	
	public boolean isRowAvailable(){
		return getDataModel().isRowAvailable();		
	}
	
	
	public int getRowCount(){
		return getDataModel().getRowCount();
	}
	
	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#restoreState(javax.faces.context.FacesContext, java.lang.Object)
	 */
	public void restoreState(FacesContext context, Object state) {
        Object values[] = (Object[]) state;
        super.restoreState(context, values[0]);
        index = ((Integer) values[1]).intValue();
	    savedValues = (HashMap) values[2];
        value = values[3];
        var = (String) values[4];
        first = ((Integer)values[5]).intValue();
        firstSet = (Boolean) values[6];
        rows = ((Integer)values[7]).intValue();
        rowsSet = (Boolean) values[8];
	}
	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#saveState(javax.faces.context.FacesContext)
	 */
	public Object saveState(FacesContext context) {
        Object values[] = new Object[9];
        values[0] = super.saveState(context);
        values[1] = new Integer(index);
        values[2] = savedValues;
        values[3] = value;
        values[4] = var;
        values[5] = new Integer(first);
        values[6] = firstSet;
        values[7] = new Integer(rows);
        values[8] = rowsSet;
        return (values);
	}
	
    /*
     * (non-Javadoc)
     * 
     * @see javax.faces.component.UIComponentBase#queueEvent(javax.faces.event.FacesEvent)
     */
    public void queueEvent(FacesEvent event) {
        super.queueEvent(new DataIndexEvent(this, event, getIndex()));
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.faces.component.UIComponentBase#broadcast(javax.faces.event.FacesEvent)
     */
    public void broadcast(FacesEvent event) throws AbortProcessingException {
        if (event instanceof DataIndexEvent) {
        	DataIndexEvent die = (DataIndexEvent) event;
            int currIndex = getIndex();
            
            //instead of xfaces code...
            setIndex(die.getIndex());
            die.getEvent().getComponent().broadcast(die.getEvent());
            setIndex(currIndex);
            
            /* xfaces code
             * 
            // set up the row item list in the stack
            FacesContext context = getFacesContext();
            createRowItem(context);

            setIndex(ife.getIndex());
            FacesEvent wrappedEvent = ife.getEvent();
            wrappedEvent.getComponent().broadcast(wrappedEvent);

            // remove row item list from the stack
            destroyRowItem(context);

            setIndex(currIndex);
            */
        }
        else {
            super.broadcast(event);
        }
    }

    private boolean isNotEndRow()
    {
    	boolean isnotendrow = true;
    	
    	if(getRows() > 0)
    		isnotendrow = !( getIndex() > ( getFirst() + getRows() -1));
    	
    	return isnotendrow;
    }
}
