/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.component;

import java.io.OutputStream;
import java.util.Iterator;
import java.util.ResourceBundle;

import javax.faces.FacesException;
import javax.faces.application.FacesMessage;

import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.fileupload.util.ContentElement;
import com.ibm.faces.util.Base64;

/**
 * <strong>UIFileupload </strong> is a
 * {@link javax.faces.component.UIComponent UIComponent}that allows the client
 * to upload a file to the server. The HTML form that the component is used on
 * must be encoded with "multipart/form-data".
 * <p>
 * <b>Tags: </b> {@link com.ibm.faces.taglib.html_extended.FileuploadTag}<br>
 * <b>Renderers: </b>
 * {@link com.ibm.faces.renderkit.html_extended.FileuploadRenderer}
 * 
 * @see <a
 *      href="http://www.ietf.org/rfc/rfc1867.txt">http://www.ietf.org/rfc/rfc1867.txt
 *      </a>
 */
public class UIFileupload extends UIInput {
	public final static int VALUE_REF_BYTE_ARRAY = 0;

	public final static int VALUE_REF_STRING = 1;

	public final static int VALUE_REF_STREAM = 2;

	private int valueRefType;

	private ContentElement value;

	private String filename;

	private String mimetype;

	/**
	 * <p>
	 * The standard component family for this component.
	 * </p>
	 */
	public static final String COMPONENT_FAMILY = "com.ibm.faces.Fileupload"; //$NON-NLS-1$

	/**
	 * Returns component type.
	 * 
	 * @return component type
	 */
	public String getComponentType() {
		return UIFileupload.class.getName();
	}

	/**
	 * Sets value.
	 * 
	 * @param valueRef
	 *            new value
	 * 
	 * @see javax.faces.component.UIOutput#setValue(java.lang.Object)
	 */
	public void setValue(Object valueRef) {
		value = (ContentElement) valueRef;
	}

	/**
	 * Returns value.
	 * 
	 * @return current value
	 * 
	 * @see javax.faces.component.UIOutput#getValue()
	 */
	public Object getValue() {
		return value;
	}

	/**
	 * Returns current value.
	 * 
	 * @return current value
	 *  
	 */
	public Object getLocalValue() {
		return value;
	}

	/**
	 * Returns a flag indicating whether this component is responsible for
	 * rendering its child components.
	 * 
	 * @see javax.faces.component.UIComponent#getRendersChildren()
	 */
	public boolean getRendersChildren() {
		return true;
	}

	/**
	 * Updates the model data associated with this component.
	 * <p>
	 * <ul>
	 * <li>Creates a data header block containing filename and mime type, so
	 * that the uploaded data can be interpreted later (for example, during
	 * download)
	 * <li>Data processing depends on the type of the object being used to
	 * store data (referenced by <code>valueRef</code> attribute)
	 * <ul>
	 * <li>If stored as a String, encodes data using base64 encoding
	 * <li>If stored as a byte array, no further processing is required
	 * </ul>
	 * <li>Stores created header and processed data in the model.
	 * <ul>
	 * 
	 * @param context
	 *            {@link FacesContext}for the request we are processing
	 * 
	 * @exception NullPointerException
	 *                if <code>context</code> is <code>null</code>
	 * 
	 * @see javax.faces.component.UIComponent#updateModel(javax.faces.context.FacesContext)
	 */
	public void updateModel(FacesContext context) {
		if (context == null) {
			throw new NullPointerException();
		}
		if (!isValid()) {
			return;
		}
		if (value == null) {
			return;
		}
		//updateModel for child UIFileProps
		//RATLC01076914 - update these even if a value attribute hasn't been set fileupload tag
		Iterator kids = getChildren().iterator();
		while (kids.hasNext()) {
			Object kid = kids.next();
			if (kid instanceof UIFileProp) {
				UIFileProp prop = (UIFileProp) kid;

				if ((prop.getName() == null)
						|| ((!prop.getName().equalsIgnoreCase(
								"filename")) && (!prop.getName()
								.equalsIgnoreCase("contenttype")))) {
					continue;
				}
				if (prop.getName().equalsIgnoreCase("filename")) {
					prop.setValue(getFilename());
				} else {
					prop.setValue(getMimetype());
				}
				ValueBinding pvb = prop.getValueBinding("value");
				if (pvb != null) {
					pvb.setValue(context, prop.getValue());
				}
			}

		}
		
		try {
			ValueBinding vb = getValueBinding("value"); //$NON-NLS-1$
			if (vb != null) {
				Class valueType = vb.getType(context);
				if (valueType.getName().equals("java.lang.String")) { //$NON-NLS-1$
					valueRefType = VALUE_REF_STRING;
				} else if ((valueType.getName()).equals("[B")) { //$NON-NLS-1$
					valueRefType = VALUE_REF_BYTE_ARRAY;
				} else {
					Object val = vb.getValue(context);
					if (val instanceof OutputStream) {
						valueRefType = VALUE_REF_STREAM;
					} else {
						throw new IllegalArgumentException(ResourceHandler.getServerString("FileUpload.UnsupportedValueType"));
					}
				}

				switch (valueRefType) {
				case VALUE_REF_BYTE_ARRAY: {
					//RATLC00285146 - make more memory efficient
					/*
					 * ByteArrayOutputStream baos = new ByteArrayOutputStream();
					 * baos.write(value.getContentValue()); vb.setValue(context,
					 * baos.toByteArray());
					 */
					vb.setValue(context, value.getContentValue());
					break;
				}
				case VALUE_REF_STRING: {
					//RATLC00285146 - make more memory efficient
					/*
					 * byte[] buf = value.getContentValue(); buf =
					 * Base64.encode(buf); String raw_data = new String(buf,
					 * "ISO-8859-1"); //$NON-NLS-1$ vb.setValue(context,
					 * raw_data);
					 */
					vb
							.setValue(context, Base64.encode(value
									.getContentValue()));
					break;
				}
				case VALUE_REF_STREAM: {
					//RATLC00285146 - make more memory efficient
					//byte[] buf = value.getContentValue();
					OutputStream os = (OutputStream) vb.getValue(context);
					os.write(value.getContentValue());
					vb.setValue(context, os);
					break;
				}
				}

				setValid(true);
				//SAVING FILE TO TEMP DIR
				//String fname = writeToTempDir(value.getName());
				//System.out.println("FILE WRITTEN:"+fname);
				return;
			}
			//RATLC00285146
		} catch (OutOfMemoryError e) {
			ResourceBundle rb = ResourceHandler.getBundle(context);
			System.err.println("OutOfMemoryError=fileupload : "
					+ ResourceHandler.getString(rb,
							"FileUpload.ServerOutOfMemoryErr"));
			FacesMessage m = new FacesMessage(FacesMessage.SEVERITY_ERROR,
					ResourceHandler.getString(rb,
							"FileUpload.ServerOutOfMemoryErr"), ResourceHandler
							.getString(rb, "FileUpload.ServerOutOfMemoryErr"));
			context.addMessage(getClientId(context), m);
			setValid(false);
		} catch (FacesException e) {
			System.err.println("FacesException=" + e.getMessage()); //$NON-NLS-1$
			setValid(false);
			throw e;
		} catch (IllegalArgumentException e) {
			System.err.println("IllegalArgumentException=" + e.getMessage()); //$NON-NLS-1$
			setValid(false);
			throw e;
		} catch (Exception e) {
			System.err.println("Exception=" + e.getMessage()); //$NON-NLS-1$
			setValid(false);
			throw new FacesException(e);
		}
	}

	/**
	 * Writes uploaded item to java.io.tmpdir directory.
	 * 
	 * @param fname
	 *            name for the file
	 * 
	 * @return String representing path to the newly created file including
	 *         filename
	 * @throws IOException
	 * 
	 * protected String writeToTempDir(String fname) throws IOException { String
	 * name = ""; //$NON-NLS-1$ String ext = ""; //$NON-NLS-1$ int npos =
	 * fname.lastIndexOf(File.separator); int epos = fname.lastIndexOf(".");
	 * //$NON-NLS-1$
	 * 
	 * if (epos != -1) { name = fname.substring(npos + 1, epos); ext =
	 * fname.substring(epos); } else { name = fname.substring(npos + 1); ext =
	 * ""; //$NON-NLS-1$ }
	 * 
	 * File tempFile = File.createTempFile(name, ext); FileOutputStream fos =
	 * new FileOutputStream(tempFile);
	 * 
	 * InputStream in = value.getInputStream();
	 * 
	 * while (in.available() > 0) { fos.write(in.read()); } fos.close();
	 * in.close(); return tempFile.getPath(); }
	 */

	/**
	 * @return String file name of the uploaded item. This is the name that came
	 *         from the client including path to the file e.g.
	 *         <code>c:\temp\readme.txt</code>
	 */
	public String getFilename() {
		return filename;
	}

	/**
	 * @return String mimetype of the uploaded item
	 */
	public String getMimetype() {
		return mimetype;
	}

	/**
	 * Setter for the filename property
	 * 
	 * @param string
	 *            represents filename of the uploaded item
	 */
	public void setFilename(String string) {
		filename = string;
	}

	/**
	 * Setter for the mimetype property
	 * 
	 * @param string
	 *            mimetype of the uploaded item
	 */
	public void setMimetype(String string) {
		mimetype = string;
	}

	public String getFamily() {
		return (COMPONENT_FAMILY);
	}

	public void validate(FacesContext context) {
		// just need to enforce the required attribute here
		if (isValid() && isRequired() && null == getValue()) {
			ResourceBundle rb = ResourceHandler.getBundle(context);
			FacesMessage message = new FacesMessage(ResourceHandler.getString(
					rb, "ValidationMessage.value_is_required"));
			message.setSeverity(FacesMessage.SEVERITY_ERROR);
			context.addMessage(getClientId(context), message);
			setValid(false);
		}
	}

	 /*JSF state saving/restoration methods*/
	 
	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#restoreState(javax.faces.context.FacesContext, java.lang.Object)
	 */
	public void restoreState(FacesContext context, Object state) {
       Object values[] = (Object[]) state;
       super.restoreState(context, values[0]);
       
       filename 		= (String)values[1];
       //mimetype 		= (String)values[2];
       //valueRefType 	= ((Integer)values[3]).intValue();
       //value 			= (ContentElement)restoreAttachedState(context, values[4]);
   
       
	}
	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#saveState(javax.faces.context.FacesContext)
	 */
	public Object saveState(FacesContext context) {
       Object values[] = new Object[2];
       values[0] = super.saveState(context);
       
       values[1] =  filename;
      //values[2] =  mimetype;
      //values[3] =  new Integer(valueRefType);
      //values[4] =  saveAttachedState(context, value);
               
       return (values);
	}	
	
}
