/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/
package com.ibm.faces.component;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.faces.FacesException;
import javax.faces.component.UICommand;
import javax.faces.component.UIComponent;
import javax.faces.component.UIData;
import javax.faces.context.FacesContext;
import javax.faces.event.AbortProcessingException;
import javax.faces.event.ActionEvent;
import javax.faces.event.ActionListener;
import javax.faces.event.FacesEvent;

import com.ibm.faces.event.PagerEvent;
import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;
import com.ibm.faces.renderkit.html_extended.Utils;

/**
 * <p>Base component for data paging components.</p>
 * @author Alasdair McDowell/Paul Andrews
 */
public class UIPager extends UICommand implements ActionListener {
	
	private PagerDataWrapper pagerData = null;
	protected String UNSUPPORTED_PAGED_COMPONENT = "com.ibm.faces.UNSUPPORTED_PAGED_COMPONENT";
	public static final String PAGED_INNER_TABLE_IDS = "com.ibm.faces.PAGED_INNER_TABLE_IDS";
	
	
	private boolean innerTable = false;
	private Boolean innerTableSet = Boolean.FALSE;
	
	
	
	/**
	 * @return Returns true if the pager is for a inner dataTable.
	 */
	public boolean isInnerTable() {
		if(!innerTableSet.booleanValue()){
			UIData d = Utils.findParentUIData(this);
			if(d != null){
				d = Utils.findParentUIData(d.getParent());
				if(d != null){
					innerTable = true;	
				}
			}
			innerTableSet = Boolean.TRUE;
		}
		return innerTable;
	}

	/**
	 * @return Returns the pager data component wrapper.
	 */
	public PagerDataWrapper getPagerDataWrapper() {
		
		String dataId = (String)getAttributes().get("for"); //$NON-NLS-1$

        UIComponent data = null;

        if (dataId != null) {
            data = findComponent(dataId);
        }
        if (data == null) {
            data = findDataParent(this);
        }
        
        if(data == null){
        	return null;
        }
        
        this.pagerData = new PagerDataWrapper(data,isInnerTable());
		return pagerData;
	}
	
	public void encodeBegin(FacesContext context) throws IOException {
		resetId();
		//inner table pager state storing mechanism
		if(isInnerTable()){
			//get parent table
			UIData data = Utils.findParentUIData(this);
			if(data != null){//which it should if we get this far
				Map tableIdMap = (Map) context.getExternalContext().getSessionMap().get(PAGED_INNER_TABLE_IDS);
				if(tableIdMap == null){
					tableIdMap = new HashMap();
					context.getExternalContext().getSessionMap().put(PAGED_INNER_TABLE_IDS,tableIdMap);
				}
				String tableId = data.getClientId(context);

				
				int rows = data.getRows();
				int rowCount = data.getRowCount();
				
				//add this id with first|rows|rowcount int[] property to the map if it doesn't already exist
				if(!tableIdMap.containsKey(tableId)){
					/*
					 *if hasn't been added before, start at 0 otherwise when outer
					 *table is paged, new pages inner tables will start at last innertable of 
					 *previous outer pager page.
					 */
					tableIdMap.put(tableId,new int[]{0,rows,rowCount});
					data.setFirst(0);
				} else {
					//update rows and rowcount
					int[] vals = (int[]) tableIdMap.get(tableId);
					vals[1] = rows;
					vals[2] = rowCount;
				}
			}
		}
		super.encodeBegin(context);
	}

	
	public void decode(FacesContext context) {
		resetId();
		super.decode(context);
	}
	
	/*
	 * Used to reset client ids, required when pager is on an inner table
	 */
	private void resetId(){
		if(HxClientRenderUtil.isContainedInPanel(this)){
			String id = this.getId();
			this.setId(id);
			
		}
	}
	
    /**
     * @see javax.faces.event.ActionListener#processAction(javax.faces.event.ActionEvent)
     * @param event
     * @throws AbortProcessingException
     */
    public void processAction(ActionEvent event)
        throws AbortProcessingException {
        if (event instanceof PagerEvent) {
            processAction((PagerEvent) event);
        }
    }

    private void processAction(PagerEvent pagerEvent) {
    	
    	PagerDataWrapper pdw = getPagerDataWrapper();
    	if(pdw == null){
    		return;
    	}

    	int rows = pdw.getRows();
        int first = pdw.getFirst();
    	
        if (rows <= 0) {
            //then no paging
            return;
        }
        int page = pagerEvent.getPage();

        //check page lies within appropriate range
        page = getCheckRange(page, rows);
        
        int currentPage = (first + 1) / rows;
        //if rows per page is just 1 then first is the correct current page 
        if(rows == 1) {
        	currentPage = first;
        }
         
        if((first + 1) % rows > 1){
        	currentPage++;
        }
        //now set the page relative to current page
        if(page > currentPage)
        	first += (page - currentPage) * rows;
        if(page < currentPage)
        	first -= (currentPage - page) * rows;
        
        if(first < 0)
        	first = 0;
        
        if(isInnerTable()){
        	int[] vals = getInnerTableIdMapVals();
        	vals[0] = first;
        } else {
        	//outer table
        	pdw.setFirst(first);	
        }
        
    }

    /**
     * Checks the target page lies between the minimum and maximum page
     * @param data
     * @param page
     * @param rows
     * @return
     */
    private int getCheckRange(int page, int rows) {
    	PagerDataWrapper pdw = getPagerDataWrapper();
    	
        if (page <= 0 || pdw == null) {
            return 0;
        }
        
        int max = pdw.getRowCount();
        int first = pdw.getFirst();	
        
        if (max > 0) {
        	//RATLC01087304
        	//need to calc last page based on first attribute being set by user
        	
        	//lastPage = current  + no of pages from this first and no of pages behind
        	//before
        	int lastPage = first / rows;
        	if(first % rows > 0 )
        		lastPage++;
        	//after
        	lastPage += (max - (first + rows)) / rows;
        	if ((first + rows) <= max && (max - (first + rows)) % rows > 0 )
        		lastPage++;
			//check	
			if(page > lastPage){
				page = lastPage;
			}        	
        }
        return page;
    }
    
    /*
     * conviently get vals[] for inner table 
     */
    private int[] getInnerTableIdMapVals(){
    	Map tableMap = (Map) FacesContext.getCurrentInstance().getExternalContext().getSessionMap().get(PAGED_INNER_TABLE_IDS);
    	if(tableMap == null){
    		return null;
    	}
    	UIData data = Utils.findParentUIData(this);
    	if(data != null){
    		return (int[])tableMap.get(data.getClientId(FacesContext.getCurrentInstance()));	
    	} 
    	return null;
    }

    /**
     * @see javax.faces.component.UIComponent#broadcast(javax.faces.event.FacesEvent)
     * @param event
     * @throws AbortProcessingException
     */
    public void broadcast(FacesEvent event) throws AbortProcessingException {
        if (event instanceof ActionEvent) {
            this.processAction((ActionEvent) event);
        }

        super.broadcast(event);
    }

	/**
	 * <p>The standard component family for this component.</p>
	 */
	public static final String COMPONENT_FAMILY = "com.ibm.faces.Pager"; //$NON-NLS-1$

	public String getFamily() {
		return (COMPONENT_FAMILY);
	}
	
	/*
	 * <p>Returns the current page no.</p>
	 */
	public int getCurrentPage(PagerDataWrapper pdw){
		if(pdw == null){
			return 0;
		}

		int rows = pdw.getRows();
        int first = pdw.getFirst();
        		
        if (rows <= 0) {
            return 0;
        }
        
		//RATLC01087304        
        int current = first / rows;
        if(first % rows > 0){
        	current++;
        }
        
        if(current > getLastPage(pdw)){
        	current = 0;
        	pdw.setFirst(0);
        }
        return current;
	}
	
	/*
	 * <p>Returns the last page number</p>
	 */
	public int getLastPage(PagerDataWrapper pdw){
		
		if(pdw == null){
			return -1;
		}

		int rows  = pdw.getRows();
        int first = pdw.getFirst();
        int rowCount = pdw.getRowCount();
        
        if (rows <= 0) {
            return 0;
        }
        int max = rowCount;
        if (max < 0) {
            return -1;
        }
        else if (max > 0) {
        	//RATLC01087304
        	//need to calc last page based on first being set by user
        	//lastPage = current  + no of pages from this first and no of pages behind
        	//before
        	int lastPage = first / rows;
        	if(first % rows > 0 )
        		lastPage++;
        	//after
        	lastPage += (max - (first + rows)) / rows;
        	if ((first + rows) <= max && (max - (first + rows)) % rows > 0 )
        		lastPage++;
        	
            return lastPage;
        }
        else {
            return 0;
        }
	}
	
	
    /**
     * Determines whether the data component is paged.
     * @return  <code>true</code> if rows &gt; 0
     */
    public boolean isPaged(PagerDataWrapper pdw) {
    	if(pdw == null){
    		return false;
    	}
    	if(isInnerTable()){
    		int[] vals = getInnerTableIdMapVals();
    		if(vals != null) {
    			//vals[1] is rows field
    			return vals[1] > 0;
    		}
    	}
    	return pdw.getRows() > 0;
    
    }
	
    private UIComponent findDataParent(UIComponent component) {
        if (component == null) {
            return null;
        }
        if (component instanceof UIData || component instanceof UIDataIterator) {
            return component;
        }
        return findDataParent(component.getParent());
    }
    
    /*
     * <p>Wrapper for accessing methods on data components to be paged. 
     * These can be UIData and UIDataIterator components only/</p>
     * @author pandrews
     */
    public class PagerDataWrapper {
    	
    	UIComponent dataComponent = null;
    	boolean innerTable = false;
    	
    	/*
    	 * <p>Component must be UIData or UIDataIterator based.</p>
    	 */
    	public PagerDataWrapper(UIComponent component, boolean innerTable){
    		this.dataComponent = component;
    		this.innerTable = innerTable;
    	}
    	
    	/*
    	 * <p>Return the number of rows display.</p>
    	 */
    	public int getRows(){
    		if(this.innerTable){
    			int[] vals = this.getInnerTableIdMapVals();
    			if(vals != null){
    				return vals[1];
    			}
    		}
            if(dataComponent instanceof UIData){
            	return ((UIData)dataComponent).getRows();
            } else if (dataComponent instanceof UIDataIterator){
            	return ((UIDataIterator)dataComponent).getRows();
            } else {
            	throw new FacesException(UNSUPPORTED_PAGED_COMPONENT);
            }
    	}

    	/*
    	 * <p>Sets the number of rows display.</p>
    	 */
    	public void setRows(int rows){
    		if(this.innerTable){
    			int[] vals = this.getInnerTableIdMapVals();
    			if(vals != null){
    				vals[1] = rows;
    				return;
    			}
    		}
            if(dataComponent instanceof UIData){
            	((UIData)dataComponent).setRows(rows);
            } else if (dataComponent instanceof UIDataIterator){
            	((UIDataIterator)dataComponent).setRows(rows);
            } else {
            	throw new FacesException(UNSUPPORTED_PAGED_COMPONENT);
            }
    	}
    	/*
    	 * <p>Returns the zero relative index of the first row to display.</p>
    	 */
    	public int getFirst(){
    		if(this.innerTable){
    			int[] vals = this.getInnerTableIdMapVals();
    			if(vals != null){
    				return vals[0];
    			}
    		}
            if(dataComponent instanceof UIData){
            	return ((UIData)dataComponent).getFirst();
            } else if (dataComponent instanceof UIDataIterator){
            	return ((UIDataIterator)dataComponent).getFirst();
            } else {
            	throw new FacesException(UNSUPPORTED_PAGED_COMPONENT);
            }
    	}
    	/*
    	 * <p>Sets the zero relative index of the first row to display.</p>
    	 */
    	public void setFirst(int first){
    		if(this.innerTable){
    			int[] vals = this.getInnerTableIdMapVals();
    			if(vals != null){
    				vals[0] = first;
    				return;
    			}
    		}
            if(dataComponent instanceof UIData){
            	((UIData)dataComponent).setFirst(first);
            } else if (dataComponent instanceof UIDataIterator){
            	((UIDataIterator)dataComponent).setFirst(first);
            } else {
            	throw new FacesException(UNSUPPORTED_PAGED_COMPONENT);
            }
    	}
    	/*
    	 * <p>Returns the total count of rows in the data.</p>
    	 */
    	public int getRowCount(){
    		if(this.innerTable){
    			int[] vals = this.getInnerTableIdMapVals();
    			if(vals != null){
    				return vals[2];
    			}
    		}
            if(dataComponent instanceof UIData){
            	return ((UIData)dataComponent).getRowCount();
            } else if (dataComponent instanceof UIDataIterator){
            	return ((UIDataIterator)dataComponent).getRowCount();
            } else {
            	throw new FacesException(UNSUPPORTED_PAGED_COMPONENT);
            }
    	}
    	
        /*
         * conviently get vals[] for inner table, performance improvement for this class
         * to have its own version 
         */
        private int[] getInnerTableIdMapVals(){
        	Map tableMap = (Map) FacesContext.getCurrentInstance().getExternalContext().getSessionMap().get(PAGED_INNER_TABLE_IDS);
        	UIData data = Utils.findParentUIData(this.dataComponent);
        	if(data != null){
        		return (int[])tableMap.get(data.getClientId(FacesContext.getCurrentInstance()));	
        	} 
        	return null;
        }
    }
    
    

}
