/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/
package com.ibm.faces.component;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import javax.faces.component.UIComponent;
import javax.faces.component.UIComponentBase;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.el.MethodBinding;

import com.ibm.faces.util.AjaxUtil;
import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;
import com.ibm.faces.util.JavaScriptUtil;
import com.ibm.faces.util.LifecycleUtil;

/**
 * <strong>UIScriptCollector</strong> is a {@link javax.faces.component.UIComponent UIComponent} 
 * that wraps other components on a JSF page.
 * <p>
 * It provides the following services:
 * <ul>
 * <li>It collects names of JavaScript libraries that would be needed in the
 * following SCRIPT block by the collected and contributed scripts. <br>
 * Components that need a JavaScript library included in the page should call
 * <code>addLibrary</code> method to have their libraries registered with the
 * ScriptCollector.
 * <li>It can create and write an include statement for a JavaScript library
 * directly into ResponseWriter.<br>
 * Components that need a JavaScript library included at certain places inside
 * the page should call <code>writeLibraryReference</code> method to have
 * their libraries included and registered with the ScriptCollector.
 * <li>It collects JavaScript code from its children to be outputted after all these
 * components are rendered.<br>
 * Components that want to add JavaScript code at the end of the page should call
 * <code>addScript</code> or <code>addScriptOnce</code> methods to have their scripts
 * registered with the ScriptCollector. These scripts will be outputed as a part of
 * <code>encodeEnd</code> phase of ScriptCollector's rendering.
 * <li>It collects script contributors (classes that implement {@link IScriptContributor})
 * and lets them output JavaScript code after all children components are rendered.<br>
 * Components that do not want to (or can't) add scripts as described in the previous
 * paragraph, can implenent {@link IScriptContributor} interface and register themselves
 * with the ScriptCollector. In this case their <code>contributeScript</code> methods
 * will be called as a part of <code>encodeEnd</code> phase of ScriptCollector's rendering.
 * <li>It allows event scripting by exposing <code>preRender</code>, 
 * <code>postRender</code> and <code>decode</code> attributes. These attributes
 * can contain names of the methods in a Java class (code behind class, for example)
 * that take a {@link javax.faces.context.FacesContext FacesContext} parameter
 * and return <code>void</code>. These methods are called at the corresponding
 * phases of the ScriptCollector's rendering:
 * <ul>
 * <li>preRender is called before ScriptCollector and all its children render
 * <li>postRender is called after ScriptCollector and all its children have been rendered
 * <li>decode is called when ScriptCollector is being decoded, ie the page is being
 * submitted to the server
 * </ul>
 * </ul>
 * <p>
 * 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.ScriptCollectorTag}<br>
 * <b>Renderers:</b> {@link com.ibm.faces.renderkit.html_extended.ScriptCollectorRenderer}
 */

public class UIScriptCollector extends UIComponentBase {
	/**
	 * <p>The standard component type for this component.</p>
	 */
	public static final String COMPONENT_TYPE = "com.ibm.faces.ScriptCollector"; //$NON-NLS-1$

	/**
	 * <p>The standard component family for this component.</p>
	 */
	public static final String COMPONENT_FAMILY = "com.ibm.faces.ScriptCollector"; //$NON-NLS-1$

	/**
	 * Scope constant for find methods. 
	 * Looks for the nearest parent tag.
	 */
	public static final int FIND_NEAREST = 1;
	/**
	 * Scope constant for find methods. 
	 * Looks for the topmost parent tag.
	 */
	public static final int FIND_TOPMOST = 2;
	
	/**
	 * Constant for addScript methods.
	 * Scripts added with ADD_AT_TOP will be outputed
	 * at the top of the SCRIPT block, before
	 * the contributers.
	 */
	public static final int ADD_AT_TOP 		= 1;

	/**
	 * Constant for addScript methods.
	 * Scripts added with ADD_AT_BOTTOM will be outputed
	 * at the bottom of the SCRIPT block, after
	 * the contributers.
	 */
	public static final int ADD_AT_BOTTOM 	= 2;
	
	/**
	 * Constant for addScript methods.
	 * Scripts added with ADD_VERY_AT_BOTTOM will be outputed
	 * at the very bottom of the SCRIPT block, after
	 * the ADD_AT_BOTTOM scripts.
	 * FOR USE BY JWL ONPAGELOADONLY
	 */
	public static final int ADD_AT_VERY_BOTTOM 	= 3;
		
	private Vector scriptsTop = null;
	private Vector scriptsBottom = null;
	private Vector scriptsVeryBottom = null;
	private Vector libraryReferencesTop = null;
	private HashMap contributors = null;
	private HashMap libraries = null;
	private Vector writtenLibraries = null;
	private Vector writtenScripts = null;
	
	private MethodBinding preRender = null;
	private MethodBinding postRender = null;
	private MethodBinding decode = null;
	
	/**
	 * Constructor.
	 * Allocates buffers to store scripts, script contributors and libraries.
	 */
	public UIScriptCollector() {
		super();
		setRendererType("ScriptCollector"); //$NON-NLS-1$
		scriptsTop = new Vector();
		libraryReferencesTop = new Vector();
		scriptsBottom = new Vector();
		scriptsVeryBottom = new Vector();
		contributors = new HashMap();
		libraries = new HashMap();
		writtenLibraries = new Vector();
		writtenScripts = new Vector();
	}
	
	// -------------------------------------------------------------- Properties
	public String getFamily() {

		return (COMPONENT_FAMILY);

	}
    
	/**
	 * Adds a new script to the collection.
	 * @param newScript new script to add
	 * @param addWhere specifies where to add the script (top or bottom of the script block)
	 */
	public void addScript(String newScript, int addWhere) 
	{
		//Check to see if this script should be added to the page
		if(this.isRendering())
		{
			if(addWhere == ADD_AT_TOP) {
				scriptsTop.add(newScript);
			} else if(addWhere == ADD_AT_VERY_BOTTOM)
			{
				scriptsVeryBottom.add(newScript);
			}else {
				scriptsBottom.add(newScript);    
			}
		}
	}

	/**
	 * Adds a new script for the bottom of the script block.
	 * @param newScript new script to add
	 */
	public void addScript(String newScript) {
		addScript(newScript, ADD_AT_BOTTOM);
	}

	/**
	 * Adds a new script for the bottom of the script block, 
	 * making sure the exact same script wasn't already added.
	 * @param newScript new script to add
	 */
	public void addScriptOnce(String newScript) {
		addScriptOnce(newScript, ADD_AT_BOTTOM);
	}

	/**
	 * Adds a new script to the collection, making sure the exact same script
	 * wasn't already added.
	 * @param newScript new script to add
	 * @param addWhere specifies where to add the script (top or bottom of the script block)
	 */
	public void addScriptOnce(String newScript, int addWhere) {
		//Check to see if this script should be added to the page
		if(this.isRendering())
		{
			Vector scripts;
			if(addWhere == ADD_AT_TOP) {
				scripts = scriptsTop;
			}
			else if(addWhere == ADD_AT_VERY_BOTTOM)
			{
				scripts = scriptsVeryBottom;
			}else {
				scripts = scriptsBottom;    
			}
			
			for(int i = 0; i < scripts.size(); i++) {
				String str = (String) scripts.get(i);
				if(str.equals(newScript)) {
					return;
				}
			}	
			scripts.add(newScript);
		}
	}
	
	/**
	 * Returns all collected scripts for the top of the script block.
	 * @return collected scripts
	 */
	public Vector getScriptsTop() {
		return scriptsTop;
	}
	/**
	 * Returns all the collected script library references to be output 
	 * at the top of the page ---- Graham YOU added this
	 * @return the collected Library References.
	 */
	public Vector getLibraryReferencesTop()
	{
		return libraryReferencesTop;
	}

	/**
	 * Returns all collected scripts for the bottom of the script block.
	 * @return collected scripts
	 */
	public Vector getScriptsBottom() {
		return scriptsBottom;
	}
	
	/**
	 * Returns all collected scripts for the very bottom of the script block.
	 * @return collected scripts
	 */
	public Vector getScriptsVeryBottom() {
		return scriptsVeryBottom;
	}	
	
	/**
	 * Discards all collected scripts, script contributors and libraries.
	 * Resets buffers to zero length.
	 */
	public void reset() {
		scriptsTop.clear();
		scriptsBottom.clear();
		scriptsVeryBottom.clear();
		contributors.clear();
		libraries.clear();
		writtenLibraries.clear();
		writtenScripts.clear();
		libraryReferencesTop.clear();  //Graham YOU added this
	}
	
	/**
	 * Registers a new script contributor.
	 * 
	 * @param contributor contributor to register
	 * @param component component for the contributor to act on
	 */
	public void register(IScriptContributor contributor, UIComponent component) {
		if(!isRendering())
				return; // if you are in an ajax context and you're not meant to render so
						// you don't need to add script at all	
		contributors.put(component, contributor);
	}
	
	/**
	 * Returns all registerd script contributors.
	 * 
	 * @return all contributors in component/contributor Map
	 */
	public HashMap getContributors() {
		return contributors;
	}
	
	/**
	 * Finds the right {@link com.ibm.faces.component.UISriptCollector UISriptCollector} component for
	 * any given {@link javax.faces.component.UIComponent UIComponent},
	 * that is passed as a parameter.
	 * <p>
	 * In case there are multiple ScriptCollector tags on the page,
	 * it is important to be able to find the correct Collector
	 * for a component to register its scripts with. This method
	 * can find either the closest or the topmost ScriptCollector on the page.
	 * 
	 * @param component {@link javax.faces.component.UIComponent UIComponent} to
	 * start search with.
	 * @param scope scope for the search. This method can look either
	 * for the closest {@link com.ibm.faces.component.UISriptCollector UISriptCollector} 
	 * (FIND_NEAREST) or the topmost (FIND_TOPMOST).
	 * @return {@link com.ibm.faces.component.UISriptCollector UISriptCollector} 
	 * that was found according to the scope parameter.
	 */
	public static UIScriptCollector find(UIComponent component, int scope) {
		UIScriptCollector scriptCollector = null;
		while (null != (component =	component.getParent())) {
			if (component instanceof UIScriptCollector) {
				scriptCollector = (UIScriptCollector) component;
				if(scope == FIND_NEAREST) {
					return scriptCollector;
				}
			}
		}
		return scriptCollector;
	}

	/**
	 * Finds the topmost {@link com.ibm.faces.component.UISriptCollector UISriptCollector} component for
	 * any given {@link javax.faces.component.UIComponent UIComponent},
	 * that is passed as a parameter.
	 * 
	 * @param component {@link javax.faces.component.UIComponent UIComponent} to
	 * start search with.
	 * @return {@link com.ibm.faces.component.UISriptCollector UISriptCollector} 
	 * that was found
	 * 
	 * @see com.ibm.faces.component.UISriptCollector#find(javax.faces.component.UIComponent, int scope)
	 */
	public static UIScriptCollector find(UIComponent component) {
		return find(component, FIND_TOPMOST);
	}

	/**
	 * Adds a new script library to the collection, making sure the exact same library
	 * wasn't already added.
	 * @param newLib new library to add
	 */
	public void addLibrary(String newLib, String libBlock) {
		libraries.put(newLib, libBlock);
	}
	
	/**
	 * Adds a new script library to the collection, making sure the exact same library
	 * wasn't already added.
	 * @param newLib new library to add
	 */
	public void addLibrary(String newLib) {
		if (!libraries.containsKey(newLib)){
			StringBuffer libBlock = new StringBuffer(); 
			libBlock.append(JavaScriptUtil.getStartTag(newLib));
			libBlock.append(JavaScriptUtil.getEndTag());
			addLibrary(newLib, libBlock.toString());
		}
	}
	
	/**
	 * 
	 * Adds a library include for a javaScript file to a page once and once only
	 * 
	 * @param newLib new library to add
	 * @param context {@link javax.faces.context.FacesContext FacesContext} for the request being processed
	 * @throws IOException
	 */
	public void addLibraryReference(String newLib, FacesContext context) 
	{
		if(newLib != null) 
		{		
			if(context!=null)
			{
				newLib = context.getExternalContext().encodeResourceURL(LifecycleUtil.getResourceServletContextPath(context) + "/" + newLib); //$NON-NLS-1$
			}
			
			Iterator i = libraryReferencesTop.iterator();
			while(i.hasNext()) 
			{
				String str = (String) i.next();
				if(str.equals(newLib)) {
					return;
				}
			}
			
			for(int x = 0; x < writtenLibraries.size(); x++) {
				String str = (String) writtenLibraries.get(x);
				if(str.equals(newLib)) {
					return;
				}
			}
			
			libraryReferencesTop.add(newLib); 
			
		}
	}
	
		/**
	 * Writes an include statement for a script library into ResponseWriter, 
	 * making sure the exact same library wasn't already included.
	 * The URL for the library is encoded for the Portal using 
	 * ExternalContext#encodeResourceURL.
	 * 
	 * @param newLib new library to add
	 * @param context {@link javax.faces.context.FacesContext FacesContext} for the request being processed
	 * @throws IOException
	 */
	public void writeLibraryReference(String newLib, FacesContext context) {
		for(int i = 0; i < writtenLibraries.size(); i++) {
			String str = (String) writtenLibraries.get(i);
			if(str.equals(newLib)) {
				return;
			}
		}

		for(int i = 0; i < libraryReferencesTop.size(); i++)
		{
			String str = (String) libraryReferencesTop.get(i);
			if(str.equals(newLib))
			{
				return;
			}
		}
		
		ResponseWriter writer = context.getResponseWriter();

		// 278788
//		String lib = context.getExternalContext().encodeResourceURL(newLib);
		
		//RATLC00285401
		String lib = LifecycleUtil.getContextPath(context) + "/" + newLib; //$NON-NLS-1$

		try {
			writer.write(JavaScriptUtil.getStartTag(lib));
			writer.write(JavaScriptUtil.getEndTag());
			writer.write(JavaScriptUtil.JS_NEWLINE);
		} catch (IOException e) {
			e.printStackTrace();
		}

		writtenLibraries.add(newLib);
	}

	/**
	 * Writes an include statement for a script library into ResponseWriter, 
	 * making sure the exact same library wasn't already included.
	 * The URL for the library is encoded for the Portal using 
	 * ExternalContext#encodeResourceURL.
	 * 
	 * @param newLib new library to add
	 * @param context {@link javax.faces.context.FacesContext FacesContext} for the request being processed
	 * @throws IOException
	 */
	public void writeLibraryReference(FacesContext context, String newLib, String libBlock) {
		for(int i = 0; i < writtenLibraries.size(); i++) {
			String str = (String) writtenLibraries.get(i);
			if(str.equals(newLib)) {
				return;
			}
		}
		
		for(int i = 0; i < libraryReferencesTop.size(); i++)
		{
			String str = (String) libraryReferencesTop.get(i);
			if(str.equals(newLib))
			{
				return;
			}
		}

		ResponseWriter writer = context.getResponseWriter();

		// 278788
//		String lib = context.getExternalContext().encodeResourceURL(newLib);
		
		//RATLC00285401
		//String lib = LifecycleUtil.getContextPath(context) + "/" + newLib; //$NON-NLS-1$

		try {
			writer.write(JavaScriptUtil.JS_NEWLINE);
			writer.write(libBlock);
		} catch (IOException e) {
			e.printStackTrace();
		}

		writtenLibraries.add(newLib);
	}

 	/* Writes an include statement for a script library into ResponseWriter, 
	 * making sure the exact same library wasn't already included. The name
	 * passed to the call is the basic library name, without the directory
	 * or version numbers.
	 * 
	 * @param context {@link javax.faces.context.FacesContext FacesContext} for the request being processed
	 * @param newLib new library to add
	 *
	 */
	public void addSimpleLibraryReference(FacesContext context, String newLib) {
		String jsLib = JavaScriptUtil.getJavaScriptPath(context)
		              + newLib + HxClientRenderUtil.HX_VERSION
					  + ".js"; //$NON-NLS-1$
		this.addLibrary(jsLib);

	}
	public void addSimpleLibraryReference(String newLib) {
		addSimpleLibraryReference(FacesContext.getCurrentInstance(), newLib);
	}	

	/**
	 * Writes a Java Script into ResponseWriter, 
	 * making sure the exact same script wasn't already written.
	 * 
	 * @param newScript new script to write
	 * @param context {@link javax.faces.context.FacesContext FacesContext} for the request being processed
	 * @throws IOException
	 */
	public void writeScriptOnce(String newScript, FacesContext context, boolean bWrapInScriptBlock) {
		if(!isRendering())
			return;
		
		for(int i = 0; i < writtenScripts.size(); i++) {
			String str = (String) writtenScripts.get(i);
			if(str.equals(newScript)) {
				return;
			}
		}

		ResponseWriter writer = context.getResponseWriter();
		try {
			if(bWrapInScriptBlock) {		
				writer.write(JavaScriptUtil.JS_NEWLINE);
				writer.write(JavaScriptUtil.getStartTag());
			}
	
			writer.write(JavaScriptUtil.JS_NEWLINE);
			writer.write(newScript);

			if(bWrapInScriptBlock) {		
				writer.write(JavaScriptUtil.JS_NEWLINE);
				writer.write(JavaScriptUtil.getEndTag());
			}

		} catch (IOException e) {
			e.printStackTrace();
		} 

		writtenScripts.add(newScript);
	}
	
	/**
	 * Returns all collected libraries 
	 * @return Vector of strings with library names
	 */
	public Vector getLibraries() {
		return new Vector(libraries.keySet());
	}

	/**
	 * Returns all collected libraries 
	 * @return Vector of strings with library names
	 */
	public HashMap getLibraryBlocks() {
		return libraries;
	}



	/**
	 * @return
	 */
	public MethodBinding getDecode() {
		return decode;
	}

	/**
	 * @return
	 */
	public MethodBinding getPostRender() {
		return postRender;
	}

	/**
	 * @return
	 */
	public MethodBinding getPreRender() {
		return preRender;
	}

	/**
	 * @param binding
	 */
	public void setDecode(MethodBinding binding) {
		decode = binding;
	}

	/**
	 * @param binding
	 */
	public void setPostRender(MethodBinding binding) {
		postRender = binding;
	}

	/**
	 * @param binding
	 */
	public void setPreRender(MethodBinding binding) {
		preRender = binding;
	}
	
	
	public Object saveState(FacesContext context) {
		Object values[] = new Object[4];
		values[0] = super.saveState(context);
		values[1] = saveAttachedState(context,this.decode);
		values[2] = saveAttachedState(context,this.preRender);
		values[3] = saveAttachedState(context,this.postRender);
		return values;
	}
	
	public void restoreState(FacesContext context, Object valueArray) {
		Object values[] = (Object[])valueArray;
		super.restoreState(context, values[0]);
		this.decode = (MethodBinding) restoreAttachedState(context,values[1]);
		this.preRender = (MethodBinding) restoreAttachedState(context,values[2]);
		this.postRender = (MethodBinding) restoreAttachedState(context,values[3]);
	}
	
	/**
	 * This method checks to see if the current request is an Ajax request and whether
	 * or not the AjaxFacesContext is set to render. If the AjaxFacesContext has rendering
	 * set to true then the component that is being process at that time will be rendered
	 * to the page. If the component is being re-rendered to the page then so too must its
	 * javaScript. So this indicates whether or not the scriptCollector should add the
	 * script for the current component. 
	 * 
	 * @return boolean whether or not to add script blocks to the page
	 */
	public boolean isRendering()
	{
		boolean isRendering = true;
		FacesContext context = FacesContext.getCurrentInstance();
		if(AjaxUtil.isAjaxRequest(context))
		{
			if(AjaxUtil.isRendering(context)==false)
			{
				isRendering = false;
			}
		}
		return isRendering;
	}
}
