/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.context;

import java.util.Iterator;

import javax.faces.application.Application;
import javax.faces.application.FacesMessage;
import javax.faces.application.FacesMessage.Severity;
import javax.faces.component.UIViewRoot;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseStream;
import javax.faces.context.ResponseWriter;
import javax.faces.render.RenderKit;

import com.ibm.faces.util.AjaxUtil;

/**
 * AjaxFacesContext is a FacesContext implementation which is used for ajax.
 * 
 * @author mark_wallace@ie.ibm.com
 */
public class AjaxFacesContext extends FacesContext {
	
    private FacesContext _context;
    private AjaxExternalContext _externalContext;
    private ResponseWriter _nullResponseWriter = null;
	
	public AjaxFacesContext(FacesContext context) {
		_context = context;
      
        FacesContext.setCurrentInstance(this);
        _externalContext = new AjaxExternalContext(context);
	}

	private ResponseWriter getNullResponseWriter() {
		if(_nullResponseWriter==null) {
			_nullResponseWriter = new NullResponseWriter(_context);
		}
		return _nullResponseWriter;
	}
	
    /**
     * @since 1.0
     * @see javax.faces.context.FacesContext#getApplication()
     */
    public Application getApplication() {
        return _context.getApplication();
    }

    /**
     * @since 1.0
     * @see javax.faces.context.FacesContext#getClientIdsWithMessages()
     */
    public Iterator getClientIdsWithMessages() {
        return _context.getClientIdsWithMessages();
    }

    /**
     * @since 1.0
     * @see javax.faces.context.FacesContext#getExternalContext()
     */
    public ExternalContext getExternalContext() {
    	if(AjaxUtil.getAjaxComponentId(_context) == null){
        	return _context.getExternalContext();
    	}
   		return _externalContext;
    }

    /**
     * @since 1.0
     * @see javax.faces.context.FacesContext#getMaximumSeverity()
     */
    public Severity getMaximumSeverity() {
        return _context.getMaximumSeverity();
    }

    /**
     * @since 1.0
     * @see javax.faces.context.FacesContext#getMessages()
     */
    public Iterator getMessages() {
        return _context.getMessages();
    }

    /**
     * @since 1.0
     * @see javax.faces.context.FacesContext#getMessages(java.lang.String)
     */
    public Iterator getMessages(String clientId) {
        return _context.getMessages(clientId);
    }

    /**
     * @since 1.0
     * @see javax.faces.context.FacesContext#getRenderKit()
     */
    public RenderKit getRenderKit() {
        return _context.getRenderKit();
    }

    /**
     * @since 1.0
     * @see javax.faces.context.FacesContext#getRenderResponse()
     */
    public boolean getRenderResponse() {
        return _context.getRenderResponse();
    }

    /**
     * @since 1.0
     * @see javax.faces.context.FacesContext#getResponseComplete()
     */
    public boolean getResponseComplete() {
        return _context.getResponseComplete();
    }

    /**
     * @since 1.0
     * @see javax.faces.context.FacesContext#getResponseStream()
     */
    public ResponseStream getResponseStream() {
        return _context.getResponseStream();
    }

    /**
     * @since 1.0
     * @see javax.faces.context.FacesContext#setResponseStream(javax.faces.context.ResponseStream)
     */
    public void setResponseStream(ResponseStream responseStream) {
        _context.setResponseStream(responseStream);
    }

    /**
     * @since 1.0
     * @see javax.faces.context.FacesContext#getResponseWriter()
     */
    public ResponseWriter getResponseWriter() {
    	// ensure the response writer has been setup
    	ResponseWriter responseWriter = _context.getResponseWriter();
    	if (responseWriter == null) {
    		// this is the signal for the tag lib or XSP runtime to 
    		// create a response writer for this context
    		return null;
    	}
    	
    	// return based on the rendering state
    	if (AjaxUtil.isRendering(_context)) {
    		return responseWriter;
    	}
    	
    	return getNullResponseWriter();
    	
    }

    /**
     * @since 1.0
     * @see javax.faces.context.FacesContext#setResponseWriter(javax.faces.context.ResponseWriter)
     */
    public void setResponseWriter(ResponseWriter responseWriter) {
    	if(_externalContext.isWrappedDispatchRequest()) {
    		//then the response writer being passed here wraps a null stream passed into dispatch method
    		_context.setResponseWriter(_externalContext.getRealResponseWriter());
    	} else {
    		_context.setResponseWriter(responseWriter);
    	}
    }

    /**
     * @since 1.0
     * @see javax.faces.context.FacesContext#getViewRoot()
     */
    public UIViewRoot getViewRoot() {
        return _context.getViewRoot();
    }

    /**
     * @since 1.0
     * @see javax.faces.context.FacesContext#setViewRoot(javax.faces.component.UIViewRoot)
     */
    public void setViewRoot(UIViewRoot viewRoot) {
        _context.setViewRoot(viewRoot);
    }

    /**
     * @since 1.0
     * @see javax.faces.context.FacesContext#addMessage(java.lang.String, javax.faces.application.FacesMessage)
     */
    public void addMessage(String clientId, FacesMessage facesMessage) {
        _context.addMessage(clientId, facesMessage);
    }

    /**
     * @since 1.0
     * @see javax.faces.context.FacesContext#release()
     */
    public void release() {
        _context.release();
    }

    /**
     * @since 1.0
     * @see javax.faces.context.FacesContext#renderResponse()
     */
    public void renderResponse() {
        _context.renderResponse();
    }

    /**
     * @since 1.0
     * @see javax.faces.context.FacesContext#responseComplete()
     */
    public void responseComplete() {    	
        _context.responseComplete();
    }
    

	
}
