/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2000, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.databind;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.faces.application.Application;
import javax.faces.context.FacesContext;
import javax.faces.el.PropertyResolver;
import javax.faces.model.SelectItem;

/**
 * Used for computing the results of a "selectitems" EL expression, which
 * will result in a List of SelectItem entries to populate a JSF UISelect* 
 * component. The variable resolver for "selectitems" creates an instance of 
 * this class, after which the associated property resolver for this type 
 * stores each segment of the EL expression in the instance.  The general form
 * of the expression is "selectitems.beanName.labelFieldName.valueFieldName".
 * Since beanName itself could be replaced with a valid compound 
 * (multi-".") name that resolves to a scoped object, the evaluation of
 * the fields for labels and values is deferred until first access to the List.
 * At that point property resolution on the label and value fields is delegated
 * back to the regular JSF EL processing an internal List is set up to back the 
 * subsequent get method calls.  Only an extremely minimal implementation is 
 * provided for List
 * 
 *  or the internal
 * keySet, so changes to the implementation of UISelectBase in the JSF API
 * may impact this class.
 * 
 * @author spaxton
 */
public class SelectItemsWrapper {
	// list of the String segments for the EL expression
	private List segments = new ArrayList();
	
	// used to store the name/value pairs - null until first access
	private List internalList = null;
			
	public void addExpressionSegment(String segment) {
		segments.add(segment);
	}

	// initialize the internal List - this does the variable and property resolution
	private void initList() {
		if(internalList != null) {
			return;
		}
		
		internalList = new ArrayList();
		
		// strip off and store the last two expression segments, which are
		// the label and value fields of the bean
		int segs = segments.size();
		if(segs < 3) {
			// we don't have at least a bean name and the label and value fields
			return;
		}
		
		String valueFieldName = (String) segments.remove(segs-1);
		String labelFieldName = (String) segments.remove(segs-2); 
		
		// all remaining segment comprise the bean name - concat them
		StringBuffer buf = new StringBuffer();
		for(int i=0;i<segs-2;i++) {
			buf.append((String) segments.get(i));
			buf.append(".");		 //$NON-NLS-1$
		}
		
		// remove the last extra "." to arrive at the bean's name
		String beanName = buf.substring(0, buf.length()-1);
		
		FacesContext context = FacesContext.getCurrentInstance();
		Application app = context.getApplication();

		// find the bean object
		Object bean = app.createValueBinding("#{"+beanName+"}").getValue(context);
		
		if(bean == null) {
			return;
		}

		// get the property resolver
		PropertyResolver propResolver = app.getPropertyResolver();
		
		// slightly different processing for several bean types
		
/*		YK: Commented out Map case as a result of RATLC00975787.
 * 		considering Map is not really needed. If the bean is a Map,
 * 		it will fall into the 'else' case below and will be treated
 * 		correctly. There is no reason to have a special case for a Map
 * 		at the top level and then reimplement handling of all different
 * 		types, that happens in the 'else'-case anyway already.
 *  
  		if(bean instanceof Map) {
			// a Map as the bean assumes we're creating a single SelectItem with the appropriate fields from the map
			//RATLC00975787 fix for map field containing a list of strings 
			Map m = (Map) bean;
			Object elValue = m.get(valueFieldName);
			if(elValue instanceof List){
				Iterator iter = ((List)elValue).iterator();
				while(iter.hasNext()){
					Object item = iter.next();
					Object label=item.toString();
					Object value=item;
					if((label != null) && (value != null)){
						internalList.add(new SelectItem(value, label.toString()));
					}
				}
			} else {
				internalList.add(new SelectItem(elValue, m.get(labelFieldName).toString()));	
			}			
		} else 
*/
		if(bean instanceof List) {
			// given a List as the bean, we'll loop through and resolve each property and create SelectItem instances
			// for each key/value pair
			Iterator iter = ((List) bean).iterator();
			while(iter.hasNext()) {
				Object item = iter.next();
				Object label = propResolver.getValue(item, labelFieldName); 
				Object value = propResolver.getValue(item, valueFieldName);
				if((label != null) && (value != null)) {
					internalList.add(new SelectItem(value, label.toString()));
				}
			}
		} else if (bean instanceof Object[]) {
			// bean is an array, so iterate through each element - behavior is similar to the List case
			Object[] array = (Object[]) bean;
			for(int i=0;i<array.length;i++) {
				Object item = array[i];
				Object label = propResolver.getValue(item, labelFieldName); 
				Object value = propResolver.getValue(item, valueFieldName);
				if((label != null) && (value != null)) {
					internalList.add(new SelectItem(value, label.toString()));
				}
			}			
		} else {
			// the bean is single-valued - we'll need to examine the label and value properties now
			// to further determine special cases
			Object label = propResolver.getValue(bean, labelFieldName); 
			Object value = propResolver.getValue(bean, valueFieldName);

			if((label == null) || (value == null)) {
				return;
			} 
			
			if((label instanceof List) && (value instanceof List)) {
				List labels = (List) label;
				List values = (List) value;
				if(labels.size() == values.size()) {
					for(int i=0;i<values.size();i++) {
						internalList.add(new SelectItem(values.get(i), labels.get(i).toString()));
					}
				} else {
					// mismatched list sizes, don't know what the user wanted
					return;
				}
			} else if((label instanceof Collection) && (value instanceof Collection)) {
				// does this one make any sense?
				Collection labels = (Collection) label;
				Collection values = (Collection) value;
				Iterator labelsIter = labels.iterator();
				Iterator valuesIter = values.iterator();
				while(labelsIter.hasNext() && valuesIter.hasNext()) {
					internalList.add(new SelectItem(valuesIter.next(), labelsIter.next().toString()));
				}
			} else if((label instanceof Object[]) && (value instanceof Object[])) {
				// the properties are arrays, so create internal Lists from the values
				Object[] labels = (Object[]) label;
				Object[] values = (Object[]) value;
				for(int i=0;i<labels.length;i++) {
					internalList.add(new SelectItem(values[i], labels[i].toString()));
				}
			} else {
				// this appears to be a single name/value pair, so just stick them on the List as such
				internalList.add(new SelectItem(value, label.toString()));
			}
		}
	}

	public SelectItem[] toArray() {
		initList();
		return (SelectItem[]) internalList.toArray(new SelectItem[0]);
	}
}
