/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.fileupload.util;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.StringTokenizer;

/**
 * This class covers one part of a multipart content, which is parsed by the MultpiartContent class.
 *
 * @author Andreas Nagel
 */
public class ContentElement 
    {
    private static String TAG_CONTENT_TYPE = "Content-Type";
    private static String TAG_CONTENT_DISP = "Content-Disposition";
    private static String TAG_NAME = "name";
    private static String TAG_FILENAME = "filename";
    private String name;
    private String fileName;
    private String mimeType;
    boolean headerSection;
	private String encoding;
    /**
     * Content data store.
     */
    private ByteArrayOutputStream contentValue;
    private byte [] content;

    /**
     *  Constructor method.
     */
    public ContentElement (String enc)
       {
       headerSection = true; // we first except some content header information
       contentValue = new ByteArrayOutputStream (); 
       content = null;
	   encoding=enc;
       }

    /**
     * Set the name of the content part
     * 
     * @param n      Coneent part name
     */
    public void setName (String n)
       { name = new String (n); }

    /**
     * Return the name of the content part
     * 
     * @return String content part name
     */
    public String getName ()
       { return name; }

    /**
     * Set the file name of the conent part
     * 
     * @param n      file name of the content part.
     */
    public void setFileName (String n)
       { fileName = new String (n); }

    /**
     * Return the file name of the content part
     * 
     * @return Strigng file name
     */
    public String getFileName ()
       { return fileName; }

    /**
     * Set the mime type of the content part
     * 
     * @param n      Mime type (String representation)
     */
    public void setMimeType (String n)
       { mimeType = new String (n); }

    /**
     * Return the MIME type of the content part.
     * 
     * @return String Mime Type
     */
    public String getMimeType ()
       { return mimeType; }

    /**
     * Return the value of the content part.
     * 
     * @return byte [] Content part value
     */
    public byte [] getContentValue ()
       { 
       if (content == null)
          {
          // We must throw away the last CRLF characters.
          int realLength = this.getContentSize();
          //RATLC00285146 - remove temp field and set contentValue to null after correct number of bytes are copied to content field
          //byte [] temp = contentValue.toByteArray ();
          content = new byte [realLength];
          System.arraycopy (contentValue.toByteArray(), 0, content, 0, realLength);
          contentValue = null;
          }
       return content;
       }

    /**
     * Return the value of the content part as a String object.
     * 
     * @return String Content part value
     */
    public String getContentValueAsString ()
       {
       	
       String rc = null;
		try {
			rc = new String(getContentValue(), 0, getContentSize(), encoding);
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
       return rc;
       }

    /**
     * Return the size of the content value part.
     * 
     * @return int Content size
     */
    public int getContentSize ()
       { 
       int rc = 0;
       if (content == null)
          content = contentValue.toByteArray ();

       if (content != null)
          {
          if (content.length >= 2 && content[content.length-2] == 0x0d && content[content.length-1] == 0x0a)
             rc = content.length - 2;
          else
             rc = content.length; // this should never occur
          }
       return rc;
       }


    /**
     * Add a content line to the current content.
     * The line is either a part of the conent header, or a piece of the content data. Depending on its internal state,
     * the object either tries to extract header information, or appends the line to its internal content data store.
     * 
     * @param line   Content line object.
     */
    public void addToContent (ContentLine line)
       {
       // process headers  

       if (headerSection == true)
          {
          String s = line.getString ();
          String tag;
          String value;
          String val;
          int ix = s.indexOf (':');
          if (ix < 0) { tag = s; value = ""; } else { tag = s.substring (0, ix); value = s.substring (ix+1, s.length()); }

          // an empty line indicates that the content data section is started.
          if ( tag.equals("") && value.equals("")) 
             headerSection = false;  

          else if (tag.equalsIgnoreCase (TAG_CONTENT_TYPE))
             {
             mimeType = value.trim ();
             }
          else if (tag.equalsIgnoreCase (TAG_CONTENT_DISP))
             {
             StringTokenizer st = new StringTokenizer (value, ";");
             while (st.hasMoreTokens ())
                {
                String tok = st.nextToken().trim();
                ix = tok.indexOf ('=');
                if (ix < 0) { tag = tok; val = ""; } else { tag = tok.substring (0, ix); val = tok.substring (ix+1, tok.length()); }
                if (tag.equalsIgnoreCase (TAG_NAME))
                   name = new String (val.trim());
                if (tag.equalsIgnoreCase (TAG_FILENAME)){
                	//RATLC00477329 : filename will be truncated if contains a ; (tokenize char)
                	val = val.trim();
                	if(val.startsWith("\"") && !(val.endsWith("\""))){
                    	int x = value.indexOf(val);
                    	int y = value.indexOf("\"",x + 1);
                    	//filename may contain quotes also, can't assume y points to last one
                    	int z = value.indexOf("\"",y + 1);
                    	while(z > -1){
                    		if(value.charAt(z-1) == '='){
                    			//this is a quote belonging to next field
                    			break;
                    		} else {
                    			y = z;
                    		}
                    		z = value.indexOf("\"",z + 1);
                    	}
                    	fileName = value.substring(x,y + 1);
                	} else {
                		fileName = new String (val);
                	}
                }
             }
          }
          else if (tag.startsWith (" ") || tag.startsWith ("\t"))
             {
             StringTokenizer st = new StringTokenizer (tag.trim(), ";");
             while (st.hasMoreTokens ())
                {
                String tok = st.nextToken().trim();
                ix = tok.indexOf ('=');
                if (ix < 0) { tag = tok; val = ""; } else { tag = tok.substring (0, ix); val = tok.substring (ix+1, tok.length()); }
                //if (tag.equalsIgnoreCase (TAG_NAME))
                //   name = new String (val.trim());
                if (tag.equalsIgnoreCase (TAG_FILENAME))
                   fileName = new String (val.trim());
                }
             }
          }

       // process content values
       else
          {
          try {
             contentValue.write (line.getBytes ());
             }
          catch (IOException e) { System.err.println ("Exception!"); } // this never occurs!
          }
       }

    /**
     * Return the string representation of this object (only for tracing and logging).
     * 
     * @return 
     */
    public String toString ()
       {
       return TAG_CONTENT_TYPE + "=" + mimeType + "; "+ TAG_NAME + "=" + name;
       }

    }
  
