/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.jwlajaxencoder;

import javax.faces.context.FacesContext;

import com.ibm.faces.util.ContextParams;


/**
 * This is a factory class that creates ajaxurlencoder.
 * 
 */
public class AjaxUrlFactory {

	private static final String ERROR_MESSAGE_NO_LOAD_ENCODER_START = "the specified encoder class - '";	
	private static final String ERROR_MESSAGE_NO_LOAD_ENCODER_END = "' could not be instancied";
	
	private static final boolean IS_PORTAL_SUPPORTED = determineSupported();
	
	private static final boolean IS_SPECIFIED_ENCODER = determineSpecifiedSupported();
	
	private static final AjaxUrlEncoder DEFAULT_ENCODER = new DefaultAjaxUrlEncoder();
	
	private static final AjaxUrlEncoder SPECIFIED_ENCODER = createSpecifiedEncoder();
	
	private static final AjaxUrlEncoder PORTAL_ENCODER = createPortalEncoder();
		
	private static String USE_SPECIFIED_CLASS;
	
	public static AjaxUrlEncoder getAjaxUrlEncoder(FacesContext context) {
		if(IS_SPECIFIED_ENCODER)
		{
			return SPECIFIED_ENCODER;
		}
		else
		{
			if(PORTLET_REQUEST==null) {			
				return DEFAULT_ENCODER;
			}
			Object request = context.getExternalContext().getRequest();
			if(PORTLET_REQUEST.isInstance(request)) {			
				return PORTAL_ENCODER;
			} else {
				return DEFAULT_ENCODER;
			}
		}
	}

	/*
	 * This trys to create and return the Portal 6.0/IBM faces 
	 * portlet bridge ajaxurlencoder	 
	 * It will create it if we are on portal 6.0 and the 
	 * bridge is used. 
	 * Otherwise if returns the default encoder 
	 * */
	private static AjaxUrlEncoder createPortalEncoder() {

		try{
			
			if(IS_SPECIFIED_ENCODER) {
				return SPECIFIED_ENCODER;
			}
			
			if(!IS_PORTAL_SUPPORTED) {			
				return DEFAULT_ENCODER;
			}
			
			return new PortalAjaxUrlEncoder();
			
		}
		catch(Exception pe)
		{			
			return DEFAULT_ENCODER;
		}
	}
	
	private static Class PORTLET_REQUEST = loadRequestClass("javax.portlet.PortletRequest");
	
	private static Class loadRequestClass(String str) {
		try {
			return loadClass(str);
		} catch(ClassNotFoundException e) {
			return null;
		}
	}

	/*
	 * This trys to create and return the specified ajaxurlencoder
	 * will create it, if the class is specified and can 
	 * be loaded and instancised
	 * Otherwise if returns the default encoder 
	 * */
	private static AjaxUrlEncoder createSpecifiedEncoder() {
		if(IS_SPECIFIED_ENCODER)
		{		
			try {
				return (AjaxUrlEncoder)(loadClass(USE_SPECIFIED_CLASS)).newInstance();
			} catch(ClassNotFoundException e) 
			{
				System.err.println(ERROR_MESSAGE_NO_LOAD_ENCODER_START + USE_SPECIFIED_CLASS + ERROR_MESSAGE_NO_LOAD_ENCODER_START);
				return DEFAULT_ENCODER;
			}
			catch(Exception e)
			{
				System.err.println(ERROR_MESSAGE_NO_LOAD_ENCODER_START + USE_SPECIFIED_CLASS + ERROR_MESSAGE_NO_LOAD_ENCODER_START);
				return DEFAULT_ENCODER;
			}
		}
		else
		{
			return DEFAULT_ENCODER;
		}
	}

	/*
	 * This is used to figure out if we should create/use the specfied encoder
	 * It does this by seeing if the context param is set.	 
	 * */
	private static boolean determineSpecifiedSupported() {
		USE_SPECIFIED_CLASS = FacesContext.getCurrentInstance().getExternalContext().getInitParameter(ContextParams.JWL_AJAX_ENCODER_CLASS);
		if((USE_SPECIFIED_CLASS != null) && (USE_SPECIFIED_CLASS.trim().length()>0))
			return true;
		
		return false;
	}
	
	/*
	 * This is used to figure out if we are running on IBM Portal 6.0
	 * It does this by seeing if a required class and method call are available
	 * Both of which are private api calls available only in portal 6.0.
	 * */
	private static boolean determineSupported() {
		// this is here in case portal 5.1 breaks again, we can get prople to set this context param
		String TURN_OFF_PORTAL_60_PATH = FacesContext.getCurrentInstance().getExternalContext().getInitParameter(ContextParams.TURN_OFF_AJAX_PORTAL_60_PATH);
		if((TURN_OFF_PORTAL_60_PATH != null) && ("true".equalsIgnoreCase(TURN_OFF_PORTAL_60_PATH)))
			return false;
		
		String soloController ="com.ibm.portal.state.accessors.solo.SoloAccessorController";
		String soloMethod = "setSoloPortlet";
		String [] soloMethodParams = {"com.ibm.portal.ObjectID"};
		
		String stateManagerService ="com.ibm.portal.state.service.StateManagerService";

		String localeAcessor ="com.ibm.portal.state.accessors.locale.LocaleAccessor";
		String localeMethod = "getLocale";
		String [] localeMethodParams = {""};
		
		MethodDescription [] MethodDesc1 = {new MethodDescription(soloMethod,soloMethodParams)};		
		Check chk1 = new Check(soloController,MethodDesc1);
		MethodDescription [] MethodDesc2 = {new MethodDescription(localeMethod,localeMethodParams)};		
		Check chk2 = new Check(localeAcessor,MethodDesc2);		

		Check[] checks = {chk1, chk2};
		
		for(int i=0; i<checks.length; i++) {
			if(!checks[i].isSupported()) {
				return false;
			}
		}
		
		try{			// extra check that only needs to see if class is there
			Class x = loadClass(stateManagerService);			
			if(x == null)
				return false;			
		}catch(ClassNotFoundException e) {
			return false;
		}
		
		return true;
		
	}
	
	private static class Check {
		private String _className;
		private MethodDescription[] _methods;
		protected Check(String className, MethodDescription[] methods) {
			_className = className;
			_methods = methods;
		}
		public boolean isSupported() {
			try {
				Class c = loadClass(_className);
				
				for(int i=0; i<_methods.length; i++) {
					if(!_methods[i].isSupported()) {
						return false;
					}
					
					if((_methods[i].getParams())[0] != null)
						c.getMethod(_methods[i].getName(), _methods[i].getParams());
					else // this deals with a bean-like get
						c.getMethod(_methods[i].getName(), null);
				}
			} catch(ClassNotFoundException e) {				
				return false;
			} catch(NoSuchMethodException e) {
				return false;
			} 
			return true;
		}
	}
	
	private static class MethodDescription {
		private String _methodName;
		private String[] _paramClasses;
		private Class[] _params;
		protected MethodDescription(String methodName, String[] paramClasses) {
			_methodName = methodName;
			_paramClasses = paramClasses;
		}
		public boolean isSupported() {
			_params = new Class[_paramClasses.length];
			for(int i=0; i<_paramClasses.length; i++) {
				try {
					_params[i] = loadClass(_paramClasses[i]);
				} catch(ClassNotFoundException e) {
					return false;
				}
			}
			return true;
		}
		public Class[] getParams() {
			return _params;
		}
		public String getName() {
			return _methodName;
		}
	}
	
	protected static Class loadClass(String className) throws ClassNotFoundException {
		if("".equalsIgnoreCase(className))
			return null;
		ClassLoader cl = Thread.currentThread().getContextClassLoader();
		if(cl==null) {
			cl = AjaxUrlFactory.class.getClassLoader();
		}
		if(cl==null) {
			cl = ClassLoader.getSystemClassLoader();
		}
		return cl.loadClass(className);
	}

	
}
