/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.ResourceBundle;

import javax.faces.component.UIComponent;
import javax.faces.component.UIData;
import javax.faces.component.UIParameter;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.util.JavaScriptUtil;
import com.ibm.faces.util.LifecycleUtil;


/**
 * Abstract base class for renderers that are aware they are
 * used by UIData children.
 */
abstract class AbstractRowRenderer extends Renderer {

    /**
     * Finds the parent UIData of component.
     * @param component     the component
     * @return              the UIData parent or null if not found
     */
    protected static UIData findUIData(UIComponent component) {
        if (component == null) {
            return null;
        }
        if (component instanceof UIData) {
            return (UIData) component;
        }
        return findUIData(component.getParent());
    }

    /**
     * Finds the parent UIData's id.
     * @param context       the FacesContext
     * @param component     the component
     * @return              the UIData parent id or null if not found
     */    
    protected String findUIDataId(
        FacesContext context,
        UIComponent component) {
        UIComponent data = findUIData(component);
        if (data == null) {
            return null;
        }
        String dataId = data.getClientId(context);
        //comes in form someId:n, need to remove row #
        int idx = dataId.lastIndexOf(':');
        if (idx >= 0) {
            dataId = dataId.substring(0, idx);
        }
        //end of id fixing
        return dataId;
    }

    /**
     * Writes a script directly to the ResponseWriter
     * @param context       the context
     * @param src           the script library source (can be null)
     * @param script        the script (can be null)
     * @throws IOException
     */
    private void writeJavaScript(
        FacesContext context,
        String script)
        throws IOException {

        ResourceBundle bundle = ResourceHandler.getBundle(context);
        String src = JavaScriptUtil.getJavaScriptPath(context) + ResourceHandler.buildLocalizedLibName(bundle, "hxclient_v1.js"); //$NON-NLS-1$
        ResponseWriter writer = context.getResponseWriter();
        if (src != null) {
            writer.write(JavaScriptUtil.getStartTag(src));
            writer.write(JavaScriptUtil.getEndTag());
        }
        if (script != null) {
            writer.write(JavaScriptUtil.getStartTag());
            writer.write(script);
            writer.write(JavaScriptUtil.getEndTag());
        }
    }

    /**
     * Writes the given script to the script collector.
     * Fails safe by writing to the ResponseWriter if the script collector is null.
     * @param context           the FacesContext
     * @param scriptCollector   the script collector (can be null)
     * @param src               the script source library
     * @param script            the script
     * @throws IOException
     */
    protected void writeJavaScript(
        FacesContext context,
        UIScriptCollector scriptCollector,
        String script)
        throws IOException {

        if (scriptCollector != null) {
			HxClientRenderUtil.initJSLibraries(scriptCollector, context);
            if (script != null) {
                scriptCollector.addScript(script);
            }
        }
        else {
            //failsafe
            writeJavaScript(context, script);
        }
    }

    /**
     * Encodes parameters as hidden fields.
     * @param context       the FacesContext
     * @param component     the UIComponent
     * @throws IOException
     */
    protected void encodeParameters(
        FacesContext context,
        UIComponent component)
        throws IOException {
            
        UIParameter[] params = LifecycleUtil.getParameters(component);
        if (params == null || params.length == 0) {
            return;
        }

        ResponseWriter writer = context.getResponseWriter();
        for (int i = 0, len = params.length; i < len; i++) {
            String key = component.getClientId(context) + ":" + params[i].getName(); //$NON-NLS-1$
            Object value = params[i].getValue();
            if (value == null) {
                continue;
            }
            writer.startElement("input",component);
            writer.writeAttribute("type","hidden",null);
			writer.writeAttribute("id",key,null);
			writer.writeAttribute("name",key,null);
			writer.writeAttribute("value",value.toString(),null);
			writer.endElement("input");
			writer.write("\n");
        }
    }

    /**
     * Decodes the parameters from hidden fields.
     * @param context       the FacesContext
     * @param component     the UIComponent
     * @return              a Map of parameters or null if the component has no parameters
     */
    protected Map decodeParameters(
        FacesContext context,
        UIComponent component) {
        return this.decodeParameters(context, component, null);
    }

    /**
     * Decodes the parameters from hidden fields.
     * @param context       the FacesContext
     * @param component     the UIComponent
     * @param parameterMap  the map to store the decoded parameters on; if null, a new Map will be created
     * @return              a Map of parameters or null if the component has no parameters
     */
    protected Map decodeParameters(
        FacesContext context,
        UIComponent component,
        Map parameterMap) {
        UIParameter[] params = LifecycleUtil.getParameters(component);
        if ((params == null) || (params.length == 0)) {
            return null;
        }
        Map requestMap =
            FacesContext
                .getCurrentInstance()
                .getExternalContext()
                .getRequestParameterMap();
        if (parameterMap == null) {
            parameterMap = new HashMap();
        }
        for (int i = 0, len = params.length; i < len; i++) {
            String paramName = component.getClientId(context) + ":" + params[i].getName(); //$NON-NLS-1$
            Object value = requestMap.get(paramName);
            parameterMap.put(params[i].getName(), value);
        }
        return parameterMap;
    }

}
