/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.List;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.component.UIOutput;
import javax.faces.component.UIPanel;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlPanelActionbar;

/**
 * Renderer for {@link javax.faces.component.UIPanel UIPanel} component, 
 * such as a toolbar or a navigator that renders all children 
 * in a vertical or horizontal order.
 * 
 * <p>
 * Although all components are supported as children, this component
 * is geared towards hyperlinks and separators. Nested Actionbars are supported.
 * <p>
 * The renderer uses a table to arrange all children.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.Panel_ActionbarTag}<br>
 * <b>Components:</b> {@link javax.faces.component.UIPanel}
 */
public class ActionbarRenderer extends Renderer implements IScriptContributor {
	private static final String DEFAULT_LAYOUT = "pageDirection"; //$NON-NLS-1$
	private static final String DEFAULT_AUTOSEPARATE = "false"; //$NON-NLS-1$
	private static final String DEFAULT_SEPARATORSTYLE = "line"; //$NON-NLS-1$
	private static final String DEFAULT_SEPARATORSIZE = "12"; //$NON-NLS-1$
	private static final String DEFAULT_CLASS = "s-actionbar-Nav"; //$NON-NLS-1$
	private static final String DEFAULT_INDENT = "20"; //$NON-NLS-1$

	/** 
	 * Renders the beginning of the component.
	 * <p>
	 * Creates opening table tag. Children will be rendered
	 * in table cells later on in {@link ActionbarRenderer#encodeChildren encodeChildren}.
	 * <p>
	 * If this is a nested Actionbar, visibility is set according <code>nestedExpanded</code>
	 * attribute.
	 * <p>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeBegin(FacesContext context, UIComponent component)
		throws IOException {

		if (context == null || component == null) {
			throw new NullPointerException();
		}
		
		HtmlPanelActionbar actionbarComponent = null;
		if ( component instanceof HtmlPanelActionbar ){
			actionbarComponent = (HtmlPanelActionbar)component;
		}
		
		HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component);
			
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		
		String id;
		String styleClass;
		String title;
		
		if ( actionbarComponent != null ){
			styleClass = actionbarComponent.getStyleClass();
			title = actionbarComponent.getTitle();
		}else{
			styleClass = (String) component.getAttributes().get("styleClass");
			title = (String) component.getAttributes().get("title");
		}
		
		if (styleClass == null) styleClass = DEFAULT_CLASS;
		
		//if (id == null)
			id = component.getClientId(context);

		ResponseWriter writer = context.getResponseWriter();

		//add hidden field to keep track of expansion status
		if (isNestedActionbar(component)) {
			writer.startElement("input",component);
			writer.writeAttribute("id",id + "_display",null);
			writer.writeAttribute("name",id + "_display",null);
			writer.writeAttribute("type","hidden",null);
			writer.endElement("input");
		}
		writer.startElement("table",component);
		writer.writeAttribute("id",id/*+"_table"*/,null);
		writer.writeAttribute("class",styleClass,"styleClass");
		writer.writeAttribute("border","0",null);
		writer.writeAttribute("cellspacing","0",null);
		writer.writeAttribute("cellpadding","0",null);
		if(title != null){
			writer.writeAttribute("title",title,"title");
		}

		if (isNestedActionbar(component)) {
			//get and check if there is a valid expansion status for the nested actionbar 
			Map reqMap = context.getExternalContext().getRequestParameterMap();
			String dispStatus = (String) reqMap.get(id + "_display"); //$NON-NLS-1$

			if (dispStatus == null) { //$NON-NLS-1$
				
				String expanded;
				if ( actionbarComponent != null ){
					expanded = actionbarComponent.getNestedExpanded();
				}else{
					expanded = (String) component.getAttributes().get("nestedExpanded");
				}
				
				if (expanded == null || expanded.equalsIgnoreCase("false")) { //$NON-NLS-1$
					writer.writeAttribute("style","display:none",null);
					dispStatus = "none"; //$NON-NLS-1$
				} else {
					dispStatus = "visible"; //$NON-NLS-1$
				}
			} else {
				if(dispStatus.equals("none")){ //$NON-NLS-1$
					writer.writeAttribute("style","display:none",null);
				}
			}
			
			
			String style;
			if ( actionbarComponent != null ) {
				style = actionbarComponent.getStyle();
			}else{		
				style = (String)component.getAttributes().get("style");
			}
			if(style != null){
				writer.writeAttribute("style",style,"style");
			}
			
			dispStatus = null;
		}else{
			//not nested
			String style;
			if ( actionbarComponent != null ) {
				style = actionbarComponent.getStyle();
			}else{		
				style = (String)component.getAttributes().get("style");
			}
			if(style != null){
				writer.writeAttribute("style",style,"style");
			}
		}
	}

	public boolean getRendersChildren() {
		return true;
	}

	/** 
	 * Renders component's children.
	 * <p>
	 * Renders all child components as table cells either vertically
	 * or horizontally, depending on <code>layout</code> attribute.<br>
	 * Creates separators between items if <code>autoseparate</code> was set
	 * to <code>true</code>.
	 * <p>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeChildren(FacesContext context, UIComponent component)
		throws IOException {

		if (context == null || component == null) {
			throw new NullPointerException();
		}

		String layout = getLayout(component);

		if (layout.equalsIgnoreCase("pageDirection")) { //$NON-NLS-1$
			encodeChildrenVertical(context, component);
		} else {
			encodeChildrenHorizontal(context, component);
		}
	}

	private void encodeChildrenVertical(
		FacesContext context,
		UIComponent component)
		throws IOException {

		HtmlPanelActionbar actionbarComponent = null;
		if ( component instanceof HtmlPanelActionbar ){
			actionbarComponent = (HtmlPanelActionbar)component;
		}

		String autoseparate;
		String separatorstyle;
		String separatorsize;
		String indent;
		String parentId;
		
		if ( actionbarComponent != null ){
			autoseparate = actionbarComponent.getAutoseparate();
			separatorstyle = actionbarComponent.getSeparatorStyle();
			separatorsize = actionbarComponent.getSeparatorSize();
			indent = actionbarComponent.getNestedIndent();
			//parentId = actionbarComponent.getId();
		}else{
			autoseparate = (String) component.getAttributes().get("autoseparate"); //$NON-NLS-1$
			separatorstyle = (String) component.getAttributes().get("separatorStyle"); //$NON-NLS-1$
			separatorsize = (String) component.getAttributes().get("separatorSize"); //$NON-NLS-1$
			indent = (String) component.getAttributes().get("nestedIndent"); //$NON-NLS-1$
			//parentId = (String) component.getAttributes().get("id"); //$NON-NLS-1$
		}

		
		if (autoseparate == null) {
			autoseparate = DEFAULT_AUTOSEPARATE;
		}
		
		if (separatorstyle == null) {
			separatorstyle = DEFAULT_SEPARATORSTYLE;
		}
		
		if (separatorsize == null) {
			separatorsize = DEFAULT_SEPARATORSIZE;
		}
		
		if (indent == null) {
			indent = DEFAULT_INDENT;
		}
		
		//if (parentId == null)
			parentId = component.getClientId(context);

		List children = component.getChildren();
		ResponseWriter writer = context.getResponseWriter();
		for (int i = 0; i < children.size(); i++) {
			UIComponent child = (UIComponent) children.get(i);
			//RATLC00289488
			if(child.isRendered()){
				writer.startElement("tr",child);
				writer.startElement("td",child);
				writer.writeAttribute("colspan","2",null);
				
				if (child instanceof UIPanel) { //nested actionbar
					writeNestedLink(context,child,writer);

					writer.endElement("td");
					writer.endElement("tr");
					writer.startElement("tr",child);
					writer.startElement("td",child);
					writer.writeAttribute("style","width:"+indent+"px",null);
					writer.endElement("td");
					writer.startElement("td",child);
				}
				Utils.encodeComponent(context, child);
				writer.endElement("td");
				writer.endElement("tr");

				if (autoseparate.equalsIgnoreCase("true") && (i < children.size() - 1)) { //$NON-NLS-1$
					writer.startElement("tr",child);
					writer.writeAttribute("height",separatorsize,null);
					if (separatorstyle.equalsIgnoreCase("line")) { //$NON-NLS-1$
						writer.startElement("td",child);
						writer.writeAttribute("colspan","2",null);
						writer.startElement("hr",child);
						writer.endElement("hr");
						writer.endElement("td");
						writer.endElement("tr");
					} else {
						writer.startElement("td",child);
						writer.endElement("td");
						writer.endElement("tr");
					}
				}
			}
		}
		writer.endElement("table");
	}

	private void encodeChildrenHorizontal(
		FacesContext context,
		UIComponent component)
		throws IOException {

		HtmlPanelActionbar actionbarComponent = null;
		if ( component instanceof HtmlPanelActionbar ){
			actionbarComponent = (HtmlPanelActionbar)component;
		}

		String autoseparate;
		String separatorstyle;
		String separatorsize;
		String indent;
		String parentId;

		if (actionbarComponent != null) {
			autoseparate = actionbarComponent.getAutoseparate();
			separatorstyle = actionbarComponent.getSeparatorStyle();
			separatorsize = actionbarComponent.getSeparatorSize();
			indent = actionbarComponent.getNestedIndent();
			//parentId = actionbarComponent.getId();
		} else {
			autoseparate = (String) component.getAttributes().get("autoseparate"); //$NON-NLS-1$
			separatorstyle = (String) component.getAttributes().get("separatorStyle"); //$NON-NLS-1$
			separatorsize = (String) component.getAttributes().get("separatorSize"); //$NON-NLS-1$
			indent = (String) component.getAttributes().get("nestedIndent"); //$NON-NLS-1$
			//parentId = (String) component.getAttributes().get("id"); //$NON-NLS-1$
		}

		if (autoseparate == null) {
			autoseparate = DEFAULT_AUTOSEPARATE;
		}

		if (separatorstyle == null) {
			separatorstyle = DEFAULT_SEPARATORSTYLE;
		}

		if (separatorsize == null) {
			separatorsize = DEFAULT_SEPARATORSIZE;
		}

		if (indent == null) {
			indent = DEFAULT_INDENT;
		}
		
		String childId;
		//if (parentId == null)
			parentId = component.getClientId(context);

		List children = component.getChildren();
		ResponseWriter writer = context.getResponseWriter();

		writer.startElement("tr",component);
		
		int numberOfChildren=children.size();
		for (int i = 0; i < numberOfChildren ; i++) {
			UIComponent child = (UIComponent) children.get(i);
			//RATLC00289488
			if(child.isRendered()){
				writer.startElement("td",child);
				
				if (child instanceof UIPanel) { //nested actionbar)
					if ( child instanceof HtmlPanelActionbar ){
						childId = ((HtmlPanelActionbar)child).getId();	
					}else{
						childId = (String) child.getAttributes().get("id"); //$NON-NLS-1$
					}
					if (childId == null)
						childId = child.getClientId(context);
					
					writeNestedLink(context,child,writer);
					
				} else if ((child instanceof UIOutput) && child.getRendererType().equals("com.ibm.faces.SeparatorRenderer")) { // separator //$NON-NLS-1$
					writer.startElement("hr",child);
					writer.writeAttribute("width","1",null);
					writer.writeAttribute("size","10",null);
					writer.endElement("hr");
				} else {
					Utils.encodeComponent(context, child);
				}
				writer.endElement("td");
				
				if(i < (numberOfChildren-1)){			
					if (autoseparate.equalsIgnoreCase("true") && (i < children.size() - 1)) { //$NON-NLS-1$
						writer.startElement("td",child);
						writer.writeAttribute("style","width:"+separatorsize+"px",null);
						
						if (separatorstyle.equalsIgnoreCase("line")) { //$NON-NLS-1$
							writer.startElement("hr",child);
							writer.writeAttribute("width","1",null);
							writer.writeAttribute("size","10",null);
							writer.endElement("hr");
						} 
						writer.endElement("td");
					} else {
						writer.startElement("td",child);
						writer.writeAttribute("style","width:5px",null);
						writer.endElement("td");
					}
				}
			}
		}
		writer.endElement("tr");
		writer.endElement("table");

		for (int i = 0; i < children.size(); i++) {
			UIComponent child = (UIComponent) children.get(i);
			if (child instanceof UIPanel) { //nested actionbar
				Utils.encodeComponent(context, child);
			}
		}

	}

	public void encodeEnd(FacesContext context, UIComponent component)
		throws IOException {
		if (context == null || component == null) {
			throw new NullPointerException();
		}
	}

	private boolean isNestedActionbar(UIComponent component) {
		UIComponent parent = component.getParent();
		if (parent == null) {
			return false;
		}
		if (!(parent instanceof UIPanel)) {
			return false;
		}
		String renderer = component.getRendererType();
		String parentRenderer = parent.getRendererType();
		if (parentRenderer != null && parentRenderer.equals(renderer)) {
			return true;
		}
		return false;
	}

	private String getLayout(UIComponent component) {
		String layout = null;
		if (!isNestedActionbar(component)) {
			layout = (String) component.getAttributes().get("layout"); //$NON-NLS-1$
		} else {
			UIComponent parent = component.getParent();
			layout = getLayout(parent);
		}
		if (layout == null) {
			layout = DEFAULT_LAYOUT;
		}
		return layout;
	}
	
	private void writeNestedLink(FacesContext context,UIComponent component,ResponseWriter writer) throws IOException {
	
		HtmlPanelActionbar actionbarComponent = null;
		if ( component instanceof HtmlPanelActionbar ){
			actionbarComponent = (HtmlPanelActionbar)component;
		}
		
		String label;
		String nestedImage;
		String nestedImagePosition;
		String childId;
		String tabindex;
		
		if ( actionbarComponent != null ){
			label = actionbarComponent.getNestedTitleText();
			nestedImage = actionbarComponent.getNestedTitleImage();
			nestedImagePosition = actionbarComponent.getNestedImagePosition();
			tabindex = actionbarComponent.getTabindex();
		}else{
			label = (String) component.getAttributes().get("nestedTitleText"); //$NON-NLS-1$
			nestedImage = (String) component.getAttributes().get("nestedTitleImage"); //$NON-NLS-1$
			nestedImagePosition = (String) component.getAttributes().get("nestedImagePosition"); //$NON-NLS-1$
			tabindex = (String) component.getAttributes().get("tabindex");
		}

		//if (childId == null) 
			childId = component.getClientId(context);
		
			String parentId;
			if ( actionbarComponent != null ){
				parentId = actionbarComponent.getId();
			}else{
				parentId = (String) component.getAttributes().get("id"); //$NON-NLS-1$
			}
		
		if (parentId == null)parentId = component.getClientId(context);

		if (nestedImage == null && label == null) {
			label = component.getClientId(context);
		}
		
		/*
		 * adding tabindex, preference is setting it on the label as rendered <a> tag supports tabindex attribute,
		 * however if image is set with no label tabindex will be set on the rendered <img> tag
		 */
		String imgLabel = "image_label";
		if (nestedImagePosition == null || nestedImagePosition.equalsIgnoreCase(imgLabel)) { //$NON-NLS-1$
			if (nestedImage != null) {
				if(label == null){
					//just an image displayed so set tabindex on it
					writeMenubarImage(writer,component,childId,nestedImage,imgLabel, context, tabindex);
				} else {
					//tabindex not set on img - to be set on the label 
					writeMenubarImage(writer,component,childId,nestedImage,imgLabel, context, null);
				}
			}
			writeMenubarLink(writer,component,childId,label,tabindex);
		} else {
			if(nestedImage == null){
				//no image so write tabindex on the link
				writeMenubarLink(writer,component,childId,label,tabindex);
			} else {
				writeMenubarLink(writer,component,childId,label,tabindex);
				if(label != null){
					//then tabindex has been already written on the label, set to null so not added to img
					tabindex = null;
				}
				writeMenubarImage(writer,component,childId,nestedImage,nestedImagePosition,context,tabindex);
			}
		}

		//get and check if there is a valid expansion status for the nested actionbar 
		Map reqMap = context.getExternalContext().getRequestParameterMap();
		String id = component.getClientId(context);
		
		String dispStatus = (String) reqMap.get(id + "_display"); //$NON-NLS-1$

		if (dispStatus == null) { //$NON-NLS-1$
			String expanded;
			if ( actionbarComponent != null ){
				expanded = actionbarComponent.getNestedExpanded();
			}else{
				expanded = (String) component.getAttributes().get("nestedExpanded"); //$NON-NLS-1$
			}
			
			if (expanded == null || expanded.equalsIgnoreCase("false")) { //$NON-NLS-1$
				dispStatus = "none"; //$NON-NLS-1$
			} else {
				dispStatus = "visible"; //$NON-NLS-1$
			}
		}

		//add script for label and img links if present
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		scriptCollector.addScript(HxClientRenderUtil.HX_VAR_NAME + ".addBehavior(\"" + childId + "_link\", \"onclick\"," + //$NON-NLS-1$ //$NON-NLS-2$
		 						  "new "+ HxClientRenderUtil.HX_VAR_NAME + ".JSFBehaviorHS (\"hide-object:" + childId + "\", \"hide-tree:" + childId + "\", "+ //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		 						  "\"hide-field:" + childId + "_display\", \"hide-field-value:" + dispStatus +"\"));");  //$NON-NLS-1$ //$NON-NLS-2$//$NON-NLS-3$
		if (nestedImage!=null) {
			scriptCollector.addScript(HxClientRenderUtil.HX_VAR_NAME + ".addBehavior(\"" + childId + "_img\", \"onclick\"," +  //$NON-NLS-1$//$NON-NLS-2$
									  "new " + HxClientRenderUtil.HX_VAR_NAME + ".JSFBehaviorHS (\"hide-object:" + childId + "\", \"hide-tree:" + childId + "\", "+    //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
									  "\"hide-field:" + childId + "_display\", \"hide-field-value:" + dispStatus +"\"));"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
	}

	private void writeMenubarImage(ResponseWriter writer,UIComponent component,String idStr, String imgStr,String position, FacesContext context, String tabindex) throws IOException{
		
		String alt = ResourceHandler.getBundle(context).getString("DatagridCategoryRenderer.twistieAlt");

		imgStr = Utils.getEncodedResourceURL(context, imgStr);
		writer.startElement("img",component);
		writer.writeAttribute("id",idStr+"_img",null);
		writer.writeAttribute("src",imgStr,"nestedTitleImage");
		writer.writeAttribute("align","middle",null);
		if (position.equalsIgnoreCase("image_label")){
			writer.writeAttribute("style","cursor:pointer;margin-right:5px",null);	
		} else {
			writer.writeAttribute("style","cursor:pointer;margin-left:5px",null);
		}
		if(tabindex != null){
			writer.writeAttribute("tabindex",tabindex,"tabindex");
		}
		writer.writeAttribute("alt",alt,null);
		writer.writeAttribute("title",alt,null);
		writer.endElement("img");		
	}
	
	private void writeMenubarLink(ResponseWriter writer,UIComponent component,String idStr, String labelStr, String tabindex) throws IOException{
		//if labelStr is null then there is an image set without nestedTitleText set - in this case assume user wants image only
		//(without an image this would default to clientId)
		if(labelStr != null){
			writer.startElement("a",component);
			writer.writeAttribute("id",idStr+"_link",null);
			writer.writeAttribute("href","#",null);
			if(tabindex != null){
				writer.writeAttribute("tabindex",tabindex,"tabindex");
			}
			writer.writeText(labelStr,"nestedTitleText");
			writer.endElement("a");	
		}
				
	}

	/* (non-Javadoc)
	 * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {
		// TODO Auto-generated method stub
		
	}

		

	
}
