/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.Locale;

import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.convert.Converter;
import javax.faces.convert.DateTimeConverter;
import javax.faces.render.Renderer;
import javax.faces.validator.LengthValidator;
import javax.faces.validator.Validator;

import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlInputHelperAssist;
import com.ibm.faces.converter.MaskConverter;
import com.ibm.faces.converter.NumberConverterEx;
import com.ibm.faces.converter.NumberConverterExICU;
import com.ibm.faces.util.ScriptObject;
import com.ibm.faces.validator.ConstraintValidator;
import com.ibm.faces.validator.ExpressionValidator;
import com.ibm.faces.validator.ModulusSelfCheckValidator;


/**
 * Renderer for {@link com.ibm.faces.component.UIInputHelper UIInputHelper} component.
 * 
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.InputHelper_AssistTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.component.UIInputHelper}
 */
public class AssistRenderer extends Renderer implements IScriptContributor {

	/**
	 * Renders the beginning of the component.
	 * <p>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 *
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException {

		if (context == null || component == null) {
			throw new NullPointerException();
		}
		
		BehaviorArgs behavior = new BehaviorArgs(component);
		if (!behavior.isScriptRendered()){
			//none on the assist enabling attributes are set on the component
			return;
		}
		
		if (!(component.getParent() instanceof UIInput)) {
			return;
		}
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		// we need to output our javascript library
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component);
		
		UIInput input = (UIInput) component.getParent();
		Converter converter = input.getConverter();
		if(converter instanceof DateTimeConverter) {
			DateTimeConverter dtConverter = (DateTimeConverter)converter;
			Locale locale = dtConverter.getLocale();
			HxClientRenderUtil.addLocalizedStringLibraryIfNeeded(locale.toString(), component);			
		}

		if (HxClientRenderUtil.isContainedInPanel(component.getParent())) {
			encode(context, component, behavior, false);
		} else {
			scriptCollector.register(this, component);
		}
	}

	/* (non-Javadoc)
	 * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {
		
		BehaviorArgs behavior = new BehaviorArgs(component);
		if (!behavior.isScriptRendered()){
			//none on the assist enabling attributes are set on the component
			return;
		}
		
		encode(context, component, behavior, true);
	}

	private void encode(FacesContext context, UIComponent component, BehaviorArgs behavior, boolean writeNow) throws IOException {
		

		
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		UIInput input = (UIInput) component.getParent();
		
		ScriptObject converterScript = null;
		ScriptObject validatorScript = null;
		String maxLength = null; // we may need this for auto tab
		
		//don't skip converters if prompt character is set
		if( 	behavior.getPromptCharacter() == null && (
				(behavior.getAutoTab() != null && behavior.getAutoTab().equalsIgnoreCase("true") && (!(input.getConverter() instanceof DateTimeConverter || input.getConverter() instanceof NumberConverterEx)))
			||	(behavior.getImeMode() != null && (behavior.getImeMode().equalsIgnoreCase("auto") || behavior.getImeMode().equalsIgnoreCase("active"))))
		){
					
			//autoTab (and converter not datetime or number) or imeMode enabled so converter and validator script processing will be skipped
			//Can get strings from converters or validators if needed
			
			//if LengthValidator is set we need maxLength from for autoTab
			if(behavior.getAutoTab() != null && behavior.getAutoTab().equalsIgnoreCase("true")){
				Validator[] validators = input.getValidators();
				LengthValidator length = null;
			
				for (int i = 0; i < validators.length; i++) {
					Validator validator = validators[i];
					if (validator instanceof LengthValidator) {
						length = (LengthValidator)validator;
					}
				}
			
				if(length != null && length.getMaximum() > 0){
					maxLength = Integer.toString(length.getMaximum());
				}
			}
		} else {
			Converter converter = input.getConverter();
			if (!((converter instanceof DateTimeConverter || converter instanceof NumberConverterEx
					|| converter instanceof NumberConverterExICU || converter instanceof MaskConverter ))) {
				Validator[] validators = input.getValidators();
				LengthValidator length = null;

				for (int i = 0; i < validators.length; i++) {
					Validator validator = validators[i];
					if (validator instanceof LengthValidator) {
						length = (LengthValidator)validator;
					}
				}
				maxLength = (null != length) && (0 < length.getMaximum()) ? 
				                   Integer.toString(length.getMaximum()) : null;
				
			}
			converterScript = getConverterScript(context, input);
			validatorScript = getValidatorScript(context, input, converterScript);
		}

		
		String assistScript = null;
		String validateScript = null;
		assistScript = genAssistBehavior(
					input.getClientId(context),
					converterScript,
					validatorScript,
					behavior,
					maxLength,
					component);
		
		if (behavior.isValidation){
			behavior.setDefaultClass((String) input.getAttributes().get("styleClass")); //$NON-NLS-1$
			validateScript =
				genValidateBehavior(
					input.getClientId(context),
					converterScript,
					validatorScript,
					behavior,
					component);
		}
		if (writeNow) {
			ResponseWriter writer = context.getResponseWriter();
			if (null != converterScript) {
				scriptCollector.writeScriptOnce(converterScript.getScript(), context, false);
			}
			if (null != validatorScript) {
				scriptCollector.writeScriptOnce(validatorScript.getScript(), context, false);
			}
			if (null != assistScript) {
				writer.write('\n');
				writer.write(assistScript);
			}
			if (null != validateScript) {
				writer.write('\n');
				writer.write(validateScript);
			}
		} else {
			if (null != converterScript) {
				scriptCollector.addScriptOnce(converterScript.getScript());
			}
			if (null != validatorScript) {
				scriptCollector.addScriptOnce(validatorScript.getScript());
			}
			if (null != assistScript) {
				scriptCollector.addScript(assistScript);
			}
			if (null != validateScript) {
				scriptCollector.addScript(validateScript);
			}
		}
	}
	
	/**
	 * reused by formItem renderer
	 */
	protected static ScriptObject getConverterScript(FacesContext context, UIInput input) {
		ScriptObject converterScript = null;
		//process converters and validtors will be processed and get added to rendered page
		Converter converter = input.getConverter();
		if (converter instanceof DateTimeConverter) {
			converterScript = HxClientRenderUtil.buildJsDateConverter(context, (DateTimeConverter)converter);
		} else if ((converter instanceof NumberConverterEx) || (converter instanceof NumberConverterExICU)) {
			converterScript =
				HxClientRenderUtil.buildJsNumberConverter(context, (NumberConverterEx) converter);
		} else  if (converter instanceof MaskConverter) {
			converterScript = HxClientRenderUtil.buildJsMaskConverter(context, (MaskConverter)converter);
		}			
		return converterScript;
	}
	
	/**
	 * reused by formItem renderer
	 */
	protected static ScriptObject getValidatorScript(FacesContext context, UIInput input, ScriptObject converterScript) {
		ScriptObject validatorScript = null;
		
		//process converters and validtors will be processed and get added to rendered page
		Converter converter = input.getConverter();
		ExpressionValidator expressionValidator = HxClientRenderUtil.getExpressionValidator(input);
		ModulusSelfCheckValidator modulusValidator = HxClientRenderUtil.getModulusValidator(input);
		
		if (converter instanceof DateTimeConverter) {
			if (null != converterScript) {
				//We can't have a validator without a converter 
				validatorScript = HxClientRenderUtil.buildJsAddDateTimeValidator(context, HxClientRenderUtil.getDateRangeValidator(input), input.isRequired(), expressionValidator);
			}
		} else if ((converter instanceof NumberConverterEx) || (converter instanceof NumberConverterExICU)) {
			if (null != converterScript) {
				//We can't have a validator without a converter 
				validatorScript = HxClientRenderUtil.buildJsAddNumberValidator(context, HxClientRenderUtil.getNumberRangeValidator(input), input.isRequired(), expressionValidator, modulusValidator);
			}
		} else  if (converter instanceof MaskConverter) {
			if (input.isRequired() || expressionValidator != null) {
				validatorScript = HxClientRenderUtil.buildJsAddStringValidator(context, null, null, null, input.isRequired(), expressionValidator);
			}
		} else /*if (null == converter) */ {
			// RATLC00288560 - Converters cannot be used for RATLC0108240
			// If no converter or custom converter, consider it as the string  
			Validator[] validators = input.getValidators();
			ConstraintValidator constraint = null;
			LengthValidator length = null;

			for (int i = 0; i < validators.length; i++) {
				Validator validator = validators[i];
				if (validator instanceof ConstraintValidator) {
					constraint = (ConstraintValidator)validator;
				}
				if (validator instanceof LengthValidator) {
					length = (LengthValidator)validator;
				}
			}
			String regexp = (null != constraint) ? constraint.getRegex() : null;
			String minLength = (null != length) && (0 < length.getMinimum()) ? 
							   Integer.toString(length.getMinimum()) : null;
			String maxLength = (null != length) && (0 < length.getMaximum()) ? 
			                   Integer.toString(length.getMaximum()) : null;
			validatorScript = null;
			validatorScript = HxClientRenderUtil.buildJsAddStringValidator(context, regexp, minLength, maxLength, input.isRequired(), expressionValidator);
		}
		return validatorScript;
	}

	protected String genAssistBehavior(String scope, ScriptObject converter, ScriptObject validator, BehaviorArgs behavior, String maxLength, UIComponent component){
		
		StringBuffer script = new StringBuffer();
		String[] basicArgs = new String[6+1];
		int iBaseArg = 0;

		String s = null;
		if (null != (s = behavior.getPromptCharacter())) {
			if (s.equals("")) s = "default";  //$NON-NLS-1$//$NON-NLS-2$
			basicArgs[iBaseArg++] = "prompt-char:" + s; //$NON-NLS-1$
		}
		if (null != (s = behavior.getImeMode())) {
			basicArgs[iBaseArg++] = "ime-mode:" + s; //$NON-NLS-1$
		}
		
		if(null != behavior.getAutoSubmit()){
			if(behavior.getAutoSubmit().length() > 0){
				//get client id of the component idspecified
				UIComponent comp = component.findComponent(behavior.getAutoSubmit());
				if(comp != null){
					basicArgs[iBaseArg++] = "auto-submit:" + comp.getClientId(FacesContext.getCurrentInstance());
				} else {
					//just set the arg to unmodified string - could be a non jsf rendered html tag
					basicArgs[iBaseArg++] = "auto-submit:" + behavior.getAutoSubmit();
				}
			} else {
				basicArgs[iBaseArg++] = "auto-submit:";
			}
			
		}
		
		if(null != behavior.getAutoTab()){
			if (behavior.getAutoTab().equalsIgnoreCase("true")) { //$NON-NLS-1$
				basicArgs[iBaseArg++] = "auto-tab:" + ((maxLength==null)?"0":maxLength); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
		/*
		if (null != behavior.getValidation()) {
			basicArgs[iBaseArg++] = "auto-char:true"; //$NON-NLS-1$
		}
		*/
		if (iBaseArg == 0){
			return null; // there's nothing to do here
		}
		if (null != validator) {
			basicArgs[iBaseArg++] = "validator:" + validator.getId(); //$NON-NLS-1$
		}
		if (null != converter) {
			basicArgs[iBaseArg++] = "converter:" + converter.getId(); //$NON-NLS-1$
		}
		if (basicArgs.length > 0){
			script.append(HxClientRenderUtil.HX_VAR_NAME) //$NON-NLS-1$
			.append(".addBehavior(\"")
			.append(scope).append("\", \"oninput\", new ") //$NON-NLS-1$
			.append(HxClientRenderUtil.HX_VAR_NAME)
			.append(".JSFBehaviorAssist(\"")
			.append(basicArgs[--iBaseArg])
			.append('\"');
			while (iBaseArg > 0){
				script.append(", \"") //$NON-NLS-1$
				.append(basicArgs[--iBaseArg])
				.append('\"');
			}
			script.append("));"); //$NON-NLS-1$
		}
		return script.toString();
	}
	
	//Unused component param used by Designer, do not remove.
	protected String genValidateBehavior(String scope, ScriptObject converter, ScriptObject validator, BehaviorArgs behavior, UIComponent component){
		StringBuffer script = new StringBuffer();
		String[] blurArgs = new String[10+1];
		String[] focusArgs = new String[10+1];
		int iBlurArgs = 0;
		int iFocusArgs = 0;
		String s = null;
		
		// Generate onblur Behavior

		if (null != (s = behavior.getSuccessAction())) {
			blurArgs[iBlurArgs++] = "success-action:" + s; //$NON-NLS-1$
		}
		if (null != (s = behavior.getErrorAction())) {
			blurArgs[iBlurArgs++] = "error-action:" + s; //$NON-NLS-1$
		}
		if (null != (s = behavior.getSuccessClass())) {
			blurArgs[iBlurArgs++] = "success-class:" + s; //$NON-NLS-1$
		}
		if (null != (s = behavior.getErrorClass())) {
			blurArgs[iBlurArgs++] = "error-class:" + s; //$NON-NLS-1$
		}
		if (null != (s = behavior.getOnsuccess())) {
			blurArgs[iBlurArgs++] = "success-function:" + s; //$NON-NLS-1$
		}
		if (null != (s = behavior.getOnerror())) {
			blurArgs[iBlurArgs++] = "error-function:" + s; //$NON-NLS-1$
		}
		if (null != (s = behavior.getTargetErrorAction())) {
			s = HxClientRenderUtil.convertComponentIdsToClientIds(s, component);
			blurArgs[iBlurArgs++] = "error-target:" + s; //$NON-NLS-1$
		}
		if (null != (s = behavior.getTargetSuccessAction())) {
			s = HxClientRenderUtil.convertComponentIdsToClientIds(s, component);
			blurArgs[iBlurArgs++] = "success-target:" + s; //$NON-NLS-1$
		}
		
		if (iBlurArgs > 0){
			if (null != validator) {
				blurArgs[iBlurArgs++] = "validator:" + validator.getId(); //$NON-NLS-1$
			}
			if (null != converter) {
				blurArgs[iBlurArgs++] = "converter:" + converter.getId(); //$NON-NLS-1$
			}
			if (null != (s = behavior.getDefaultClass())) {
				blurArgs[iBlurArgs++] = "default-class:" + s; //$NON-NLS-1$
			}

			final int argSize = iBlurArgs;
			script.append("if (") //$NON-NLS-1$
				.append(HxClientRenderUtil.HX_VAR_NAME)
				.append(".getBehaviorById(\"") //$NON-NLS-1$
				.append(scope)
				.append("\", \"validate\", \"onblur\")==null) \n\t"); //$NON-NLS-1$
			
			script.append(HxClientRenderUtil.HX_VAR_NAME)
			.append(".addBehavior(\"") //$NON-NLS-1$
			.append(scope).append("\", \"onblur\", new ") //$NON-NLS-1$
			.append(HxClientRenderUtil.HX_VAR_NAME)
			.append(".JSFBehaviorValidate(\"") //$NON-NLS-1$
			.append(blurArgs[--iBlurArgs])
			.append('\"');
			while (iBlurArgs > 0){
				script.append(", \"") //$NON-NLS-1$
				.append(blurArgs[--iBlurArgs])
				.append('\"');
			}
			script.append("));"); //$NON-NLS-1$
			
			iBlurArgs = argSize;
			script.append("\nelse\t"); //$NON-NLS-1$
			script.append(HxClientRenderUtil.HX_VAR_NAME)
				.append(".getBehaviorById(\"") //$NON-NLS-1$
				.append(scope)
				.append("\", \"validate\", \"onblur\").setAttribute(\"") //$NON-NLS-1$
				.append(blurArgs[--iBlurArgs])
				.append('\"');
			while (iBlurArgs > 0){
				script.append(", \"") //$NON-NLS-1$
				.append(blurArgs[--iBlurArgs])
				.append('\"');
			}
			script.append(");"); //$NON-NLS-1$
		}
		
		// Generate onfocus Behavior

		if (null != (s = behavior.getOnfocusSuccessAction())) {
			focusArgs[iFocusArgs++] = "success-action:" + s; //$NON-NLS-1$
		}
		if (null != (s = behavior.getOnfocusErrorAction())) {
			focusArgs[iFocusArgs++] = "error-action:" + s; //$NON-NLS-1$
		}
		if (null != (s = behavior.getOnfocusSuccessClass())) {
			focusArgs[iFocusArgs++] = "success-class:" + s; //$NON-NLS-1$
		}
		if (null != (s = behavior.getOnfocusErrorClass())) {
			focusArgs[iFocusArgs++] = "error-class:" + s; //$NON-NLS-1$
		}
		if (null != (s = behavior.getOnfocussuccess())) {
			focusArgs[iFocusArgs++] = "success-function:" + s; //$NON-NLS-1$
		}
		if (null != (s = behavior.getOnfocuserror())) {
			focusArgs[iFocusArgs++] = "error-function:" + s; //$NON-NLS-1$
		}
		if (null != (s = behavior.getTargetOnfocusErrorAction())) {
			s = HxClientRenderUtil.convertComponentIdsToClientIds(s, component);
			focusArgs[iFocusArgs++] = "error-target:" + s; //$NON-NLS-1$
		}
		if (null != (s = behavior.getTargetOnfocusSuccessAction())) {
			s = HxClientRenderUtil.convertComponentIdsToClientIds(s, component);
			focusArgs[iFocusArgs++] = "success-target:" + s; //$NON-NLS-1$
		}
		
		if (iFocusArgs > 0){
			if (null != validator) {
				focusArgs[iFocusArgs++] = "validator:" + validator.getId(); //$NON-NLS-1$
			}
			if (null != converter) {
				focusArgs[iFocusArgs++] = "converter:" + converter.getId(); //$NON-NLS-1$
			}
			if (null != (s = behavior.getDefaultClass())) {
				focusArgs[iFocusArgs++] = "default-class:" + s; //$NON-NLS-1$
			}

			if(script.length() > 0) {
				script.append("\n"); //$NON-NLS-1$
			}
			
			final int argSize = iFocusArgs;
			script.append("if (") //$NON-NLS-1$
				.append(HxClientRenderUtil.HX_VAR_NAME)
				.append(".getBehaviorById(\"") //$NON-NLS-1$
				.append(scope)
				.append("\", \"validate\", \"onfocus\")==null) \n\t"); //$NON-NLS-1$
			
			script.append(HxClientRenderUtil.HX_VAR_NAME)
			.append(".addBehavior(\"") //$NON-NLS-1$
			.append(scope).append("\", \"onfocus\", new ") //$NON-NLS-1$
			.append(HxClientRenderUtil.HX_VAR_NAME)
			.append(".JSFBehaviorValidate(\"") //$NON-NLS-1$
			.append(focusArgs[--iFocusArgs])
			.append('\"');
			while (iFocusArgs > 0){
				script.append(", \"") //$NON-NLS-1$
				.append(focusArgs[--iFocusArgs])
				.append('\"');
			}
			script.append("));"); //$NON-NLS-1$
			
			iFocusArgs = argSize;
			script.append("\nelse\t"); //$NON-NLS-1$
			script.append(HxClientRenderUtil.HX_VAR_NAME)
				.append(".getBehaviorById(\"") //$NON-NLS-1$
				.append(scope)
				.append("\", \"validate\", \"onfocus\").setAttribute(\"") //$NON-NLS-1$
				.append(focusArgs[--iFocusArgs])
				.append('\"');
			while (iFocusArgs > 0){
				script.append(", \"") //$NON-NLS-1$
				.append(focusArgs[--iFocusArgs])
				.append('\"');
			}
			script.append(");"); //$NON-NLS-1$
		}
		
		return script.toString();
	}

	/**
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 * @param context
	 * @param component
	 * @throws IOException
	 */
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
		//defect 280926
	}
	


	protected class BehaviorArgs {
		boolean scriptRendered = true;
		boolean isValidation = false;
		//boolean isAssist = false;
		String defaultClass = null;
		String inputAssist;
		String validation;
		String promptCharacter;
		String autoTab;
		String autoSubmit;
		String imeMode;
		String errorClass; 
		String successClass;
		String errorAction;
		String successAction;
		String onerror;
		String onsuccess;
		String onfocuserror;
		String onfocussuccess;
		String onfocusErrorClass;
		String onfocusSuccessClass;
		String onfocusErrorAction;
		String onfocusSuccessAction;
		String targetErrorAction;
		String targetSuccessAction;
		String targetOnfocusErrorAction;
		String targetOnfocusSuccessAction;

	
		BehaviorArgs(UIComponent component){	
			HtmlInputHelperAssist assistComponent = null;
			if ( component instanceof HtmlInputHelperAssist ){
				assistComponent = (HtmlInputHelperAssist)component;
			}
		
			if ( assistComponent != null ){
				validation = assistComponent.getValidation();
				if (validation != null && validation.equalsIgnoreCase("true")){
					isValidation = true;
					errorClass = assistComponent.getErrorClass();
					successClass = assistComponent.getSuccessClass();
					errorAction = assistComponent.getErrorAction();
					successAction = assistComponent.getSuccessAction();
					onerror = assistComponent.getOnerror();
					onsuccess = assistComponent.getOnsuccess();
					onfocuserror = assistComponent.getOnfocuserror();
					onfocussuccess = assistComponent.getOnfocussuccess();
					onfocusErrorClass = assistComponent.getOnfocusErrorClass();
					onfocusSuccessClass = assistComponent.getOnfocusSuccessClass();
					onfocusErrorAction = assistComponent.getOnfocusErrorAction();
					onfocusSuccessAction = assistComponent.getOnfocusSuccessAction();
					targetErrorAction = assistComponent.getTargetErrorAction();
					targetSuccessAction = assistComponent.getTargetSuccessAction();
					targetOnfocusErrorAction = assistComponent.getTargetOnfocusErrorAction();
					targetOnfocusSuccessAction = assistComponent.getTargetOnfocusSuccessAction();
					
				}
				
				
				inputAssist = assistComponent.getInputAssist();
				promptCharacter = assistComponent.getPromptCharacter();
				autoTab = assistComponent.getAutoTab();
				imeMode = assistComponent.getImeMode();
				
				//if none these are enabled we don't want to emit script
				if (	
						isValidation ||
						(inputAssist != null && inputAssist.equalsIgnoreCase("true")) ||
						promptCharacter != null ||
						(autoTab != null && autoTab.equalsIgnoreCase("true")) ||
						(imeMode != null && (imeMode.equalsIgnoreCase("auto") || imeMode.equalsIgnoreCase("active")))  
				){
					//assist script should be rendered
					setScriptRendered(true);
					//section from previous code....
					
					// check v512 support
					if ((null == promptCharacter) && ("true".equalsIgnoreCase(inputAssist))){ //$NON-NLS-1$
						promptCharacter = ""; // use default char //$NON-NLS-1$
					}
					
				} else {
					//don't render script
					setScriptRendered(false);
				}
				
				/*
				if ("false".equalsIgnoreCase(inputAssist = assistComponent.getInputAssist())){ //$NON-NLS-1$
					// explicity look for false now
					isAssist = false;
				} else {
					isAssist = true; // really depends on the other args
					promptCharacter = assistComponent.getPromptCharacter();
					// check v512 support
					if ((null == promptCharacter) && ("true".equalsIgnoreCase(inputAssist))){ //$NON-NLS-1$
						promptCharacter = ""; // use default char //$NON-NLS-1$
					}
					autoSubmit = assistComponent.getAutoSubmit();
					autoTab = assistComponent.getAutoTab();
					imeMode = assistComponent.getImeMode();
				}
				*/
			}else{
				if (null != (validation = (String)component.getAttributes().get("validation"))){ //$NON-NLS-1$
					isValidation = true;
					errorClass = (String)component.getAttributes().get("errorClass"); //$NON-NLS-1$
					successClass = (String)component.getAttributes().get("successClass"); //$NON-NLS-1$
					errorAction = (String)component.getAttributes().get("errorAction"); //$NON-NLS-1$
					successAction = (String)component.getAttributes().get("successAction"); //$NON-NLS-1$
					onerror = (String)component.getAttributes().get("onerror"); //$NON-NLS-1$
					onsuccess = (String)component.getAttributes().get("onsuccess"); //$NON-NLS-1$
					onfocuserror = (String)component.getAttributes().get("onfocussrror"); //$NON-NLS-1$
					onfocussuccess = (String)component.getAttributes().get("onfocussuccess"); //$NON-NLS-1$
					onfocusErrorClass = (String)component.getAttributes().get("onfocusErrorClass"); //$NON-NLS-1$
					onfocusSuccessClass = (String)component.getAttributes().get("onfocusSuccessClass"); //$NON-NLS-1$
					onfocusErrorAction = (String)component.getAttributes().get("onfocusErrorAction"); //$NON-NLS-1$
					onfocusSuccessAction = (String)component.getAttributes().get("onfocusSuccessAction"); //$NON-NLS-1$
					targetErrorAction = (String)component.getAttributes().get("targetErrorAction"); //$NON-NLS-1$
					targetSuccessAction = (String)component.getAttributes().get("targetSuccessAction"); //$NON-NLS-1$
					targetOnfocusErrorAction = (String)component.getAttributes().get("targetOnfocusErrorAction"); //$NON-NLS-1$
					targetOnfocusSuccessAction = (String)component.getAttributes().get("targetOnfocusSuccessAction"); //$NON-NLS-1$

				}
				
				inputAssist = (String)component.getAttributes().get("inputAssist");
				promptCharacter = (String)component.getAttributes().get("promptCharacter");
				autoTab = (String)component.getAttributes().get("autoTab");
				imeMode = (String)component.getAttributes().get("imeMode");
				
				//if none these are enabled we don't want to emit script
				if (	
						isValidation ||
						(inputAssist != null && inputAssist.equalsIgnoreCase("true")) ||
						promptCharacter != null ||
						(autoTab != null && autoTab.equalsIgnoreCase("true")) ||
						(imeMode != null && (imeMode.equalsIgnoreCase("auto") || imeMode.equalsIgnoreCase("active")))  
				){
					//assist script should be rendered
					setScriptRendered(true);
					//section from previous code....
					
					// check v512 support
					if ((null == promptCharacter) && ("true".equalsIgnoreCase(inputAssist))){ //$NON-NLS-1$
						promptCharacter = ""; // use default char //$NON-NLS-1$
					}
					
				} else {
					//don't render script
					setScriptRendered(false);
				}
				
				/*
				if ("false".equalsIgnoreCase(inputAssist = (String)component.getAttributes().get("inputAssist"))){ //$NON-NLS-1$ //$NON-NLS-2$
					isAssist = false;
				} else {
					isAssist = true; // really depends on the other args
					autoSubmit = (String)component.getAttributes().get("autoSubmit");
					autoTab = (String)component.getAttributes().get("autoTab"); //$NON-NLS-1$
					imeMode = (String)component.getAttributes().get("imeMode"); //$NON-NLS-1$
					promptCharacter = (String)component.getAttributes().get("promptCharacter"); //$NON-NLS-1$
					// check v512 support
					if ((null == promptCharacter) && ("true".equalsIgnoreCase(inputAssist))){ //$NON-NLS-1$
						promptCharacter = ""; // use default char //$NON-NLS-1$
					}
				}
				*/
			}
		}
		
		public String getAutoSubmit(){
			return autoSubmit;
		}
		public String getAutoTab() {
			return autoTab;
		}
		public String getErrorAction() {
			return errorAction;
		}
		public String getErrorClass() {
			return errorClass;
		}
		public String getImeMode() {
			return imeMode;
		}
		public String getInputAssist() {
			return inputAssist;
		}

		public boolean isValidation() {
			return isValidation;
		}
		public String getOnerror() {
			return onerror;
		}
		public String getOnsuccess() {
			return onsuccess;
		}
		public String getPromptCharacter() {
			return promptCharacter;
		}
		public String getSuccessAction() {
			return successAction;
		}
		public String getSuccessClass() {
			return successClass;
		}
		public String getValidation() {
			return validation;
		}
		public void setAutoSubmit(String string){
			autoSubmit = string;
		}
		public void setAutoTab(String string) {
			autoTab = string;
		}
		public void setErrorAction(String string) {
			errorAction = string;
		}
		public void setErrorClass(String string) {
			errorClass = string;
		}
		public void setImeMode(String string) {
			imeMode = string;
		}
		public void setInputAssist(String string) {
			inputAssist = string;
		}

		public void setValidation(boolean b) {
			isValidation = b;
		}
		public void setOnerror(String string) {
			onerror = string;
		}
		public void setOnsuccess(String string) {
			onsuccess = string;
		}
		public void setPromptCharacter(String string) {
			promptCharacter = string;
		}
		public void setSuccessAction(String string) {
			successAction = string;
		}
		public void setSuccessClass(String string) {
			successClass = string;
		}
		public void setValidation(String string) {
			validation = string;
		}
		public String getDefaultClass() {
			return defaultClass;
		}
		public void setDefaultClass(String string) {
			defaultClass = string;
		}
		public String getOnfocuserror() {
			return onfocuserror;
		}
		public void setOnfocuserror(String onfocuserror) {
			this.onfocuserror = onfocuserror;
		}
		public String getOnfocusErrorAction() {
			return onfocusErrorAction;
		}
		public void setOnfocusErrorAction(String onfocusErrorAction) {
			this.onfocusErrorAction = onfocusErrorAction;
		}
		public String getOnfocusErrorClass() {
			return onfocusErrorClass;
		}
		public void setOnfocusErrorClass(String onfocusErrorClass) {
			this.onfocusErrorClass = onfocusErrorClass;
		}
		public String getOnfocussuccess() {
			return onfocussuccess;
		}
		public void setOnfocussuccess(String onfocussuccess) {
			this.onfocussuccess = onfocussuccess;
		}
		public String getOnfocusSuccessAction() {
			return onfocusSuccessAction;
		}
		public void setOnfocusSuccessAction(String onfocusSuccessAction) {
			this.onfocusSuccessAction = onfocusSuccessAction;
		}
		public String getOnfocusSuccessClass() {
			return onfocusSuccessClass;
		}
		public void setOnfocusSuccessClass(String onfocusSuccessClass) {
			this.onfocusSuccessClass = onfocusSuccessClass;
		}
		public String getTargetErrorAction() {
			return targetErrorAction;
		}
		public void setTargetErrorAction(String targetErrorAction) {
			this.targetErrorAction = targetErrorAction;
		}
		public String getTargetOnfocusErrorAction() {
			return targetOnfocusErrorAction;
		}
		public void setTargetOnfocusErrorAction(String targetOnfocusErrorAction) {
			this.targetOnfocusErrorAction = targetOnfocusErrorAction;
		}
		public String getTargetOnfocusSuccessAction() {
			return targetOnfocusSuccessAction;
		}
		public void setTargetOnfocusSuccessAction(
				String targetOnfocusSuccessAction) {
			this.targetOnfocusSuccessAction = targetOnfocusSuccessAction;
		}
		public String getTargetSuccessAction() {
			return targetSuccessAction;
		}
		public void setTargetSuccessAction(String targetSuccessAction) {
			this.targetSuccessAction = targetSuccessAction;
		}
		
		public boolean isScriptRendered() {
			return scriptRendered;
		}
		public void setScriptRendered(boolean scriptRendered) {
			this.scriptRendered = scriptRendered;
		}
	}
}
