/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.Calendar;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.faces.FacesException;
import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.convert.DateTimeConverter;
import javax.faces.render.Renderer;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlInputHelperDatePicker;
import com.ibm.faces.util.ScriptObject;



/**
 * Renderer for {@link com.ibm.faces.component.UIInputHelper UIInputHelper} component, 
 * that creates an image, which when being clicked on, pops up
 * a Date Picker control.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.InputHelper_DatePickerTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.component.UIInputHelper}
 */

public class DatePickerRenderer extends Renderer  implements IScriptContributor {
	static private final String DEFAULT_BUTTON_BORDER = "-1"; //$NON-NLS-1$
	
	/**
	 * Renders the beginning of the component.
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 *
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException {
		if (context == null || component == null) {
			throw new NullPointerException();
		}
		UIComponent parent = component.getParent();
		if (parent == null || !(parent instanceof UIInput)) {
			return; // no point in going any further if we don't have these
		}

		HxClientRenderUtil.addScriptLibrary("hxclient_dp.js", component);
		HxClientRenderUtil.addScriptLibrary("hxclient_sp.js", component);
		
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		// we need to output our javascript library
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		
		UIInput input = (UIInput) component.getParent();
		DateTimeConverter dtConverter = (DateTimeConverter)input.getConverter();
		if(dtConverter != null) {
			Locale locale = dtConverter.getLocale();
			HxClientRenderUtil.addLocalizedStringLibraryIfNeeded(locale.toString(), component);			
		}
		
		if (HxClientRenderUtil.isContainedInPanel(component.getParent())) {
			encode(context, component, false);
		} else {
			scriptCollector.register(this, component);
		}
	}

	/* (non-Javadoc)
	 * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {
		encode(context, component, true);
	}
	
	private void encode(FacesContext context, UIComponent component, boolean writeNow) throws IOException {
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		UIInput input = (UIInput) component.getParent();

		DateTimeConverter dtConverter = (DateTimeConverter)input.getConverter();
        if(dtConverter == null)
        {
        	ResourceBundle rb = ResourceHandler.getBundle(context);	            	
        	throw new FacesException(ResourceHandler.getString(rb, "DatePickerNoConveterFoundError"));
        }
		ScriptObject converterScript = HxClientRenderUtil.buildJsDateConverter(context, dtConverter);
		ScriptObject validatorScript = null;
		if (null != converterScript) {
			 // We can't have a validator without a converter so only get the 
			 // validator (as follows) if we've found a converter. 
			validatorScript = HxClientRenderUtil.buildJsRangeValidator(context,
																dtConverter,
																HxClientRenderUtil.getRangeValidator(input), 
																input.isRequired(), 
																HxClientRenderUtil.getExpressionValidator(input), null);
		}
		String assistScript = null;
		if (dtConverter.getType().indexOf("date") != -1){ //$NON-NLS-1$
					assistScript = getDatePickerScript(context, component,dtConverter.getLocale(), 
														converterScript!=null?converterScript.getId():null,
														validatorScript!=null?validatorScript.getId():null);
		} else {//if (dtConverter.getType().indexOf("time") != -1){ //$NON-NLS-1$
			assistScript = getTimePickerScript(context, component, 
									converterScript!=null?converterScript.getId():null,
									validatorScript!=null?validatorScript.getId():null);
		}
		if (writeNow) {
			ResponseWriter writer = context.getResponseWriter();
			if (null != converterScript) {
				scriptCollector.writeScriptOnce(converterScript.getScript(), context, false);
			}
			if (null != validatorScript) {
				scriptCollector.writeScriptOnce(validatorScript.getScript(), context, false);
			}
			if (null != assistScript) {
				writer.write("\n"); //$NON-NLS-1$
				writer.write(assistScript);
			}
		} else {
			if (null != converterScript) {
				scriptCollector.addScriptOnce(converterScript.getScript());
			}
			if (null != validatorScript) {
				scriptCollector.addScriptOnce(validatorScript.getScript());
			}
			if (null != assistScript) {
				scriptCollector.addScript(assistScript);
			}
		}
	}
	
		/* (non-Javadoc)
	 * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	private String getDatePickerScript(FacesContext context, UIComponent component, Locale converterLocale, String converterId, String validatorId) {
		String sRequired = null;
		UIInput input = null;

		HtmlInputHelperDatePicker dateComponent = null;
		if ( component instanceof HtmlInputHelperDatePicker ){
			dateComponent = (HtmlInputHelperDatePicker)component;
		}

		/* Get a default value of the first day of week - locale sensitive 
		 * Use converter locale if set, otherwise use viewRoot locale
		 * */
		Locale locale;
		if(converterLocale==null){
			locale = context.getViewRoot().getLocale();
		} else {
			locale=converterLocale;
		}
		//Locale locale = context.getViewRoot().getLocale();
		Calendar cal = Calendar.getInstance(locale);
		int DefaultFirstDay = cal.getFirstDayOfWeek();
		DefaultFirstDay--;

		// 1st find the UIInput field we're assisting and read whatever
		// relevant attributes that my be set on it. 
		UIComponent parent = component.getParent();
		if (parent != null && parent instanceof UIInput) {
			input = (UIInput) parent;
		} else {
			return ""; // no point in going any further if we don't have these //$NON-NLS-1$
		}

		Boolean bRequired = (Boolean) input.getAttributes().get("required"); //$NON-NLS-1$
		if (bRequired.booleanValue())
			sRequired = "true"; //$NON-NLS-1$
			
		String firstDay, buttonBorder;
		if ( dateComponent != null ){
			firstDay = dateComponent.getFirstDay();
			buttonBorder = dateComponent.getButtonBorder();
		}else{
			firstDay = (String)component.getAttributes().get("firstDay"); //$NON-NLS-1$
			buttonBorder = (String)component.getAttributes().get("buttonBorder"); //$NON-NLS-1$
		}	
			
		String sFirstDay =
			getNumericAttribute(
				firstDay, //$NON-NLS-1$
				"first-day-of-week:", //$NON-NLS-1$
				new Integer(DefaultFirstDay).toString(),
				new Integer(0),
				new Integer(6));
		String sBtnBorder =
			getNumericAttribute(
				buttonBorder, //$NON-NLS-1$
				"button-border:", //$NON-NLS-1$
				DEFAULT_BUTTON_BORDER,
				new Integer(-1),
				new Integer(3));
		
		String sCSSPrefix,sBtnColor;		
		if ( dateComponent != null ){
			sCSSPrefix = dateComponent.getStyleClass();
			sBtnColor = dateComponent.getButtonColor();
		}else{
			sCSSPrefix = (String) component.getAttributes().get("styleClass"); //$NON-NLS-1$
			sBtnColor = (String) component.getAttributes().get("buttonColor"); //$NON-NLS-1$
		}				
				
		// where does this value come from - converter???

		StringBuffer script = new StringBuffer(HxClientRenderUtil.HX_VAR_NAME) //$NON-NLS-1$
		.append(".addComponent(\"")
		 .append(parent.getClientId(context))
		  .append("\", new ")
			.append(HxClientRenderUtil.HX_VAR_NAME)
			.append(".JSFDatePicker(");  //$NON-NLS-1$//$NON-NLS-2$
		if (null != converterId) {
			script.append("\"converter:"); //$NON-NLS-1$
			script.append(converterId);
			script.append('\"'); //$NON-NLS-1$ 
		}
		if (null != validatorId) {
			script.append(", \""); //$NON-NLS-1$
			script.append("validator:"); //$NON-NLS-1$
			script.append(validatorId);
			script.append('\"'); //$NON-NLS-1$ 
		}
		if (sCSSPrefix != null && sCSSPrefix != "") { //$NON-NLS-1$
			script.append(", \"css-prefix:"); //$NON-NLS-1$
			script.append(Utils.getFirstCSSClassName(sCSSPrefix));
			script.append('\"'); //$NON-NLS-1$ 
		}
		if (sBtnColor != null && sBtnColor != "") { //$NON-NLS-1$
			script.append(", \"button-color:"); //$NON-NLS-1$
			script.append(sBtnColor);
			script.append('\"'); //$NON-NLS-1$ 
		}
		if (sRequired != null && sRequired != "") { //$NON-NLS-1$
			script.append(", \"required:"); //$NON-NLS-1$
			script.append(sRequired);
			script.append('\"'); //$NON-NLS-1$
		}
		
		
		boolean multiLine = true; // This is the default

		Object ml = component.getAttributes().get("multiLine"); // only where to tell if attribaute was set
		if(ml != null && ml instanceof String){
			if("false".equalsIgnoreCase(ml.toString()))
				multiLine = true;			
		}
		if(ml != null && ml instanceof Boolean){
			multiLine = ((Boolean)ml).booleanValue();
		}
		
		if (multiLine) { //$NON-NLS-1$ // this is the normal case
			script.append(", \"multi-line:"); //$NON-NLS-1$
			script.append("true");
			script.append('\"'); //$NON-NLS-1$
		}
		else { //$NON-NLS-1$ // this is the special case
			script.append(", \"multi-line:"); //$NON-NLS-1$
			script.append("false");
			script.append('\"'); //$NON-NLS-1$
		}		
		
		
		//	no need to check if following attributes have values as defaults are automatically set if not on page
		script.append(", \""); //$NON-NLS-1$
		script.append(sFirstDay);
		script.append('\"'); //$NON-NLS-1$ 
		script.append(", \""); //$NON-NLS-1$
		script.append(sBtnBorder);
		script.append('\"'); //$NON-NLS-1$ 
		script.append("));"); //$NON-NLS-1$
		return script.toString();
	}
	
	/**
	 * Fetch, validate and transform a numeric component attribute embedded in a string
	 * @param component		- JSF component holding the requested attribute
	 * @param sAttInName	- name of JSF attribute, e.g. ShowTabs
	 * @param sAttOutName	- "show-tabs:"
	 * @param DefaultValue	- value to be assigned if nothing is found
	 * @param MinValue		- smallest valid value, if any
	 * @param MaxValue		- largest valid value, if any
	 * @return 				- e.g. "show-tabs:1"
	 */
	// TODO MWD: abstract to utility class so any renderer can use this
	private String getNumericAttribute(String sValue, String sAttOutName,String DefaultValue, Integer MinValue, Integer MaxValue) {

		if (sValue == null || sValue == "") //$NON-NLS-1$
			sValue = DefaultValue;
		else {
			try {
				int iValue = Integer.parseInt(sValue);
				if (MinValue != null && iValue < MinValue.intValue())
					sValue = DefaultValue;
				else if (MaxValue != null && iValue > MaxValue.intValue())
					sValue = DefaultValue;
				} catch (NumberFormatException e){
					sValue = DefaultValue;
			}
		}		
		sValue = sAttOutName + sValue; 
		return sValue;
	}
	
	private String getTimePickerScript(FacesContext context, UIComponent component, String converterId, String validatorId) {
	
		// Get own attributes		
		
		HtmlInputHelperDatePicker dateComponent = null;
		if ( component instanceof HtmlInputHelperDatePicker ){
			dateComponent = (HtmlInputHelperDatePicker)component;
		}
		
		
		Boolean HMS_individual;		
		if ( dateComponent != null ){
			HMS_individual = new Boolean(dateComponent.isPositional());
		}else{
			HMS_individual = (Boolean) component.getAttributes().get("positional"); //$NON-NLS-1$
		}		

		
		
		// Check if the parent has a validation tag, and if yes - get validation attributes
	
		StringBuffer script = new StringBuffer(HxClientRenderUtil.HX_VAR_NAME); //$NON-NLS-1$ 
		script.append(".addComponent(\"");
		script.append(component.getParent().getClientId(context));
		script.append("\", new "); //$NON-NLS-1$
		script.append(HxClientRenderUtil.HX_VAR_NAME);
		script.append(".JSFSpinner(\"type:datetime\"");
		
		
		if(HMS_individual.booleanValue()) // allows the hours, minutes and secs to be updated seperately
		{
			
			String delta_hours_str; //$NON-NLS-1$
			String delta_minutes_str; //$NON-NLS-1$
			String delta_secs_str; //$NON-NLS-1$	
			
			if ( dateComponent != null ){
				delta_hours_str = dateComponent.getDeltaHours();
				delta_minutes_str = dateComponent.getDeltaMinutes();
				delta_secs_str = dateComponent.getDeltaSeconds();
			}else{
				delta_hours_str = (String) component.getAttributes().get("deltaHours"); //$NON-NLS-1$
				delta_minutes_str = (String) component.getAttributes().get("deltaMinutes"); //$NON-NLS-1$
				delta_secs_str = (String) component.getAttributes().get("deltaSeconds"); //$NON-NLS-1$
			}
			
			if(delta_hours_str != null) // Do the hours
			{
				int delta_num;
				try
				{
					delta_num = Integer.parseInt(delta_hours_str);
				}catch(NumberFormatException nfe)
				{
					delta_num = 1; // this is the default - 1 hour
				}				
				int delta_hours = delta_num*3600; // value * seconds per hour = delta in secs
				script.append(", \"increment:");
				script.append(delta_hours+"\"");
			}
			else
				script.append(", \"increment:3600\""); // this is the default - 1 hour

			if(delta_minutes_str != null) // Do the minutes
			{
				int delta_num;
				try
				{
					delta_num = Integer.parseInt(delta_minutes_str);
				}catch(NumberFormatException nfe)
				{
					delta_num = 1; // this is the default - 1 minute
				}				
				int delta_minutes = delta_num*60; // value * seconds per minute = delta in secs				
				script.append(", \"increment1:"); // minutes delta			
				script.append(delta_minutes+"\"");				
			}
			else
			{
				script.append(", \"increment1:"); // minutes delta	
				script.append("60\""); // this is the default - 1 minute
			}

			if(delta_secs_str != null) // Do the secs
			{
				int delta_secs;
				try
				{
					delta_secs = Integer.parseInt(delta_secs_str);
				}catch(NumberFormatException nfe)
				{
					delta_secs = 1; // this is the default - 1 second
				}				
				script.append(", \"increment2:");
				script.append(delta_secs+"\"");
			}
			else
			{
				script.append(", \"increment2:");
				script.append("1\""); // this is the default - 1 sec
			}
			
		}
		else // This was the normal way of doing the timechooser delta
		{
			String delta; //$NON-NLS-1$
			if ( dateComponent != null ){
				delta = dateComponent.getDelta();
			}else{
				delta = (String) component.getAttributes().get("delta"); //$NON-NLS-1$
			}
			
			if (delta == null) {
				delta = "1800"; //$NON-NLS-1$
			}			
			script.append(", \"increment:"+delta); // normal delta
			script.append(('\"')); //$NON-NLS-1$ 	
		}
		
		if (converterId != null) {
			script.append(",\"converter:"); //$NON-NLS-1$ 
			script.append(converterId);
			script.append(('\"')); //$NON-NLS-1$
		}
		if (validatorId != null) {
			script.append(",\"validator:"); //$NON-NLS-1$ 
			script.append(validatorId);
			script.append(('\"')); //$NON-NLS-1$
		}
		script.append("));"); //$NON-NLS-1$
		return script.toString();
	}
    
    /**
     * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     * @throws IOException
     */
    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {
        //defect 280926
    }
}
