/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.component.UIForm;
import javax.faces.component.UIInput;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.convert.ConverterException;
import javax.faces.render.Renderer;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.component.UIFileupload;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.fileupload.util.ContentElement;
import com.ibm.faces.component.html.HtmlFileupload;
import com.ibm.faces.context.MultipartExternalContextImpl;

/**
 * Renderer for {@link com.ibm.faces.component.UIFileupload UIFileupload} component
 * that creates an input field and a browse button. 
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.FileuploadTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.component.UIFileupload}
 */
public class FileuploadRenderer extends Renderer {
	private static final String FILEUPLOAD_ERR1 = "Unaccepted Content Type:"; //$NON-NLS-1$
	private static final String FILEUPLOAD_ERR2 = "Unaccepted Extention Type:"; //$NON-NLS-1$
	private static final String EXCLUDE_ATTR = "exclude";
	private static final String ACCEPT_ATTR = "accept";

	//
	// Constructors and Initializers    
	//

	public FileuploadRenderer() {
		super();
	}
	/** 
	 * Renders the beginning of the component.
	 * <p>
	 * Doesn't need to do anything; all processing is done in {@link FileuploadRenderer#encodeEnd encodeEnd}.
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeBegin(FacesContext context, UIComponent component)
		throws IOException {
		if (context == null || component == null) {
			throw new NullPointerException();
		}
	}

	/** 
	 * Renders component's children.
	 * <p>
	 * Encodes nested (if any){@link com.ibm.faces.component.UIFileProp} components. 
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request being processed
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeChildren(FacesContext context, UIComponent component)
		throws IOException {
		if (context == null || component == null) {
			throw new NullPointerException();
		}
		Iterator kids = component.getChildren().iterator();
		while (kids.hasNext()) {
			UIComponent kid = (UIComponent) kids.next();
			kid.encodeBegin(context);
			if (kid.getRendersChildren()) {
				kid.encodeChildren(context);
			}
			kid.encodeEnd(context);
		}
	}
	/**
	 * Renders the ending of the component.
	 * <p>
	 * Renders the entire component.
	 * <p>
	 * Creates an INPUT tag with "file" type. Browsers render
	 * this as an text input field and a file browse button.
	 * <p>
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 *
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeEnd(FacesContext context, UIComponent component)
		throws IOException {

		if (context == null || component == null) {
			throw new NullPointerException();
		}
		ResponseWriter writer = context.getResponseWriter();

		writer.startElement("input",component);
		writer.writeAttribute("type","file",null);
		writer.writeAttribute("id",component.getClientId(context),"id");
		writer.writeAttribute("name",component.getClientId(context),"id");
		
		HtmlFileupload uploadComponent = null;
		if ( component instanceof HtmlFileupload ){
			uploadComponent = (HtmlFileupload)component;
		}
		
		String styleClass;
		if ( uploadComponent != null ){
			styleClass = uploadComponent.getStyleClass();
		}else{
			styleClass = (String) component.getAttributes().get("styleClass"); //$NON-NLS-1$
		}
		
		if (styleClass != null) {
			writer.writeAttribute("class",styleClass,"styleClass");
		}

		if ( uploadComponent != null ) {
			if (uploadComponent.getAccesskey() != null) {
				writer.writeAttribute(
					"accesskey",
					uploadComponent.getAccesskey(),
					null);
			}
			if (uploadComponent.isDisabled()) {
				writer.writeAttribute("disabled", "disabled", null);
			}
			if (uploadComponent.getOnblur() != null) {
				writer.writeAttribute(
					"onblur",
					uploadComponent.getOnblur(),
					null);
			}
			if (uploadComponent.getOnchange() != null) {
				writer.writeAttribute(
					"onchange",
					uploadComponent.getOnchange(),
					null);
			}
			if (uploadComponent.getOnclick() != null) {
				writer.writeAttribute(
					"onclick",
					uploadComponent.getOnclick(),
					null);
			}
			if (uploadComponent.getOndblclick() != null) {
				writer.writeAttribute(
					"ondblclick",
					uploadComponent.getOndblclick(),
					null);
			}
			if (uploadComponent.getOnfocus() != null) {
				writer.writeAttribute(
					"onfocus",
					uploadComponent.getOnfocus(),
					null);
			}
			if (uploadComponent.getOnkeydown() != null) {
				writer.writeAttribute(
					"onkeydown",
					uploadComponent.getOnkeydown(),
					null);
			}
			if (uploadComponent.getOnkeypress() != null) {
				writer.writeAttribute(
					"onkeypress",
					uploadComponent.getOnkeypress(),
					null);
			}
			if (uploadComponent.getOnkeyup() != null) {
				writer.writeAttribute(
					"onkeyup",
					uploadComponent.getOnkeyup(),
					null);
			}
			if (uploadComponent.getOnmousedown() != null) {
				writer.writeAttribute(
					"onmousedown",
					uploadComponent.getOnmousedown(),
					null);
			}
			if (uploadComponent.getOnmousemove() != null) {
				writer.writeAttribute(
					"onmousemove",
					uploadComponent.getOnmousemove(),
					null);
			}
			if (uploadComponent.getOnmouseout() != null) {
				writer.writeAttribute(
					"onmouseout",
					uploadComponent.getOnmouseout(),
					null);
			}
			if (uploadComponent.getOnmouseover() != null) {
				writer.writeAttribute(
					"onmouseover",
					uploadComponent.getOnmouseover(),
					null);
			}
			if (uploadComponent.getOnmouseup() != null) {
				writer.writeAttribute(
					"onmouseup",
					uploadComponent.getOnmouseup(),
					null);
			}
			if (uploadComponent.getOnselect() != null) {
				writer.writeAttribute(
					"onselect",
					uploadComponent.getOnselect(),
					null);
			}
			if (uploadComponent.isReadonly()) {
				writer.writeAttribute("readonly", "readonly", null);
			}
			if (uploadComponent.getStyle() != null) {
				writer.writeAttribute(
					"style",
					uploadComponent.getStyle(),
					null);
			}
			if (uploadComponent.getTabindex() != null) {
				writer.writeAttribute(
					"tabindex",
					uploadComponent.getTabindex(),
					null);
			}
			if (uploadComponent.getTitle() != null) {
				writer.writeAttribute(
					"title",
					uploadComponent.getTitle(),
					null);
			}
			if (uploadComponent.getAlt() != null) {
				writer.writeAttribute(
					"alt",
					uploadComponent.getAlt(),
					null);
			}
			if (uploadComponent.getDir() != null) {
				writer.writeAttribute(
					"dir",
					uploadComponent.getDir(),
					null);
			}
			if (uploadComponent.getSize() != null) {
				writer.writeAttribute(
					"size",
					uploadComponent.getSize(),
					null);
			}
		} else {
			String accesskey = (String)component.getAttributes().get("accesskey");
			if(accesskey != null){
				writer.writeAttribute("accesskey",accesskey,"accesskey");
			}
			String onblur = (String)component.getAttributes().get("onblur");
			if(onblur != null){
				writer.writeAttribute("onblur",onblur,"onblur");
			}
			String onchange = (String)component.getAttributes().get("onchange");
			if(onchange != null){
				writer.writeAttribute("onchange",onchange,"onchange");
			}
			String onclick = (String)component.getAttributes().get("onclick");
			if(onclick != null){
				writer.writeAttribute("onclick",onclick,"onclick");
			}
			String ondblclick = (String)component.getAttributes().get("ondblclick");
			if(ondblclick != null){
				writer.writeAttribute("ondblclick",ondblclick,"ondblclick");
			}
			String onfocus = (String)component.getAttributes().get("onfocus");
			if(onfocus != null){
				writer.writeAttribute("onfocus",onfocus,"onfocus");
			}
			String onkeydown = (String)component.getAttributes().get("onkeydown");
			if(onkeydown != null){
				writer.writeAttribute("onkeydown",onkeydown,"onkeydown");
			}
			String onkeypress = (String)component.getAttributes().get("onkeypress");
			if(onkeypress != null){
				writer.writeAttribute("onkeypress",onkeypress,"onkeypress");
			}
			String onkeyup = (String)component.getAttributes().get("onkeyup");
			if(onkeyup != null){
				writer.writeAttribute("onkeyup",onkeyup,"onkeyup");
			}
			String onmousedown = (String)component.getAttributes().get("onmousedown");
			if(onmousedown != null){
				writer.writeAttribute("onmousedown",onmousedown,"onmousedown");
			}
			String onmousemove = (String)component.getAttributes().get("onmousemove");
			if(onmousemove != null){
				writer.writeAttribute("onmousemove",onmousemove,"onmousemove");
			}
			String onmouseout = (String)component.getAttributes().get("onmouseout");
			if(onmouseout != null){
				writer.writeAttribute("onmouseout",onmouseout,"onmouseout");
			}
			String onmouseover = (String)component.getAttributes().get("onmouseover");
			if(onmouseover != null){
				writer.writeAttribute("onmouseover",onmouseover,"onmouseover");
			}
			String onmouseup = (String)component.getAttributes().get("onmouseup");
			if(onmouseup != null){
				writer.writeAttribute("onmouseup",onmouseup,"onmouseup");
			}
			String onselect = (String)component.getAttributes().get("onselect");
			if(onselect != null){
				writer.writeAttribute("onselect",onmouseup,"onselect");
			}
			String style = (String)component.getAttributes().get("style");
			if(style != null){
				writer.writeAttribute("style",style,"style");
			}
			String tabindex = (String)component.getAttributes().get("tabindex");
			if(tabindex != null){
				writer.writeAttribute("tabindex",tabindex,"tabindex");
			}
			String title = (String)component.getAttributes().get("title");
			if(title != null){
				writer.writeAttribute("title",title,"title");
			}
			String alt = (String)component.getAttributes().get("alt");
			if(alt != null){
				writer.writeAttribute("alt",alt,"alt");
			}
			String dir = (String)component.getAttributes().get("dir");
			if(dir != null){
				writer.writeAttribute("dir",dir,"dir");
			}
			String size = (String)component.getAttributes().get("size");
			if(size != null){
				writer.writeAttribute("size",size,"size");
			}
			
			Object disabled = component.getAttributes().get("disabled");
			if(disabled != null){
				Boolean isDisabled = Boolean.FALSE;
				if(disabled instanceof String && ((String)disabled).equalsIgnoreCase("true")){
					isDisabled = Boolean.TRUE;
				}
				if(disabled instanceof Boolean && ((Boolean)disabled).booleanValue()){
					isDisabled = Boolean.TRUE;
				}
				if(isDisabled.booleanValue()){
					writer.writeAttribute("disabled","disabled","disabled");
				}
			}
			Object readonly = component.getAttributes().get("readonly");
			if(readonly != null){
				Boolean isReadonly = Boolean.FALSE;
				if(readonly instanceof String && ((String)readonly).equalsIgnoreCase("true")){
					isReadonly = Boolean.FALSE;
				}
				if(readonly instanceof Boolean && ((Boolean)readonly).booleanValue()){
					isReadonly = Boolean.FALSE;
				}
				if(isReadonly.booleanValue()){
					writer.writeAttribute("readonly","readonly","readonly");
				}
			}
		}
		writer.endElement("input");
		
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		UIForm form = getForm(component);
		if(form != null) {
			String formId = form.getClientId(context);
			scriptCollector.addScriptOnce("if(" + HxClientRenderUtil.HX_VAR_NAME + ".getElementById(\"" + formId + "\")!=null) " + //$NON-NLS-1$ //$NON-NLS-2$
			HxClientRenderUtil.HX_VAR_NAME + ".getElementById(\"" + formId + "\").encoding=\"multipart/form-data\";\n"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		else {
			// TODO log a warning that there is no FORM
		}
	}

	/**
	 * Decodes the current state of the component.
	 * <p>
	 * Performs various checks of the data being uploaded to make sure it's valid.<br>
	 * Verifies data to be acceptable, based on <code>accept</code> and
	 * <code>exclude</code>attributes.
	 * <p>
	 * @param context {@link FacesContext} for the request we are processing
	 * @param component {@link UIComponent} to be decoded.
	 *
	 * @exception IOException if an input/output error occurs while decoding
	 * 
	 * @see javax.faces.render.Renderer#decode(javax.faces.context.FacesContext, javax.faces.component.UIComponent) 
	 */
	public void decode(FacesContext context, UIComponent component) {
		if (context == null || component == null) {
			throw new NullPointerException();
		}

		if (!(component instanceof UIFileupload)) {
			return;
		}
		
		HtmlFileupload uploadComponent = null;
		if ( component instanceof HtmlFileupload ){
			uploadComponent = (HtmlFileupload)component;
		}
		
		((UIInput)component).setSubmittedValue(null);
		ResourceBundle bundle = ResourceHandler.getBundle(context);
		String localizedErr1 = null;
		String localizedErr2 = null;
		String bundleTxt = ResourceHandler.getString(bundle, "FileUpload.ErrMsgs"); //$NON-NLS-1$
		if ((bundleTxt == null) || (bundleTxt.indexOf(",") == -1)) {
			localizedErr1 = FILEUPLOAD_ERR1;
			localizedErr2 = FILEUPLOAD_ERR2;
		} else {
			try {
				localizedErr1 = bundleTxt.substring(0, bundleTxt.indexOf(","));
				localizedErr2 = bundleTxt.substring(bundleTxt.indexOf(",") + 1);
			} catch (IndexOutOfBoundsException iobe) {
				localizedErr1 = FILEUPLOAD_ERR1;
				localizedErr2 = FILEUPLOAD_ERR2;
			}
		}

		//Get uploaded item from the context
		ExternalContext excontext = context.getExternalContext();
		UIFileupload uiFileupload = (UIFileupload) component;

//		if (!(excontext instanceof MultipartExternalContext)) {
//			uiFileupload.setValid(false);
//			return;
//		}
		Map items = (Map) excontext.getRequestMap().get(MultipartExternalContextImpl.FILE_ITEM_MAP_PARAM);
		if (null == items) {
			uiFileupload.setValid(true);
			uiFileupload.setFilename(null);
			uiFileupload.setMimetype(null);
			uiFileupload.setValue(null);
			return;
		}
		String clientId = component.getClientId(context);
		ContentElement item = (ContentElement) items.get(clientId);

		//check if there's an uploaded item and it has name
		if (null == item || item.getName().length() == 0) {
			uiFileupload.setValid(true);
			uiFileupload.setFilename(null);
			uiFileupload.setMimetype(null);
			uiFileupload.setValue(null);
			return;
		}
		//check if there's a valid filename
		String fname = item.getFileName();
		if ((fname == null) || (fname.equals("\"\""))) {
			uiFileupload.setValid(true);
			uiFileupload.setFilename(null);
			uiFileupload.setMimetype(null);
			uiFileupload.setValue(null);
			return;
		}
		fname = fname.substring(1, fname.length() - 1);
		//check if there's a valid mimetype
		String itemMimeType = item.getMimeType();
		if (itemMimeType == null) {
			uiFileupload.setValid(true);
			uiFileupload.setFilename(null);
			uiFileupload.setMimetype(null);
			uiFileupload.setValue(null);
			return;
		}

		//Check for accepted mimetypes
		String expected_attr;
		if ( uploadComponent != null ){
			expected_attr = uploadComponent.getAccept();
		}else{
			expected_attr = (String) component.getAttributes().get(ACCEPT_ATTR);
		}

		final Properties expected = parseAcceptIntoProperties(expected_attr); //$NON-NLS-1$
		if (expected != null) {
			String first = itemMimeType.substring(0, itemMimeType.indexOf("/")); //$NON-NLS-1$
			String second = itemMimeType.substring(itemMimeType.indexOf("/") + 1); //$NON-NLS-1$

			Vector lookup = (Vector) expected.get(first);

			if ((lookup == null) || ((!lookup.contains(second)) && (!lookup.contains("*")))) //$NON-NLS-1$
				{
				uiFileupload.setValid(false);
				String err = localizedErr1 + itemMimeType; //$NON-NLS-1$
				context.addMessage(
					clientId,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, err, err));
				return;
			}
		}

		//check for excluded file extensions
		String exclude_attr;
		if ( uploadComponent != null ){
			exclude_attr = uploadComponent.getExclude();
		}else{
			exclude_attr = (String) component.getAttributes().get(EXCLUDE_ATTR);
		}
				
		final Vector exclude_expected = parseExcludeIntoVector(exclude_attr);
		if (exclude_expected != null) {
			String ext = ""; //$NON-NLS-1$
			int epos = fname.lastIndexOf("."); //$NON-NLS-1$
			if (epos != -1) {
				ext = fname.substring(epos + 1);
			}
			final String lookup = ext;
			if (exclude_expected.contains(lookup)) {
				uiFileupload.setValid(false);
				String err = localizedErr2 + lookup; //$NON-NLS-1$
				context.addMessage(
					clientId,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, err, err));
				return;
			}

		}
		uiFileupload.setMimetype(itemMimeType);
		uiFileupload.setFilename(fname);
		uiFileupload.setValue(item);
		uiFileupload.setValid(true);
	}

	/**
	 * Method for finding the form id; useful when doing a JavaScript submit.
	 * @param component
	 * @return String
	 */
	private UIForm getForm(UIComponent component) {
		if (component == null) {
			return null;
		}
		if (component instanceof UIForm) {
			return (UIForm) component;
		}
		return getForm(component.getParent());
	}
	private Properties parseAcceptIntoProperties(String acceptString) {
		if ((acceptString == null) || (acceptString.trim() == "")) //$NON-NLS-1$
			return null;
		Properties prop = new Properties();
		Vector values = new Vector();
		StringTokenizer tokenizer = new StringTokenizer(acceptString, ","); //$NON-NLS-1$
		while (tokenizer.hasMoreTokens()) {
			String token = tokenizer.nextToken();
			int pos = token.indexOf("/"); //$NON-NLS-1$
			if (pos == -1) {
				//System.out.println("FILEUPLOAD: Bad \"accept\" attribute provided:" + acceptString); //$NON-NLS-1$
				throw new IllegalArgumentException("Bad \"accept\" attribute. Expected valid mimetypes e.g. image/gif comma-delimeted. Value provided: " + acceptString); //$NON-NLS-1$
			}
			String first = token.substring(0, pos);
			String second = token.substring(pos + 1);
			values = (Vector) prop.get(first);
			if (values == null) {
				Vector newvec = new Vector();
				newvec.add(second);
				prop.put(first, newvec);
			} else {
				values.add(second);
				prop.put(first, values);
			}

		}
		return prop;
	}

	private Vector parseExcludeIntoVector(String excludeString) {
		if ((excludeString == null) || (excludeString.trim() == "")) //$NON-NLS-1$
			return null;
		Vector values = new Vector();
		StringTokenizer tokenizer = new StringTokenizer(excludeString, ","); //$NON-NLS-1$
		while (tokenizer.hasMoreTokens()) {
			values.add(tokenizer.nextToken());
		}
		return values;
	}

	public Object getConvertedValue(
		FacesContext context,
		UIComponent component,
		Object submittedValue)
		throws ConverterException {

		return submittedValue;
	}

}
