/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Method;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;
import javax.faces.convert.DateTimeConverter;
import javax.faces.validator.Validator;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.converter.MaskConverter;
import com.ibm.faces.converter.NumberConverterEx;
import com.ibm.faces.util.CacheHashMap;
import com.ibm.faces.util.JavaScriptUtil;
import com.ibm.faces.util.LifecycleUtil;
import com.ibm.faces.util.ScriptObject;
import com.ibm.faces.validator.DateTimeRangeValidator;
import com.ibm.faces.validator.DoubleRangeValidatorEx;
import com.ibm.faces.validator.ExpressionValidator;
import com.ibm.faces.validator.LongRangeValidatorEx;

/**
 * @author Bob Gallagher
 *
 */
public class HxClientV3{
	
	public static final String HX_LIB_FILENAME = "hxclient_v3.js";
	public static final String HX_VAR_NAME = "hX_3";
	public static final String HXG_FUNCTION_NAME = "HxG_3";
	
	private static final ClassLoader hxClientClassLoader = (new HxClientV3()).getClass().getClassLoader();

	//    static private final String DEFAULT_BASE_YEAR_OFFSET = "80"; //$NON-NLS-1$
	private static final String DATESTYLE_SHORT = "short"; //$NON-NLS-1$
	private static final String DATESTYLE_MEDIUM = "medium"; //$NON-NLS-1$
	private static final String DATESTYLE_LONG = "long"; //$NON-NLS-1$
	private static final String DATESTYLE_FULL = "full"; //$NON-NLS-1$

	private static final String NUMBERSTYLE_NUMBER = "number"; //$NON-NLS-1$
	private static final String NUMBERSTYLE_CURRENCY = "currency"; //$NON-NLS-1$
	private static final String NUMBERSTYLE_PERCENT = "percent"; //$NON-NLS-1$

	// generate an id
	private static int idSeed = 0;
	private synchronized static String genScriptId(FacesContext context) {
		return context.getExternalContext().encodeNamespace(Integer.toHexString(idSeed++));
	}

	// Some maps to hold on to converters/validators we may already have generated
	private static CacheHashMap maskConverterJs = new CacheHashMap(25);
	private static CacheHashMap dateConverterJs = new CacheHashMap(25);
	private static CacheHashMap numberConverterJs = new CacheHashMap(25);
	private static CacheHashMap rangeValidatorJs = new CacheHashMap(25);
	private static CacheHashMap constraintValidatorJs = new CacheHashMap(25);
	private static CacheHashMap libraryInclude = new CacheHashMap(12);
	
	private static String numberStrictnessLevel = null;
	private static String dateStrictnessLevel = null;
	
	public static final String DATETIMESTRICTNESSPARAM = "com.ibm.faces.DATETIME_ASSIST_STRICTNESS"; //$NON-NLS-1$
	public static final String NUMBERSTRICTNESSPARAM = "com.ibm.faces.NUMBER_ASSIST_STRICTNESS"; //$NON-NLS-1$
	
	public static final String BODY_SCOPE = "#body"; //$NON-NLS-1$
	/**
	 * Builds the javascript statement(s) to generate the setResourceServer 
	 * call to init resources in hxclient.js. This could return null if this call is not
	 * required.
	 *  
	 * @param context FacesContext
	 * @return return null if is not need else the setResourceServer call to init resource in hxclient.js
	 */
	public static final String buildSetResourcePathCall(FacesContext context) {
		StringBuffer script = new StringBuffer();
		script.append("if (hX_3)  hX_3.setResourceServer(\""); //$NON-NLS-1$
		
		//RATLC01083006
		script.append(LifecycleUtil.getResourceServletContextPath(context));
		
		script.append("\");"); //$NON-NLS-1$
		return script.toString();
	}

	/**
	 * Builds the javascript statement(s) to instanciate a converter and register it. 
	 * The javascript is cached to be available for requests for a converter configure 
	 * the same.
	 * This method is synchronized to avoid multiple copies of same converter being added.
	 * It is banking on getting a high hit rate for cache otherwise it performance may
	 * be poor due to blocking for the creationg of coverter code. The hit rate will
	 * come from the same page being served up to multiple users and use in a dataTable
	 *  
	 * @param context FacesContext
	 * @param converter DateTimeConverter
	 * @return ScriptObject containing the id and javascript of the converter.
	 */
	public synchronized static final ScriptObject buildJsMaskConverter(
		FacesContext context,
		MaskConverter maskConverter) {

		if (null == maskConverter) {
			return null;
		}
		String pattern = maskConverter.getMask();
		// let's see if we already have one
		ScriptObject cachedScript = (ScriptObject) maskConverterJs.get(pattern);
		if (null != cachedScript) {
			return cachedScript;
		}
		// ok, build one

		String converterId = genScriptId(context);

		if (pattern != null && !pattern.equals("")) { //$NON-NLS-1$
			StringBuffer javascript = new StringBuffer("hX_3.addConverter(\""); //$NON-NLS-1$
			javascript.append(converterId);
			javascript.append("\", "); //$NON-NLS-1$
			javascript.append(buildJsMaskConverter(maskConverter));
			javascript.append(");"); //$NON-NLS-1$
			ScriptObject so = new ScriptObject(converterId, javascript.toString());
			maskConverterJs.put(pattern, so);
			return so;
		}
		return null;
	}
	/**
	 * Builds the javascript statement to create a new MaskConverter. 
	 *  
	 * @param MaskConverter
	 * @return the javascript new converter statement without trailing ';'
	 */
	public static final String buildJsMaskConverter(MaskConverter maskConverter) {
		if (null == maskConverter) {
			return null;
		}
		String pattern = maskConverter.getMask();
		if (pattern != null && !pattern.equals("")) { //$NON-NLS-1$
			StringBuffer javascript = new StringBuffer("new hX_3.MaskConverter(\"mask:"); //$NON-NLS-1$
			javascript.append(pattern);
			javascript.append("\")"); //$NON-NLS-1$
			return javascript.toString();
		}
		return null;
	}

	/**
	 * Builds the javascript statement(s) to instanciate a converter and register it. 
	 * The javascript is cached to be available for requests for a converter configure 
	 * the same.
	 * This method is synchronized to avoid multiple copies of same converter being added.
	 * It is banking on getting a high hit rate for cache otherwise it performance may
	 * be poor due to blocking for the creationg of coverter code. The hit rate will
	 * come from the same page being served up to multiple users and use in a dataTable
	 *  
	 * @param context FacesContext
	 * @param converter DateTimeConverter
	 * @return ScriptObject containing the id and javascript of the converter.
	 */
	public synchronized static final ScriptObject buildJsDateConverter(
		FacesContext context,
		DateTimeConverter datetimeConverter) {

		if (null == datetimeConverter) {
			datetimeConverter = defaultDateTimeConverter;
		}
		String converterKey = createDateConverterKey(datetimeConverter);
		// let's see if we already have one
		ScriptObject cachedScript = (ScriptObject) dateConverterJs.get(converterKey);
		if (null != cachedScript) {
			return cachedScript;
		}
		// ok, build one
		
		setupStrictnessLevel(context);

		String converterId = genScriptId(context);
		String newConverter = buildJsDateTimeConverter(datetimeConverter);
		if (null != newConverter) {
			StringBuffer javascript = new StringBuffer("hX_3.addConverter(\""); //$NON-NLS-1$
			javascript.append(converterId);
			javascript.append("\", "); //$NON-NLS-1$
			javascript.append(newConverter);
			javascript.append(");"); //$NON-NLS-1$
			ScriptObject so = new ScriptObject(converterId, javascript.toString());
			dateConverterJs.put(converterKey, so);
			return so;
		}
		return null;
	}
	private static DateTimeConverter defaultDateTimeConverter = new DateTimeConverter();

	private static String createDateConverterKey(DateTimeConverter converter) {
		StringBuffer key = new StringBuffer(converter.getLocale().toString());
		key.append(converter.getType());
		key.append(converter.getPattern());
		key.append(converter.getDateStyle());
		key.append(converter.getTimeStyle());
		return key.toString();
	}
	/**
	 * Builds the javascript statement to create a new DateTimeConverter. 
	 *  
	 * @param datetimeConverter
	 * @return the javascript new converter statement without trailing ';'
	 */
	public static final String buildJsDateTimeConverter(DateTimeConverter datetimeConverter) {

		String pattern = datetimeConverter.getPattern();
		String type = datetimeConverter.getType();
		String dateStyle = datetimeConverter.getDateStyle();
		String timeStyle = datetimeConverter.getTimeStyle();
		String format = null;

		if (pattern != null && !pattern.equals("")) //$NON-NLS-1$
			format = pattern;
		else {
			Locale locale = datetimeConverter.getLocale();
			DateFormat df = null;
			if (type.equals("both")) { //$NON-NLS-1$
				df = DateFormat.getDateTimeInstance(getStyleCode(dateStyle), getStyleCode(timeStyle), locale);
			} else if (type.equals("date")) { //$NON-NLS-1$
				df = DateFormat.getDateInstance(getStyleCode(dateStyle), locale);
			} else if (type.equals("time")) { //$NON-NLS-1$
				df = DateFormat.getTimeInstance(getStyleCode(timeStyle), locale);
			} else {
				df = DateFormat.getDateInstance(getStyleCode(dateStyle), locale);
			}
			format = ((SimpleDateFormat) df).toPattern();
		}
		if (null != format) {
			StringBuffer javascript = new StringBuffer("new hX_3.DateTimeConverter(\"strict:" + dateStrictnessLevel +"\", \"format:"); //$NON-NLS-1$ //$NON-NLS-2$
			javascript.append(format);
			javascript.append("\")"); //$NON-NLS-1$
			return javascript.toString();
		}
		return null;
	}

	/**
	 * Builds the javascript statement(s) to instanciate a converter and register it. 
	 * The javascript is cached to be available for requests for a converter configure 
	 * the same.
	 * This method is synchronized to avoid multiple copies of same converter being added.
	 * It is banking on getting a high hit rate for cache otherwise it performance may
	 * be poor due to blocking for the creationg of coverter code. The hit rate will
	 * come from the same page being served up to multiple users and use in a dataTable
	 *  
	 * @param context FacesContext
	 * @param converter NumberConverterEx
	 * @return ScriptObject containing the id and javascript of the converter.
	 */
	public synchronized static final ScriptObject buildJsNumberConverter(
		FacesContext context,
		NumberConverterEx numConverter) {

		String numConverterKey = createNumberConverterKey(numConverter);
		// let's see if we already have one
		ScriptObject cachedScript = (ScriptObject) numberConverterJs.get(numConverterKey);
		if (null != cachedScript) {
			return cachedScript;
		}
		// ok, build one
		
		setupStrictnessLevel(context);

		String converterId = genScriptId(context);

		StringBuffer javascript = new StringBuffer();
		String newConverter = buildJsNumberConverter(numConverter);
		if (null != newConverter && newConverter.length() > 0) {
			javascript.append("hX_3.addConverter(\""); //$NON-NLS-1$
			javascript.append(converterId);
			javascript.append("\", "); //$NON-NLS-1$
			javascript.append(newConverter);
			javascript.append(");"); //$NON-NLS-1$
			ScriptObject so = new ScriptObject(converterId, javascript.toString());
			numberConverterJs.put(numConverterKey, so);
			return so;
		}
		return null;
	}
	
	private static String createNumberConverterKey(NumberConverterEx numConverter) {
		StringBuffer key = new StringBuffer(numConverter.getLocale().toString());
		key.append(numConverter.getPattern());
		key.append(numConverter.getType());
		key.append(numConverter.isCurrencySymbolSet() ? numConverter.getCurrencySymbol() : null);
		key.append(numConverter.isCurrencyCodeSet() ? numConverter.getCurrencyCode() : null);
		key.append(numConverter.isGroupingUsed());
		key.append(numConverter.isIntegerOnly());
		key.append(
			numConverter.isMaxFractionDigitsSet() ? Integer.toString(numConverter.getMaxFractionDigits()) : null);
		key.append(
			numConverter.isMinFractionDigitsSet() ? Integer.toString(numConverter.getMinFractionDigits()) : null);
		key.append(numConverter.isMaxIntegerDigitsSet() ? Integer.toString(numConverter.getMaxIntegerDigits()) : null);
		key.append(numConverter.isMinIntegerDigitsSet() ? Integer.toString(numConverter.getMinIntegerDigits()) : null);
		return key.toString();
	}
	private static String createNumberValidatorKey(Object a, Object b, Object c, Object d, boolean required, String expression) {
		StringBuffer key = new StringBuffer("n"); //$NON-NLS-1$
		key.append(a);
		key.append(b);
		key.append(c);
		key.append(d);
		key.append(required);
		key.append(expression);
		return key.toString();
	}
	private static String createDateTimeValidatorKey(Object a, Object b, Object c, Object d, boolean required, String expression) {
		StringBuffer key = new StringBuffer("d"); //$NON-NLS-1$
		key.append(a);
		key.append(b);
		key.append(c);
		key.append(d);
		key.append(required);
		key.append(expression);
		return key.toString();
	}
	private static String createStringValidatorKey(String constraint, String minLength, String maxLength, boolean required, String expression) {
		StringBuffer key = new StringBuffer();
		key.append(constraint);
		key.append(minLength);
		key.append(maxLength);
		key.append(required);
		key.append(expression);
		return key.toString();
	}
	/**
	 * @deprecated - use buildJsNumberConverter(NumberConverterEx)
	 * Builds the javascript statement(s) to instanciate a converter and register it. 
	 * It returns the id of the converter so it can be looked up in the converter map.
	 *  
	 * @param numConverter
	 * @param locale
	 * @return the javascript new converter statement
	 */
	public static final String buildJsNumberConverter(NumberConverterEx numConverter, Locale locale) {
		return buildJsNumberConverter(numConverter);
	}
	/**
	 * Builds the javascript statement to create a new NumberConverter. 
	 *  
	 * @param numConverter
	 * @param locale
	 * @return the javascript new converter statement without trailing ';'
	 */
	public static final String buildJsNumberConverter(NumberConverterEx numConverter) {

		String converterPattern = null;
		Locale locale = numConverter.getLocale();
		NumberFormat formatter = getNumberFormat(numConverter, locale);
		try {
			// Configure the formatter
			String pattern = numConverter.getPattern();
			String type = numConverter.getType();
			if (((pattern != null) && !pattern.equals("")) || NUMBERSTYLE_CURRENCY.equals(type)) { //$NON-NLS-1$
				configureCurrency(numConverter, formatter);
			}
			configureFormatter(numConverter, formatter);
			if (formatter instanceof DecimalFormat) {
				if(pattern != null && !pattern.equals("")){ //$NON-NLS-1$
					((DecimalFormat) formatter).applyPattern(pattern);
					converterPattern=pattern;
				} else {
					converterPattern = ((DecimalFormat) formatter).toPattern();	
				}
				
				//RATLC00670050 - output cannot handle the currency pattern character in japanese locale(there maybe others locales affected) 
				//replacing it with unicode string for the character
				while(converterPattern.indexOf("\u00A4")>-1){ //$NON-NLS-1$
					int pos=converterPattern.indexOf("\u00A4"); //$NON-NLS-1$
					StringBuffer b=new StringBuffer();
					b.append(converterPattern.substring(0,pos));
					b.append("\\u00A4"); //$NON-NLS-1$
					b.append(converterPattern.substring(pos+1,converterPattern.length()));
					converterPattern=b.toString();
				}
				
			} else {
				String formattedResult = formatter.format(new Double(111111111111111111.111111111111111111));
				converterPattern = formattedResult.replace('1', '#');
			}
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
		if (null != converterPattern && converterPattern.length() > 0) {
			StringBuffer javascript = new StringBuffer("new hX_3.NumberConverter(\"strict:" + numberStrictnessLevel + "\", \"pattern:"); //$NON-NLS-1$ //$NON-NLS-2$
			javascript.append(converterPattern);
			javascript.append("\", \"locale:"); //$NON-NLS-1$
			//RATLC00976099 fix, client side validation not working correctly when currency symbol set
			//This section of code was not using the symbols from the configured formatter set up previously in this method 
			DecimalFormatSymbols symbols=null;
			if(formatter instanceof DecimalFormat){
				symbols=((DecimalFormat)formatter).getDecimalFormatSymbols();
			} else {
				symbols = new DecimalFormatSymbols(locale);
			}
			javascript.append(symbols.getGroupingSeparator());
			javascript.append(symbols.getDecimalSeparator());
			javascript.append(symbols.getPercent());
			javascript.append(symbols.getPerMill());
			javascript.append(symbols.getMinusSign());
		    javascript.append(Utils.escapeBackslash(symbols.getCurrencySymbol()));
			//javascript.append(symbols.getCurrencySymbol());
			javascript.append("\")"); //$NON-NLS-1$		
			return javascript.toString();
		}
		return null;
	}

	private static int getStyleCode(String s) {
		int dateCode = DateFormat.DEFAULT;
		if (s != null && s != "") { //$NON-NLS-1$
			if (DATESTYLE_SHORT.equalsIgnoreCase(s)) {
				dateCode = DateFormat.SHORT;
			} else if (DATESTYLE_MEDIUM.equalsIgnoreCase(s)) {
				dateCode = DateFormat.MEDIUM;
			} else if (DATESTYLE_LONG.equalsIgnoreCase(s)) {
				dateCode = DateFormat.LONG;
			} else if (DATESTYLE_FULL.equalsIgnoreCase(s)) {
				dateCode = DateFormat.FULL;
			}
		}
		return dateCode;
	}

	/**
	 * Builds the javascript statement(s) to instanciate a validator and register it. 
	 * The javascript is cached to be available for requests for a validator configure 
	 * the same.
	 * This method is synchronized to avoid multiple copies of same validator being added.
	 * It is banking on getting a high hit rate for cache otherwise it performance may
	 * be poor due to blocking for the creationg of coverter code. The hit rate will
	 * come from the same page being served up to multiple users and use in a dataTable
	 *  
	 * @param context FacesContext
	 * @param validator Range validator
	 * @return ScriptObject containing the id and javascript of the validator.
	 */
	public synchronized static final ScriptObject buildJsRangeValidator(FacesContext context, Validator validator, boolean required, ExpressionValidator expressionValidator) {

		if (validator instanceof DateTimeRangeValidator) {
			return buildJsAddDateTimeValidator(context, (DateTimeRangeValidator) validator, required, expressionValidator);
		} else if ((validator instanceof LongRangeValidatorEx) || (validator instanceof DoubleRangeValidatorEx)) {
			return buildJsAddNumberValidator(context, validator, required, expressionValidator);
		}
		return null;
	}

	/**
	 * Builds the javascript statement(s) to instanciate a datetime validator and register it. 
	 * The javascript is cached to be available for requests for a validator configure 
	 * the same.
	 * This method is synchronized to avoid multiple copies of same validator being added.
	 * It is banking on getting a high hit rate for cache otherwise it performance may
	 * be poor due to blocking for the creationg of coverter code. The hit rate will
	 * come from the same page being served up to multiple users and use in a dataTable
	 *  
	 * @param context FacesContext
	 * @param validator Range validator
	 * @return ScriptObject containing the id and javascript of the validator.
	 */
	public synchronized static final ScriptObject buildJsAddDateTimeValidator(FacesContext context, DateTimeRangeValidator validator, boolean required, ExpressionValidator expressionValidator) {
	
		Object minBound = null;
		Object maxBound = null;
	
		String validatorKey = null;
		String validatorId = null;
		String newValidator = null;
	
		String expression = null;
		if(expressionValidator != null) {
			expression = JavaScriptUtil.convertEL2JavaScript(expressionValidator.getExpression());
		}

		validatorKey =
			createDateTimeValidatorKey(
				null == validator ? null:validator.getMinimum(),
				null == validator ? null:validator.getMaximum(),
				minBound,
				maxBound,
				required,
				expression);
		// let's see if we already have one
		ScriptObject cachedScript = (ScriptObject) rangeValidatorJs.get(validatorKey);
		if (null != cachedScript) {
			return cachedScript;
		}
		newValidator =
			buildJsDateRangeValidator(validator, (Date) minBound, (Date) maxBound, required, expression);
		validatorId = genScriptId(context);
		if (null != newValidator) {
			StringBuffer javascript = new StringBuffer("hX_3.addValidator(\""); //$NON-NLS-1$
			javascript.append(validatorId);
			javascript.append("\", "); //$NON-NLS-1$
			javascript.append(newValidator);
			javascript.append(");"); //$NON-NLS-1$
			ScriptObject so = new ScriptObject(validatorId, javascript.toString());
			rangeValidatorJs.put(validatorKey, so);
			return so;
		}
		return null;
	}


	/**
	 * Builds the javascript statement(s) to instanciate a number validator and register it. 
	 * The javascript is cached to be available for requests for a validator configure 
	 * the same.
	 * This method is synchronized to avoid multiple copies of same validator being added.
	 * It is banking on getting a high hit rate for cache otherwise it performance may
	 * be poor due to blocking for the creationg of coverter code. The hit rate will
	 * come from the same page being served up to multiple users and use in a dataTable
	 *  
	 * @param context FacesContext
	 * @param validator Range validator
	 * @return ScriptObject containing the id and javascript of the validator.
	 */
	public synchronized static final ScriptObject buildJsAddNumberValidator(FacesContext context, Validator validator, boolean required, ExpressionValidator expressionValidator) {

		Object minBound = null;
		Object maxBound = null;

		String validatorKey = null;
		String validatorId = null;
		String newValidator = null;

		String expression = null;
		if(expressionValidator != null) {
			expression = JavaScriptUtil.convertEL2JavaScript(expressionValidator.getExpression());
		}

		if ((validator instanceof LongRangeValidatorEx) || (validator instanceof DoubleRangeValidatorEx)) {
			if (validator instanceof LongRangeValidatorEx) {
				validatorKey =
					createNumberValidatorKey(
						null == validator ? null:new Long(((LongRangeValidatorEx) validator).getMinimum()),
						null == validator ? null:new Long(((LongRangeValidatorEx) validator).getMaximum()),
						minBound,
						maxBound,
						required,
						expression);
			} else {
				validatorKey =
					createNumberValidatorKey(
						null == validator ? null:new Double(((DoubleRangeValidatorEx) validator).getMinimum()),
						null == validator ? null:new Double(((DoubleRangeValidatorEx) validator).getMaximum()),
						minBound,
						maxBound,
						required,
						expression);
			}
		} else {
			validatorKey =
				createNumberValidatorKey(
					null,
					null,
					null,
					null,
					required,
					expression);
		}
		// let's see if we already have one
		ScriptObject cachedScript = (ScriptObject) rangeValidatorJs.get(validatorKey);
		if (null != cachedScript) {
			return cachedScript;
		}
		newValidator = buildJsNumberRangeValidator(validator, (Number) minBound, (Number) maxBound, required, expression);
		validatorId = genScriptId(context);
		if (null != newValidator) {
			StringBuffer javascript = new StringBuffer("hX_3.addValidator(\""); //$NON-NLS-1$
			javascript.append(validatorId);
			javascript.append("\", "); //$NON-NLS-1$
			javascript.append(newValidator);
			javascript.append(");"); //$NON-NLS-1$
			ScriptObject so = new ScriptObject(validatorId, javascript.toString());
			rangeValidatorJs.put(validatorKey, so);
			return so;
		}
		return null;
	}

	public static final Validator getRangeValidator(UIInput input) {
		// Check if the component has a validation tag
		Validator validators[] = input.getValidators();
		for (int i = 0; i < validators.length; i++) {
			Validator validator = validators[i];
			if ((validator instanceof DateTimeRangeValidator)
				|| (validator instanceof LongRangeValidatorEx)
				|| (validator instanceof DoubleRangeValidatorEx)) {
				return validator;
			}
		}
		return null;
	}

	public static final Validator getNumberRangeValidator(UIInput input) {
		// Check if the component has a validation tag
		Validator validators[] = input.getValidators();
		for (int i = 0; i < validators.length; i++) {
			Validator validator = validators[i];
			if ((validator instanceof LongRangeValidatorEx)
				|| (validator instanceof DoubleRangeValidatorEx)) {
				return validator;
			}
		}
		return null;
	}

	public static final DateTimeRangeValidator getDateRangeValidator(UIInput input) {
		// Check if the component has a validation tag
		Validator validators[] = input.getValidators();
		for (int i = 0; i < validators.length; i++) {
			Validator validator = validators[i];
			if (validator instanceof DateTimeRangeValidator) {
				return (DateTimeRangeValidator)validator;
			}
		}
		return null;
	}
	
	public static final ExpressionValidator getExpressionValidator(UIInput input) {
		// Check if the component has a validation tag
		Validator validators[] = input.getValidators();
		for (int i = 0; i < validators.length; i++) {
			Validator validator = validators[i];
			if (validator instanceof ExpressionValidator) {
				return (ExpressionValidator)validator;
			}
		}
		return null;
	}

	/**
	 * Builds the javascript statement(s) to instanciate a number validator. 
	 *  
	 * @param validator
	 * @param minBound override of validator min
	 * @param maxBound override of validator max
	 * @return the javascript new validator statement
	 */
	public static final String buildJsNumberRangeValidator(Validator validator) {
		return buildJsNumberRangeValidator(validator, null, null, false, null);
	}

	/**
	 * Builds the javascript statement(s) to instanciate a number validator. 
	 *  
	 * @param validator
	 * @param minBound override of validator min
	 * @param maxBound override of validator max
	 * @return the javascript new validator statement
	 */
	public static final String buildJsNumberRangeValidator(Validator validator, Number minBound, Number maxBound,boolean required,String expression) {

		StringBuffer javascript = new StringBuffer();
		if (validator instanceof LongRangeValidatorEx) {
			LongRangeValidatorEx longValidator = (LongRangeValidatorEx) validator;
			if (null == minBound && longValidator.isMinimumSet()) {
				minBound = new Long(longValidator.getMinimum());
			}
			if (null == maxBound && longValidator.isMaximumSet()) {
				maxBound = new Long(longValidator.getMaximum());
			}
		}
		if (validator instanceof DoubleRangeValidatorEx) {
			DoubleRangeValidatorEx doubleValidator = (DoubleRangeValidatorEx) validator;
			if (null == minBound && doubleValidator.isMinimumSet()) {
				minBound = new Double(doubleValidator.getMinimum());
			}
			if (null == maxBound && doubleValidator.isMaximumSet()) {
				maxBound = new Double(doubleValidator.getMaximum());
			}

		}
		javascript.append("new hX_3.NumberValidator("); //$NON-NLS-1$
		int nArgs = 0;
		if (minBound != null) {
			javascript.append(" \"min-bound:"); //$NON-NLS-1$
			javascript.append(minBound);
			javascript.append('\"'); //$NON-NLS-1$ //$NON-NLS-2$
			nArgs++;
		}
		if (maxBound != null) {
			if (nArgs++ > 0) {
				javascript.append(", "); //$NON-NLS-1$
			}
			javascript.append(" \"max-bound:"); //$NON-NLS-1$
			javascript.append(maxBound);
			javascript.append('\"'); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if(required){
			if (nArgs++ > 0){
				javascript.append(", "); //$NON-NLS-1$
			}
			javascript.append(" \"required:true\""); //$NON-NLS-1$
		}
		if(expression != null) {
			if (nArgs++ > 0){
				javascript.append(", "); //$NON-NLS-1$
			}
			javascript.append(" \"constraint-expression:"); //$NON-NLS-1$
			javascript.append(expression);
			javascript.append('\"'); //$NON-NLS-1$ 
		}
		javascript.append(")"); //$NON-NLS-1$		
		return javascript.toString();
	}
	/**
	 * Builds the javascript statement(s) to instanciate a number validator. 
	 *  
	 * @param validator
	 * @param minBound override of validator min
	 * @param maxBound override of validator max
	 * @return the javascript new validator statement
	 */
	public static final String buildJsDateRangeValidator(DateTimeRangeValidator validator) {
		return buildJsDateRangeValidator(validator, null, null,false, null);
	}

	/**
	 * Builds the javascript statement(s) to instanciate a number validator. 
	 *  
	 * @param validator
	 * @param minBound override of validator min
	 * @param maxBound override of validator max
	 * @return the javascript new validator statement
	 */
	public static final String buildJsDateRangeValidator(
		DateTimeRangeValidator validator,
		Date minBound,
		Date maxBound,
		boolean required,
		String expression) {

		StringBuffer javascript = new StringBuffer();
		if (null != validator){
			if (null == minBound) {
				minBound = validator.getMinimum(); // override component's own settings
			}
			if (null == maxBound) {
				maxBound = validator.getMaximum(); // override component's own settings
			}
		}
		javascript.append("new hX_3.DateTimeValidator("); //$NON-NLS-1$
		SimpleDateFormat sd = new SimpleDateFormat("yyyyMMddHHmmss"); //$NON-NLS-1$
		int nArgs = 0;
		if (minBound != null) {
			javascript.append(" \"min-bound:"); //$NON-NLS-1$
			javascript.append(sd.format(minBound));
			javascript.append('\"'); //$NON-NLS-1$ //$NON-NLS-2$
			nArgs++;
		}
		if (maxBound != null) {
			if (nArgs++ > 0) {
				javascript.append(", "); //$NON-NLS-1$
			}
			javascript.append(" \"max-bound:"); //$NON-NLS-1$
			javascript.append(sd.format(maxBound));
			javascript.append('\"'); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if(required){
			if (nArgs++ > 0){
				javascript.append(", "); //$NON-NLS-1$
			}
			javascript.append(" \"required:true\""); //$NON-NLS-1$
		}
		if(expression != null) {
			if (nArgs++ > 0){
				javascript.append(", "); //$NON-NLS-1$
			}
			javascript.append(" \"constraint-expression:"); //$NON-NLS-1$
			javascript.append(expression);
			javascript.append('\"'); //$NON-NLS-1$ 
		}

		javascript.append(")"); //$NON-NLS-1$	
		return javascript.toString();
	}

	/**
	 * <p>Return a <code>NumberFormat</code> instance to use for formatting
	 * and parsing in this {@link Converter}.</p>
	 *          
	 * @param locale The <code>Locale</code> used to select formatting
	 *  and parsing conventions
	 *
	 * @exception ConverterException if no instance can be created
	 */
	static private NumberFormat getNumberFormat(NumberConverterEx numConverter, Locale locale) {
		NumberFormat formatter = null;
		String pattern = numConverter.getPattern();
		String type = numConverter.getType();
		if (pattern == null && type == null) {
			throw new IllegalArgumentException("Either pattern or type must" + " be specified."); //$NON-NLS-1$ //$NON-NLS-2$
		}


		// If pattern is specified, type is ignored
		if (pattern != null) {
			DecimalFormatSymbols symbols = new DecimalFormatSymbols(locale);
			formatter = new DecimalFormat(pattern, symbols);
		}

		// Create an instance based on the specified type
		else if (type.equals(NUMBERSTYLE_CURRENCY)) {
			formatter = NumberFormat.getCurrencyInstance(locale);
		} else if (type.equals(NUMBERSTYLE_NUMBER)) {
			formatter = NumberFormat.getNumberInstance(locale);
		} else if (type.equals(NUMBERSTYLE_PERCENT)) {
			formatter = NumberFormat.getPercentInstance(locale);
		} else {
			throw new ConverterException(new IllegalArgumentException(type));
		}
		if (numConverter.isIntegerOnly()) {
			formatter.setMaximumFractionDigits(0);
		}
		return formatter;
	}

	private static Boolean curClassAvailable = null;
	private static Class curClass = null;
	private static Class[] stringArg = new Class[]{String.class};
	
	private static void configureCurrency(NumberConverterEx numConverter, NumberFormat formatter) throws Exception {
		//if code or symbol not set do nothing
		if(numConverter.getCurrencyCode() == null && numConverter.getCurrencySymbol() == null){
			return;
		}
		if(curClassAvailable == null) {
				try {
					curClass = Class.forName("java.util.Currency");
					curClassAvailable = Boolean.TRUE;
				} catch (ClassNotFoundException e) {
					//jre version < 1.4
					curClassAvailable = Boolean.FALSE;
				}
		}
		
		//if currencycode is set and currency class is available
		if(numConverter.getCurrencyCode() != null && curClassAvailable.booleanValue()){
			
			Method method = curClass.getMethod("getInstance",stringArg);
			Object cur = method.invoke(null, new Object[]{numConverter.getCurrencyCode()});
			
			Class numFormatClass = Class.forName("java.text.NumberFormat");
			method = numFormatClass.getMethod("setCurrency",new Class[]{curClass});
			method.invoke(formatter, new Object[]{cur});
			return;
		}
		String curSymbol = null;
		
		if (numConverter.getCurrencySymbol() != null){
			curSymbol = numConverter.getCurrencySymbol();
		} else if (numConverter.getCurrencyCode() != null){
			curSymbol = numConverter.getCurrencyCode();
			
		}

		if (curSymbol != null){
			DecimalFormat df = (DecimalFormat) formatter;
			DecimalFormatSymbols dfs = df.getDecimalFormatSymbols();
			dfs.setCurrencySymbol(curSymbol);
			df.setDecimalFormatSymbols(dfs);
		}
	}

	/**
	 * <p>Configure the specified <code>NumberFormat</code> based on the
	 * formatting properties that have been set.</p>
	 *
	 * @param formatter The <code>NumberFormat</code> instance to configure
	 */
	private static void configureFormatter(NumberConverterEx numConverter, NumberFormat formatter) {

		formatter.setGroupingUsed(numConverter.isGroupingUsed());
		if (numConverter.isMaxIntegerDigitsSet()) {
			formatter.setMaximumIntegerDigits(numConverter.getMaxIntegerDigits());
		}
		if (numConverter.isMinIntegerDigitsSet()) {
			formatter.setMinimumIntegerDigits(numConverter.getMinIntegerDigits());
		}
		if (numConverter.isMaxFractionDigitsSet()) {
			formatter.setMaximumFractionDigits(numConverter.getMaxFractionDigits());
		}
		if (numConverter.isMinFractionDigitsSet()) {
			formatter.setMinimumFractionDigits(numConverter.getMinFractionDigits());
		}

	}

	/**
	 * @param component
	 * @param context
	 */
	public static void initJSLibraries(UIComponent component, FacesContext context) {
		initJSLibraries(component, context, false);
	}

	/**
	 * @deprecated use initJSLibraries with version
	 * @param component
	 * @param context
	 * @param bWriteNow
	 */
	public static void initJSLibraries(UIComponent component, FacesContext context, boolean bWriteNow) {
		UIScriptCollector scriptCollector = null;
		if (component instanceof UIScriptCollector) {
			scriptCollector = (UIScriptCollector) component;
		} else {
			scriptCollector = UIScriptCollector.find(component);
		}
		ScriptObject script = buildLibraryIncludeBlock(context);
		if (bWriteNow) {
			scriptCollector.writeLibraryReference(context, script.getId(), script.getScript());
		} else {
			scriptCollector.addLibrary(script.getId(), script.getScript());
			scriptCollector.addScriptOnce("hX_3.onPageLoad();"); //$NON-NLS-1$ 
		}
	}

	/**
	 * Build hxclient library include block
	 * @param context
	 * @param version
	 * @return ScriptObject where the Id is the library name and the script is the library include block
	 * which includes the script file include and string definitions and call to resource init.
	 */
	public synchronized static ScriptObject buildLibraryIncludeBlock(FacesContext context) {

		String libKey = context.getViewRoot().getLocale().toString();		
		ScriptObject script = (ScriptObject)libraryInclude.get(libKey);
		if (null != script){
			return script;
		}
		ResourceBundle bundle = ResourceHandler.getBundle(context);
		String libStrings = ResourceHandler.buildLocalizedLibName(bundle, "ibmjsfres/hxclient_S_3.js"); //$NON-NLS-1$ //$NON-NLS-2$
	
		String strings = null;
		InputStream stream = hxClientClassLoader.getResourceAsStream(libStrings);
		if (stream != null) {
			byte[] buffer = new byte[10000];
			byte[] bytes = new byte[0];
			int read = 0;
			try {
				while ((read = stream.read(buffer)) > 0) {
					int newSize = bytes.length + read;
					byte[] newBytes = new byte[newSize];
					System.arraycopy(bytes, 0, newBytes, 0, bytes.length);
					System.arraycopy(buffer, 0, newBytes, bytes.length, read);
					bytes = newBytes;
				}
				stream.close();
			} catch (IOException e) {
				e.printStackTrace();
			}

			try {
				String encoding = bundle.getString("Locale.encoding"); //$NON-NLS-1$
				if (encoding == null || encoding.trim().length() == 0 || encoding.equals("!Locale.encoding!")) { //$NON-NLS-1$
					encoding = "ISO-8859-1"; //$NON-NLS-1$
				}
				strings = new String(bytes, encoding);
			} catch (UnsupportedEncodingException e1) {
				// Shouldn't happen!
				e1.printStackTrace();
				strings = new String(bytes);
			}
		}
		StringBuffer buffer = new StringBuffer();
		buffer.append("<script type=\"text/JavaScript\" language=\"JavaScript\""); //$NON-NLS-1$
		buffer.append(" src=\""); //$NON-NLS-1$
		
		//RATLC01083006
		buffer.append(LifecycleUtil.getResourceServletContextPath(context));
		buffer.append("/hxclient_v3.js\"></script>\n"); //$NON-NLS-1$
		
		buffer.append("<script type=\"text/JavaScript\" language=\"JavaScript\">\n"); //$NON-NLS-1$
		if (null != strings){
			buffer.append(strings);
		}
		buffer.append(buildSetResourcePathCall(context));
		buffer.append("\n</script>"); //$NON-NLS-1$
		libraryInclude.put(libKey, script = (new ScriptObject(libKey, buffer.toString())));
		return script;
	}

	/**
	* <p>Return <code>true</code> if the component is contained in a another
	* component that renders its children.</p>
	*/
	public static boolean isContainedInPanel(UIComponent component) {

		while (component != null) {
			if (component.getRendersChildren()) {
				return (true);
			}
			component = component.getParent();
		}
		return (false);

	}

	/**
        * @deprecated - use buildJsAddStringValidator()
		*  
		* @param context FacesContext
		* @param validators validators on component
		* @return ScriptObject containing the id and javascript of the validator.
		*/
	public synchronized static final ScriptObject buildJsStringValidator(
		FacesContext context,
		String constraint,
		String minLength,
		String maxLength,
		boolean required,
		ExpressionValidator expressionValidator) {
			return buildJsAddStringValidator(context, constraint, minLength, maxLength, required, expressionValidator);
		}

	/**
		* Builds the javascript statement(s) to instanciate a validator and register it. 
		* The javascript is cached to be available for requests for a validator configure 
		* the same.
		* This method is synchronized to avoid multiple copies of same validator being added.
		* It is banking on getting a high hit rate for cache otherwise it performance may
		* be poor due to blocking for the creationg of coverter code. The hit rate will
		* come from the same page being served up to multiple users and use in a dataTable
		*  
		* @param context FacesContext
		* @param validators validators on component
		* @return ScriptObject containing the id and javascript of the validator.
		*/
	public synchronized static final ScriptObject buildJsAddStringValidator(
		FacesContext context,
		String regexp,
		String minLength,
		String maxLength,
		boolean required,
		ExpressionValidator expressionValidator) {

		String validatorKey = null;
		String validatorId = null;
		String expression = null;
		if(expressionValidator != null) {
			expression = JavaScriptUtil.convertEL2JavaScript(expressionValidator.getExpression());
		}
		validatorKey = createStringValidatorKey(regexp, minLength, maxLength, required, expression);
		// let's see if we already have one
		ScriptObject cachedScript = (ScriptObject) constraintValidatorJs.get(validatorKey);
		if (null != cachedScript) {
			return cachedScript;
		}
		String[] args = new String[5];
		int iArg = 0;
		if (null != minLength) {
			args[iArg++] = "min-length:" + minLength; //$NON-NLS-1$
		}
		if (null != maxLength) {
			args[iArg++] = "max-length:" + maxLength; //$NON-NLS-1$
		}
		if (required){
			args[iArg++] = "required:true"; //$NON-NLS-1$
		}
		if (null != expression) {
			args[iArg++] = "constraint-expression:" + expression; //$NON-NLS-1$
		}
		if (null != regexp) {
			if(regexp.equals("AlphabetOnly") || regexp.equals("DigitOnly") || regexp.equals("AlnumOnly") ||  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			   regexp.equals("AlphabetExOnly") || regexp.equals("AlnumExOnly")) { //$NON-NLS-1$ //$NON-NLS-2$

				args[iArg++] = "constraint:" + regexp; //$NON-NLS-1$
			}
			else {
				// JavaScript can't handle back slashes in regular expressions correctly.
				// We need to replace \ with \u005c for JS to work.
				regexp = regexp.replaceAll("\\\\", "\\\\u005c"); //$NON-NLS-1$ //$NON-NLS-2$
				
				args[iArg++] = "constraint-regexpression:" + regexp; //$NON-NLS-1$
			}
		}

		if (iArg > 0) {
			validatorId = genScriptId(context);
			StringBuffer javascript = new StringBuffer("hX_3.addValidator(\""); //$NON-NLS-1$
			javascript.append(validatorId);
			javascript.append("\", "); //$NON-NLS-1$

			javascript.append("new hX_3.StringValidator(\"") //$NON-NLS-1$
			.append(args[--iArg]).append('\"');
			while (iArg > 0) {
				javascript.append(", \"") //$NON-NLS-1$
				.append(args[--iArg]).append('\"');
			}
			javascript.append("));"); //$NON-NLS-1$
			ScriptObject so = new ScriptObject(validatorId, javascript.toString());
			constraintValidatorJs.put(validatorKey, so);
			return so;
		}
		return null;
	}
	
	public static void setupStrictnessLevel(FacesContext context){
		if(dateStrictnessLevel == null){
			dateStrictnessLevel = context.getExternalContext().getInitParameter(DATETIMESTRICTNESSPARAM);
			if(dateStrictnessLevel == null || (!dateStrictnessLevel.equals("1") && !dateStrictnessLevel.equals("2"))){ //$NON-NLS-1$ //$NON-NLS-2$
				dateStrictnessLevel = "1"; //$NON-NLS-1$
			}
		}
		if(numberStrictnessLevel == null){
			numberStrictnessLevel = context.getExternalContext().getInitParameter(NUMBERSTRICTNESSPARAM);
			if(numberStrictnessLevel == null || (!numberStrictnessLevel.equals("1") && !numberStrictnessLevel.equals("2"))){ //$NON-NLS-1$ //$NON-NLS-2$
				numberStrictnessLevel = "1"; //$NON-NLS-1$
			}
		}
	}
	
	/**
	 * @return Returns the dateStrictnessLevel.
	 */
	public static String getDateStrictnessLevel() {
		return dateStrictnessLevel;
	}
	/**
	 * @return Returns the numberStrictnessLevel.
	 */
	public static String getNumberStrictnessLevel() {
		return numberStrictnessLevel;
	}
}
