/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.el.ValueBinding;
import javax.faces.render.Renderer;

import com.ibm.faces.webapp.ValueResourcePhaseListener;



import com.ibm.faces.component.html.HtmlGraphicImageEx;

/**
 * Renderer for {@link javax.faces.component.UIGraphic UIGraphic} component, 
 * that creates an image.
 * <p> 
 * This renderer is based on the standard Image renderer and extends it
 * to support binding to a data source and different image types.
 * <p>
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.Graphic_ImageExTag}<br>
 * <b>Components:</b> {@link javax.faces.component.UIGraphic}
 */

public class ImageExRenderer extends Renderer {
	private final static String MIMETYPE_ATTR = "mimeType"; //$NON-NLS-1$
	/** 
	 * Renders the beginning of the component.
	 * <p>
	 * Doesn't need to do anything; all processing is done in {@link ImageExRenderer#encodeEnd encodeEnd}.
	 * <p>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeBegin(FacesContext context, UIComponent component)
		throws IOException {
		if (context == null || component == null) {
			throw new NullPointerException();
		}
	}

	/** 
	 * Renders component's children.
	 * <p>
	 * Doesn't need to do anything, all processing is done in {@link ImageExRenderer#encodeEnd encodeEnd}.
	 * <p>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeChildren(FacesContext context, UIComponent component) {
		if (context == null || component == null) {
			throw new NullPointerException();
		}
	}

	/**
	 * Renders the ending of the component.
	 * <p>
	 * Renders the entire component.
	 * <p>
	 * Creates an IMG tag.
	 * <p>
	 * <ul>
	 * <li>If a regular image file is used for the image, <code>src</code>
	 * attribute is set to point to that file. 
	 * <li>If the component is bound to a data source, <code>src</code>
	 * attribute is set to use JsfResourceServlet mapping. File name is constructed
	 * from "/jsfresource/" prefix and the actual file name in the data source (if file name
	 * is stored in the data source. If not, a temporary file name is generated).
	 * </ul>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 *
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */

	public void encodeEnd(FacesContext context, UIComponent component)
		throws IOException {
		ResponseWriter writer = null;

		if (context == null || component == null) {
			throw new NullPointerException();
		}

		HtmlGraphicImageEx imageComponent = null;
		if ( component instanceof HtmlGraphicImageEx ){
			imageComponent = (HtmlGraphicImageEx)component;
		}

		writer = context.getResponseWriter();

		/*
		writer.write("<img id=\""); //$NON-NLS-1$
		writer.write(component.getClientId(context));
		writer.write("\""); //$NON-NLS-1$
		writer.write(" src=\""); //$NON-NLS-1$
		writer.write(image);
		writer.write("\""); //$NON-NLS-1$
		*/
		
		String image = getImageURL(context, component);
		
		writer.startElement("img",component);
		writer.writeAttribute("id",component.getClientId(context),"id");
		writer.writeAttribute("src",image,null);
		
		if ( imageComponent != null  ) {
			if (imageComponent.getAccesskey() != null) {
				writer.writeAttribute(
					"accesskey",
					imageComponent.getAccesskey(),
					null);
			}
			if (imageComponent.getAlt() != null) {
				writer.writeAttribute(
					"alt",
					imageComponent.getAlt(),
					null);
			}
			else
			{
				writer.writeAttribute(
					"alt",
					"",
					null);			
			}
			
			if (imageComponent.getDir() != null) {
				writer.writeAttribute(
					"dir",
					imageComponent.getDir(),
					null);
			}
			if (imageComponent.getHeight() != null) {
				writer.writeAttribute(
					"height",
					imageComponent.getHeight(),
					null);
			}
			if (imageComponent.isIsmap()) {
				writer.writeAttribute("ismap", "ismap", null);
			}
			if (imageComponent.getLang() != null) {
				writer.writeAttribute(
					"lang",
					imageComponent.getLang(),
					null);
			}
			if (imageComponent.getLongdesc() != null) {
				writer.writeAttribute(
					"longdesc",
					imageComponent.getLongdesc(),
					null);
			}
			if (imageComponent.getOnblur() != null) {
				writer.writeAttribute(
					"onblur",
					imageComponent.getOnblur(),
					null);
			}
			if (imageComponent.getOnclick() != null) {
				writer.writeAttribute(
					"onclick",
					imageComponent.getOnclick(),
					null);
			}
			if (imageComponent.getOndblclick() != null) {
				writer.writeAttribute(
					"ondblclick",
					imageComponent.getOndblclick(),
					null);
			}
			if (imageComponent.getOnkeydown() != null) {
				writer.writeAttribute(
					"onkeydown",
					imageComponent.getOnkeydown(),
					null);
			}
			if (imageComponent.getOnkeypress() != null) {
				writer.writeAttribute(
					"onkeypress",
					imageComponent.getOnkeypress(),
					null);
			}
			if (imageComponent.getOnkeyup() != null) {
				writer.writeAttribute(
					"onkeyup",
					imageComponent.getOnkeyup(),
					null);
			}
			if (imageComponent.getOnmousedown() != null) {
				writer.writeAttribute(
					"onmousedown",
					imageComponent.getOnmousedown(),
					null);
			}
			if (imageComponent.getOnmousemove() != null) {
				writer.writeAttribute(
					"onmousemove",
					imageComponent.getOnmousemove(),
					null);
			}
			if (imageComponent.getOnmouseout() != null) {
				writer.writeAttribute(
					"onmouseout",
					imageComponent.getOnmouseout(),
					null);
			}
			if (imageComponent.getOnmouseover() != null) {
				writer.writeAttribute(
					"onmouseover",
					imageComponent.getOnmouseover(),
					null);
			}
			if (imageComponent.getOnmouseup() != null) {
				writer.writeAttribute(
					"onmouseup",
					imageComponent.getOnmouseup(),
					null);
			}
			if (imageComponent.getStyle() != null) {
				writer.writeAttribute(
					"style",
					imageComponent.getStyle(),
					null);
			}
			if (imageComponent.getTabindex() != null) {
				writer.writeAttribute(
					"tabindex",
					imageComponent.getTabindex(),
					null);
			}
			if (imageComponent.getTitle() != null) {
				writer.writeAttribute(
					"title",
					imageComponent.getTitle(),
					null);
			}
			if (imageComponent.getUsemap() != null) {
				writer.writeAttribute(
					"usemap",
					imageComponent.getUsemap(),
					null);
			}
			if (imageComponent.getWidth() != null) {
				writer.writeAttribute(
					"width",
					imageComponent.getWidth(),
					null);
			}
			if (imageComponent.getBorder() != null){
				writer.writeAttribute(
						"border",
						imageComponent.getBorder(),
						null);
			}
		} else {
			/*
			Util.renderPassThruAttributes(writer, component);
			Util.renderBooleanPassThruAttributes(writer, component);
			*/
			String accesskey = (String)component.getAttributes().get("accesskey");
			if(accesskey != null){
				writer.writeAttribute("accesskey",accesskey,"accesskey");
			}
			String alt = (String)component.getAttributes().get("alt");
			if(alt != null){
				writer.writeAttribute("alt",alt,"alt");
			}
			String dir = (String)component.getAttributes().get("dir");
			if(dir != null){
				writer.writeAttribute("dir",dir,"dir");
			}
			String height = (String)component.getAttributes().get("height");
			if(height != null){
				writer.writeAttribute("height",height,"height");
			}
			String width = (String)component.getAttributes().get("width");
			if(width != null){
				writer.writeAttribute("width",width,"width");
			}
			String lang = (String)component.getAttributes().get("lang");
			if(lang != null){
				writer.writeAttribute("lang",lang,"lang");
			}
			String longdesc = (String)component.getAttributes().get("longdesc");
			if(longdesc != null){
				writer.writeAttribute("longdesc",longdesc,"longdesc");
			}
			String onblur = (String)component.getAttributes().get("onblur");
			if(onblur != null){
				writer.writeAttribute("onblur",onblur,"onblur");
			}
			String onclick = (String)component.getAttributes().get("onclick");
			if(onclick != null){
				writer.writeAttribute("onclick",onclick,"onclick");
			}
			String ondblclick = (String)component.getAttributes().get("ondblclick");
			if(ondblclick != null){
				writer.writeAttribute("ondblclick",ondblclick,"ondblclick");
			}
			String onkeydown = (String)component.getAttributes().get("onkeydown");
			if(onkeydown != null){
				writer.writeAttribute("onkeydown",onkeydown,"onkeydown");
			}
			String onkeypress = (String)component.getAttributes().get("onkeypress");
			if(onkeypress != null){
				writer.writeAttribute("onkeypress",onkeypress,"onkeypress");
			}
			String onkeyup = (String)component.getAttributes().get("onkeyup");
			if(onkeyup != null){
				writer.writeAttribute("onkeyup",onkeyup,"onkeyup");
			}
			String onmousedown = (String)component.getAttributes().get("onmousedown");
			if(onmousedown != null){
				writer.writeAttribute("onmousedown",onmousedown,"onmousedown");
			}
			String onmousemove = (String)component.getAttributes().get("onmousemove");
			if(onmousemove != null){
				writer.writeAttribute("onmousemove",onmousemove,"onmousemove");
			}
			String onmouseout = (String)component.getAttributes().get("onmouseout");
			if(onmouseout != null){
				writer.writeAttribute("onmouseout",onmouseout,"onmouseout");
			}
			String onmouseover = (String)component.getAttributes().get("onmouseover");
			if(onmouseover != null){
				writer.writeAttribute("onmouseover",onmouseover,"onmouseover");
			}
			String onmouseup = (String)component.getAttributes().get("onmouseup");
			if(onmouseup != null){
				writer.writeAttribute("onmouseup",onmouseup,"onmouseup");
			}
			String style = (String)component.getAttributes().get("style");
			if(style != null){
				writer.writeAttribute("style",style,"style");
			}
			String tabindex = (String)component.getAttributes().get("tabindex");
			if(tabindex != null){
				writer.writeAttribute("tabindex",tabindex,"tabindex");
			}
			String title = (String)component.getAttributes().get("title");
			if(title != null){
				writer.writeAttribute("title",title,"title");
			}
			String border = (String)component.getAttributes().get("border");
			if(border != null){
				writer.writeAttribute("border",border,"border");
			}
			String usemap = (String)component.getAttributes().get("usemap");
			if(usemap != null){
				writer.writeAttribute("usemap",usemap,"usemap");
			}
			Object ismap = component.getAttributes().get("ismap");
			if(ismap != null){
				Boolean isIsmap = Boolean.FALSE;
				if(ismap instanceof String && ((String)ismap).equalsIgnoreCase("true")){
					isIsmap = Boolean.TRUE;
				}
				if(ismap instanceof Boolean && ((Boolean)ismap).booleanValue()){
					isIsmap = Boolean.TRUE;
				}
				if(isIsmap.booleanValue()){
					writer.writeAttribute("ismap","ismap",null);
				}
			}
		}
		
		String align;
		String hspace;
		String vspace;
		String graphicClass;
		
		if ( imageComponent != null ){
			
			align = imageComponent.getAlign();
			hspace = imageComponent.getHspace();
			vspace = imageComponent.getVspace();
			graphicClass = imageComponent.getStyleClass();
			
		}else{
			
			align = (String) component.getAttributes().get("align"); //$NON-NLS-1$
			hspace = (String) component.getAttributes().get("hspace"); //$NON-NLS-1$
			vspace = (String) component.getAttributes().get("vspace"); //$NON-NLS-1$
			graphicClass = (String) component.getAttributes().get("styleClass"); //$NON-NLS-1$
			
		}
		
		
		if (align != null) {
			//writer.write(" align=\"" + align + "\""); //$NON-NLS-1$ //$NON-NLS-2$
			
			writer.writeAttribute("align",align,"align");
		}
		
		if (hspace != null) {
			//writer.write(" hspace=\"" + hspace + "\""); //$NON-NLS-1$ //$NON-NLS-2$
			
			writer.writeAttribute("hspace",hspace,"hspace");
		}
		
		if (vspace != null) {
			//writer.write(" vspace=\"" + vspace + "\""); //$NON-NLS-1$ //$NON-NLS-2$
			
			writer.writeAttribute("vspace", vspace,"vspace");
		}

		if (graphicClass != null) { //$NON-NLS-1$
			//writer.write(" class=\"" + graphicClass + "\""); //$NON-NLS-1$ //$NON-NLS-2$
			
			writer.writeAttribute("class",graphicClass,"styleClass");
		}
		
		writer.endElement("img");
	}

	private String getImageURL(FacesContext context, UIComponent component) {
		String value = ""; //$NON-NLS-1$
		ValueBinding vb = component.getValueBinding("value"); //$NON-NLS-1$
		if ((component.getValueBinding(MIMETYPE_ATTR) != null)
			|| (component.getAttributes().get(MIMETYPE_ATTR) != null)) {
				value = ValueResourcePhaseListener.getUrl(context,component);
		} else {
			if (vb != null) {
				Object obj = vb.getValue(context);
				if (obj instanceof String) {
					value = (String) obj;
				}
			} else {
				value = (String) component.getAttributes().get("value"); //$NON-NLS-1$
			}
			if (value == null) {
				value = ""; //$NON-NLS-1$
			}
		}
		
		String serverRelative = null;
		if(component instanceof HtmlGraphicImageEx){
			serverRelative = ((HtmlGraphicImageEx)component).getServerRelative();
		} else {
			serverRelative = (String)component.getAttributes().get("serverRelative");
		}
		
		if(serverRelative != null && serverRelative.equalsIgnoreCase("true")){
			//user has specified this url as server relative so no attempt made to prepend with context path
			return context.getExternalContext().encodeResourceURL(value);
		}

		if (value.startsWith("/")) {
       		String cpath = context.getExternalContext().getRequestContextPath();
           	if (!value.startsWith(cpath)){
           		value = context.getApplication().getViewHandler().getResourceURL(context, value);
           	}
       	}
		return context.getExternalContext().encodeResourceURL(value);
	}
}