/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import javax.faces.component.UIComponent;
import javax.faces.component.UIForm;
import javax.faces.context.FacesContext;
import javax.faces.render.Renderer;

import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.util.JavaScriptUtil;
import com.ibm.faces.component.html.HtmlInputHelperKeybind;



/**
 * Renderer for {@link com.ibm.faces.component.UIInputHelper UIInputHelper} component.
 * 
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.InputHelper_KeybindTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.component.UIInputHelper}
 */
public class KeybindRenderer extends Renderer implements IScriptContributor {

	/**
	 * Renders the ending of the component.
	 * <p>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 *
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException {

		if (context == null || component == null) {
			throw new NullPointerException();
		}
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		scriptCollector.register(this, component);

		HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component);	
	}

	/* (non-Javadoc)
	 * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {

		HtmlInputHelperKeybind keyComponent = null;
		if ( component instanceof HtmlInputHelperKeybind ){
			keyComponent = (HtmlInputHelperKeybind)component;
		}

		String scope = null;
		String key, targetAction, target, cancelBubble, onpress;
		if ( keyComponent != null ){
			key = keyComponent.getKey();
			targetAction = keyComponent.getTargetAction();
			target = keyComponent.getTarget();
			cancelBubble = keyComponent.getCancelBubble();
			onpress = keyComponent.getOnpress();
		}else{
			key = (String)component.getAttributes().get("key"); //$NON-NLS-1$
			targetAction = (String)component.getAttributes().get("targetAction"); //$NON-NLS-1$
			target = (String)component.getAttributes().get("target"); //$NON-NLS-1$
			cancelBubble = (String)component.getAttributes().get("cancelBubble"); //$NON-NLS-1$
			onpress = (String)component.getAttributes().get("onpress"); //$NON-NLS-1$
		}

		if (key == null)
			return ;

		UIComponent parent = component.getParent();
		while ((null != parent) && !(parent instanceof UIForm)) {
			parent = parent.getParent();
		}
		if (parent != null) {
			scope = parent.getClientId(context);
		} else {
			// not within a form must mean they want it for the whole page
			scope = HxClientRenderUtil.BODY_SCOPE;
		}
		if ( target != null) { 
			
			UIComponent finding = null;
			
			//check component, this will search from components parent NamingContainer
			finding = component.findComponent(target);
			//check from view root if not found
			if(finding == null){
				finding = context.getViewRoot().findComponent(target);
				if(finding == null){
					//method to resursively search through all components
					finding = Utils.findComponent(context.getViewRoot(),target);
				}
			}
			
			if (finding != null){
				target = finding.getClientId(context);
			} else {
				target = null;
			}
			
		}

		StringBuffer script = new StringBuffer();
		genScriptKeyBind(script, scope, key, target, targetAction, onpress, cancelBubble);

		if (script.length() > 0)
			context.getResponseWriter().write(script.toString());
	}
	

	private void genScriptKeyBind(
		StringBuffer script, 
		String scope, 
		String key,
		String target,
		String action,
		String onpressAction,
		String cancelBubble){

		String _keyCode;
		String _modKey;

		// break up key into its two parts
		int x = key.indexOf('+');
		if (x == -1) {
			_keyCode = JavaScriptUtil.getKeyCode(key);
			_modKey = null;
		} else {
			_modKey = key.substring(0, x);
			_keyCode = JavaScriptUtil.getKeyCode(key.substring(x+1));
		}

		script.append("\n"); //$NON-NLS-1$
		script.append(HxClientRenderUtil.HX_VAR_NAME);
		script.append(".addBehavior(\"");
		script.append(scope);
		script.append("\", \"onkeydown\", new "); //$NON-NLS-1$
		script.append(HxClientRenderUtil.HX_VAR_NAME);
		script.append(".JSFBehaviorKeybind (");
		script.append("\"keycode:"); //$NON-NLS-1$
		script.append(_keyCode);
		if (null != _modKey) {
			script.append("\", \"modifier:"); //$NON-NLS-1$
			script.append(_modKey);
		}
		if (null != action) {
			script.append("\", \"action:"); //$NON-NLS-1$
			script.append(action);
		}
		if (null != onpressAction) {
			script.append("\", \"function:"); //$NON-NLS-1$
			script.append(onpressAction);
		}
		if (null != target) {
			script.append("\", \"target:"); //$NON-NLS-1$
			script.append(target);
		}
		if (null != cancelBubble) {
			script.append("\", \"cancel-bubble:"); //$NON-NLS-1$
			script.append(cancelBubble);
		}
		script.append("\"));"); //$NON-NLS-1$
	}

	/**
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 * @param context
	 * @param component
	 * @throws IOException
	 */
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
		//defect 280926
	}
	

}
