/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;

import javax.faces.FacesException;
import javax.faces.component.UIComponent;
import javax.faces.component.UIPanel;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlOutputSeparator;
import com.ibm.faces.component.html.HtmlPanelMenu;

/**
 * Renderer for {@link javax.faces.component.UIPanel UIPanel} component, 
 * such as a toolbar or a navigator that renders all children 
 * in a vertical or horizontal order.
 * 
 * <p>
 * Although all components are supported as children, this component
 * is geared towards hyperlinks and separators. Nested Menus are supported.
 * <p>
 * The renderer uses a table to arrange all children.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.Panel_MenuTag}<br>
 * <b>Components:</b> {@link javax.faces.component.UIPanel}
 */
/**
 * @author ykats
 *
 */
public class MenuRenderer extends Renderer implements IScriptContributor {

	private final static String MENU_ITEM_DELIMITER = "-"; //$NON-NLS-1$
	private final static String _SELECTION = "_selection"; //$NON-NLS-1$
	private final static String MENU_FACET = "header"; //$NON-NLS-1$
	private final static String MENU_ITEM_ALT_CLASS = "JWL-MENU-alt-Class"; //$NON-NLS-1$
	private final static String ALTCLASS = "altClass"; //$NON-NLS-1$
	private final static String ACTIONCLICK = "actionClick";//$NON-NLS-1$
	private final static String ITEMDISABLED = "itemDisabled";//$NON-NLS-1$
	
	
	//private final static String NestedMenuMissingFacetError = "nested menu's require a menu owner - some item that is contained in a f:facet with name = \"header\" ";
	
	
	public void decode(FacesContext context, UIComponent component) {
		
		if(context == null)
			throw new NullPointerException();

		if(component == null)
			throw new NullPointerException();		

		if(isNestedMenu(component)) {
			return;
		}
				
		HtmlPanelMenu menu_comp = null;
		
		if (component instanceof HtmlPanelMenu) // check if this is a button component
			menu_comp = (HtmlPanelMenu)component;
		
								//	 get the request parameters
		Map paramMap = context.getExternalContext().getRequestParameterMap(); 
								// get the client id
		String clientId = component.getClientId(context);		
		
		String selection = (String)paramMap.get(clientId + _SELECTION);// get value on the input
		// should be set if it is null, have someone cleared it, but I'm propably wrong 
		if(selection != null && selection.trim().length() > 0)
		{
			if(menu_comp != null)
				menu_comp.setInitialSelection(selection);
			else
				component.getAttributes().put("initialSelection",selection);			
		}
								
	}
	
	
	
	/** 
	 * Renders the beginning of the component.
	 * <p>
	 * Creates opening table tag. Children will be rendered
	 * in table cells later on in {@link MenuRenderer#encodeChildren encodeChildren}.
	 * <p>
	 * If this is a nested Menu, visibility is set according <code>nestedExpanded</code>
	 * attribute.
	 * <p>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeBegin(FacesContext context, UIComponent component)
		throws IOException {

		if (context == null || component == null) {
			throw new NullPointerException();
		}

		if(!(component.isRendered()))
			return;
		
		if(isNestedMenu(component)) {
			return;
		}
		
		String id = component.getClientId(context);
	
		HxClientRenderUtil.addScriptLibrary("hxclient_menu.js", component);
		HxClientRenderUtil.addScriptLibrary("hxclient_im.js", component);
		
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		//scriptCollector.register(this, component);
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		/*scriptCollector.addScript(cS(context, component), UIScriptCollector.ADD_AT_BOTTOM);*/

		HtmlPanelMenu menuComponent = null;
		if(component instanceof HtmlPanelMenu){
			menuComponent = (HtmlPanelMenu)component;
		}

		
		String style, styleClass;
		if(menuComponent != null ) {
			styleClass = menuComponent.getStyleClass();
			style = menuComponent.getStyle();
		} else {
			styleClass = (String) component.getAttributes().get("styleClass"); //$NON-NLS-1$
			style = (String) component.getAttributes().get("style"); //$NON-NLS-1$
		}
		
		ResponseWriter writer = context.getResponseWriter();

		writer.startElement("div", component); //$NON-NLS-1$
		writer.writeAttribute("id", id, "id"); //$NON-NLS-1$ //$NON-NLS-2$
		if(styleClass != null) {
			writer.writeAttribute("class", styleClass, "styleClass"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if(style != null) {
			writer.writeAttribute("style", style, "style"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		
		writer.startElement("input", component);
		writer.writeAttribute("type", "hidden", null);
		writer.writeAttribute("id", id+_SELECTION, null);
		writer.writeAttribute("name", id+_SELECTION, null);		
		writer.writeAttribute("value", "", null);
		writer.endElement("input"); //$NON-NLS-1$
		
		writer.endElement("div"); //$NON-NLS-1$
		
		writer.startElement("div", component); //$NON-NLS-1$
		writer.writeAttribute("style", "display:none;position:absolute", null); //$NON-NLS-1$ //$NON-NLS-2$
	}

	public boolean getRendersChildren() {
		return true;
	}

	/** 
	 * Renders component's children.
	 * <p>
	 * Renders all child components as table cells either vertically
	 * or horizontally, depending on <code>layout</code> attribute.<br>
	 * Creates separators between items if <code>autoseparate</code> was set
	 * to <code>true</code>.
	 * <p>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeChildren(FacesContext context, UIComponent component)
		throws IOException {

		if (context == null || component == null) {
			throw new NullPointerException();
		}
		
		if(component instanceof HtmlPanelMenu) {
			UIComponent child = component.getFacet(MENU_FACET);
			if(child != null) {
				Utils.encodeComponent(context, child);
			}
		}
		
		List children = component.getChildren();
		for (int i = 0; i < children.size(); i++) {
			UIComponent child = (UIComponent) children.get(i);
			Utils.encodeComponent(context, child);
		}

	}


	/**
	 * Renders the ending of the component.
	 * <p>
	 * Creates the closing table tag.
	 * <p>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 *
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeEnd(FacesContext context, UIComponent component)
		throws IOException {

		if (context == null || component == null) {
			throw new NullPointerException();
		}
		
		if(!(component.isRendered()))
			return;
		
		if(isNestedMenu(component)) {
			return;
		}

		ResponseWriter writer = context.getResponseWriter();

		writer.endElement("div"); //$NON-NLS-1$
		
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);		
		scriptCollector.addScript(cS(context, component));
		
	}

	private boolean isNestedMenu(UIComponent component) {
		UIComponent parent = component.getParent();
		if (parent == null) {
			return false;
		}
		if (!(parent instanceof UIPanel)) {
			return false;
		}
		String renderer = component.getRendererType();
		String parentRenderer = parent.getRendererType();
		if (parentRenderer != null && parentRenderer.equals(renderer)) {
			return true;
		}
		return false;
	}

	
	
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {}
	
	/* (non-Javadoc)
	 * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	/*public void contributeScript(FacesContext context, UIComponent component) throws IOException {
		ResponseWriter writer = context.getResponseWriter();

		HtmlPanelMenu menuComponent = null;
		if(component instanceof HtmlPanelMenu) {
			menuComponent = (HtmlPanelMenu) component; 
		}
		
		String vertical, verticalBody, flyOpen, initialSelection, tabIndex;
		if(menuComponent != null) {
			vertical = menuComponent.getVertical();
			verticalBody = menuComponent.getVerticalBody();
			flyOpen = menuComponent.getFlyOpen();
			initialSelection = menuComponent.getInitialSelection();
			tabIndex = menuComponent.getTabIndex();
		}
		else {
			vertical = (String) component.getAttributes().get("vertical"); //$NON-NLS-1$
			verticalBody = (String) component.getAttributes().get("verticalBody"); //$NON-NLS-1$
			flyOpen = (String) component.getAttributes().get("flyOpen"); //$NON-NLS-1$
			initialSelection = (String) component.getAttributes().get("initialSelection"); //$NON-NLS-1$
			tabIndex = (String) component.getAttributes().get("tabIndex"); //$NON-NLS-1$
		}
		
		StringBuffer params = new StringBuffer();
		if(vertical != null && vertical.length() > 0) {
			params.append("\"vertical:"); //$NON-NLS-1$
			params.append(vertical);
			params.append("\","); //$NON-NLS-1$
		}
		if(verticalBody != null && verticalBody.length() > 0) {
			params.append("\"vertical-body:"); //$NON-NLS-1$
			params.append(verticalBody);
			params.append("\","); //$NON-NLS-1$
		}
		if(flyOpen != null && flyOpen.length() > 0) {
			params.append("\"fly-open:"); //$NON-NLS-1$
			params.append(flyOpen);
			params.append("\","); //$NON-NLS-1$
		}
		if(initialSelection != null && initialSelection.length() > 0) {
			params.append("\"initial-selection:"); //$NON-NLS-1$
			params.append(initialSelection);
			params.append("\","); //$NON-NLS-1$
		}
		if(tabIndex != null && tabIndex.length() > 0) {
			params.append("\"tab-index:"); //$NON-NLS-1$
			params.append(tabIndex);
			params.append("\","); //$NON-NLS-1$
		}
		if(params.length() > 0) {
			params.deleteCharAt(params.length() - 1); // remove the last comma
		}

		String id = component.getClientId(context);
		
		StringBuffer menu = new StringBuffer();
		menu.append(HxClientRenderUtil.HX_VAR_NAME);
		menu.append(".addComponent(\""); //$NON-NLS-1$
		menu.append(id);
		menu.append("\", new "); //$NON-NLS-1$
		menu.append(HxClientRenderUtil.HX_VAR_NAME);
		menu.append(".JSFMenu("); //$NON-NLS-1$
		menu.append(params);
		menu.append("));\n"); //$NON-NLS-1$
		
		writer.write(menu.toString());
		
		String parentId = id;
		UIComponent headerFacet = component.getFacet(MENU_FACET);
		if(headerFacet != null) {
			parentId = id + MENU_ITEM_DELIMITER + "1"; //$NON-NLS-1$
			writeMenuItem(context, headerFacet, id, parentId);
		}
		
		writeMenuItems(context, component, parentId);
	}

	protected void writeMenuItems(FacesContext context, UIComponent parentMenu, String parentId) throws IOException {
		List children = parentMenu.getChildren();
		for (int i = 0; i < children.size(); i++) {
			UIComponent item = (UIComponent) children.get(i);
			UIComponent nestedMenu = null;
			
			if(isNestedMenu(item)) {
				nestedMenu = item;
				item = item.getFacet(MENU_FACET);
			}

			String itemId = parentId + MENU_ITEM_DELIMITER + (i+1);
			writeMenuItem(context, item, parentId, itemId);
			
			if(nestedMenu != null) {
				writeMenuItems(context, nestedMenu, itemId);
			}
		}
	}
	
	protected void writeMenuItem(FacesContext context, UIComponent item, String parentId, String itemId) throws IOException {
		ResponseWriter writer = context.getResponseWriter();
		
		StringBuffer menuItem = new StringBuffer();
		
		menuItem.append(HxClientRenderUtil.HX_VAR_NAME);
		menuItem.append(".addSubComponent(\""); //$NON-NLS-1$
		menuItem.append(parentId);
		menuItem.append("\", \""); //$NON-NLS-1$
		menuItem.append(itemId);
		menuItem.append("\", new "); //$NON-NLS-1$
		menuItem.append(HxClientRenderUtil.HX_VAR_NAME);
		menuItem.append(".JSFMenuItem('"); //$NON-NLS-1$
		if (item instanceof HtmlOutputSeparator) {
			//menuItem.append("separator:"); //$NON-NLS-1$
			//writer.write(menuItem.toString());
			//Utils.encodeComponent(context, item);	
			//writer.write("'));\n");
			menuItem.append("separator'));\n"); //$NON-NLS-1$
		} else {
	        menuItem.append("item:");//$NON-NLS-1$
	        menuItem.append(item.getClientId(context));
	        menuItem.append("'");
	        menuItem.append(getAssist(item));
	        menuItem.append("));\n");
		}

	}
	
	*/
    public String cS(FacesContext context, UIComponent component)
    throws IOException
	{
	    StringBuffer menuJSstring = new StringBuffer();
	    HtmlPanelMenu menuComponent = null;
	    if(component instanceof HtmlPanelMenu)
	        menuComponent = (HtmlPanelMenu)component;
	    String vertical;
	    String verticalBody;
	    String flyOpen;
	    String initialSelection;
	    String tabIndex;
	    String flyOpenBody;
	    String openInPlace;
	    String onchange;
	    String openSticky;
	    String selectAll;
	   	    	    
	    if(menuComponent != null)
	    {
	        vertical = menuComponent.getVertical();
	        verticalBody = menuComponent.getVerticalBody();
	        flyOpen = menuComponent.getFlyOpen();
	        flyOpenBody = menuComponent.getFlyOpenBody();
	        initialSelection = menuComponent.getInitialSelection();
	        
	        if(initialSelection != null && initialSelection.trim().length() >0)
	        {
	        	UIComponent s = context.getViewRoot();
	        	UIComponent c = Utils.findComponent(s, initialSelection);
	        	if(c != null)
	        		initialSelection = context.getExternalContext().encodeNamespace(c.getClientId(context));	
	        }
	        		        	
	        tabIndex = menuComponent.getTabIndex();
	        openInPlace = menuComponent.getOpenInPlace();
	        onchange = menuComponent.getOnchange();
	        openSticky = menuComponent.getOpenSticky();
	        selectAll = menuComponent.getSelectAll();
	    } else
	    {
	        vertical = (String)component.getAttributes().get("vertical");
	        verticalBody = (String)component.getAttributes().get("verticalBody");			
	        flyOpen = (String)component.getAttributes().get("flyOpen");			
	        flyOpenBody = (String)component.getAttributes().get("flyOpenBody");
		    initialSelection = (String)component.getAttributes().get("initialSelection");
		
	        if(initialSelection != null && initialSelection.trim().length() >0)
	        {
	        	UIComponent c = Utils.findComponent(component, initialSelection);
	        	if(c != null)
	        		initialSelection = context.getExternalContext().encodeNamespace(c.getClientId(context));	
	        }		    
		    
	        tabIndex = (String)component.getAttributes().get("tabIndex");	     
	        openInPlace = (String)component.getAttributes().get("openInPlace");
		    onchange = (String)component.getAttributes().get("onchange");
		    openSticky = (String)component.getAttributes().get("openSticky");
	        selectAll = (String)component.getAttributes().get("selectAll");			        
	    }
 	    
	    StringBuffer params = new StringBuffer();
	    if(vertical != null && vertical.length() >0) //boolean
	    {
	        params.append("\"vertical:");
	        params.append(vertical);
	        params.append("\",");
	    }
	    if(verticalBody != null && verticalBody.length() >0)//boolean
	    {
	        params.append("\"vertical-body:");
	        params.append(verticalBody);
	        params.append("\",");
	    }
	    if(flyOpen != null && flyOpen.length()>0)//boolean
	    {
	        params.append("\"fly-open:");
	        params.append(flyOpen);
	        params.append("\",");
	    }
	    if(flyOpenBody != null && flyOpenBody.length()>0)//boolean
	    {
	        params.append("\"fly-open-body:");
	        params.append(flyOpenBody);
	        params.append("\",");
	    }
	    if(initialSelection != null && initialSelection.length() > 0)
	    {
	        params.append("\"initial-selection:");
	        params.append(initialSelection);
	        params.append("\",");
	    } 	    
	    if(tabIndex != null && tabIndex.length() > 0)
	    {
	        params.append("\"tab-index:");
	        params.append(tabIndex);
	        params.append("\",");
	    }
	    if(openInPlace != null && openInPlace.length() > 0)//boolean
	    {
	        params.append("\"open-in-place:");
	        params.append(openInPlace);
	        params.append("\",");
	    }
	    if(onchange != null && onchange.length() > 0) 
	    {
	        params.append("\"onchange:");
	        params.append(onchange);
	        params.append("\",");
	    }	    
	    if(openSticky != null && openSticky.length() > 0)//boolean
	    {
	        params.append("\"open-sticky:");
	        params.append(openSticky);
	        params.append("\",");
	    }	    
	    if(selectAll != null && selectAll.length() > 0) //boolean
	    {
	        params.append("\"select-all:");
	        params.append(selectAll);
	        params.append("\"");
	    }
       	    
	    if((params.length() > 0) && (',' == params.charAt(params.length() -1)))
	        params.deleteCharAt(params.length() - 1);
	    
	    String id = component.getClientId(context);
	    StringBuffer menu = new StringBuffer();
	    menu.append(HxClientRenderUtil.HX_VAR_NAME);
	    menu.append(".addComponent(\"");
	    menu.append(id);
	    menu.append("\", new ");
	    menu.append(HxClientRenderUtil.HX_VAR_NAME);
	    menu.append(".JSFMenu(");
	    menu.append(params);
	    menu.append("));\n");
	    menuJSstring.append(menu.toString());
	    String parentId = id;
	    
	    UIComponent headerFacet = component.getFacet(MENU_FACET);
	    
	    if(headerFacet != null)
	    {
	        parentId = id + MENU_ITEM_DELIMITER + "1";							// false passed because we assume to action-click on root-root
	        menuJSstring.append(writeMI(context, headerFacet, id, parentId, false));	        
	    }
	    menuJSstring.append(writeMIs(context, component, parentId));
	    return menuJSstring.toString();
	}
	
	protected String writeMIs(FacesContext context, UIComponent parentMenu, String parentId)
	    throws IOException
	{
	    List children = parentMenu.getChildren();
	    StringBuffer menuItems = new StringBuffer();
	    boolean isFacet = false;
	    for(int i = 0; i < children.size(); i++)
	    {
	    	isFacet = false;
	        UIComponent item = (UIComponent)children.get(i);
	        if(item.isRendered())
	        {
		        UIComponent nestedMenu = null;
		        if(isNestedMenu(item))
		        {
		            nestedMenu = item;
		            item = item.getFacet(MENU_FACET);
		            
		            if(item == null)
		            {
		            	ResourceBundle rb = ResourceHandler.getBundle(context);	            	
		            	throw new FacesException(ResourceHandler.getString(rb, "NestedMenuMissingFacetError"));
		            }
		            isFacet = true;
		        }

	        	String itemId = parentId + MENU_ITEM_DELIMITER + (i + 1);
	        	menuItems.append(writeMI(context, item, parentId, itemId, isFacet));
	        	if(nestedMenu != null)
	        		menuItems.append(writeMIs(context, nestedMenu, itemId));
	        }
	    }
	
	    return menuItems.toString();
	}
	
	protected String writeMI(FacesContext context, UIComponent item, String parentId, String itemId, boolean isFacet){
	    
		if(!item.isRendered())
			return "";
		
		
	    StringBuffer menuItem = new StringBuffer();
	    menuItem.append(HxClientRenderUtil.HX_VAR_NAME);
	    menuItem.append(".addSubComponent(\"");
	    menuItem.append(parentId);
	    menuItem.append("\", \"");
	    menuItem.append(itemId);
	    menuItem.append("\", new ");
	    menuItem.append(HxClientRenderUtil.HX_VAR_NAME);
	    menuItem.append(".JSFMenuItem(\"");
	    
	    if(item instanceof HtmlOutputSeparator)
	    {
			//menuItem.append("separator:"); //$NON-NLS-1$
			//writer.write(menuItem.toString());
			//Utils.encodeComponent(context, item);	
			//writer.write("'));\n");
	    	HtmlOutputSeparator labelItem = (HtmlOutputSeparator)item;
	    	if(labelItem.getLabel() != null && labelItem.getLabel().trim().length() >0)
	    	{
	    		menuItem.append("separator:"); 
	    		menuItem.append(labelItem.getLabel());
	    		menuItem.append("\"));\n"); //$NON-NLS-1$
	    	}
	    	else
	    		menuItem.append("separator\"));\n"); //$NON-NLS-1$
	    } 
	    else
	    {				//	a submenu root 														
		    if((isFacet)) 
		    {				// so youre in a facet 
		        menuItem.append("item:");
		        menuItem.append(item.getClientId(context));
		        menuItem.append("\"");		        
		        menuItem.append(getActionClick(item));	//	        
		        menuItem.append(getAltClass(item));
		        menuItem.append(getDisabled(item));	
		        menuItem.append("));\n");
		    }
		    else // youre a normal menuitem
		    {
		        menuItem.append("item:");
		        menuItem.append(item.getClientId(context));
		        menuItem.append("\"");
		        menuItem.append(getAltClass(item));
		        menuItem.append(getDisabled(item));	
		        menuItem.append("));\n");
		    }
	    }
	    return menuItem.toString();
	}
	
	private String getAltClass(UIComponent item)
	{
	    StringBuffer assistStr = new StringBuffer();

	    String altclass = null;

        altclass = (String)item.getAttributes().get(MENU_ITEM_ALT_CLASS);
        
        if(altclass == null)
        	altclass = (String)item.getAttributes().get(ALTCLASS);
        
        if(altclass != null && altclass.length() > 0 && (0 == "true".compareToIgnoreCase(altclass)))
        {
        	assistStr.append(",\"");
            assistStr.append("alt-class:");
            assistStr.append(altclass);
            assistStr.append("\"");
        }

	    return assistStr.toString();
	}

	private String getActionClick(UIComponent item)
	{
	    StringBuffer assistStr = new StringBuffer();

	    String actionclick = null;

        actionclick = (String)item.getAttributes().get(ACTIONCLICK);
        
        if(actionclick != null && actionclick.length() > 0 && (0 == "true".compareToIgnoreCase(actionclick)))
        {
        	assistStr.append(", \"action-click:true\"");
        }

	    return assistStr.toString();
	}

	private String getDisabled(UIComponent item)
	{
	    StringBuffer assistStr = new StringBuffer();

	    String disabled = null;

	    disabled = (String)item.getAttributes().get(ITEMDISABLED);
        
        if(disabled != null && disabled.length() > 0 && (0 == "true".compareToIgnoreCase(disabled)))
        {
        	assistStr.append(", \"disabled:true\"");
        }

	    return assistStr.toString();
	}
	
}
