/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.el.ValueBinding;



import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.webapp.ValueResourcePhaseListener;
import javax.faces.render.Renderer;


/**
 * Convenience base class for {@link com.ibm.faces.component.UIPlayer UIPlayer} component,
 * that implements the default behavior of all player controls.  
 * <p> 
 * <b>Tags:</b> n/a
 * <b>Components:</b> {@link com.ibm.faces.component.UIPlayer}
 */
public abstract class PlayerRenderer extends Renderer {
	public final static String ADDITIONAL_PLAYER_SETTINGS = "playerAttributes";
	public final static String MIMETYPE_ATTR = "mimeType";
	private final static String[] objectAttributes = { 
		"codetype", //$NON-NLS-1$
		"data", //$NON-NLS-1$
		"type", //$NON-NLS-1$
		"archive", //$NON-NLS-1$
		"declare", //$NON-NLS-1$
		"standby", //$NON-NLS-1$

		"accesskey", //$NON-NLS-1$
		"align", //$NON-NLS-1$
		"alt", //$NON-NLS-1$
		"hidden", //$NON-NLS-1$
		"style", //$NON-NLS-1$
		"tabindex", //$NON-NLS-1$
		"title", //$NON-NLS-1$

		"hspace", //$NON-NLS-1$
		"vspace", //$NON-NLS-1$
		"border", //$NON-NLS-1$

		"dir", //$NON-NLS-1$
		"lang", //$NON-NLS-1$
		"archive", //$NON-NLS-1$
		"width", //$NON-NLS-1$
		"height", //$NON-NLS-1$

		//javascript events from here on
		"onclick", //$NON-NLS-1$
		"ondblclick", //$NON-NLS-1$
		"onkeydown", //$NON-NLS-1$
		"onkeypress", //$NON-NLS-1$
		"onkeyup", //$NON-NLS-1$
		"onmousedown", //$NON-NLS-1$
		"onmousemove", //$NON-NLS-1$
		"onmouseout", //$NON-NLS-1$
		"onmouseover", //$NON-NLS-1$
		"onmouseup", //$NON-NLS-1$
		
		//focus events
		"onfocus", //$NON-NLS-1$
		"onblur" //$NON-NLS-1$
		
	};
	


	
	/**
	 * Player's parameter.
	 * <p>Matches JSF tag's attribute name to a player's parameter name.
	 */
	public static class Param {
		/** JSF tag attribute */
		protected String attrName;
		/** Player's parameter */
		protected String paramName;

		/**
		 * Constructor.
		 * 
		 * @param attrName attribute name
		 * @param paramName parameter name
		 */
		public Param(String attrName, String paramName) {
			this.attrName = attrName;
			this.paramName = paramName;
		}
	}

	/**
	 * Returns Class ID for the control.
	 * 
	 * @return Class ID
	 */
	abstract protected String getObjectClassId();

	/**
	 * Returns Code Base for the control.
	 * 
	 * @return Code Base
	 */
	abstract protected String getObjectCodebase();

	/**
	 * Returns a list of parameters for the control.
	 * 
	 * @return array of parameters
	 */
	abstract protected Param[] getObjectParams();

	/**
	 * Returns ULR to control's web site
	 * 
	 * @return url
	 */
	abstract protected String getPluginPage();

	/**
	 * Returns type of control's plug-in
	 * 
	 * @return type of plugin
	 */
	abstract protected String getPluginType();

	/**
	 * Returns a list of parameters for control's plugin.
	 * 
	 * @return array of parameters
	 */
	abstract protected Param[] getPluginParams();

	/**
	 * Returns type of control.
	 * 
	 * @return type of control. <code>null</code> if undefined.
	 */
	protected String getObjectType() {
		return null;
	}

	/**
	 * Returns default control width.
	 * 
	 * @return width. <code>null</code> if there is no default
	 */
	protected String getDefaultWidth() {
		return null;
	}

	/**
	 * Returns default control height.
	 * 
	 * @return height. <code>null</code> if there is no default
	 */
	protected String getDefaultHeight() {
		return null;
	}

	/** 
	 * Renders the ending of the component.
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeEnd(FacesContext context, UIComponent component)
		throws IOException {
		if (context == null || component == null) {
			throw new NullPointerException();
		}
		// suppress rendering if "rendered" property on the component is
		// false.
		StringBuffer objectAttr = new StringBuffer();
		StringBuffer objectParam = new StringBuffer();
		StringBuffer embedAttr = new StringBuffer();		
		
		HashMap objAttrMap = new HashMap();
		HashMap objParamMap = new HashMap();
		HashMap objEmbedMap = new HashMap();
		Object value;
		Iterator paramKeys;
		Param[] objectParams = getObjectParams();
		ResponseWriter writer = context.getResponseWriter();
		
		
		if (null != (value = component.getClientId(context))) {
			objAttrMap.put("name", value.toString());
			objAttrMap.put("id", value.toString());
		}

		if (null == (value = component.getAttributes().get("classid"))) { //$NON-NLS-1$
			value = getObjectClassId();
		}
		objAttrMap.put("classid", value.toString());

		if (null != (value = component.getAttributes().get("codebase"))) { //$NON-NLS-1$
			objAttrMap.put("codebase", value.toString());
		} else if (null != (value = getObjectCodebase())) {
			objAttrMap.put("codebase", value.toString());
		}

		String styleClass = null;
		if (null != (styleClass = (String) component.getAttributes().get("styleClass"))) { //$NON-NLS-1$
			objAttrMap.put("class", styleClass);
		}

		if (null == (value = component.getAttributes().get("width"))) { //$NON-NLS-1$
			value = getDefaultWidth();
		}
		if (null != value) {
			objAttrMap.put("width", value.toString());
		}

		if (null == (value = component.getAttributes().get("height"))) { //$NON-NLS-1$
			value = getDefaultHeight();
		}
		if (null != value) {
			objAttrMap.put("height", value.toString());
		}

		int len = objectAttributes.length;
		for (int i = 0; i < len; i++) {
			if (null
				!= (value =
					(String) component.getAttributes().get(
						objectAttributes[i]))) {
				objAttrMap.put(objectAttributes[i], value);
			}
		}
		if (null
			!= (value = (String) component.getAttributes().get(MIMETYPE_ATTR))) {
			objAttrMap.put("type", value);
			objEmbedMap.put("type", value);
		} else if (null != (value = getPluginType())) {
			objAttrMap.put("type", value);
			objEmbedMap.put("type", value);
		}

		paramKeys = objAttrMap.keySet().iterator();
		while (paramKeys.hasNext()) {
			String key = (String) paramKeys.next();
			objectAttr.append(key + "=\"" + objAttrMap.get(key) + "\"" + " ");
		}

		encodeParams(context,objParamMap,objEmbedMap,component,objectParams);
		//src attribute is the same for objEmbedMap as in objParamMap
		objEmbedMap.put("src", objParamMap.get(objectParams[0].paramName));
		encodeEmbedTagAttributes(objEmbedMap, component);

		paramKeys = objParamMap.keySet().iterator();
		while (paramKeys.hasNext()) {
			String key = (String) paramKeys.next();
			objectParam.append("<param name=\"" + key + "\" value=\"" + objParamMap.get(key)	+ "\" />");
		}
		paramKeys = objEmbedMap.keySet().iterator();
		while (paramKeys.hasNext()) {
			String key = (String) paramKeys.next();
			embedAttr.append(key + "=\"" + objEmbedMap.get(key) + "\"" + " ");
		}
		writer.startElement("span",component);
		writer.writeAttribute("id",component.getClientId(context)+"span",null);
		writer.endElement("span");

		UIScriptCollector.find(component).addScript(constructPlayerScript(context,component,objectAttr,objectParam,embedAttr));
	}

	/** 
	 * Renders control's parameters.
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param htmlOutput String buffer to append HTML to
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * @param params Array of parameters to be rendered
	 * 
	 */
	protected void encodeParams(
		FacesContext context,
		HashMap objParamMap,
		HashMap objEmbedMap,
		UIComponent component,
		Param[] params) {
		String value;
		int len = params.length;

		encodeSrcParam(context, objParamMap, component, params);

		//Note: i is set to 1 because 0 (the source) is taken care 
		//of in encodeSourceParam
		for (int i = 1; i < len; i++) {
			if (null != (value =(String) component.getAttributes().get(params[i].attrName))) {
				objParamMap.put(params[i].paramName, value);
			}
		}
		//need to get the width and height for embed attributes also
		if (null != (value =(String) component.getAttributes().get("width"))) {
			objEmbedMap.put("width", value);
		}
		if (null != (value =(String) component.getAttributes().get("height"))) {
			objEmbedMap.put("height", value);
		}
		//add additional player settings as entered into playerAttributes
		if (null
			!= (value =
				(String) component.getAttributes().get(
					ADDITIONAL_PLAYER_SETTINGS))) {
			value = value.trim();
			int x = 0;
			int y = 0;
			Vector v = new Vector();
			while (y > -1) {
				y = value.indexOf(" ", x);

				if (y != -1) {
					if (y != 0)
						v.add(value.substring(x, y).trim());
					x = y + 1;
				}
				if (y == -1 && x < value.length()) {
					v.add(value.substring(x, value.length()).trim());
				}
			}
			int vlength = v.size();
			String pair = "";
			String pairname = "";
			String pairvalue = "";
			for (int a = 0; a < vlength; a++) {
				pair = v.get(a).toString();
				if (pair.indexOf("=") != -1) {
					pairname = pair.substring(0, pair.indexOf("="));
					pairvalue =pair.substring(pair.indexOf("=") + 1, pair.length());
					//remove quotes from beginning and end of value if they exist
					if (pairvalue.startsWith("'")||pairvalue.startsWith("\"")) {
						pairvalue=pairvalue.substring(1,pairvalue.length());
					}
					if (pairvalue.endsWith("'")||pairvalue.endsWith("\"")) {
						pairvalue=pairvalue.substring(0,pairvalue.length()-1);
					}
					
					if (pairname.equals("") | pairvalue.equals("")) {
						System.err.println(
							"Bad name=value pair set in "
								+ component.getClientId(context)
								+ " "
								+ ADDITIONAL_PLAYER_SETTINGS);
					} else {
						objParamMap.put(pairname, pairvalue);
						objEmbedMap.put(pairname, pairvalue);
					}
				}
			}
		}
	}

	/** 
	 * Gets the media source parameter--this can be set uri/url pointing to a file or bound to a back end
	 * data.
	 * When binding data the 'mimeType' attribute must also be set, the 'filename' attribute is optional. 
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param htmlOutput String buffer to append HTML to
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * @param params Array of parameters to be rendered
	 * 
	 */
	protected void encodeSrcParam(FacesContext context,	HashMap objParamMap,UIComponent component,Param[] params) {
		String param = params[0].paramName;
		objParamMap.put(param,src(context,component));
	}

	/** 
	 * Renders control's plugin.
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param htmlOutput String buffer to append HTML to
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 */
	protected void encodeEmbedTagAttributes(HashMap objEmbedMap,UIComponent component) {
		String value;

		if (null != (value = getPluginPage())) { //$NON-NLS-1$
			objEmbedMap.put("pluginspage", value);
		}

		Param[] params = getPluginParams();
		int len = params.length;

		//Note: i is set to 1 because 0 has already been set with src attribute
		for (int i = 1; i < len; i++) {
			if (null
				!= (value =
					(String) component.getAttributes().get(
						params[i].attrName))) {

				objEmbedMap.put(params[i].paramName, value);
			}
		}
	}

	/** 
	 * Renders the beginning of the component.
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeBegin(FacesContext context, UIComponent component)
		throws IOException {
		if (context == null || component == null) {
			throw new NullPointerException();
		}

		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
	}

	private String constructPlayerScript(FacesContext context,UIComponent component,StringBuffer objectAttr,StringBuffer objectParam,StringBuffer embedAttr){

		StringBuffer scriptOut=new StringBuffer();
		String spanId = component.getClientId(context)+"span";

		//replace the param with '+paramTag+' in objectParam StringBuffer
		String paramStr=objectParam.toString();
		while(paramStr.indexOf("param") > -1) {
			int x=paramStr.indexOf("param");
			objectParam.replace(x,x+5,"'+ " + HxClientRenderUtil.HX_VAR_NAME + ".TAG_PARAM + '");
			paramStr=objectParam.toString();
		}
		scriptOut.append("if(null != "); //$NON-NLS-1$
		scriptOut.append(HxClientRenderUtil.HX_VAR_NAME);
		scriptOut.append(".getElementById('");
		scriptOut.append(spanId);
		scriptOut.append("')){\n");
		
		scriptOut.append("(");
		scriptOut.append(HxClientRenderUtil.HX_VAR_NAME);
		scriptOut.append(".getElementById('");
		scriptOut.append(spanId);
		scriptOut.append("')).innerHTML='<' + ");
		scriptOut.append(HxClientRenderUtil.HX_VAR_NAME);
		scriptOut.append(".TAG_OBJECT + ' ");
		scriptOut.append(objectAttr.toString());
		scriptOut.append(" >");
		scriptOut.append(objectParam);
		scriptOut.append("<'+ ");
		scriptOut.append(HxClientRenderUtil.HX_VAR_NAME);
		scriptOut.append(".TAG_EMBED +' ");
		scriptOut.append(embedAttr.toString());
		scriptOut.append("/></'+ ");
		scriptOut.append(HxClientRenderUtil.HX_VAR_NAME);
		scriptOut.append(".TAG_OBJECT +'>';}\n");
		
		return(scriptOut.toString());
	}
	private String src(FacesContext context, UIComponent component) {
		String value = ""; //$NON-NLS-1$
		ValueBinding vb = component.getValueBinding("src"); //$NON-NLS-1$

		if ((component.getValueBinding(MIMETYPE_ATTR) != null)
		|| (component.getAttributes().get(MIMETYPE_ATTR) != null)) {
			
			//need to set 'value' valuebinding for ValueResourceListener
			component.setValueBinding("value",vb);
						
			value = ValueResourcePhaseListener.getUrl(context,component);
		} else {
			if (vb != null) {
				Object obj = vb.getValue(context);
				if (obj instanceof String) {
					value = (String) obj;
				}
		} else {
			value = (String) component.getAttributes().get("src"); //$NON-NLS-1$
		}
		if (value == null) {
			value = ""; //$NON-NLS-1$
		}
		}
		value =	context.getApplication().getViewHandler().getResourceURL(context,value);
		return (context.getExternalContext().encodeResourceURL(value));
	}

}
