/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.io.Writer;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.ResourceBundle;

import javax.faces.component.UIComponent;
import javax.faces.component.UIData;
import javax.faces.component.UIInput;
import javax.faces.component.UIParameter;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;
import javax.faces.event.ValueChangeEvent;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.util.HtmlUtil;
import com.ibm.faces.util.JavaScriptUtil;
import com.ibm.faces.util.LifecycleUtil;

import com.ibm.faces.component.html.HtmlInputRowSelect;

/**
 * Renderer for providing row select support in a data table.
 * <p>
 * This component should be used as a row-level child of a javax.faces.component.UIData component.
 * <p>
 * Supported Value Types: Boolean; Boolean[]; boolean[]; int[]; Integer[]; java.util.Collection( java.util.Map )
 * <p>
 * <h3>Boolean</h3>
 * A boolean value in a row-level bean. The boolean is evaluated like any other row property.
 * <em>Other data types are treated as global to the table</em>. This is the most efficient data type.
 * <p>
 * <h3>Boolean[]/boolean[]</h3>
 * Boolean arrays. Rows are evaluated with respect to the array index at UIData.getRowIndex().
 * The range available must start at index UIData.getFirst() and be of length UIData.getFirst()+UIData.getRows().
 * The recommended array size is therefore UIData.getRowCount().
 * <p>
 * <h3>Integer[]/int[]</h3>
 * Integer arrays. The values in the array indicate selected row indices.
 * On encode, if the current row UIData.getRowIndex() is a member of the array, the control is rendered selected.
 * On decode, if the current row is selected and is not a member of the array, it is appended to the array.
 * On decode, if the current row is unselected and is a member of the array, the first instance in the array is removed.
 * The recommended initial array size is zero for all rows unselected.
 * The array will be replaced with new instances as selection changes.
 * <p>
 * <h3>java.util.Collection</h3>
 * A Collection of Maps. This data type must be used with UIParameter children.
 * The UIParameters should resolve values from row-level beans.
 * <p>
 * For each row, a Map is created entering a key/value pair for each UIParameter child.
 * Key=UIParameter.getName(); Value=UIParameter.getValue().
 * <p>
 * On encode, if the Collection contains the current Map, the row is rendered selected.
 * On decode, if the row is selected and the Collection does not contain the Map, the Map is added to the Collection.
 * On decode, if the row is unselected and the Collection contains the Map, the Map is removed from the Collection.
 * <p>
 * @see javax.faces.component.UIData
 * @see javax.faces.component.UIParameter
 */
public class RowSelectRenderer extends AbstractRowRenderer implements IScriptContributor{

    private final static String PAD_INPUT_SEL = "__input_sel"; //$NON-NLS-1$
    private final static String PAD_HIDDEN_SEL = "__hidden_sel"; //$NON-NLS-1$

    private final static String IMAGE_SELECTED = Utils.versionizeFilename("datagrid_selected.gif"); //$NON-NLS-1$
    private final static String IMAGE_UNSELECTED = Utils.versionizeFilename("datagrid_unselected.gif"); //$NON-NLS-1$
    
    private final static String SELECTONE_IMAGE_SELECTED = Utils.versionizeFilename("datagrid_radio_selected.gif"); //$NON-NLS-1$
    private final static String SELECTONE_IMAGE_UNSELECTED = Utils.versionizeFilename("datagrid_radio_unselected.gif"); //$NON-NLS-1$

    private static final String JS_SELECTION = "__jsSelection"; //$NON-NLS-1$
    private static final String JS_OBJGROUP = "__jsObjGroup"; //$NON-NLS-1$

    private static final String VALUE_STORE = "com.ibm.faces.ROW_SEL_TEMP_"; //$NON-NLS-1$

    /**
     * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     * @throws IOException
     */
    public void encodeBegin(FacesContext context, UIComponent component)
        throws IOException {
    	
    	/*
    	 * Removing this, VALUE_STORE never gets added to data's attributes
    	 * 
        UIData data = findUIData(component);
        data.getAttributes().remove(VALUE_STORE);
        */
    	
        Writer writer = context.getResponseWriter();
        String id = component.getClientId(context);
		    
        //wrap in span for possible JS manipulation
        HtmlUtil.writeTagStartOpen(writer, "span", false, false); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "id", id); //$NON-NLS-1$
        HtmlUtil.writeTagStartCloseLn(writer, false);
    }

    /**
     * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     * @throws IOException
     */
    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {

		HtmlInputRowSelect selectComponent = null;
		if ( component instanceof HtmlInputRowSelect ){
			selectComponent = (HtmlInputRowSelect)component;
		}

        Writer writer = context.getResponseWriter();
        String id = component.getClientId(context);
        
        initializeValues(context, component);
        boolean selected = isRowSelected(component);

		String noscriptString;
		String style;
		String styleClass;
		String selectOneStr;
		String onclick;
		String onmouseup;
		String title;
		String titleSelected;
		if( selectComponent != null ){
			noscriptString = selectComponent.getNoscript();
			style = selectComponent.getStyle();
			styleClass = selectComponent.getStyleClass();
			selectOneStr = selectComponent.getSelectOne();
			onclick = selectComponent.getOnclick();
			onmouseup = selectComponent.getOnmouseup();
			title = selectComponent.getTitle();
			titleSelected = selectComponent.getTitleSelected();
		}else{
			noscriptString = (String)component.getAttributes().get("noscript");
			style = (String)component.getAttributes().get("style");
			styleClass = (String)component.getAttributes().get("styleClass");
			selectOneStr = (String)component.getAttributes().get("selectOne");
			onclick = (String)component.getAttributes().get("onclick");
			onmouseup = (String)component.getAttributes().get("onmouseup");
			title = (String)component.getAttributes().get("title");
			titleSelected = (String)component.getAttributes().get("titleSelected");
		}

		boolean noscript = ("true").equalsIgnoreCase(noscriptString);
		boolean selectOne = ("true").equalsIgnoreCase(selectOneStr);
		
        if (noscript) {
            String paddedId = id + PAD_INPUT_SEL;
            HtmlUtil.writeTagStartOpen(writer, "input", false, false); //$NON-NLS-1$

            if(onclick != null){
            	HtmlUtil.writeTagAttribute(writer,"onclick",onclick);	
            }
            if(onmouseup != null){
            	HtmlUtil.writeTagAttribute(writer,"onmouseup",onmouseup);	
            }
            if(selectOne){
                HtmlUtil.writeTagAttribute(writer, "type", "radio"); //$NON-NLS-1$ //$NON-NLS-2$
                HtmlUtil.writeTagAttribute(writer, "id", component.getId()); //$NON-NLS-1$
                HtmlUtil.writeTagAttribute(writer, "name", component.getId()); //$NON-NLS-1$
                HtmlUtil.writeTagAttribute(writer, "value", component.getClientId(context)); //$NON-NLS-1$
            } else {
            	HtmlUtil.writeTagAttribute(writer, "type", "checkbox"); //$NON-NLS-1$ //$NON-NLS-2$
                HtmlUtil.writeTagAttribute(writer, "id", paddedId); //$NON-NLS-1$
                HtmlUtil.writeTagAttribute(writer, "name", paddedId); //$NON-NLS-1$
            }
            HtmlUtil.writeTagAttribute(writer, "style", style); //$NON-NLS-1$
            HtmlUtil.writeTagAttribute(writer, "class", styleClass); //$NON-NLS-1$
            
            HtmlUtil.writeTagAttribute(writer, "title", title); //$NON-NLS-1$
            
            if (selected) {
                HtmlUtil.writeTagAttribute(writer, "checked", "checked"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
            }
            HtmlUtil.writeTagStartCloseLn(writer, true);
        }
        else {
            ResourceBundle bundle = ResourceHandler.getBundle(context);

            String paddedId = id + PAD_INPUT_SEL;
            String localizedUnselected = ResourceHandler.getString(bundle, "DataRendererSelect.unselected"); //$NON-NLS-1$
            String localizedSelected = ResourceHandler.getString(bundle, "DataRendererSelect.selected"); //$NON-NLS-1$

            if(titleSelected != null && titleSelected.length()>0)
            	localizedSelected = titleSelected;
            if(title != null && title.length()>0)
            	localizedUnselected = title;
            
            String src = this.getImage(context, component, selectOne, selected);
            String alt = selected ? localizedSelected : localizedUnselected;

            HtmlUtil.writeTagStartOpen(writer, "input", false, false); //$NON-NLS-1$
            if(onclick != null){
            	HtmlUtil.writeTagAttribute(writer, "onclick", onclick); //$NON-NLS-1$ //$NON-NLS-2$	
            }
            if(onmouseup != null){
            	HtmlUtil.writeTagAttribute(writer,"onmouseup",onmouseup);	
            }
            HtmlUtil.writeTagAttribute(writer, "type", "image"); //$NON-NLS-1$ //$NON-NLS-2$
            HtmlUtil.writeTagAttribute(writer, "id", paddedId); //$NON-NLS-1$
            HtmlUtil.writeTagAttribute(writer, "name", paddedId); //$NON-NLS-1$
            HtmlUtil.writeTagAttribute(writer, "src", src); //$NON-NLS-1$
            HtmlUtil.writeTagAttribute(writer, "alt", alt); //$NON-NLS-1$
            HtmlUtil.writeTagAttribute(writer, "title", alt); //$NON-NLS-1$
            HtmlUtil.writeTagAttribute(writer, "style", style); //$NON-NLS-1$
            HtmlUtil.writeTagAttribute(writer, "class", styleClass); //$NON-NLS-1$
            HtmlUtil.writeTagStartCloseLn(writer, true);

            HtmlUtil.writeInputHidden(writer, id + PAD_HIDDEN_SEL, selected ? "true" : "false"); //$NON-NLS-1$ //$NON-NLS-2$

            this.encodeJavaScript(context, component, selectOne, selected);
        }

        encodeParameters(context, component);

        //span end
        HtmlUtil.writeTagEndLn(writer, "span"); //$NON-NLS-1$
    }

    private void encodeJavaScript(
        FacesContext context,
        UIComponent component,
		boolean selectOne,
        boolean selected)
        throws IOException {
        UIScriptCollector scriptCollector = UIScriptCollector.find(component);
        String dataId = this.findUIDataId(context, component);
        
		HtmlInputRowSelect selectComponent = null;
		if ( component instanceof HtmlInputRowSelect ){
			selectComponent = (HtmlInputRowSelect)component;
		}
        
        if (dataId != null) {
            ResourceBundle bundle = ResourceHandler.getBundle(context);
            String id = component.getClientId(context);
            String localizedUnselected = ResourceHandler.getString(bundle, "DataRendererSelect.unselected"); //$NON-NLS-1$
            String localizedSelected = ResourceHandler.getString(bundle, "DataRendererSelect.selected"); //$NON-NLS-1$

    		String titleSelected;
    		String title;
            
            if( selectComponent != null ){
    			titleSelected = selectComponent.getTitleSelected();
    			title = selectComponent.getTitle();
    		}else{
    			titleSelected = (String)component.getAttributes().get("titleSelected");
    			title = (String)component.getAttributes().get("title");
    		}
            
            if(titleSelected != null && titleSelected.length()>0)
            	localizedSelected = titleSelected;
            if(title != null && title.length()>0)
            	localizedUnselected = title;
            
            String src_sel = this.getImage(context, component, selectOne, true);
            String src_usel = this.getImage(context, component, selectOne, false);

            StringBuffer buffer = new StringBuffer();

            String varCellGroup = getJsSelectionGroupVariable(dataId);
            buffer.append("\r\n if(!") //$NON-NLS-1$
            .append(varCellGroup).append(") { var ") //$NON-NLS-1$
            .append(varCellGroup).append(" = new ") //$NON-NLS-1$
			.append(HxClientRenderUtil.HXG_FUNCTION_NAME)
			.append(".prototype.HxCheckCellGroup(")
			.append((selectOne ? "true" : "false" ))
			.append("); } \r\n");
            
            this.writeJavaScriptOnce(
                context,
				component,
                scriptCollector,
                buffer.toString());

            buffer.setLength(0);

            String varSelection = getJsSelectionVariable(id);
            buffer.append("\r\n if(!") //$NON-NLS-1$
            .append(varSelection).append(") { var ") //$NON-NLS-1$
            .append(varSelection).append(" = new ") //$NON-NLS-1$
			.append(HxClientRenderUtil.HXG_FUNCTION_NAME)
			.append(".prototype.HxSelection(")
            .append(selected ? "true" : "false") //$NON-NLS-1$ //$NON-NLS-2$
            .append("); } \r\n"); //$NON-NLS-1$
            buffer.append(varSelection).append(".addSelectionListener(new ") //$NON-NLS-1$
			.append(HxClientRenderUtil.HXG_FUNCTION_NAME)
			.append(".prototype.HxCheckImageListener(\"")
            .append(id).append(PAD_INPUT_SEL).append("\", \"") //$NON-NLS-1$
            .append(src_usel).append("\", \"") //$NON-NLS-1$
            .append(src_sel).append("\", \"") //$NON-NLS-1$
            .append(localizedUnselected).append("\", \"") //$NON-NLS-1$
            .append(localizedSelected).append("\")); \r\n"); //$NON-NLS-1$
            buffer.append("new ") //$NON-NLS-1$
			.append(HxClientRenderUtil.HXG_FUNCTION_NAME)
			.append(".prototype.HxCheckCellListener(\"")
            .append(id).append("\", \"") //$NON-NLS-1$
            .append(id).append(PAD_HIDDEN_SEL).append("\", ") //$NON-NLS-1$ 
            .append(varCellGroup).append(", ") //$NON-NLS-1$
            .append(varSelection).append("); \r\n"); //$NON-NLS-1$

			String columnClasses;
			if ( selectComponent != null ){
				columnClasses = selectComponent.getColumnClasses();
			}else{
				columnClasses = (String) component.getAttributes().get("columnClasses");
			}
	
            if (columnClasses != null) {
                buffer.append("new ") //$NON-NLS-1$
				.append(HxClientRenderUtil.HXG_FUNCTION_NAME)
				.append(".prototype.HxStyleRegister(")
                .append(varSelection).append(", \"") //$NON-NLS-1$
                .append(dataId).append("\", \"") //$NON-NLS-1$
                .append(id).append("\", \"") //$NON-NLS-1$
                .append(columnClasses).append("\"); \r\n"); //$NON-NLS-1$
            }

            this.writeJavaScript(
                context,
                scriptCollector,
                buffer.toString());
        }
    }

    private void writeJavaScriptOnce(
        FacesContext context,
		UIComponent component,
        UIScriptCollector scriptCollector,
        String script)
        throws IOException {

        if (scriptCollector != null) {
            HxClientRenderUtil.initJSLibraries(scriptCollector, context);            
            HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component);
    		HxClientRenderUtil.addScriptLibrary("hxclient_table.js", component);
            scriptCollector.addScript(script);
        }
        else {
            //failsafe
            writeJavaScript(context, null, script);
        }
    }

    /**
     * @see javax.faces.render.Renderer#decode(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     */
    public void decode(FacesContext context, UIComponent component) {

        String id = component.getClientId(context);
        Map requestMap = context.getExternalContext().getRequestParameterMap();

		HtmlInputRowSelect selectComponent = null;
		if ( component instanceof HtmlInputRowSelect ){
			selectComponent = (HtmlInputRowSelect)component;
		}

		String noscriptString;
		String selectOneStr;
		if( selectComponent != null ){
			noscriptString = selectComponent.getNoscript();
			selectOneStr = selectComponent.getSelectOne();
		}else{
			noscriptString = (String)component.getAttributes().get("noscript");
			selectOneStr = (String)component.getAttributes().get("selectOne");
		}

		boolean noscript = ("true").equalsIgnoreCase(noscriptString);
		boolean selectOne = ("true").equalsIgnoreCase(selectOneStr);
		
        boolean selected = false;

        if (noscript) {
        	if(selectOne){
            	String selectOneValue = (String)requestMap.get(component.getId());
            	if(selectOneValue != null && selectOneValue.equals(id)){
            		selected = true;
            	}
            } else {
            	selected = (requestMap.get(id + PAD_INPUT_SEL) != null);	
            }
        }
        else {
            selected = ("true").equals(requestMap.get(id + PAD_HIDDEN_SEL)); //$NON-NLS-1$
        }
        
        initializeValues(context, component);
        this.setRowSelected(context, component, selected);
    }

    static boolean isRowSelected(UIComponent component)
        throws IndexOutOfBoundsException {
        if (!(component instanceof UIInput)) {
            return false;
        }
        UIInput input = (UIInput) component;
        Object value = input.getValue();
        if (value instanceof Boolean) {
            return ((Boolean) value).booleanValue();
        }
        else if (value instanceof Boolean[]) {
            return isRowSelected(component, (Boolean[]) value);
        }
        else if (value instanceof boolean[]) {
            return isRowSelected(component, (boolean[]) value);
        }
        else if (value instanceof Integer[]) {
            return isRowSelected(component, (Integer[]) value);
        }
        else if (value instanceof int[]) {
            return isRowSelected(component, (int[]) value);
        }
        else if (value instanceof Collection) {
            return isRowSelected(component, (Collection) value);
        }
        return false;
    }

    private static boolean isRowSelected(
        UIComponent component,
        Boolean[] arr) {
        int idx = getCurrentRowIndex(component);
        if (idx < 0) {
            return false;
        }
        if (arr[idx] == null) {
            return false;
        }
        return arr[idx].booleanValue();
    }

    private static boolean isRowSelected(
        UIComponent component,
        boolean[] arr) {
        int idx = getCurrentRowIndex(component);
        if (idx < 0) {
            return false;
        }
        return arr[idx];
    }

    private static boolean isRowSelected(
        UIComponent component,
        Integer[] arr) {
        int idx = getCurrentRowIndex(component);
        if (idx < 0) {
            return false;
        }
        for (int i = 0, len = arr.length; i < len; i++) {
            if (arr[i] == null) {
                continue;
            }
            if (arr[i].intValue() == idx) {
                return true;
            }
        }
        return false;
    }

    private static boolean isRowSelected(UIComponent component, int[] arr) {
        int idx = getCurrentRowIndex(component);
        if (idx < 0) {
            return false;
        }
        for (int i = 0, len = arr.length; i < len; i++) {
            if (arr[i] == idx) {
                return true;
            }
        }
        return false;
    }

    private static boolean isRowSelected(
        UIComponent component,
        Collection arr) {
        UIParameter[] params = LifecycleUtil.getParameters(component);
        if ((params == null) || (params.length == 0)) {
            return false;
        }
        Map map = new HashMap();
        for (int i = 0, len = params.length; i < len; i++) {
            map.put(params[i].getName(), params[i].getValue());
        }
        return arr.contains(map);
    }

    private static int getCurrentRowIndex(UIComponent component) {
        UIData data = findUIData(component);
        if (data == null) {
            return -1;
        }
        return data.getRowIndex();
    }

    private void setRowSelected(FacesContext context, UIComponent component, boolean selected) {
        if (!(component instanceof UIInput)) {
            return;
        }

        UIInput input = (UIInput) component;
        Object value = input.getValue();
        
        UIData data = findUIData(component);
        Map reqMap = context.getExternalContext().getRequestMap();
        //make the value_store param unique to component as there may be more than one rowSelect component on the page
        String valueParam = VALUE_STORE + component.getId();
    	 
        if (value instanceof Boolean) {
        	
        	if(((Boolean)value).booleanValue() != selected){
        		//set a request parameter to fire a valueChangeEvent
        		//only want to do this once at the first detected change(it is possible this could be process multiple times as in a boolean property in a DataModel)
        		if (reqMap.get(valueParam) == null){
        			reqMap.put(valueParam,new Boolean[]{((Boolean)value), selected? Boolean.TRUE : Boolean.FALSE });
        		}
        	}
        	
        	input.setValue(selected ? Boolean.TRUE : Boolean.FALSE);
        	
        	//if last row check if to fire a value change event
        	if(isLastRow(data)){
        		Boolean[] b = (Boolean[])reqMap.get(valueParam);
        		if(b != null){
        			input.queueEvent(new ValueChangeEvent(component, b[0], b[1]));
        		}
        	}
        	
        }
        
        else {
        	boolean last = isLastRow(data);
        
            if ((!last) && (isRowSelected(component) == selected)) {
                return;
            }

            /* 
             * We need to overcome the row-level state saving in the UIData
             * To do this, keep a handle on the collection component
             * We also only want 1 value changed event, so do that in last row
             */
            Object newValue = reqMap.get(valueParam);

            boolean copy = false;
            if (newValue == null) {
                copy = true;
            }
            else {
                value = newValue;
            }
            
            if (value instanceof Collection) {
                Collection v = (Collection) value;
                if (copy) {
                    value = v = getCopy(v);
                }
                setRowSelected(input, v, selected);
            }
            if (value instanceof Boolean[]) {
                Boolean[] v = (Boolean[]) value;
                if (copy) {
                    value = v = getCopy(v);
                }
                setRowSelected(input, v, selected);
            }
            else if (value instanceof boolean[]) {
                boolean[] v = (boolean[]) value;
                if (copy) {
                    value = v = getCopy(v);
                }
                setRowSelected(input, v, selected);
            }
            else if (value instanceof Integer[]) {
                Integer[] v = (Integer[]) value;
                if (copy) {
                    value = v = getCopy(v);
                }
                value = setRowSelected(input, v, selected);
            }
            else if (value instanceof int[]) {
                int[] v = (int[]) value;
                if (copy) {
                    value = v = getCopy(v);
                }
                value = setRowSelected(input, v, selected);
            }

            reqMap.put(valueParam, value);

            if ((last) && ((newValue != null) || (copy))) {
                //                boolean unchanged = true;
                //
                //                if (value instanceof Collection) {
                //                    Collection v = (Collection) value;
                //                    unchanged = isEquivalent(v, (Collection) input.getValue());
                //                }
                //                if (value instanceof Boolean[]) {
                //                    Boolean[] v = (Boolean[]) value;
                //                    unchanged = isEquivalent(v, (Boolean[]) input.getValue());
                //                }
                //                else if (value instanceof boolean[]) {
                //                    boolean[] v = (boolean[]) value;
                //                    unchanged = isEquivalent(v, (boolean[]) input.getValue());
                //                }
                //                else if (value instanceof Integer[]) {
                //                    Integer[] v = (Integer[]) value;
                //                    unchanged = isEquivalent(v, (Integer[]) input.getValue());
                //                }
                //                else if (value instanceof int[]) {
                //                    int[] v = (int[]) value;
                //                    unchanged = isEquivalent(v, (int[]) input.getValue());
                //                }

                reqMap.remove(valueParam);
                
                /*
                 *Because setValue is used instead of setSubmitted value is used this will bypass the validate method, this
                 *is what we want as because a new copy will have been created components old value and new value will always
                 *be different so even when value hasn't changed a valueChangedEvent will be queued.
                 * 
                 *Check the value change by array comparison in hasValueChanged method (pandrews).
                 */
                if(hasValueChanged(input.getValue(),value)){
                	input.queueEvent(new ValueChangeEvent(input,input.getValue(),value));
                }
                input.setValue(value);
            }
        }
    }
    
    private boolean hasValueChanged(Object oldValue, Object newValue){
    	
    	Boolean changed = Boolean.FALSE;
    	if(oldValue == null){
    		if(newValue != null){
    			changed = Boolean.TRUE;
    		}
    	} else if(newValue == null){
    		//we know oldValue is not null
    		changed = Boolean.TRUE;
    	} else {
    		//both oldValue and newValue are not null
    		if(newValue instanceof boolean[]){
    			if(!(Arrays.equals((boolean[])oldValue,(boolean[])newValue))){
    				changed = Boolean.TRUE;
    			}
    		} else if (newValue instanceof Boolean[]){
    			if(!(Arrays.equals((Boolean[])oldValue,(Boolean[])newValue))){
    				changed = Boolean.TRUE;
    			}
    		} else if (newValue instanceof int[]){
    			if(!(Arrays.equals((int[])oldValue,(int[])newValue))){
    				changed = Boolean.TRUE;
    			}
    		} else if (newValue instanceof Integer[]){
    			if(!(Arrays.equals((Integer[])oldValue,(Integer[])newValue))){
    				changed = Boolean.TRUE;
    			}
    		} else if (newValue instanceof Collection){
    			if(!(Arrays.equals(((Collection)oldValue).toArray(),((Collection)newValue).toArray()))){
    				changed = Boolean.TRUE;
    			}
    		}
    	}
    	return changed.booleanValue();
    }

    private boolean isLastRow(UIData data) {
        boolean last = false;
        int currentRow = data.getRowIndex();

        if (data.getRows() >= 1) {
            int lastRow = data.getFirst() + data.getRows() - 1;
            if (lastRow == currentRow) {
                last = true;
            }
        }

        if (last) {
            return last;
        }

        int rowCount = data.getRowCount();
        if (rowCount >= 0) {
            if (currentRow + 1 == rowCount) {
                last = true;
            }
        }
        else {
            int nextRow = currentRow + 1;
            data.setRowIndex(nextRow);
            if (!data.isRowAvailable()) {
                last = true;
            }
            data.setRowIndex(currentRow);
        }

        return last;
    }

    private int[] getCopy(int[] array) {
        int[] copy = new int[array.length];
        System.arraycopy(array, 0, copy, 0, array.length);
        return copy;
    }

    private Integer[] getCopy(Integer[] array) {
        Integer[] copy = new Integer[array.length];
        System.arraycopy(array, 0, copy, 0, array.length);
        return copy;
    }

    private boolean[] getCopy(boolean[] array) {
        boolean[] copy = new boolean[array.length];
        System.arraycopy(array, 0, copy, 0, array.length);
        return copy;
    }

    private Boolean[] getCopy(Boolean[] array) {
        Boolean[] copy = new Boolean[array.length];
        System.arraycopy(array, 0, copy, 0, array.length);
        return copy;
    }

    private Collection getCopy(Collection collection) {
        if (collection instanceof Cloneable) {
            try {
                final Class[] params = {};
                Method clone = collection.getClass().getMethod("clone", params); //$NON-NLS-1$
                final Object[] args = {};
                Collection ret = (Collection) clone.invoke(collection, args);
                return ret;
            }
            catch (NoSuchMethodException e) {}
            catch (InvocationTargetException e) {}
            catch (IllegalAccessException e) {}
        }
        return collection;
    }

    private void setRowSelected(
        UIInput component,
        Boolean[] arr,
        boolean selected) {
        int idx = getCurrentRowIndex(component);
        if (idx < 0) {
            return;
        }
        if ((arr[idx] != null) && (arr[idx].booleanValue() == selected)) {
            return;
        }

        arr[idx] = selected ? Boolean.TRUE : Boolean.FALSE;
    }

    private void setRowSelected(
        UIInput component,
        boolean[] arr,
        boolean selected) {
        int idx = getCurrentRowIndex(component);
        if (idx < 0) {
            return;
        }
        if (arr[idx] == selected) {
            return;
        }

        arr[idx] = selected;
    }

    private Object setRowSelected(
        UIInput component,
        Integer[] arr,
        boolean selected) {
        int idx = getCurrentRowIndex(component);
        if (idx < 0) {
            return arr;
        }
        if (selected) {
            for (int i = 0, len = arr.length; i < len; i++) {
                if ((arr[i] != null) && (arr[i].intValue() == idx)) {
                    return arr;
                }
            }

            Integer[] copy = new Integer[arr.length + 1];
            System.arraycopy(arr, 0, copy, 0, arr.length);
            copy[arr.length] = new Integer(idx);
            return copy;
        }
        
        int found = -1;
        for (int i = 0, len = arr.length; i < len; i++) {
            if ((arr[i] != null) && (arr[i].intValue() == idx)) {
                found = i;
                break;
            }
        }
        if (found < 0) {
            return arr;
        }

        Integer[] copy = new Integer[arr.length - 1];
        System.arraycopy(arr, 0, copy, 0, found);
        System.arraycopy(
            arr,
            found + 1,
            copy,
            found,
            arr.length - found - 1);
        return copy;
        
    }

    private Object setRowSelected(
        UIInput component,
        int[] arr,
        boolean selected) {
        int idx = getCurrentRowIndex(component);
        if (idx < 0) {
            return arr;
        }
        if (selected) {
            for (int i = 0, len = arr.length; i < len; i++) {
                if (arr[i] == idx) {
                    return arr;
                }
            }

            int[] copy = new int[arr.length + 1];
            System.arraycopy(arr, 0, copy, 0, arr.length);
            copy[arr.length] = idx;
            return copy;
        }
        
        int found = -1;
        for (int i = 0, len = arr.length; i < len; i++) {
            if (arr[i] == idx) {
                found = i;
                break;
            }
        }
        if (found < 0) {
            return arr;
        }

        int[] copy = new int[arr.length - 1];
        System.arraycopy(arr, 0, copy, 0, found);
        System.arraycopy(
            arr,
            found + 1,
            copy,
            found,
            arr.length - found - 1);
        return copy;
    }

    private void setRowSelected(
        UIInput component,
        Collection arr,
        boolean selected) {
        Map map =
            this.decodeParameters(FacesContext.getCurrentInstance(), component);
        if (map == null) {
            return;
        }
        try {
            if (selected && !arr.contains(map)) {
                arr.add(map);
            }
            else if (!selected && arr.contains(map)) {
                arr.remove(map);
            }
        }
        catch (UnsupportedOperationException e) {
            //failsafe if Collection does not support optional method(s)
        }
    }

    static String getJsSelectionVariable(String id) {
        return JavaScriptUtil.makeVariableName(id + JS_SELECTION);
    }

    static String getJsSelectionGroupVariable(String id) {
        return JavaScriptUtil.makeVariableName(id + JS_OBJGROUP);
    }

    private String getImage(
        FacesContext context,
        UIComponent component,
		boolean selectOne,
        boolean selected) {
        String ret;
        
		HtmlInputRowSelect selectComponent = null;
		if ( component instanceof HtmlInputRowSelect ){
			selectComponent = (HtmlInputRowSelect)component;
		}
        
        if (selected) {
        	String img;
        	if ( selectComponent != null ){
        		img = selectComponent.getSrcSelected();
        	}else{
				img = (String) component.getAttributes().get("srcSelected"); //$NON-NLS-1$
        	}
            
            if (img != null) {
                ret = Utils.getEncodedResourceURL(context, img);
            }
            else {
            	if(selectOne){
            		ret = JavaScriptUtil.getImagePath(context, SELECTONE_IMAGE_SELECTED);	
            	} else {
            		ret = JavaScriptUtil.getImagePath(context, IMAGE_SELECTED);
            	}
            	ret = context.getExternalContext().encodeResourceURL(ret);
            }
        }
        else {
        	String img;
			if ( selectComponent != null ){
				img = selectComponent.getSrcUnselected();
			}else{
				img = (String) component.getAttributes().get("srcUnselected"); //$NON-NLS-1$
			}
			
            if (img != null) {
                ret = Utils.getEncodedResourceURL(context, img);
            } else {
            	if(selectOne){
            		ret = JavaScriptUtil.getImagePath(context, SELECTONE_IMAGE_UNSELECTED);	
            	} else {
            		ret = JavaScriptUtil.getImagePath(context, IMAGE_UNSELECTED);
            	}
            	ret = context.getExternalContext().encodeResourceURL(ret);
            }
        }
        return ret;
    }
    
    private void initializeValues(FacesContext context, UIComponent component){
    	//only want to do this once per request
    	String key = component.getId() + "_boolcheck";
    	Boolean done = (Boolean)context.getExternalContext().getRequestMap().get(key);
    	if (done != null) {
    		return;
    	} 
   		context.getExternalContext().getRequestMap().put(key,Boolean.TRUE);
    	
        ValueBinding vb = component.getValueBinding("value");
        if(vb == null){
        	return;
        }
        String name = vb.getType(context).getName();
        
        if(name.equals("[Ljava.lang.Boolean;") || name.equals("[Z") || name.equals("[Ljava.lang.Integer;") || name.equals("[I")){
        	//bound to a boolean or Boolean array
        	Object val = ((UIInput)component).getValue();
            int tableRows = Utils.findParentUIData(component).getRowCount();
            
            if(		val == null || 
            		(val instanceof Boolean[] && ( ((Boolean[])val).length == 0 || ((Boolean[])val).length < tableRows )) || 
    				(val instanceof boolean[] && ( ((boolean[])val).length == 0 || ((boolean[])val).length < tableRows )))
            {
                if(name.equals("[Ljava.lang.Boolean;")){
                	//is Boolean[]
                	vb.setValue(context,new Boolean[tableRows]);
                } else if (name.equals("[Z")){
                	//is boolean[]
                	vb.setValue(context,new boolean[tableRows]);
                } else if(name.equals("[Ljava.lang.Integer;")){
                	//is  Integer[]
                	vb.setValue(context,new Integer[0]);
                } else if(name.equals("[I")){
                	//is  nteger[]
                	vb.setValue(context,new int[0]);
                }
                
            }	
        }
        
    }

    
    //    private boolean isEquivalent(boolean[] a1, boolean[] a2) {
    //        for (int i = 0, len = a1.length; i < len; i++) {
    //            if (a1[i] != a2[i]) {
    //                return false;
    //            }
    //        }
    //        return true;
    //    }
    //
    //    private boolean isEquivalent(Boolean[] a1, Boolean[] a2) {
    //        for (int i = 0, len = a1.length; i < len; i++) {
    //            if (!a1[i].equals(a2[i])) {
    //                return false;
    //            }
    //        }
    //        return true;
    //    }
    //
    //    private boolean isEquivalent(int[] a1, int[] a2) {
    //        if (a1.length != a2.length) {
    //            return false;
    //        }
    //
    //        java.util.Arrays.sort(a1);
    //        java.util.Arrays.sort(a2);
    //
    //        for (int i = 0, len = a1.length; i < len; i++) {
    //            if (a1[i] != a2[i]) {
    //                return false;
    //            }
    //        }
    //        return true;
    //    }
    //
    //    private boolean isEquivalent(Integer[] a1, Integer[] a2) {
    //        if (a1.length != a2.length) {
    //            return false;
    //        }
    //
    //        java.util.Arrays.sort(a1);
    //        java.util.Arrays.sort(a2);
    //
    //        for (int i = 0, len = a1.length; i < len; i++) {
    //            if (a1[i].equals(a2[i])) {
    //                return false;
    //            }
    //        }
    //        return true;
    //    }
    //
    //    private boolean isEquivalent(Collection c1, Collection c2) {
    //        return c1.equals(c2);
    //    }
    

	/* (non-Javadoc)
	 * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {
		//do nothing
		
	}

}
