/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.el.MethodBinding;
import javax.faces.render.Renderer;

import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlDataXml;
import com.ibm.faces.util.ContextParams;
import com.ibm.faces.util.JavaScriptUtil;
import com.ibm.faces.util.AjaxUtil;


/**
 * Renderer for {@link com.ibm.faces.component.UIScriptCollector UIScriptCollector} component, 
 * that outputs JavaScripts provided by other components.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.ScriptCollectorTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.component.UIScriptCollector}
 */
public class ScriptCollectorRenderer extends Renderer {
	
	
	/** 
	 * Renders the beginning of the component.
	 * <p>
	 * <ul>
	 * <li>Resets {@link com.ibm.faces.component.UIScriptCollector}'s buffer for storing
	 * scripts.
	 * <li>Checks if there is <code>preRender</code> attribute present,
	 * and if yes, invokes the method specified in the attribute, passing
	 * {@link javax.faces.context.FacesContext FacesContext} as a parameter.
	 * </ul>
	 * <p>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request being processed
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */	
	public void encodeBegin(FacesContext context, UIComponent component)
		throws IOException {

		UIScriptCollector scriptCollector = (UIScriptCollector) component;
		scriptCollector.reset();
		
		MethodBinding method = scriptCollector.getPreRender();
		if(method != null) {
			method.invoke(context, new Object[] {context});
		}	
				
		/*if(context instanceof AjaxContext)
		{
			AjaxContext ajaxContext = (AjaxContext)context;
						
			String del_start = (String) context.getExternalContext().getRequestMap().get(HxClientRenderUtil.DELIM_START);
			
			((AjaxContext)context).setRendering(true);
			
			if(HxClientRenderUtil.isPortal(context.getExternalContext().getRequest()))
				if(del_start == null)
				{
					try{
						ResponseWriter writer = context.getResponseWriter();
						writer.write("<head title=\"PORTALcontent\">");
						context.getExternalContext().getRequestMap().put(DELIM_START, "true");
					}catch(IOException ioe)
					{
						
					}
				}					
			
			((AjaxContext)context).setRendering(false);
		}*/
		
	}

	/** 
	 * Renders component's children.
	 * <p>
	 * Doesn't need to do anything.
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request being processed
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeChildren(FacesContext context, UIComponent component)
		throws IOException {

	}

	/**
	 * Renders the ending of the component.
	 * <p>
	 * <ul>
	 * <li>Outputs references to all collected JS libraries
 	 * <li>Creates a JavaScript block.
	 * <li>Outputs all collected scripts.
	 * <li>Asks all registered contributors to output their scripts.
	 * <li>Checks if there is <code>postRender</code> attribute present,
	 * and if yes, invokes the method specified in the attribute, passing
	 * {@link javax.faces.context.FacesContext FacesContext} as a parameter.
	 * </ul>
	 * <p>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request being processed
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 *
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */	
	public void encodeEnd(FacesContext context, UIComponent component)
		throws IOException {
		
		
		//If this is an Ajax request then we need to set rendering to true so that the ScriptCollector will always
		//have a real response writer so that the js blocks for the rendering components it added to the page. 
		
		if(AjaxUtil.isAjaxRequest(context))
		{
			AjaxUtil.setRendering(context,true);
		}
		
		ResponseWriter writer = context.getResponseWriter();
		UIScriptCollector scriptCollector = (UIScriptCollector) component;
		HashMap contributors = scriptCollector.getContributors();
		
		//if(!(context instanceof AjaxFacesContext))
		//{
			String useFullHxVersion = context.getExternalContext().getInitParameter(ContextParams.USE_HXCLIENT_FULL);
			if(useFullHxVersion == null || (useFullHxVersion!=null && (!(useFullHxVersion.equals("true")))))
			{
				Vector libraryReferences = scriptCollector.getLibraryReferencesTop();
				Iterator libRefIterator = libraryReferences.iterator();
				//String theContextPath = LifecycleUtil.getResourceServletContextPath(context);
				writer.write("\n");	
				while(libRefIterator.hasNext())
				{
					writer.write("<script type=\"text/javascript\" src=\"");
					String ref =(String)libRefIterator.next();
					writer.write(ref);
					writer.write("\"></script>");
					writer.write("\n");
				}
			}
		//}
		// Create SCRIPT tags for all collected JS libraries
		HashMap libraries = scriptCollector.getLibraryBlocks();
		Map.Entry lib;
		for (Iterator iter = libraries.entrySet().iterator(); iter.hasNext();) 
		{
			lib = (Map.Entry) iter.next();
			scriptCollector.writeLibraryReference(
			    context,
			    (String)lib.getKey(),
			    (String)lib.getValue());
		}

		// Check if there is any script to emit
		Vector topScripts = scriptCollector.getScriptsTop();		
		Vector bottomScripts = scriptCollector.getScriptsBottom();
		Vector vbottomScripts = scriptCollector.getScriptsVeryBottom();
		boolean hasScript = (topScripts.size() > 0) || (bottomScripts.size() > 0) || (contributors.size() > 0) || (vbottomScripts.size() > 0);
		
		if(hasScript) {
			// Start a JS script block for all following scripts
			writer.write(JavaScriptUtil.JS_NEWLINE);
			writer.write(JavaScriptUtil.getStartTag());
	
			// Output all collected scripts for the top of the script block
			for (Iterator iter = topScripts.iterator(); iter.hasNext();) {
				String script = (String) iter.next();
				scriptCollector.writeScriptOnce(script, context, false);
			}
			
			// Ask all script contributors to output their scripts
			if(contributors.size() > 0) {
				
				
				// There is a dependency between the tree/graph
				// and xmldata, so output the dataxml first.
				Iterator iterator = contributors.keySet().iterator();	
				Vector xmlDatas = new Vector();
				
				while(iterator.hasNext()) {
					UIComponent comp = (UIComponent) iterator.next();
					IScriptContributor contributor = (IScriptContributor) contributors.get(comp);
					if(contributor != null && comp != null) {
						if (comp instanceof HtmlDataXml )
						{
							contributor.contributeScript(context, comp);
							xmlDatas.add(comp);
						}
					}
				}
				
				Iterator xmlDatasIterator = xmlDatas.iterator();
				while(xmlDatasIterator.hasNext()) {
					UIComponent comp = (UIComponent) xmlDatasIterator.next();
					if( comp != null) {
						contributors.remove(comp);
					}
				}
				
				
				Iterator iter = contributors.keySet().iterator();	
				while(iter.hasNext()) {
					UIComponent comp = (UIComponent) iter.next();
					IScriptContributor contributor = (IScriptContributor) contributors.get(comp);
					if(contributor != null && comp != null) {
						contributor.contributeScript(context, comp);
					}
				}
			}
			
			// Output all collected scripts for the bottom of the script block
			for (Iterator iter = bottomScripts.iterator(); iter.hasNext();) {
				String script = (String) iter.next();
				scriptCollector.writeScriptOnce(script, context, false);
			}

			
			// Output all collected scripts for the very bottom of the script block
			for (Iterator iter = vbottomScripts.iterator(); iter.hasNext();) {
				String script = (String) iter.next();
				scriptCollector.writeScriptOnce(script, context, false);
			}		
			
			// Close the script block
			writer.write(JavaScriptUtil.JS_NEWLINE);
			writer.write(JavaScriptUtil.getEndTag());
		}
		
		// invoke encodeEndRef method, if defined
		MethodBinding method = scriptCollector.getPostRender();
		if(method != null) {
			method.invoke(context, new Object[] {context});
		}
		
		//need to set rendering back to false so that if there are any custom components outside the 
		//script collector or if there is more than one scriptCollector on the page, they do not get
		//re-rendered to the page during an Ajax request when they shouldn't be. 
		if(AjaxUtil.isAjaxRequest(context))
		{
			
			String del_end = (String) context.getExternalContext().getRequestMap().get(HxClientRenderUtil.DELIM_END);
			
			String del_start = (String) context.getExternalContext().getRequestMap().get(HxClientRenderUtil.DELIM_START);
			
			if(HxClientRenderUtil.isPortal(context.getExternalContext().getRequest()))
				if(del_end == null && del_start != null)			
				{
					try{						
						writer.write("</head>");
						context.getExternalContext().getRequestMap().put(HxClientRenderUtil.DELIM_END, "true");
					}catch(IOException ioe)
					{
						
					}
				}			
			
			
			AjaxUtil.setRendering(context,false);
		}
	}
	
	/**
     * Decodes the current state of the specified {@link javax.faces.component.UIComponent UIComponent}
     * from the request contained in the specified {@link javax.faces.context.FacesContext FacesContext}.
     * <p>
	 * <ul>
	 * <li>Checks if there is <code>decode</code> attribute present,
	 * and if yes, invokes the method specified in the attribute, passing
	 * {@link javax.faces.context.FacesContext FacesContext} as a parameter.
	 * </ul>
	 * <p>
	 *
	 * @param context {@link javax.faces.context.FacesContext} for the request being processed
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @see javax.faces.render.Renderer#decode(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void decode(FacesContext context, UIComponent component) {
		UIScriptCollector scriptCollector = (UIScriptCollector) component;
		MethodBinding method = scriptCollector.getDecode();
		if(method != null) {
			method.invoke(context, new Object[] {context});
		}
	}

}
