/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
 
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

import javax.faces.render.Renderer;


import com.ibm.faces.component.html.HtmlOutputSeparator;

/**
 * Renderer for {@link javax.faces.component.UIOutput UIOutput} component, 
 * that creates a horizontal line separator.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.Output_SeparatorTag}<br>
 * <b>Components:</b> {@link javax.faces.component.UIOutput}
 */
public class SeparatorRenderer extends Renderer {

	/**
	 * Renders the beginning of the component.
	 * <p>Renders the entire component.
	 * <p>Creates an HR tag and sets various attributes that were specified in the 
	 * {@link com.ibm.faces.taglib.html_extended.Output_SeparatorTag}
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
     * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */	
	public void encodeBegin(FacesContext context, UIComponent component)
		throws IOException { 
			
		if (context == null || component == null) {
			throw new NullPointerException();
		}
		HtmlOutputSeparator separatorComponent = null;
		if ( component instanceof HtmlOutputSeparator )			
			separatorComponent = (HtmlOutputSeparator)component;		
		
		String width;
		String height;
		String color;
		String align;
		String shade;
		String style;
		String title;
		String separatorClass;
		String label;
		String lang;
		String dir;
		boolean escape = true; 
		
		if ( separatorComponent != null ){
			width = separatorComponent.getWidth();
			height = separatorComponent.getHeight();
			color = separatorComponent.getColor();
			align = separatorComponent.getAlign();
			shade = separatorComponent.getNoshade();
			style = separatorComponent.getStyle();
			title = separatorComponent.getTitle();
			separatorClass = separatorComponent.getStyleClass();
			label = separatorComponent.getLabel();
			lang = separatorComponent.getLang();
			dir = separatorComponent.getDir();
			escape = separatorComponent.isEscape(); //			
		}else{
			width = (String) component.getAttributes().get("width");		//$NON-NLS-1$
			height = (String) component.getAttributes().get("height");		//$NON-NLS-1$
			color = (String) component.getAttributes().get("color");		//$NON-NLS-1$
			align = (String) component.getAttributes().get("align");	//$NON-NLS-1$
			shade = (String) component.getAttributes().get("noshade");		//$NON-NLS-1$
			style = (String) component.getAttributes().get("style");		//$NON-NLS-1$
			title = (String) component.getAttributes().get("title");		//$NON-NLS-1$
			separatorClass = (String) component.getAttributes().get("styleClass");			//$NON-NLS-1$
			label = (String) component.getAttributes().get("label");			//$NON-NLS-1$
			lang = (String) component.getAttributes().get("lang");			//$NON-NLS-1$
			dir = (String) component.getAttributes().get("dir");			//$NON-NLS-1$
			
			Object a = component.getAttributes().get("escape"); //$NON-NLS-1$
			if(a instanceof String){
				if("true".equalsIgnoreCase(a.toString()))
					escape = true;
				else
					escape = false;
			}
			if(a instanceof Boolean){
				escape = ((Boolean)a).booleanValue();
			}
		}

		ResponseWriter writer = context.getResponseWriter();

		if(label != null && (label.trim()).length()>0)
		{			
			StringBuffer style_str = new StringBuffer();
			
			if(style!=null && style.trim().length() >0)			
				style_str.append(style);
			
			if(width!=null && width.trim().length() >0) 
			{
				width = addpx(width);
				style_str.append("width:");
				style_str.append(width);
				style_str.append(";");
			}

			if(height!=null && height.trim().length() >0) 
			{
				height = addpx(height);
				style_str.append("height:");
				style_str.append(height);
				style_str.append(";");
			}			
			
			if(color!=null && color.trim().length() >0) 
			{
				style_str.append("background-color:");
				style_str.append(color);
				style_str.append(";");
			}
			
			writer.startElement("div",component);
			if (align!=null) 
				writer.writeAttribute("align",align,"align");
			else
				writer.writeAttribute("align","center","align");
			
			writer.startElement("div",component);
			writer.writeAttribute("id",component.getClientId(context),null);
			
			if (title!=null && title.trim().length() >0) 
				writer.writeAttribute("title",title,"title");
			if (separatorClass!=null && separatorClass.trim().length() >0) 
				writer.writeAttribute("class",separatorClass,"styleClass");
			if(style_str.length() >0)
				writer.writeAttribute("style",style_str.toString(),null);

			if(lang != null && lang.trim().length() >0)
				writer.writeAttribute("lang",lang,null);
			if(dir != null && dir.trim().length() >0)
				writer.writeAttribute("dir",dir,null);
			
			if(!escape)
				writer.write(label);
			else 
				writer.writeText(label,null);			
			
			
			writer.endElement("div");		
			writer.endElement("div");
		}
		else
		{
			writer.startElement("hr",component);
			if (width!=null) writer.writeAttribute("width",width,"width");
			if (height!=null) writer.writeAttribute("size",height,"height");
			if (color!=null) writer.writeAttribute("color",color,"color");
			if (align!=null) writer.writeAttribute("align",align,"align");
			if (separatorClass!=null) writer.writeAttribute("class",separatorClass,"styleClass");
			if (style!=null) writer.writeAttribute("style",style,"style");
			if (title!=null) writer.writeAttribute("title",title,"title");
			
			if(lang != null && lang.trim().length() >0)
				writer.writeAttribute("lang",lang,null);
			if(dir != null && dir.trim().length() >0)
				writer.writeAttribute("dir",dir,null);			
			
			//noshade must be express as an attribute to be xhtml compliant
			if (shade != null && ("true").equals(shade))writer.writeAttribute("noshade","noshade",null); 
			writer.endElement("hr");
		}
	}

    /**
     * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     * @throws IOException
     */
    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {
        //defect 280926
    }

    private String addpx(String check)
    {
    	String checked = check;
    	
    	char ch = check.charAt(check.length()-1);
    	
    	if(Character.isDigit(ch)) //it - px or % not there
    	{
    		checked = checked + "px";
    	}
    	
    	return checked;	
    }
    
}
