/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.converter.NumberConverterEx;
import com.ibm.faces.util.ScriptObject;



import com.ibm.faces.component.html.HtmlInputHelperSlider;

/**
 * Renderer for {@link com.ibm.faces.component.UIInputHelper UIInputHelper} component. 
 * It creates an image, which when clicked, pops up
 * a Slider control.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.InputHelper_SliderTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.component.UIInputHelper}
 */

public class SliderRenderer extends Renderer implements IScriptContributor {
	
	/**
	 * Renders the ending of the component.
	 * <p>
	 * Renders the entire component.
	 * <p>
	 * <ul>
	 * <li>Instantiates a JavaScript Slider Object (from a JS library). 
	 * <li>Sets all required attributes of the object according to component's attributes. 
	 * <li>Creates an IMG tag for a "helper" image (little arrow pointing down) and sets
	 * its onclick event handler to pop up the Slider Object.
	 * </ul>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 *
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
    public void encodeBegin(FacesContext context, UIComponent component)
        throws IOException {

        if (context == null || component == null) {
            throw new NullPointerException();
        }

		UIComponent parent = component.getParent();
		if (parent == null || !(parent instanceof UIInput) 
			|| !(((UIInput)parent).getConverter() instanceof NumberConverterEx)) {
			return; // no point in going any further if we don't have these
		}
	
		HxClientRenderUtil.addScriptLibrary("hxclient_sl.js", component);
		
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		
		if (HxClientRenderUtil.isContainedInPanel(component.getParent())){
			NumberConverterEx nc = (NumberConverterEx) ((UIInput) parent).getConverter();
			ScriptObject converterScript =
				HxClientRenderUtil.buildJsNumberConverter(context, nc);
			ScriptObject validatorScript = null;
			if (null != converterScript) {
				scriptCollector.addScriptOnce(converterScript.getScript());
				//We can't have a validator without a converter 
				validatorScript = HxClientRenderUtil.buildJsRangeValidator(context,
																	nc,
																	HxClientRenderUtil.getRangeValidator((UIInput)parent),
																	((UIInput) parent).isRequired(),
																	HxClientRenderUtil.getExpressionValidator((UIInput)parent),null);
				if (null != validatorScript) {
					scriptCollector.addScriptOnce(validatorScript.getScript());
				}
			}
			scriptCollector.addScript(getSliderScript(context, component,
								converterScript!=null?converterScript.getId():null,
								validatorScript!=null?validatorScript.getId():null));

		} else {
			scriptCollector.register(this, component);
		}
    }

	/* (non-Javadoc)
	 * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {

		// Check if the parent has a validation tag, and if yes - get validation attributes
		UIComponent parent = component.getParent();
		UIScriptCollector scriptCollector = UIScriptCollector.find(parent);
		// Create converter and validator
		NumberConverterEx nc = (NumberConverterEx) ((UIInput) parent).getConverter();
		ScriptObject converterScript =
			HxClientRenderUtil.buildJsNumberConverter(context, nc);
		ScriptObject validatorScript = null;
		if (null != converterScript) {
			scriptCollector.writeScriptOnce(converterScript.getScript(), context, false);
			//We can't have a validator without a converter 
			validatorScript = HxClientRenderUtil.buildJsRangeValidator(context,
																nc,
																HxClientRenderUtil.getRangeValidator((UIInput)parent),
																((UIInput) parent).isRequired(),
																HxClientRenderUtil.getExpressionValidator((UIInput)parent), null);
			if (null != validatorScript) {
				scriptCollector.writeScriptOnce(validatorScript.getScript(), context, false);
			}
		}
		ResponseWriter writer = context.getResponseWriter();
		writer.write("\n"); //$NON-NLS-1$
		writer.write(
					getSliderScript(context, component,
					converterScript!=null?converterScript.getId():null,
					validatorScript!=null?validatorScript.getId():null));
	}

	public String getSliderScript(FacesContext context, UIComponent component, String converterId, String validatorId) {

		HtmlInputHelperSlider sliderComponent = null;
		if ( component instanceof HtmlInputHelperSlider ){
			sliderComponent = (HtmlInputHelperSlider)component;
		}
		
		String delta, width, hideLabels, sCSSPrefix, buttonColor, buttonBorder, labelIncrement;
		if ( sliderComponent != null ){
			delta = sliderComponent.getDelta();
			width = sliderComponent.getWidth();
			hideLabels = sliderComponent.getHideLabels();
			buttonColor = sliderComponent.getButtonColor();
			buttonBorder = sliderComponent.getButtonBorder();
			labelIncrement = sliderComponent.getLabelIncrement();
			sCSSPrefix =  sliderComponent.getStyleClass();
		}else{
			delta = (String) component.getAttributes().get("delta");  //$NON-NLS-1$
			width = (String) component.getAttributes().get("width"); //$NON-NLS-1$
			hideLabels = (String) component.getAttributes().get("hideLabels"); //$NON-NLS-1$
			buttonColor = (String) component.getAttributes().get("buttonColor"); //$NON-NLS-1$
			buttonBorder = (String) component.getAttributes().get("buttonBorder"); //$NON-NLS-1$
			labelIncrement = (String) component.getAttributes().get("labelIncrement"); //$NON-NLS-1$
			sCSSPrefix = (String) component.getAttributes().get("styleClass"); //$NON-NLS-1$
		}
		if(delta == null) {
			delta = "1"; //$NON-NLS-1$
		}
		
		// Check if the parent has a validation tag, and if yes - get validation attributes
		UIComponent parent = component.getParent();
		// Create converter and validator
		StringBuffer script = new StringBuffer(HxClientRenderUtil.HX_VAR_NAME);  //$NON-NLS-1$
		script.append(".addComponent(\"");
		script.append(parent.getClientId(context));
		script.append("\", new ");  //$NON-NLS-1$
		script.append(HxClientRenderUtil.HX_VAR_NAME);
		script.append(".JSFSlider(\"increment:");
		script.append(delta);
		script.append('\"'); //$NON-NLS-1$
		if(width != null && width.length() > 0) {
			script.append(",\"scale-width:");  //$NON-NLS-1$
			script.append(width);
			script.append('\"'); //$NON-NLS-1$
		}
		if(hideLabels != null && ("true").equalsIgnoreCase(hideLabels) ) { //$NON-NLS-1$
			script.append(",\"show-labels:false\""); //$NON-NLS-1$
		}
		if (sCSSPrefix != null && sCSSPrefix != "") { //$NON-NLS-1$
			script.append(", \"css-prefix:"); //$NON-NLS-1$
			script.append(Utils.getFirstCSSClassName(sCSSPrefix));
			script.append('\"'); //$NON-NLS-1$ 
		}
		if (converterId != null) {
			script.append(",\"converter:");  //$NON-NLS-1$
			script.append(converterId);
			script.append('\"'); //$NON-NLS-1$
		}
		if (validatorId != null) {
			script.append(",\"validator:");  //$NON-NLS-1$
			script.append(validatorId);
			script.append('\"'); //$NON-NLS-1$
		}
		if(buttonColor != null && buttonColor.trim().length() > 0) {	
			script.append(", \"button-color:"); //$NON-NLS-1$
			script.append(buttonColor);
			script.append("\""); //$NON-NLS-1$
		}
		if(buttonBorder != null && buttonBorder.trim().length() > 0) {	
			script.append(", \"button-border:"); //$NON-NLS-1$
			script.append(buttonBorder);
			script.append("\""); //$NON-NLS-1$
		}
		if(labelIncrement != null && labelIncrement.trim().length() > 0) {	
			script.append(", \"label-increment:"); //$NON-NLS-1$
			script.append(labelIncrement);
			script.append("\""); //$NON-NLS-1$
		}

		script.append("));");  //$NON-NLS-1$
		return script.toString();
	}

    /**
     * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     * @throws IOException
     */
    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {
        //defect 280926
    }


}
