/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.converter.NumberConverterEx;
import com.ibm.faces.util.ScriptObject;
import javax.faces.render.Renderer;


import com.ibm.faces.component.html.HtmlInputHelperSpinner;

/**
 * Renderer for {@link com.ibm.faces.component.UIInputHelper UIInputHelper} component.
 * It creates two "spinner" images (arrow up and arrow down), which 
 * when clicked, change the value of the input control this helper
 * is connected to.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.InputHelper_SpinnerTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.component.UIInputHelper}
 */
public class SpinnerRenderer extends Renderer implements IScriptContributor {

	/**
	 * Renders the ending of the component.
	 * <p>
	 * Renders the entire component.
	 * <p>
	 * Creates an IMG tag for an image with two arrows (up and down). Adds
	 * an image map to this image, so that mouse clicks on either arrow
	 * can be distinguished. Sets JavaScript handlers for mouseclick
	 * events.
	 * <p>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 *
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException {

		if (context == null || component == null) {
			throw new NullPointerException();
		}

		UIComponent parent = component.getParent();
		if (parent == null || !(parent instanceof UIInput) 
			|| !(((UIInput)parent).getConverter() instanceof NumberConverterEx)) {
			return; // no point in going any further if we don't have these
		}

		HxClientRenderUtil.addScriptLibrary("hxclient_sp.js", component);
		
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		if (HxClientRenderUtil.isContainedInPanel(component.getParent())){
			NumberConverterEx nc = (NumberConverterEx) ((UIInput) parent).getConverter();
			ScriptObject converterScript =
				HxClientRenderUtil.buildJsNumberConverter(context, nc);
			ScriptObject validatorScript = null;
			if (null != converterScript) {
				scriptCollector.addScriptOnce(converterScript.getScript());
				//We can't have a validator without a converter 
				validatorScript = HxClientRenderUtil.buildJsRangeValidator(context,
																	nc,
																	HxClientRenderUtil.getRangeValidator((UIInput)parent), 
																	((UIInput) parent).isRequired(),
																	HxClientRenderUtil.getExpressionValidator((UIInput)parent),null);
				if (null != validatorScript) {
					scriptCollector.addScriptOnce(validatorScript.getScript());
				}
			}
			scriptCollector.addScript(getSpinnerScript(context, component, 
										converterScript!=null?converterScript.getId():null,
										validatorScript!=null?validatorScript.getId():null));
		} else {
			scriptCollector.register(this, component);
		}
	}

	/* (non-Javadoc)
	 * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {

		// Check if the parent has a validation tag, and if yes - get validation attributes
		UIComponent parent = component.getParent();
		UIScriptCollector scriptCollector = UIScriptCollector.find(parent);
		NumberConverterEx nc = (NumberConverterEx) ((UIInput) parent).getConverter();
		// Create converter and validator
		ScriptObject converterScript =
			HxClientRenderUtil.buildJsNumberConverter(context, nc);
		ScriptObject validatorScript = null;
		if (null != converterScript) {
			scriptCollector.writeScriptOnce(converterScript.getScript(), context, false);
			//We can't have a validator without a converter 
			validatorScript = HxClientRenderUtil.buildJsRangeValidator(context,
																nc,
																HxClientRenderUtil.getRangeValidator((UIInput)parent), 
																((UIInput) parent).isRequired(),
																HxClientRenderUtil.getExpressionValidator((UIInput)parent),null);
			if (null != validatorScript) {
				scriptCollector.writeScriptOnce(validatorScript.getScript(), context, false);
			}
		}
		ResponseWriter writer = context.getResponseWriter();
		writer.write("\n"); //$NON-NLS-1$
		writer.write(getSpinnerScript(context, component,
							converterScript!=null?converterScript.getId():null,
							validatorScript!=null?validatorScript.getId():null));

	}

	public String getSpinnerScript(FacesContext context, UIComponent component, String converterId, String validatorId) {
	
		// Get own attributes		
		
		HtmlInputHelperSpinner spinnerComponent = null;
		if ( component instanceof HtmlInputHelperSpinner ){
			spinnerComponent = (HtmlInputHelperSpinner)component;
		}
		
		String delta = null;
		String buttonBorder = null;
		String buttonColor = null;

		if ( spinnerComponent != null ){
			delta = spinnerComponent.getDelta();
			buttonBorder = spinnerComponent.getButtonBorder();
			buttonColor = spinnerComponent.getButtonColor();
		}else{
			delta = (String) component.getAttributes().get("delta"); //$NON-NLS-1$
			buttonBorder = (String)component.getAttributes().get("buttonBorder");	
			buttonColor = (String)component.getAttributes().get("buttonColor");			
		}

		if (delta == null) {
			delta = "1"; //$NON-NLS-1$
		}
	
		// Check if the parent has a validation tag, and if yes - get validation attributes
	
		StringBuffer script = new StringBuffer(HxClientRenderUtil.HX_VAR_NAME); //$NON-NLS-1$
		script.append(".addComponent(\"");
		script.append(component.getParent().getClientId(context));
		script.append("\", new "); //$NON-NLS-1$
		script.append(HxClientRenderUtil.HX_VAR_NAME);
		script.append(".JSFSpinner(\"increment:");
		script.append(delta);
		script.append(('\"')); //$NON-NLS-1$ 
		if (converterId != null) {
			script.append(",\"converter:"); //$NON-NLS-1$ 
			script.append(converterId);
			script.append(('\"')); //$NON-NLS-1$
		}
		if (validatorId != null) {
			script.append(",\"validator:"); //$NON-NLS-1$ 
			script.append(validatorId);
			script.append(('\"')); //$NON-NLS-1$
		}
		if(buttonColor != null && buttonColor.trim().length() > 0) {	
			script.append(", \"button-color:"); //$NON-NLS-1$
			script.append(buttonColor);
			script.append("\""); //$NON-NLS-1$
		}
		if(buttonBorder != null && buttonBorder.trim().length() > 0) {	
			script.append(", \"button-border:"); //$NON-NLS-1$
			script.append(buttonBorder);
			script.append("\""); //$NON-NLS-1$
		}

		script.append("));"); //$NON-NLS-1$
		return script.toString();
	}
    
    /**
     * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     * @throws IOException
     */
    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {
        //defect 280926
    }
 
    
}
