/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.taglib.html_extended;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import javax.faces.context.FacesContext;
import javax.faces.convert.ConverterException;
import javax.faces.validator.Validator;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.ValidatorTag;
import javax.servlet.jsp.JspException;
import com.ibm.faces.validator.DateTimeRangeValidator;

/**
 * <p><strong>ValidateDateTimeRangeTag</strong> is a {@link MaxMinValidatorTag MaxMinValidatorTag} that 
 * is used to check the value of the corresponding component against specified minimum and
 * maximum date values.  
 * The tag is used to initialize the corresponding validator <code>DateTimeRangeValidator</code>.
 * The validator is created in the <code>createValidator</code> method.
 */
public class ValidateDateTimeRangeTag extends ValidatorTag {

	protected Boolean maxSet = Boolean.FALSE;
	protected Boolean minSet = Boolean.FALSE;
	
	protected String maximum = null;
	protected String minimum = null;	
	
	protected Date maxDate = null;
	protected Date minDate = null;	

	/**
	 * Date keyword "today".
	 */
	private static final String DATE_ATTR_TODAY = "today";	//$NON-NLS-1$

	/**
	 * Date keyword "tomorrow".
	 */
	private static final String DATE_ATTR_TOMORROW = "tomorrow";	//$NON-NLS-1$

	/**
	 * Date keyword "yesterday".
	 */
	private static final String DATE_ATTR_YESTERDAY = "yesterday";	//$NON-NLS-1$

	/**
	 * Time keyword "now".
	 */
	private static final String TIME_ATTR_NOW = "now";	//$NON-NLS-1$
	
	

	public ValidateDateTimeRangeTag() {
		super();
		super.setId("DateTimeRange"); //$NON-NLS-1$
		super.setValidatorId("DateTimeRange"); //$NON-NLS-1$
	}
	/**
	 * <p>@return String Gets the maximum date (latest date) to be enforced by this {@link Validator}.</p>
	 */
	public String getMaximum() {
		return maximum;
	}
	/**
	 * <p>Sets the maximum date (latest date) to be enforced by this {@link Validator}.
	 * @param String newMaximum. newMaximum is the latest date the validator will accept as a valid
	 * value.</p>
	 */

	public void setMaximum(String newMaximum) {
		maxSet = Boolean.TRUE;
		maximum = newMaximum;
	}

	/**
	 * <p>@return String Gets the minimum date (earliest date) to be enforced by this {@link Validator}.</p>
	 */

	public String getMinimum() {
		return minimum;
	}

	/**
	 * <p>Gets the minimum date (latestearliest date) to be enforced by this {@link Validator}.
	 * @param String newMinimum. newMinimum is the earliest date the validator will accept as a valid
	 * value.</p>
	 */

	public void setMinimum(String newMinimum) {
		minSet = Boolean.TRUE;
		this.minimum = newMinimum;
	}

	/**
	 * @return type
	 */

	/**
	 * @param string
	 */
	
	// 
	//	   Methods from ValidatorTag
	// 

	/**
	 * <p><strong>createValidator</strong> This method initializes the {@link Validator} object. It sets up
	 * the minimum, maximum and dateStyle attributes of the validator if they have been set on the tag.
	 * @return Validator - A new instance of the DateRageValidator, initialized with the attributes of the 
	 * current tag.</p>
	 */

	protected Validator createValidator() throws JspException {
		DateTimeRangeValidator result = null;

		result = (DateTimeRangeValidator) super.createValidator();
		
		evaluateExpressions();

		if (maxSet.booleanValue()) {
			result.setMaximum(maxDate);
		}

		if (minSet.booleanValue()) {
			result.setMinimum(minDate);
		}     
		return result;
	}
	
	private void evaluateExpressions() {
		FacesContext context = FacesContext.getCurrentInstance();
		if(minSet.booleanValue()) {
			if(UIComponentTag.isValueReference(minimum)) {
				minimum = changeKeywordsToLong(minimum);
				Object result = context.getApplication().createValueBinding(minimum).getValue(context);
				if(result instanceof Date) {
					Date dateObj = (Date) result;
					minimum = new Long(dateObj.getTime()).toString();	
					minDate = dateObj;		
				}
				else {
					Long longObj = (Long) result;
					minimum = new Long(longObj.longValue()).toString();	
					minDate = new Date(longObj.longValue());
				}
			} else { 
				minimum = changeDateToLong(minimum);
				minDate = new Date(Long.parseLong(minimum));
			}
		}		
		
		if(maxSet.booleanValue()) {
			if(UIComponentTag.isValueReference(maximum)) {
				maximum = changeKeywordsToLong(maximum);
				Object result = context.getApplication().createValueBinding(maximum).getValue(context);
				if(result instanceof Date) {
					Date dateObj = (Date) result;
					maximum = new Long(dateObj.getTime()).toString();	
					maxDate = dateObj;		
				}
				else {
					Long longObj = (Long) result;
					maximum = new Long(longObj.longValue()).toString();	
					maxDate = new Date(longObj.longValue());
				}
			} else {
				maximum = changeDateToLong(maximum);
				maxDate = new Date(Long.parseLong(maximum));
			}
		}		
	}
	
	/**
	 * @param date - <code>String</code>
	 * The string that was passed in as the maximum or minimum date attribute. This string can either be an actual 
	 * date or a predefined keyword:
	 * - today
	 * - tomorrow
	 * - yesterday
	 * - now
	 * @return 
	 * Returns a <code>String</code> that has been formatted as yyyy/mm/dd
	 */

	/**
	 * @param string
	 * @return
	 */
	private String changeDateToLong(String attr) {
		String dateFormat = "yyyy/MM/dd HH:mm:ss";  //$NON-NLS-1$
		//String dateFormat = "yyyy/MM/dd";		//$NON-NLS-1$
		Date date = null;		
		
		SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
		try{
			date = sdf.parse(attr);			
		}catch (ParseException d_t){	// date time conversion issue..			
			dateFormat = "yyyy/MM/dd";		//$NON-NLS-1$
			try{
				sdf.applyPattern(dateFormat);
				date = sdf.parse(attr);									
			}catch(ParseException d_o){ // date only
				dateFormat = "HH:mm:ss";
				try{
					sdf.applyPattern(dateFormat);
					date = sdf.parse(attr);									
				}catch(ParseException t_o){ // time only
					throw new ConverterException(d_t.getMessage() + "\n" 
													+ d_o.getMessage() + "\n" 
													+ t_o.getMessage());
				}												
			}
		}
		return (new Long(date.getTime()).toString());		
		
	}
	
	private String changeKeywordsToLong(String date) {
		String newDate = date;
		
		while(true) {
			String day = null;
			Date keyword = null;
			
			if(newDate.indexOf(TIME_ATTR_NOW) != -1) {				
				day = TIME_ATTR_NOW;
				keyword = new Date();
			} else if(newDate.indexOf(DATE_ATTR_TODAY) != -1) {
				day = DATE_ATTR_TODAY;
				keyword = new Date();				
			} else if(newDate.indexOf(DATE_ATTR_TOMORROW) != -1) {
				day = DATE_ATTR_TOMORROW;				
				keyword = new Date(System.currentTimeMillis() + 86400000);					
			} else if(newDate.indexOf(DATE_ATTR_YESTERDAY) != -1) {
				day = DATE_ATTR_YESTERDAY;
				keyword = new Date(System.currentTimeMillis() - 86400000);					
			} else {
				return newDate;
			}
			
			int start = 0, end = 0;		
			start = newDate.indexOf(day);
			end = start + day.length();
			String tmp = newDate.substring(0, start);
			tmp = tmp + " " + new Long(keyword.getTime()).toString();
			tmp = tmp + " " + newDate.substring(end, newDate.length());
			newDate = tmp;
		}
	}
}
