/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.util;

import java.lang.reflect.Array;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * This convenience class implements the MRU caching algorithm on top of a HashMap.
 * For the internal representation a chained list is implemented. A HashMap maps the keys 
 * to a CacheElement object. This object contains the key, the value and a link to a 
 * previous and a next element (a chained list is made). This list is used to realize 
 * the MRU-methods. This is much faster than to provide a seperate ListArray-Object 
 * because in the ListArray-Object you have to search every element with the (expensive) 
 * indexOf-method. When an entry needs to be removed it will the lru entry.
 */
public class CacheHashMap extends HashMap {

	private class LinkedValueNode {
		LinkedValueNode next, previous;
		Object key, value;
		LinkedValueNode(Object key, Object value) {
			this.key = key;
			this.value = value;
		}
		// so equals can be call by the backing HashMap for the user
		public boolean equals(Object obj) {
			return obj == null ? value == null : value.equals(obj);
		}

		public Object clone() {
			LinkedValueNode newNode = new LinkedValueNode(this.key, this.value);
			newNode.next = this.next;
			newNode.previous = this.previous; 
			return newNode;
		}

	}
	private LinkedValueNode mruNode = null;
	private int maxSize = 16;

	/**
	 * Create a default CacheHashMap (max size 16)
	 */
	public CacheHashMap() {
		super();
	}

	/**
	 * Create a CacheHashMap with the indicated max size
	 *  
	 * @param maxSize
	 */
	public CacheHashMap(int maxSize) {
		super();
		this.maxSize = maxSize;
	}

	/**
	 * Create a CacheHashMap with the indicated max size and initial capacity (see HashMap)
	 *  
	 * @param maxSize
	 * @param initialCapacity
	 */
	public CacheHashMap(int maxSize, int initialCapacity) {
		super(initialCapacity);
		this.maxSize = maxSize;
	}

	/**
	 * Create a CacheHashMap with the indicated max size, initial capacity and 
	 * load factor (see HashMap)
	 *  
	 * @param maxSize
	 * @param initialCapacity
	 * @param loadFactor
	 */
	public CacheHashMap(int maxSize, int initialCapacity, float loadFactor) {
		super(initialCapacity, loadFactor);
		this.maxSize = maxSize;
	}

	/**
	 * Retrieves the value and updates the MRU list.
	 * 
	 * @param key lookup key
	 * @see java.util.Map#get(java.lang.Object)
	 * @return Object The value to which this cache maps the specified key if it 
	 * exists. Null if no value exists.
	 */
	public Object get(Object key) {
		LinkedValueNode node = (LinkedValueNode) super.get(key);
		if (null != node) {
			// if we found a match make this node the most recently used
			removeNode(node);
			addNode(node);
			return node.value;
		}
		return null;
	}

	/**
	 * Checks the cache for an object that matches the key. If one exists, overwrites 
	 * it and moves the object to the front of the queue. If the cache is full, kicks 
	 * the last used object out of the cache and stores this one. If the cache is not 
	 * full, just adds this object.
	 * 
	 * @param key lookup key
	 * @see java.util.Map#put(java.lang.Object, java.lang.Object)
	 */
	public Object put(Object key, Object value) {
		remove(key); // to keep mru list up-to-date
		if (this.size() == maxSize) {
			remove(mruNode.previous.key);
		}
		LinkedValueNode prev = (LinkedValueNode)super.put(key, addNode(new LinkedValueNode(key, value)));
		return (null != prev) ? prev.value: null;
	}

	/**
	 * Makes calls to put for each element in t.
	 * 
	 * @param t map to copy into this one
	 * @see java.util.Map#putAll(java.util.Map)
	 */
	public void putAll(Map t) {
		if (null == t)
			return;
		for (Iterator i = t.entrySet().iterator(); i.hasNext();) {
			Map.Entry mapEntry = (Map.Entry) i.next();
			put(mapEntry.getKey(), mapEntry.getValue());
		}
	}

	/**
	 * Removes the entry and updates the MRU list.
	 * 
	 * @param key lookup key
	 * @see java.util.Map#remove(java.lang.Object)
	 * @return Object The value to which this cache maps the specified key if it 
	 * exists. Null if no value exists.
	 */
	public Object remove(Object key) {
		LinkedValueNode node = (LinkedValueNode) super.get(key);
		if (null != node) {
			removeNode(node);
			if (mruNode == node) {
				mruNode = node.next;
			}
			return node.value;
		}
		return null;
	}

	/**
	 * @see java.util.Map#containsValue(java.lang.Object)
	 * @return boolean True if the value is contained in the cache
	 */
	public boolean containsValue(Object value) {
		// get this thru base because LinkedValueNode implements equals
		return super.containsValue(value);
	}


	/**
	 * 
	 * @see java.util.Map#remove(java.lang.Object)
	 * @return Collection A collection of the values in cache and backed by this map.
	 */
	public Collection values() {
		return new CacheCollection(this);
	}

	/**
	 * Clear cache.
	 * @see java.util.Map#clear()
	 */
	public void clear() {
		mruNode = null;
		super.clear();
	}

	class CacheIterator implements Iterator {
		private CacheHashMap cache;
		private LinkedValueNode nextNode;
		private boolean hasNext;

		public CacheIterator(CacheHashMap cache) {
			this.cache = cache;
			this.nextNode = (LinkedValueNode)cache.mruNode.clone();
			hasNext = nextNode != null;
		}
		public boolean hasNext() {
			return hasNext;
		}

		public Object next() {
			Object o = nextNode.value;
			nextNode = nextNode.next;
			hasNext = nextNode != null && nextNode != mruNode;
			return o;
		}
		public void remove() {
			if (null != nextNode.previous){
				cache.remove(nextNode.previous.key);
			}
			throw new UnsupportedOperationException("remove is unsupported");
		}

	}

	class CacheCollection implements Collection {
		private CacheHashMap cache;
		/**
		 * 
		 */
		public CacheCollection(CacheHashMap cache) {
			this.cache = cache;
		}

		/**
		 * Unsupport operation
		 */
		public boolean add(Object o) {
			throw new UnsupportedOperationException("add is unsupported");
		}

		/**
		 * Unsupport operation
		 */
		public boolean addAll(Collection c) {
			throw new UnsupportedOperationException("addAll is unsupported");
		}

		/* (non-Javadoc)
		 * @see java.util.Collection#clear()
		 */
		public void clear() {
			cache.clear();
		}

		/* (non-Javadoc)
		 * @see java.util.Collection#contains(java.lang.Object)
		 */
		public boolean contains(Object o) {
			return cache.containsValue(o);
		}

		/* (non-Javadoc)
		 * @see java.util.Collection#containsAll(java.util.Collection)
		 */
		public boolean containsAll(Collection c) {
			if (null == c)
				return true;
			for (Iterator i = c.iterator(); i.hasNext();) {
				if (!cache.containsValue(i.next())) {
					return false;
				}
			}
			return true;
		}

		/* (non-Javadoc)
		 * @see java.util.Collection#isEmpty()
		 */
		public boolean isEmpty() {
			return cache.isEmpty();
		}

		/* (non-Javadoc)
		 * @see java.util.Collection#remove(java.lang.Object)
		 */
		public boolean remove(Object o) {
			if (null == cache.mruNode) {
				return false;
			}
			LinkedValueNode node = mruNode;
			do {
				if (node.equals(o)) {
					cache.remove(node.key);
					return true;
				}
				node = node.next;
			} while (node.next != mruNode && null != node);
			return false;
		}

		/* (non-Javadoc)
		 * @see java.util.Collection#removeAll(java.util.Collection)
		 */
		public boolean removeAll(Collection c) {
			if (null == c)
				return false;
			boolean collectionChanged = false;
			for (Iterator i = c.iterator(); i.hasNext();) {
				if (remove(i.next())) {
					collectionChanged = true;
				}
			}
			return false;
		}

		/* (non-Javadoc)
		 * @see java.util.Collection#retainAll(java.util.Collection)
		 */
		public boolean retainAll(Collection c) {
			throw new UnsupportedOperationException("retainAll is not supported");
		}

		/* (non-Javadoc)
		 * @see java.util.Collection#size()
		 */
		public int size() {
			return cache.size();
		}

		/* (non-Javadoc)
		 * @see java.util.Collection#toArray()
		 */
		public Object[] toArray() {
			Object[] a = new Object[cache.size()];
			int i = 0;
			for (Iterator iter = iterator(); iter.hasNext();) {
				a[i++] = iter.next();
			}
			return a;
		}

		/* (non-Javadoc)
		 * @see java.util.Collection#toArray(java.lang.Object[])
		 */
		public Object[] toArray(Object[] a) {
			int i = 0;
			Class matchClass = a.getClass();
			if (a.length < cache.size()) {
				a = (Object[]) Array.newInstance(matchClass, cache.size());
			}
			for (Iterator iter = iterator(); iter.hasNext();) {
				Object o = iter.next();
				if (null == o || a.getClass().equals(o.getClass())) {
					a[i++] = iter.next();
				}
			}
			if (i < a.length) {
				a[i] = null;
			}
			return a;
		}

		/* (non-Javadoc)
		 * @see java.util.Collection#iterator()
		 */
		public Iterator iterator() {
			return new CacheIterator(cache);
		}
	}

	private LinkedValueNode removeNode(LinkedValueNode node) {
		LinkedValueNode tempNode = node.next;
		if (null != tempNode) {
			tempNode.previous = node.previous;
		}
		tempNode = node.previous;
		if (null != tempNode) {
			tempNode.next = node.next;
		}
		return node;
	}
	private LinkedValueNode addNode(LinkedValueNode node) {
		node.next = mruNode;
		if (null != mruNode) {
			node.previous = mruNode.previous;
			mruNode.previous = node;
		} else {
			node.previous = null;
		}
		mruNode = node;
		return mruNode;
	}

	public static void main(java.lang.String[] Args) {
	}
}
