/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.util;

import java.io.IOException;
import java.io.Writer;

import javax.faces.context.FacesContext;

/**
 * @author Alasdair McDowell
 * Utilities methods for working with HTML.
 */
public class HtmlUtil {

    private static final String[][] ENTITIES = setEntities();
    private static final int S_CHAR = 0;
    private static final int S_ENITITY = 1;

    /**
     * Replaces reserved characters in a Java String with their HTML character entity values.
     * This is for attribute values and plaintext.
     * For example, &amp; goes to &amp;amp;
     * @param str
     * @return
     */
    public static String addHtmlEntities(String str) {
        if (str == null) {
            return str;
        }

        boolean found = false;
        for (int i = 0, len = ENTITIES.length; i < len; i++) {
            if (str.indexOf(ENTITIES[i][S_CHAR]) >= 0) {
                found = true;
                break;
            }
        }

        if (!found) {
            return str;
        }

        StringBuffer buffer = new StringBuffer(str);
        for (int i = 0, len = ENTITIES.length; i < len; i++) {
            int offset = 0;
            while (true) {
                offset = indexOf(buffer, ENTITIES[i][S_CHAR], offset);
                if (offset < 0) {
                    break;
                }

                buffer.replace(
                    offset,
                    (offset + ENTITIES[i][S_CHAR].length()),
                    ENTITIES[i][S_ENITITY]);
                offset += ENTITIES[i][S_ENITITY].length();
            }
        }

        return buffer.toString();
    }

    private synchronized static String[][] setEntities() {
        //when replacing Java->HTML, always do ampersands first
        //if going HTML->Java, always do ampersands last
        String[][] entities = { { "&", "&amp;" }, { //$NON-NLS-1$ //$NON-NLS-2$
                "<", "&lt;" }, { //$NON-NLS-1$ //$NON-NLS-2$
                ">", "&gt;" } //$NON-NLS-1$ //$NON-NLS-2$
        };
        return entities;
    }

    private static int indexOf(StringBuffer buffer, String find, int offset) {
        for (int i = offset, ilen = (buffer.length() - find.length() + 1);
            i < ilen;
            i++) {
            int lastChar = find.length() - 1;
            for (int j = 0, jlen = find.length(); j < jlen; j++) {
                if (buffer.charAt(i + j) != find.charAt(j)) {
                    continue;
                }
                if (j == lastChar) {
                    return i;
                }
            }
        }
        return -1;
    }

    /**
     * Writes a start tag.
     * If <i>close</i> &lt;<i>tag</i>&gt;.
     * If <i>end</i> &lt;<i>tag</i> /&gt;.
     * Else &lt;<i>tag</i>.
     * @param writer
     * @param tag
     * @param close
     * @param end
     * @throws IOException
     */
    public static final void writeTagStartOpen(
        Writer writer,
        String tag,
        boolean close,
        boolean end)
        throws IOException {
        writer.write("<"); //$NON-NLS-1$
        writer.write(tag);
        if (close) {
            writeTagStartClose(writer, end);
        }
    }

    public static final void writeTagStartOpenLn(
        Writer writer,
        String tag,
        boolean close,
        boolean end)
        throws IOException {
        writeTagStartOpen(writer, tag, close, end);
        writeLn(writer);
    }

    /**
     * Writes a tag attribute.
     * <i>name</i>="<i>value</i>"
     * @param writer
     * @param name
     * @param value
     * @throws IOException
     */
    public static final void writeTagAttribute(
        Writer writer,
        String name,
        String value)
        throws IOException {
        if (value == null) {
            return;
        }

        value = addHtmlEntities(value);

        writer.write(" "); //$NON-NLS-1$
        writer.write(name);
        writer.write("=\""); //$NON-NLS-1$
        writer.write(value);
        writer.write("\""); //$NON-NLS-1$
    }

    /**
     * Closes a start tag (after attributes).
     * If <i>end</i> writes /&gt; else writes &gt;
     * @param writer
     * @param end
     * @throws IOException
     */
    public static final void writeTagStartClose(Writer writer, boolean end)
        throws IOException {
        if (end) {
            writer.write(" />"); //$NON-NLS-1$
        }
        else {
            writer.write(">"); //$NON-NLS-1$
        }
    }

    public static final void writeTagStartCloseLn(Writer writer, boolean end)
        throws IOException {
        writeTagStartClose(writer, end);
        writeLn(writer);
    }

    /**
     * Writes tag end.
     * &lt;<i>tag</i>&gt;
     * @param writer
     * @param tag
     * @throws IOException
     */
    public static final void writeTagEnd(Writer writer, String tag)
        throws IOException {
        writer.write("</"); //$NON-NLS-1$
        writer.write(tag);
        writer.write(">"); //$NON-NLS-1$
    }

    public static final void writeTagEndLn(Writer writer, String tag)
        throws IOException {
        writeTagEnd(writer, tag);
        writeLn(writer);
    }

    /**
     * Writes hidden input field.
     * &lt;input type="hidden" id="<i>idName</i> name="<i>idName</i>" value="<i>value</i>" /&gt;
     * @param writer
     * @param idName
     * @param value
     * @throws IOException
     */
    public static final void writeInputHidden(
        Writer writer,
        String idName,
        String value)
        throws IOException {
        writeTagStartOpen(writer, "input", false, false); //$NON-NLS-1$
        writeTagAttribute(writer, "type", "hidden"); //$NON-NLS-1$ //$NON-NLS-2$
        writeTagAttribute(writer, "id", idName); //$NON-NLS-1$
        writeTagAttribute(writer, "name", idName); //$NON-NLS-1$
        writeTagAttribute(writer, "value", value); //$NON-NLS-1$
        writeTagStartCloseLn(writer, true);
    }

    /**
     * Writes a non-visible message that will be picked up by screen readers.
     * Accessibility method.
     * @param writer
     * @param label
     * @throws IOException
     */
    public static final void writeHiddenAudioLabel(Writer writer, String label)
        throws IOException {
        String image = JavaScriptUtil.getImagePath(FacesContext.getCurrentInstance()) + "1x1.gif"; //$NON-NLS-1$

        writeTagStartOpen(writer, "image", false, false); //$NON-NLS-1$
        writeTagAttribute(writer, "src", image); //$NON-NLS-1$
        writeTagAttribute(writer, "alt", label); //$NON-NLS-1$
        writeTagAttribute(writer, "style", "width: 0px; height: 0px; overflow: hidden; font-size: 0px; position: absolute"); //$NON-NLS-1$ //$NON-NLS-2$
        writeTagStartCloseLn(writer, true);
    }

    /**
     * Writes text with entities replaced.
     * @param writer    the output stream
     * @param label     the text
     * @throws IOException
     */
    public static final void writeText(Writer writer, String text)
        throws IOException {
        text = addHtmlEntities(text);
        writer.write(text);
    }

    /**
     * Writes an option tag (child attribute of the select tag).
     * @param writer    the output stream
     * @param label     the text label
     * @param value     the value
     * @throws IOException
     */
    public static final void writeOption(
        Writer writer,
        String label,
        String value)
        throws IOException {
        HtmlUtil.writeTagStartOpen(writer, "option", false, false); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "label", label); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "value", value); //$NON-NLS-1$
        HtmlUtil.writeTagStartClose(writer, false);
        HtmlUtil.writeText(writer, label);
        HtmlUtil.writeTagEndLn(writer, "option"); //$NON-NLS-1$
    }

    private static final void writeLn(Writer writer) throws IOException {
        writer.write("\r\n"); //$NON-NLS-1$
    }

}
