/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.util;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;
import java.util.SimpleTimeZone;
import java.util.TimeZone;

import javax.faces.FacesException;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;
import javax.servlet.http.Cookie;

import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;

/**
 * @author Martin Donnelly
 *
 * Utility methods for working with JavaScript.
 */
public class JavaScriptUtil{

	public static final String JS_NEWLINE = "\n";  //$NON-NLS-1$
	/**
	 * Standard variable name prefix to be used by all JavaScript
	 * variables we declare on a page.
	 */
	private static final String VAR_NAME_PREFIX = "hx_"; //$NON-NLS-1$
	
	/**
	 * This array contains the list of JavaScript keywords and
	 * words reserved for future use. When encoding JavaScript 
	 * code JavaScript variable names may need to be validated.
	 * 
	 */ 
    private static String javascriptReservedWords[] = {
		"abstract", //$NON-NLS-1$
		"boolean", //$NON-NLS-1$
		"break", //$NON-NLS-1$
		"byte", //$NON-NLS-1$
		"case", //$NON-NLS-1$
		"catch", //$NON-NLS-1$
		"char", //$NON-NLS-1$
		"class", //$NON-NLS-1$
		"const", //$NON-NLS-1$
		"continue", //$NON-NLS-1$
		"default", //$NON-NLS-1$
		"do", //$NON-NLS-1$
		"double", //$NON-NLS-1$
		"do",	 //$NON-NLS-1$
		"double", //$NON-NLS-1$
		"else", //$NON-NLS-1$
		"extends", //$NON-NLS-1$
		"false", //$NON-NLS-1$
		"final", //$NON-NLS-1$
		"finally", //$NON-NLS-1$
		"float", //$NON-NLS-1$
		"for", //$NON-NLS-1$
		"function", //$NON-NLS-1$
		"goto", //$NON-NLS-1$
		"if", //$NON-NLS-1$
		"implements", //$NON-NLS-1$
		"imports", //$NON-NLS-1$
		"in", //$NON-NLS-1$
		"instanceof", //$NON-NLS-1$
		"int", //$NON-NLS-1$
		"interface", //$NON-NLS-1$
		"long", //$NON-NLS-1$
		"native", //$NON-NLS-1$
		"new", //$NON-NLS-1$
		"null", //$NON-NLS-1$
		"package", //$NON-NLS-1$
		"private", //$NON-NLS-1$
		"protected", //$NON-NLS-1$
		"public", //$NON-NLS-1$
		"return", //$NON-NLS-1$
		"short", //$NON-NLS-1$
		"static", //$NON-NLS-1$
		"super", //$NON-NLS-1$
		"switch", //$NON-NLS-1$
		"super", //$NON-NLS-1$
		"switch", //$NON-NLS-1$
		"synchronized", //$NON-NLS-1$
		"this", //$NON-NLS-1$
		"throw", //$NON-NLS-1$
		"throws", //$NON-NLS-1$
		"transient", //$NON-NLS-1$
		"true", //$NON-NLS-1$
		"try", //$NON-NLS-1$
		"var", //$NON-NLS-1$
		"void", //$NON-NLS-1$
		"while", //$NON-NLS-1$
		"with" //$NON-NLS-1$
    };
    
    /**
     * Given a string, return true if it is a JavaScript keyword.
     * Otherwise false. Does a binary search.
     * @return boolean
     * 
     */
   	public static boolean isJavaScriptReservedWord(String sString) {
		int i; 
		boolean jsrw = false;
		int x = 0, y = javascriptReservedWords.length, z = 0;
		
		while (x <= y) {
			z = (x + y)/2;
			
			if ((i = sString.compareTo(javascriptReservedWords[z])) == 0) {
				jsrw = true;
				break;
			}
			else if (i > 0)
				x = z + 1 ;
			else
				y = z - 1;
		}
		
		return jsrw;
	}
	
	/**
	 * Given a string, construct a valid JavaScript variable name.
	 * When emitting JavaScript via a renderer this is useful in
	 * ensuring any JS variable declarations you make are valid.
	 * @return String
	 */
	public static String makeVariableName(String sString) {
		StringBuffer sBuffer = new StringBuffer(sString);
		int i = 0, j = 0;
		char ch;

		/**
		 * Verify input string is comprised of alphanumeric or underscore
		 * characters only. Replace anything else with underscores.
		 */	   		
		for (i = 0, j = sBuffer.length(); i < j; i++) {
				ch = sBuffer.charAt(i);
				if (!Character.isLetterOrDigit(ch))
					sBuffer.replace(i, i+1, "_");    			 //$NON-NLS-1$
		}

		return JavaScriptUtil.VAR_NAME_PREFIX + sBuffer.toString();   		
	}
	
	/**
	 * Return the standard JavaScript start tag for code snippet. 
	 * The 'language' attribute is actually deprecated and superceded with
	 * the 'type' attribute. By using both then all browsers are accounted
	 * for, since attributes that are not understood are ignored.
	 * 'src' and other attributes are not handled here.
	 * 
	 * @param src src attribute for the <code>script</code> tag. Can be null.
	 * @return String JavaScript start tag
	 */
	public static String getStartTag(String src) {
		StringBuffer buffer = new StringBuffer();
		buffer.append("<script type=\"text/javascript\""); //$NON-NLS-1$
		if(src != null) {
			buffer.append(" src=\""); //$NON-NLS-1$
			buffer.append(src);
			buffer.append("\""); //$NON-NLS-1$
		}
		buffer.append(">"); //$NON-NLS-1$

		return buffer.toString();
	}

	/**
	 * @see getStartTag(String src)
	 */
	public static String getStartTag() {
		return getStartTag(null);
	}
	
	/**
	 * standard JavaScript end tag
	 * @return String
	 */
	public static String getEndTag() {
		return "</script>"; //$NON-NLS-1$
	}
	
	/*
	 * This section has just two simple functions for validating 
	 * width/height (possibly others) element parameters. Useful 
	 * for controls which take a size parameter which can either
	 * be in absolute pixels or expressed as a percentage of screen
	 * real estate, e.g. "300" or "100%" for element WIDTH.
	 * 
	 */
	 
	/**
	 * return true if the input string is a number or percentage
	 * @return boolean
	 */
	public static boolean isValidWidth(String sWidth) {
    	boolean vw = true;
    	    	
    	if (sWidth == null || sWidth == "") //$NON-NLS-1$
    		vw = false;
    	else
		{
			int i, j = sWidth.length();	
			if (sWidth.endsWith("%")) //$NON-NLS-1$
				j--;
			
			for (i = 0; i < j; i++) {
				char ch = sWidth.charAt(i);
				if (!Character.isDigit(ch)) {
					vw = false;
					break;
				}	
			}
		}
		
		return vw;
    }
    
	/**
	 * return true if the input string is a number or percentage
	 * @return boolean
	 */
    public static boolean isValidHeight(String sHeight) {
    	return isValidWidth(sHeight);
    }
    
    /*
     * Next section deals with our standard resource paths,
     * i.e. the places where gif, html, css and possibly other
     * resource types are stored in a Seoul J2EE project.
     * There is a hidden subdir below the webcontent folder
     * called .ibmjsfres and this contains JavaScript file(s)
     * as well as the aforementioned resources organised via
     * sub-directories. 
     * 
     * NB Using these static functions means that the paths
     * need only be defined once in the backend.  The same
     * path definitions are also made on the client side in
     * the standard JavaScript library (seoullib.js currently).
     * 
     * If you change any of the following definitions you must
     * also make the equivalent changes in the client side!
     * 
     */
        
    //private static String JS_LIB_PATH = RESOURCE_ROOT_DIR;
	private static String IMG_SUB_DIR = "img/"; //$NON-NLS-1$
	private static String CSS_SUB_DIR = "css/"; //$NON-NLS-1$
	private static String DLG_SUB_DIR = "dlg/"; //$NON-NLS-1$
	private static String ODC_PRJ_DIR = "odc/"; //$NON-NLS-1$
	private static String ODC_RTE_DIR = "rte/"; //$NON-NLS-1$
	private static String ODC_RTE_IMG_DIR = "images/"; //$NON-NLS-1$
    
	/**
	 * The absolute path to the folder containing Seoul resources for
	 * the specified resource type
	 * @return String
	 */
	public static String getResourcePath(FacesContext context, String sSubDir) {
		if(context == null){
			throw new FacesException("Cannot resolve context path as faces context is null.");
		}
		StringBuffer str = new StringBuffer(context.getExternalContext().getRequestContextPath());
		str.append("/")  
		.append(LifecycleUtil.getJSResourceURLPattern(FacesContext.getCurrentInstance()))
		.append("/")  //$NON-NLS-1$ 
		.append(sSubDir);
		return str.toString();
	}
	
	/**
	 * The absolute path to the folder containing Seoul image resources
	 * @return String
	 */
	public static String getImagePath(FacesContext context) {
		return getResourcePath(context, IMG_SUB_DIR); 
	}
    
	/**
	 * @param context current FacesContext
	 * @param src image ibm built in image source path
	 * @return Return full image path
	 */
	public static String getImagePath(FacesContext context, String src) {
		StringBuffer sb = new StringBuffer(getResourcePath(context, IMG_SUB_DIR));
		sb.append(src);
		return sb.toString();
	}

	/**
	 * The absolute path to the folder containing Seoul the CSS files
	 * @return String
	 */    
	public static String getCSSPath(FacesContext context) {
		return getResourcePath(context, CSS_SUB_DIR); 
	}
    
	/**
	 * The absolute path to the folder containing the Seoul dialogs
	 * @return String
	 */ 
	public static String getDlgPath(FacesContext context) {
		return getResourcePath(context, DLG_SUB_DIR); 
	}


	/**
	 * The absolute path to the folder containing the Seoul JS lib(s)
	 * @return String
	 */
	public static String getJavaScriptPath(FacesContext context) {
		return LifecycleUtil.getJSResourceURLPattern(FacesContext.getCurrentInstance()) + "/";  //$NON-NLS-1$   
	}

	/**
	 * The absolute path to the root folder containing Odyssey resources
	 * @return String
	 */
	public static String getOdysseyPath(FacesContext context) {
		return getResourcePath(context, ODC_PRJ_DIR); 
	}
	
	/**
	 * The absolute path to the folder containing Odyssey RTE JSP resources
	 * @return String
	 */
	public static String getOdysseyEditorPath(FacesContext context) {
		return getResourcePath(context, ODC_PRJ_DIR + ODC_RTE_DIR); 
	}
	
	/**
	 * The absolute path to the folder containing Odyssey RTE gif resources
	 * @return String
	 */
	public static String getOdysseyEditorImagesPath(FacesContext context) {
		return getResourcePath(context, ODC_PRJ_DIR + ODC_RTE_DIR + ODC_RTE_IMG_DIR); 
	}

	/**
	 * @param keyname 
	 * @return keycode value for key
	 */
	public static String getKeyCode(String keyname) {

		keyname = keyname.trim();
		
		if (_keyTable == null) {
			_createKeyTable();
		}
		String code = (String)_keyTable.get(keyname.toLowerCase());
		if(code == null){
			//if code not resolved return original value
			return keyname;
		}
		return code;
	}
	
	private static Map _keyTable = null;
	synchronized private static void _createKeyTable () {
		_keyTable = new HashMap();

		_keyTable.put("tab", 		"9"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("enter", 		"13"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("space", 		"32"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f1",			"112"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f2",			"113"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f3",			"114"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f4",			"115"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f5",			"116"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f6",			"117"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f7",			"118"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f8",			"119"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f9",			"120"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f10",		"121"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f11",		"122"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f12",		"123"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("a",			"65"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("b",			"66"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("c",			"67"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("d",			"68"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("e",			"69"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f",			"70"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("g",			"71"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("h",			"72"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("i",			"73"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("j",			"74"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("k",			"75"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("l",			"76"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("m",			"77"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("n",			"78"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("o",			"79"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("p",			"80"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("q",			"81"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("r",			"82"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("s",			"83"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("t",			"84"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("u",			"85"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("v",			"86"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("w",			"87"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("x",			"88"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("y",			"89"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("z",			"90"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("arrow_up",	"38"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("arrow_down",	"40"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("arrow_left",	"37"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("arrow_right","39"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("page_up",	"33"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("page_down",	"34"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("home",		"36"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("end",		"35"); //$NON-NLS-1$ //$NON-NLS-2$
	}

	
	/**
	 * Converts an expression from EL to JavaScript.
	 * 1. Remove #{ and }
	 * 2. Replace EL operators with JS: eq (==), ne(!=), lt(<), gt(>), le(<=), ge(>=), and(&&), or(||), not(!), mod(%)
	 * 3. Replace 'value' keyword with @testValue
	 * 4. Replace 'value.property' with @testValue.property
	 * 5. Resolve any ValueBindings (such as requestScope.myVar) and replace with their String representation
	 * 6. Replace double quotes with single quotes
	 * 
	 * @param elExpression EL Expression
	 * @return JS Expression
	 */
	public static String convertEL2JavaScript(String elExpression) {
		StringBuffer jsExpression = new StringBuffer();
		
		// #1
		if(elExpression.startsWith("#{")) {
			elExpression = elExpression.substring(2);
		}
		if(elExpression.endsWith("}")) {
			elExpression = elExpression.substring(0, elExpression.length() -  1);
		}
		
		// #2
		elExpression = elExpression.replaceAll(" eq ", " == ");
		elExpression = elExpression.replaceAll(" ne ", " != ");
		elExpression = elExpression.replaceAll(" lt ", " < ");
		elExpression = elExpression.replaceAll(" gt ", " > ");
		elExpression = elExpression.replaceAll(" le ", " <= ");
		elExpression = elExpression.replaceAll(" ge ", " >= ");
		elExpression = elExpression.replaceAll(" and ", " && ");
		elExpression = elExpression.replaceAll(" or ", " || ");
		elExpression = elExpression.replaceAll(" not ", " ! ");
		elExpression = elExpression.replaceAll(" mod ", " % ");
		
		//String[] elements = elExpression.split("[\\s\\+\\-\\*\\/\\%\\(\\)\\&\\|\\!]");
		
		boolean bInDoubleQuotes = false;
		boolean bInSingleQuotes = false;
		boolean bInIdentifier = false;
		StringBuffer identifier = null;
		
		for (int i = 0; i < elExpression.length(); i++) {
			char ch = elExpression.charAt(i);

			if(bInDoubleQuotes && ch != '"') {
				jsExpression.append(ch);
				continue;
			}
			
			if(bInSingleQuotes && ch != '\'') {
				jsExpression.append(ch);
				continue;
			}
			
			if(bInIdentifier) {
				if(Character.isJavaIdentifierPart(ch) || ch == '.' || ch == '[' || ch == ']') {
					identifier.append(ch);
					continue;
				}
				// #3
				// #4
				bInIdentifier = false;
				jsExpression.append(evaluateIdentifier(identifier.toString()));
			}
			
			if(ch == '"') {
				bInDoubleQuotes = !bInDoubleQuotes;
				jsExpression.append(ch);
				continue;
			}
			
			if(ch == '\'') {
				bInSingleQuotes = !bInSingleQuotes;
				jsExpression.append(ch);
				continue;
			}

			if(Character.isJavaIdentifierStart(ch)) {
				bInIdentifier = true;
				identifier = new StringBuffer();
				identifier.append(ch);
				continue;
			}
			
			jsExpression.append(ch);
			
			if((ch == '\\') && (i != elExpression.length() - 1)) {
				jsExpression.append(elExpression.charAt(++i));
			}
		}

		if(bInIdentifier) {
			jsExpression.append(evaluateIdentifier(identifier.toString()));
		}
		
		String result = jsExpression.toString();
		
		// #6
		result = result.replaceAll("\\\"", "'");
		
		return result;
	}
	
	private static String evaluateIdentifier(String identifier) {
		if(identifier.equals("value") || identifier.startsWith("value.")) {
			String result = identifier.replaceFirst("value", "@testValue");
			return result;
		}
		// #5
		String vbExp = "#{" + identifier + "}";
		try {
			FacesContext context = FacesContext.getCurrentInstance();
			ValueBinding vb = context.getApplication().createValueBinding(vbExp);
			Object vbResult = vb.getValue(context);
			if(vbResult instanceof String) {
				return "'" + vbResult + "'";
			}
			return vbResult.toString();
		}
		catch (Exception e) {
			return identifier;
		}
	}

	// See if we already have a timezone cookie with the given ID
	public static TimeZone addTimeZoneCookieScript(String TimeZoneCookieName) {
		TimeZone tz = getClientTimeZone(TimeZoneCookieName);
		if (null == tz) {
			// OK, we don't have one, so we have to add it to the page
			// This assumes that the topmost component is the scriptcollector
			UIScriptCollector scriptCollector = UIScriptCollector
			.find((UIComponent) FacesContext.getCurrentInstance()
					.getViewRoot().getChildren().get(0));
			scriptCollector.addSimpleLibraryReference("hxclient_tz");
			String varName = FacesContext.getCurrentInstance()
			.getExternalContext().encodeNamespace(
					"__" + TimeZoneCookieName + "__");
			StringBuffer sb = new StringBuffer();
			sb.append("var ").append(varName).append(" = new ").append(
					HxClientRenderUtil.HX_VAR_NAME).append(".DSTCookie(\"")
					.append(TimeZoneCookieName).append("\");").append(varName)
					.append(".CreateCookie();");
			scriptCollector.addScriptOnce(sb.toString());
		}
		return tz;
	}
	
	// Process timezone for the cookie
	public static TimeZone getClientTimeZone(String TimeZoneCookieName) {
		Map requestCookieMap = FacesContext.getCurrentInstance()
		.getExternalContext().getRequestCookieMap();
		if (requestCookieMap.containsKey(TimeZoneCookieName)) {
			Cookie cookie = (Cookie) requestCookieMap.get(TimeZoneCookieName);
			return computeClientTimeZone(cookie);
		}
		return null;
	}
	
	/**
	 * Compute a TimeZone based on the specified cookie
	 */
	private static TimeZone computeClientTimeZone(Cookie cook) {
		SimpleTimeZone stz = null;
		try {
			boolean problem = false; // if cookie data is not correct, TZ
			// can't be computed....
			String valueCookie = cook.getValue();
			String[] tab = StringUtil.splitToArray(valueCookie, ' ');
			if (tab.length == 10 && !StringUtil.isEmpty(tab[0])
					&& !StringUtil.isEmpty(tab[1])
					&& !StringUtil.isEmpty(tab[2])
					&& !StringUtil.isEmpty(tab[3])
					&& !StringUtil.isEmpty(tab[4])
					&& !StringUtil.isEmpty(tab[5])
					&& !StringUtil.isEmpty(tab[6])
					&& !StringUtil.isEmpty(tab[7])
					&& !StringUtil.isEmpty(tab[8])
					&& !StringUtil.isEmpty(tab[9])) {
				int janOffset = Integer.parseInt(tab[0]);
				int julOffset = Integer.parseInt(tab[1]);
				
				// STEP 0 : in hemisphere North, winter offset is the right
				// offset (withoutDST)
				// contrary in hemisphere South
				int withoutDSTOffset = 0;
				if (janOffset > julOffset) { // North
					withoutDSTOffset = janOffset;
				} else if (janOffset < julOffset) { // South
					withoutDSTOffset = julOffset;
				} else {
					withoutDSTOffset = janOffset; // both are equal : no DST
				}
				
				// STEP 1: get the constant rawOffset (in ms) (note : JS method
				// returns offset opposite)
				int rawOffset = -withoutDSTOffset * 60000;
				
				// STEP 2 : compute the dstOffset
				int dstOffset = 0;
				if (janOffset != julOffset) {
					dstOffset = Math.abs(janOffset - julOffset) * 60000;
				}
				
				// STEP 3 : read start and end daylight saving dates from cookie
				int mMar = 0, dMar = 0, tMar = 0, mOct = 0, dOct = 0, tOct = 0;
				int mBegin = 0, dBegin = 0, tBegin = 0, mEnd = 0, dEnd = 0, tEnd = 0;
				
				mMar = Integer.parseInt(tab[2]);
				Integer.parseInt(tab[3]);
				Integer.parseInt(tab[4]);
				tMar = Integer.parseInt(tab[5]);
				
				mOct = Integer.parseInt(tab[6]);
				dOct = Integer.parseInt(tab[7]);
				Integer.parseInt(tab[8]);
				tOct = Integer.parseInt(tab[9]);
				
				// STEP 4 : which date is start, and which is end ?? (depends on
				// hemisphere)
				if (janOffset > julOffset) { // North
					mBegin = mMar;
					dBegin = dMar;
					tBegin = tMar;
					mEnd = mOct;
					dEnd = dOct;
					tEnd = tOct;
				} else if (janOffset < julOffset) {
					mBegin = mOct;
					dBegin = dOct;
					tBegin = tOct;
					mEnd = mMar;
					dEnd = dMar;
					tEnd = tMar;
				}
				// start DST time must be adjusted (because it is wall clock
				// time)
				tBegin = tBegin - dstOffset;
				// end DST time must be adjusted
				tEnd = tEnd + dstOffset;
				
				// STEP 5 : adjustement may cause problem (24h instead of next
				// day 0h)
				int millisPerDay = 24 * 60 * 60 * 1000;
				if (tBegin < 0 || tBegin >= millisPerDay) {
					// there is a prob with begin date
					GregorianCalendar cal = new GregorianCalendar(TimeZone
							.getTimeZone("GMT")); //$NON-NLS-1$
					cal.set(Calendar.MONTH, mBegin);
					cal.set(Calendar.DAY_OF_MONTH, dBegin);
					cal.set(Calendar.MILLISECOND, tBegin);
					mBegin = cal.get(Calendar.MONTH);
					dBegin = cal.get(Calendar.DAY_OF_MONTH);
					tBegin = cal.get(Calendar.MILLISECOND);
				}
				
				if (tEnd < 0 || tEnd >= millisPerDay) {
					// there is a prob with end date
					GregorianCalendar cal = new GregorianCalendar(TimeZone
							.getTimeZone("GMT")); //$NON-NLS-1$
					cal.set(Calendar.MONTH, mEnd);
					cal.set(Calendar.DAY_OF_MONTH, dEnd);
					cal.set(Calendar.MILLISECOND, tEnd);
					mEnd = cal.get(Calendar.MONTH);
					dEnd = cal.get(Calendar.DAY_OF_MONTH);
					tEnd = cal.get(Calendar.MILLISECOND);
				}
				
				if (problem == false) {
					if (dstOffset == 0) {
						stz = new SimpleTimeZone(rawOffset, ""); //$NON-NLS-1$
					} else {
						try {
							stz = new SimpleTimeZone(
									rawOffset,
									"", mBegin, dBegin, 0, tBegin, mEnd, dEnd, 0, tEnd, dstOffset); //$NON-NLS-1$
						} catch (IllegalArgumentException e) {
							// ignore tBegin and tEnd
							stz = new SimpleTimeZone(
									rawOffset,
									"", mBegin, dBegin, 0, 0, mEnd, dEnd, 0, 0, dstOffset); //$NON-NLS-1$
						}
					}
				}
			}
		} catch (Throwable ex) {
			// Ignore
		}
		
		// return the computed time zone
		return stz;
	}
	
	/*
	 * <p>Encodes chars in a string as unicode hex chars \uFFFF, if encodeAll parameter is false just chars > 127
	 * (outside normal ascii range) will be encoded.</p>
	 * 
	 * @param res String resource to encode
	 * @param encodeAll If true encode all characters in the string otherwise those outside the ascii range (>127) 
	 */
	public static String encodeScriptResourceString(String res, boolean encodeAll){
		
		if(res != null){
			StringBuffer buf = new StringBuffer();
			String hexVal = null;
			for(int x = 0; x < res.length(); x++){
				if(encodeAll || res.charAt(x) > 127){
					buf.append("\\u");
					hexVal = Integer.toHexString(res.charAt(x));
					
					//the unicode value should be 4 digits, add 0s if required
					int z = 4 - hexVal.length();
					if(z > 0){
						for(int i = 0; i < z; i++){
							buf.append("0");
						}	
					}
					buf.append(hexVal);
				} else {
					//add the char
					buf.append(res.charAt(x));
				}
			}
			return buf.toString();
		}
		return res;
	}
}
