/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.util;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;

/**
 * This class encapsulates the functionality exposed in javax.faces.utils.ResponseWriter
 * to allow components to aggregate their output in a stringbuffer. The idea is that the
 * standard methods used by renderers are replicated here to simplify the conversion of
 * components to use this instead of multiple writes to the ResposeWriter.
 * 
 * This Class currently deals with only the xml that should be generated for the tree and 
 * Graph. It does not read or create xml against DTD.
 * It does not actually write to a stream and requires that the content of buffer be
 * pushed into a real reponsewriter via a write call on that real response writer. 
 * and is based on the ResposeWriterBuffer.
 * 
 *  @author womahony
 *
 */
public class XMLResponseWriterBuffer {
	private StringBuffer sb;
	private boolean TagOpen;
	private boolean SimpleTag;
	
	public XMLResponseWriterBuffer() {
		sb = new StringBuffer();
	}
	
	public String toString() {
		return sb.toString();
	}
	
	public void write(String s) {
		sb.append(s);
	}
	
   /**
	* <p>Write the start of an element, up to and including the
    * element name.  Once this method has been called, clients can
    * call the <code>writeAttribute()</code> or
    * <code>writeURIAttribute()</code> methods to add attributes and
    * corresponding values.  The starting element will be closed
    * (that is, the trailing '>' character added)
    * on any subsequent call to <code>startElement()</code>,
    * <code>writeComment()</code>,
    * <code>writeText()</code>, <code>endElement()</code>,
    * <code>endDocument()</code>, <code>close()</code>,
    * <code>flush()</code>, or <code>write()</code>.</p>
    *
    * @param name Name of the element to be started
    * @param component The {@link UIComponent} (if any) to which
    *  this element corresponds
    *
    * @exception IOException if an input/output error occurs
    */
	public void startElement(String name, UIComponent component) throws IOException {
    	String outName = name.toLowerCase();
    	checkTagOpenState();

		// we could use some logic here to determine of hte tag was a simple tag or not
    	// propably requires reading a DTD or something.
    	//if (outName.equals("br") || outName.equals("hr") || outName.equals("input)"))
    	//	setSimpleTag(true);
    	//else
    		setSimpleTag(false);
    	sb.append("<" + name + " ");
		setTagOpen(true);
    }
   
	/**
     * <p>Write the end of an element, after closing any open element
     * created by a call to <code>startElement()</code>.  Elements must be
     * closed in the inverse order from which they were opened; it is an
     * error to do otherwise.</p>
     *
     * @param name Name of the element to be ended
     *
     * @exception IOException if an input/output error occurs
     */
	public void endElement(String name) throws IOException {
    	checkTagOpenState();
   		sb.append("</")
		  .append(name)
		  .append(">");
    }
 
    /**
     * <p>Write an attribute name and corresponding value, after converting
     * that text to a String (if necessary), and after performing any escaping
     * appropriate for the markup language being rendered.
     * This method may only be called after a call to
     * <code>startElement()</code>, and before the opened element has been
     * closed.</p>
     *
     * @param name Attribute name to be added
     * @param value Attribute value to be added
     * @param property Name of the property or attribute (if any) of the
     *  {@link UIComponent} associated with the containing element,
     *  to which this generated attribute corresponds
     *
     * @exception IOException if an input/output error occurs
     */	
    public void writeAttribute(String name, Object value, 
							   String property) throws IOException {
    	if (isTagOpen()) {
    		sb.append(name)
			  .append("=\"")
			  .append(value.toString())
			  .append("\" ");
    	}
    }

    /**
     * <p>Write a URI attribute name and corresponding value, after converting
     * that text to a String (if necessary), and after performing any encoding
     * appropriate to the markup language being rendered.
     * This method may only be called after a call to
     * <code>startElement()</code>, and before the opened element has been
     * closed.</p>
     *
     * @param name Attribute name to be added
     * @param value Attribute value to be added
     * @param property Name of the property or attribute (if any) of the
     *  {@link UIComponent} associated with the containing element,
     *  to which this generated attribute corresponds
     *
     * @exception IOException if an input/output error occurs
     */
    public void writeURIAttribute(String name, Object value, 
					              String property) throws IOException {
    	if (isTagOpen()) {
    		sb.append(name)
			  .append("=\"")
			  .append(FacesContext.getCurrentInstance().getExternalContext().encodeResourceURL(value.toString()))
			  .append("\" ");
    	}
    }


    /**
     * <p>Write a comment containing the specified text, after converting
     * that text to a String (if necessary), and after performing any escaping
     * appropriate for the markup language being rendered.  If there is
     * an open element that has been created by a call to
     * <code>startElement()</code>, that element will be closed first.</p>
     *
     * @param comment Text content of the comment
     *
     * @exception IOException if an input/output error occurs
    */
    public void writeComment(Object comment) throws IOException {
    	checkTagOpenState();
    	sb.append("<!-- ")
		  .append(comment.toString())
		  .append(" -->");
    }


    /**
     * <p>Write an object, after converting it to a String (if necessary),
     * and after performing any escaping appropriate for the markup language
     * being rendered.  If there is an open element that has been created
     * by a call to <code>startElement()</code>, that element will be closed
     * first.</p>
     *
     * @param text Text to be written
     * @param property Name of the property or attribute (if any) of the
     *  {@link UIComponent} associated with the containing element,
     *  to which this generated text corresponds
     * 
     * @exception IOException if an input/output error occurs
     */
    public void writeText(Object text, String property) throws IOException {
    	checkTagOpenState();
    	sb.append(text.toString());
    }


    /**
     * <p>Write text from a character array, after any performing any
     * escaping appropriate for the markup language being rendered.
     * If there is an open element that has been created by a call to
     * <code>startElement()</code>, that element will be closed first.</p>
     *
     * @param text Text to be written
     * @param off Starting offset (zero-relative)
     * @param len Number of characters to be written
     *
     * @exception IOException if an input/output error occurs
     */
    public void writeText(char text[], int off, int len) throws IOException {
    	checkTagOpenState();
    	sb.append(text, off, len);   	
    }
	/**
	 * @return Returns the simpleTag.
	 */
	private boolean isSimpleTag() {
		return SimpleTag;
	}
	/**
	 * @param simpleTag The simpleTag to set.
	 */
	private void setSimpleTag(boolean simpleTag) {
		SimpleTag = simpleTag;
	}
	/**
	 * @return Returns the tagOpen.
	 */
	private boolean isTagOpen() {
		return TagOpen;
	}
	/**
	 * @param tagOpen The tagOpen to set.
	 */
	private void setTagOpen(boolean tagOpen) {
		TagOpen = tagOpen;
	}
	
	public void checkTagOpenState() {
		if (isTagOpen()) {
			if (isSimpleTag())
				sb.append(" /");
			sb.append(">");
			setTagOpen(false);
		}
	}
	
	public void clearBuffer() {
		sb.delete(0,sb.length());		
	}
}
