/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.validator;

//import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.ResourceBundle;

import javax.faces.FacesException;
import javax.faces.application.FacesMessage;
import javax.faces.component.StateHolder;
import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.convert.DateTimeConverter;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.util.FacesMessageUtil;

import javax.faces.validator.Validator;


/**
 * <p><strong>DateTimeRangeValidator</strong> is a {@link Validator} that checks
 * the value of the corresponding component against specified minimum and
 * maximum date values.  The following algorithm is implemented:</p>
 * <ul>
 * <li>Call getValue() to retrieve the current value of the component.
 *     If it is <code>null</code>, exit immediately.</li>
 * <li>If the current component value is not a Date type, or
 *     a Long that is convertible to Date,
 *     add a TYPE_MESSAGE_ID message to the {@link FacesContext} for this
 *     request, and skip subsequent checks.</li>
 * <li>If a <code>maximum</code> property has been configured on this
 *     {@link Validator}, check the component value against
 *     this limit.  If the component value is greater than the
 *     specified minimum, add a MAXIMUM_MESSAGE_ID message to the
 *     {@link FacesContext} for this request.</li>
 * <li>If a <code>minimum</code> property has been configured on this
 *     {@link Validator}, check the component value against
 *     this limit.  If the component value is less than the
 *     specified minimum, add a MINIMUM_MESSAGE_ID message to the
 *     {@link FacesContext} for this request.</li>
 * </ul>
 */
public class DateTimeRangeValidator implements Validator, StateHolder {

	private static String _validatorMessage = "validatorMessage"; // JSF 1.2
	
	/**
	 * <p>The message identifier of the {@link Message} to be created if
	 * the maximum length check fails.  The message format string for this
	 * message may optionally include a <code>{0}</code> placeholder, which
	 * will be replaced by the configured maximum length.</p>
	 */
	public static final String MAXIMUM_MESSAGE_ID = "javax.faces.validator.LengthValidator.MAXIMUM"; //$NON-NLS-1$

	/**
	 * <p>The message identifier of the {@link Message} to be created if
	 * the minimum length check fails.  The message format string for this
	 * message may optionally include a <code>{0}</code> placeholder, which
	 * will be replaced by the configured minimum length.</p>
	 */
	public static final String MINIMUM_MESSAGE_ID = "javax.faces.validator.LengthValidator.MINIMUM"; //$NON-NLS-1$
	
	//	------------------------------------------------------------- Properties
	

	/** THESE ARE NOT USED
	 * <p>A patern used to set up the SimpleDateFormatter.</p>
	 
	private String _globalDateFormat = "yyyy/MM/dd HH:mm:ss";		//$NON-NLS-1$
	
	/**
	 * Date keyword "today".
	 
	private static final String DATE_ATTR_TODAY = "today";	//$NON-NLS-1$
	
	/**
	 * Date keyword "tomorrow".
	 
	private static final String DATE_ATTR_TOMORROW = "tomorrow";	//$NON-NLS-1$
	
	/**
	 * Date keyword "yesterday".
	 
	private static final String DATE_ATTR_YESTERDAY = "yesterday";	//$NON-NLS-1$
	
	/**
	 * Time keyword "now".
	 
	private static final String TIME_ATTR_NOW = "now";	//$NON-NLS-1$
	*/
	
	/**
	 * <p>The minimum date (earliest date) to be enforced by this {@link Validator}, if
	 * <code>minimumSet</code> is <code>true</code>.</p>
	 */
	private Date minimum = null;

	/**
	 * <p>The maximum date (latest date) to be enforced by this {@link Validator}, if
	 * <code>maximumSet</code> is <code>true</code>.</p>
	 */
	
	private Date maximum = null;	
	
	
	/**
	 * <p>Flag indicating whether a minimum date has been set.</p>
	 */
	
	private boolean minimumSet = false;
	
	/**
	 * <p>Flag indicating whether a maximum date has been set.</p>
	 */
	private boolean maximumSet = false;	
	
	

	// ----------------------------------------------------------- Constructors

	/**
	 * <p>Construct a {@link Validator} with no preconfigured limits.</p>
	 */
	public DateTimeRangeValidator() {
		super();
	}

	/**
	 * <p>Construct a {@link Validator} with the specified preconfigured upper
	 * limit.</p>
	 *
	 * @param maximum Maximum value to allow
	 */
	public DateTimeRangeValidator(Date maximum) {
		super();
		setMaximum(maximum);
	}		

	/**
	 * <p>Construct a {@link Validator} with the specified preconfigured
	 * limits.</p>
	 *
	 * @param maximum Maximum value allowed
	 * @param minimum Minimum value allowed
	 *
	 * @exception IllegalArgumentException if a specified maximum value is
	 *  less than a specified minimum value
	 */
	public DateTimeRangeValidator(Date maximum, Date minimum) {
		super();
		setMaximum(maximum);
		setMinimum(minimum);
	}	

	/**
	 * <p>Return the latest date to be enforced by this {@link
	 * Validator}, or <code>0</code> if the latest date has not been
	 * set.</p>
	 */
	public Date getMaximum() {
		return (this.isMaximumSet() ? this.maximum : null);
	}

	/**
	 * <p>Set the latest date (maximum date) to be enforced by this {@link Validator}.</p>
	 *
	 * @param maximum The new maximum value
	 *
	 * @exception IllegalArgumentException if a specified maximum value is
	 *  less than a specified minimum value
	 */
	public void setMaximum(Date newMaximum) {		
		this.maximum = newMaximum;
		this.maximumSet = true;		
	}	

	/**
	 * <p>Return a flag indicating whether a latest date has been set.</p>
	 */
	public boolean isMaximumSet() {
		return (this.maximumSet);
	}
	
	/**
	 * <p>Return the earliest date to be enforced by this {@link
	 * Validator}, or <code>0</code> if the earliest date has not been
	 * set.</p>
	 */
	public Date getMinimum() {		
		return (this.isMinimumSet() ? this.minimum : null);
	}

	/**
	 * <p>Set the earliest date to be enforced by this {@link Validator}.</p>
	 *
	 * @param minimum The new earliest date value
	 *
	 * @exception IllegalArgumentException if a specified maximum value is
	 *  less than a specified minimum value
	 */
	public void setMinimum(Date newMinimum) {
		this.minimum = newMinimum;
		this.minimumSet = true;	
	}

	/**
	 * <p>Return a flag indicating whether a minimum date limit has been set.</p>
	 */	
	public boolean isMinimumSet() {
		return (this.minimumSet);
	}	
		
	/*
	 * <p>Returns the type of validator this tag is.</p>
	 */
/*	public String getType() {
		return type;
	}
*/
	/*
	 * @param string
	 * Sets the type attribute of the validator.
	 */
/*
	public void setType(String string) {
		type = string;
		if(this.type.equalsIgnoreCase("datetime")){ //$NON-NLS-1$
			this.isDateTime = true;
		}else{
			this.isDateTime = false;
		}
		
		if(this.type.equalsIgnoreCase("time")){ //$NON-NLS-1$
			this.isTime = true;
		}else{
			this.isTime = false;
		}
	}
*/
	// --------------------------------------------------------- Public Methods
	
	/**
	 * <p>Perform the correctness checks implemented by this
	 * {@link Validator} against the specified {@link UIInput}.
	 * If any violations are found:</p>
	 * <ul>
	 * <li>Add zero or more {@link Message}s to the specified
	 *     {@link FacesContext}, specifying this {@link UIInput} as
	 *     associated with the message, describing the nature of the
	 *     violation(s) encountered.</li>
	 * <li>Set the <code>valid</code> property on the specified
	 *     {@link UIInput} to <code>false</code>.</li>
	 * </ul>
	 *
	 * @param context FacesContext for the request we are processing
	 * @param component UIInput we are checking for correctness
	 *
	 * @exception NullPointerException if <code>context</code>
	 *  or <code>component</code> is <code>null</code>
	 */
	
	public void validate(FacesContext context, UIComponent component, Object value) throws FacesException {
		if ((context == null) || (component == null)) {
			throw new NullPointerException();
		}
		
		Date convertedValue = null;        
	
		if(value != null){		
				if(value instanceof Date){
					convertedValue = (Date)value;
				}else if(value instanceof Long){
					convertedValue = new Date( ((Long) value).longValue() );					
				}else{
					throw new FacesException("Validation Error: Specified attribute cannot be converted to proper type."); //$NON-NLS-1$
				}			
			calculateTimeAndPrintErrors(convertedValue, context, (UIInput) component);				
		}
	
	}

	// ------------------------------------------------------ Protected Methods
	
	
	/**
	 * <p>Return the specified attribute value, converted to a
	 * <code>String</code>.</p>
	 *
	 * @param attributeValue The attribute value to be converted
	 */
	protected String stringValue(Object attributeValue) {

		if (attributeValue == null) {
			return (null);
		} else if (attributeValue instanceof String) {
			return ((String) attributeValue);
		} else {
			return (attributeValue.toString());
		}
	}


	protected void calculateTimeAndPrintErrors(Date convertedValue, FacesContext context, UIInput component){
		FacesMessage errMsg = null; 
		//NOTE: Date.after() works as follows:
		// x.after(y) => true if x is after y.
		//NOTE: Date.before() works as follows:
		// x.before(y) => true if x is before y.		
		Date minDate = null, maxDate = null;
		
		DateTimeConverter dtconverter = null;
		if(component.getConverter() instanceof DateTimeConverter){
			dtconverter = (DateTimeConverter)component.getConverter();
		}
		
		if(this.isMinimumSet()){
			if(dtconverter != null && dtconverter.getType().equalsIgnoreCase("time") && this.getMinimum().getTime() > 86400000){
				Calendar cal = Calendar.getInstance();
				cal.setTime(this.getMinimum());
				cal.set(1970,0,1);
				minDate = cal.getTime();
			} else {
				minDate = this.getMinimum();
			}
		}
		
		if(this.isMaximumSet()){
			if(dtconverter != null && dtconverter.getType().equalsIgnoreCase("time") && this.getMaximum().getTime() > 86400000){
				Calendar cal = Calendar.getInstance();
				cal.setTime(this.getMaximum());
				cal.set(1970,0,1);
				maxDate = cal.getTime();
			}
			maxDate = this.getMaximum();
		}
		
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss"); //$NON-NLS-1$

		if(minDate != null && convertedValue.before(minDate)){
			//ERROR convertedValue BEFORE MINIMUM				
			//Object[] tmpMin = {sdf.format(minDate)};
			ResourceBundle bundle = ResourceHandler.getBundle(context);
			//MessageFormat message = new MessageFormat(ResourceHandler.getFacesMessageString(bundle,"ValidationMessage.less_than_minimum"));
			//errMsg = new FacesMessage(message.format(tmpMin));
			//errMsg.setSeverity(FacesMessage.SEVERITY_ERROR);
			errMsg = getFacesMessage(bundle ,FacesMessage.SEVERITY_ERROR, "ValidationMessage.less_than_minimum", null, component, sdf.format(minDate));
			context.addMessage(component.getClientId(context), errMsg);	
			component.setValid(false);		
		}
		if(maxDate != null && convertedValue.after(maxDate)){
			//ERROR convertedValue AFTER MAXIMUM
			//Object[] tmpMax = {sdf.format(maxDate)};
			ResourceBundle bundle = ResourceHandler.getBundle(context);
			//MessageFormat message = new MessageFormat(ResourceHandler.getFacesMessageString(bundle,"ValidationMessage.more_than_maximum"));
			//errMsg = new FacesMessage(message.format(tmpMax));
			errMsg = getFacesMessage(bundle, null, "ValidationMessage.more_than_maximum", null, component, sdf.format(maxDate));
			context.addMessage(component.getClientId(context), errMsg);
			component.setValid(false);		
		}
	}
	
//	TODO: IBM Change.. Had to add this here as we cannot extend ValidatorBase, and instead have to implement
// 	the Validator interface!
	  
   

	public Object saveState(FacesContext context) {

		 Object values[] = new Object[4];
		 values[0] = maximum;
		 values[1] = maximumSet ? Boolean.TRUE : Boolean.FALSE;
		 values[2] = minimum;
		 values[3] = minimumSet ? Boolean.TRUE : Boolean.FALSE;
		 return (values);

	 }


	 public void restoreState(FacesContext context, Object state) {

		 Object values[] = (Object[]) state;
		 maximum = (Date)values[0];
		 maximumSet = ((Boolean) values[1]).booleanValue();
		 minimum = (Date) values[2];
		 minimumSet = ((Boolean) values[3]).booleanValue();

	 }

    /**
     * @see javax.faces.component.StateHolder#setTransient(boolean)
     * @param newTransientValue
     */
	private boolean transientValue = false;


	public boolean isTransient() {

		return (this.transientValue);

	}


	public void setTransient(boolean transientValue) {

		this.transientValue = transientValue;

	}

	//------ a method to get message -----------
	
	protected FacesMessage getFacesMessage(ResourceBundle rb, FacesMessage.Severity severity, String summaryKey, String detailKey, UIComponent component, Object value)
	{
		return FacesMessageUtil.getValidatorFacesMessage(rb, severity, summaryKey, detailKey, component, value);
	}
	
	/*protected FacesMessage getFacesMessage(FacesMessage.Severity severity, String summary, String detail, UIComponent component)
	{
		String validatormessage = (String)component.getAttributes().get(_validatorMessage);
		// this is to cover JSF 1.2, done this way for backward compatability
		if(validatormessage != null)
		{
			summary = detail = validatormessage; // if message is set on the input, then it is both the summary and detail messages
		}		
		
		FacesMessage mess = new FacesMessage(summary);
		if(detail != null)
			mess.setDetail(detail);
		if(severity != null)	
			mess.setSeverity(severity);
		return mess; 
	}*/
	
}
