/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2006 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

// ------------------------------------------------
// JSF Backchannel
//
// JSF Backchannels are used to retrieve data in the background, avoiding screen
// refreshes. The data retrieval can be synchronous, i.e. the page waits for the
// data to arrive, and then processes it, or it can be asynchronous, where the
// the page raises a data request to the server, and the page's data handler is
// called when the data arrive back. 
//
// ------------------------------------------------
// Public:  The backchannel object
// ------------------------------------------------

// @class HTTPBackchannel

// Data connection
// ==========================
// Specifying the name of the IFrame causes a hidden IFrame to be created. This
// is used to issue HTTP requests (either through forms via POST, or URLs via
// GET) to a server for data exchanges without requiring a page refresh. This is
// the best solution in situations where ActiveX is disabled, or when there's a
// very strong need to access and cache the request headers.
//
// Alternatively, the connection is established using an XmlHttpRequest object.
// This can be further enhanced to consume a web service using a SOAP request.


// ------------------------------------------------
// Public: Constructor

// The constructor's parameters are:
//  ID:         the ID of the JavaScript variable defined to hold an instance of
//              this class. This should be the same value as the name provided
//              for the on-page variable, e.g.:
//                     var ABC = new HTTPBackchannel("ABC", .....);
//
//  Callback:   The method to be called when the server request returns
//              The method must take two params - the XML data returned, and
//              the string version of these data.
//
//  Post:       Boolean to indicate that the request is to be sent as a POST
//              
//  Async:      Boolean to indicate that the request is asynchronous, which will
//              cause this to use an XmlHttpRequest object. The default, false,
//              indicates that this is to use a hidden IFrame.
//
//  iframename: The Name/Id of the iframe to use as the backchannel
//
//  onerror:    The method to use as a callback
//
//              example:
//                   var ABC = new HTTPBackchannel("ABC", AppAcknowledge, true, false);
//                   function AppAcknowledge(Data) {
//                      // Do whatever with Data.
//                   }
//
//

///// PUBLIC METHODS

// Object constructor
HxG_5.prototype.HTTPBackchannel = function(id, callback, post, async, iframename, onerror, tryHTTP, inPortal) {
   // The object's ID - Note that the XMLHttpRequest code doesn't use this
   this._Id          = id;
   
   // The user-supplied callback function
   this._AppCallback = (typeof callback == 'function')? callback : hX_5.getElementById(callback);

   // The user-supplied error function
   if (onerror) {
      this._Onerror = (typeof onerror == 'function')? onerror : hX_5.getElementById(onerror);
   } else {
      this._Onerror = this.defaultOnError;
   }
         
   // If this is to use POST. Default is GET
   this._Post = (post == true) ? true : false;
   
   // If this is to use asynchronous connection (XmlHttpRequest).
   // Default is to use synchronous in a hidden IFrame
   this._Async = (async == true) ? true : false;
   
   // The XmlHttprequest object for async connection
   this._XmlHttp = null;

   // Whether this should use XmlHttpRequest (true) or IFrame (false)
   this._UseXmlHttp = (tryHTTP!=null && tryHTTP) ? true : (iframename == null);
   this._retryXmlHTTP = (tryHTTP!=null && tryHTTP) ? true : false;

   // The iframe object we want to use for synchronous access
   this._Iframe  = null;
   this._IframeName  = iframename;

   // Whether it's OK to do cross-domain stuff
   this._AllowCrossDomain = false;

   // A username and password
   this._Username = this._Password = null;

   // Optional Soap settings
   this._SoapRequest = this._SoapAction = null;

   // Optional extra parameter and this pointer
   this._extraParam = this._this = null;

   // Optional: flag indicating we're running in portal
   this._portal = (inPortal!=null && inPortal)?true:false;

   // Gunk to manage using an iframe
   this._timer = this._contentCount = null;
}

// Some public accessors for the above
// Set the callback method
HxG_5.prototype.HTTPBackchannel.prototype.setCallback = function(callback) {
   this._AppCallback = callback;
}

// Set the post status
HxG_5.prototype.HTTPBackchannel.prototype.setPost = function(post) {
   this._Post = (post!=null && post) ? true : false;
}

// Set the callback method
HxG_5.prototype.HTTPBackchannel.prototype.setOnerror = function(onerror) {
   this._Onerror = onerror ? onerror : defaultOnError;
}

// Get the XmlHttpRequest object
HxG_5.prototype.HTTPBackchannel.prototype.getXmlHttpRequest = function() {
   return this._XmlHttp;
}

// Set the user & password
HxG_5.prototype.HTTPBackchannel.prototype.setUser = function(user, pwd) {
   this._Username = user;
   this._Password = pwd;
}

HxG_5.prototype.HTTPBackchannel.prototype.enableCrossDomain = function(allow) {
   this._AllowCrossDomain = (allow == true) ? true : false;
}

// Public: Reset the channel
HxG_5.prototype.HTTPBackchannel.prototype.reset = function() {
   if (this._UseXmlHttp && this._XmlHttp) this._XmlHttp.abort();
   if (this._timer) {
      hX_5.i.killTimerTimed(this._timer);
      this._timer = null;
   }
}

// Public: Destructor (not called automatically -- should be explicitly called when deleting a backchannel
// to reduce memory leaking)
HxG_5.prototype.HTTPBackchannel.prototype.destroy = function () {
	this.reset();
	if (this._Iframe) {
		this._Iframe.parentNode.removeChild(this.DOMParser);
		delete this._Iframe;
	}
	if (this._XmlHttp) delete this._XmlHttp;
	
}

// Public:  Advanced call to get data 
//			This call wrappers the basic "load" call and handles encoding parameters into the request
//			and/or determining which form values should be submitted
//
// 			Url		The url to make the request on. Parameters may already have encoded parameters onto it
//			thisP	The this pointer to use when making a callback, normally the this pointer of the
//					JS object making the call (e.g., a JSFBehavior)
//			ajaxid	The id of the DOMobj that the request is about (alternatively the DOMobj itself)
//			ajaxtyp	The type of call being made ("axpartial" or "axfull" or "axnone")
//			paramN	(optional, POST or GET) An array of parameter names
//			paramF	(optional, POST or GET) A matching array of field ids, the values of the fields are used as attributes
//			FisValue(optional, POST or GET) If true, paramF is not an array of field ids, it's an array of values to be used directoy
//					If paramN/F are provided, each field's value will be checked and if it's
//					OK (e.g., not disabled), the value of the field is encoded as a parameter in the request
//					URL along with the parameter name. 
//					If portal is in force, the parameters are encodede into the request differently, but
//					in a way that the server will handle as parameters
//			tgt		(optional) If parameters can be "table row based" (i.e., field repeats in a table column), this
//					is the event target that can identify the row containing the field
//			sName	(optional) If this is a "submit" ajax acall, if sName is included, then "sname='Submit" will be added
//					to the "submit package".  This will cause JSF to execute the action associated with this submit button
HxG_5.prototype.HTTPBackchannel.prototype.LoadEx = function(Url, thisP, ajaxid, ajaxtype, paramN, paramF, FisValue, tgt, sName) {
	var i,j,k,l,n,p,v,x, f=null, url=Url, obj=hX_5.getElementById(ajaxid), fl=[], fl1=[], fl2=[];

	// Compute the params that we are going to use to retrieve content
	if (paramN) {
		var hit,suff,pre,_AR="$$AJAXROW$$";
		for (i=0; i<paramN.length; i++) {
			if (FisValue!=null) {
				fl.push([paramN[i],paramF[i]]);
			} else {
				x = (paramF[i]) ? paramF[i] : paramN[i];
				// There are two types of parameter-name/field-name pairs
				// "Normal" ones are basically "form-submittable" values 
				// "Special" ones are based on the event object
				if (paramN[i].indexOf(_AR)>=0) {
					hit = false;
					n = hX_5.replaceAll(paramN[i], _AR, hX_5._E);
					x = hX_5.replaceAll(x, _AR, hX_5._E);
					while (tgt!=null && tgt.tagName!=undefined && tgt.tagName != hX_5._TR) tgt = tgt.parentNode;
					if (tgt!=null && tgt.tagName!=undefined && tgt.tagName==hX_5._TR && tgt.parentNode.tagName==hX_5._TBDY) {
						k = x.lastIndexOf(hX_5._CL);
						if (k>=0) {
							suff = x.substr(k+1);
							pre = x.substr(0,k-1);
							l=tgt.cells.length;
							for (k=0; !hit && k<l; k++) hit = this.checkCell(tgt.cells[k].firstChild, fl, pre, suff);
						}
					}
				} else if (hX_5.isValuable(x,true)) {
					v = hX_5.getValuableValue(x,true,true);
					if (v!=null) {
						if (hX_5.isString(v))
							fl.push([paramN[i],v]);
						else
							for (j=0; j<v.length; j++) fl.push([paramN[i],v[j]]);
					}
				}
			}
		}
	}

	// Add in the ajax params
	if (ajaxtype && ajaxtype!="axnone") {
		if (url.indexOf("$$ajaxmode")<0 && url.indexOf("%24%24ajaxmode")<0) {
			fl.push(["$$ajaxmode",ajaxtype]);
			fl.push(["$$ajaxid",obj.id]);
		}
		// Add in the uniquer (so that IE won't cache requests).
		// Note that posts aren't cached so it's only needed for requests
		if (!this._Post && !this._portal) {
			var dt = new Date();
			fl.push(["$$ajaxTS",dt.getTime()+hX_5._E]);
		}
	}

	// Encode the args and add them to the URL
	if (!this._portal) {
		// For portal, the URL is immutable.  We'll pass args differenting
		url = url + hX_5.encodeURIParams(url, fl);
		delete fl;
		fl=[];
	}

	// At this point, the url is encoded with the parameters EXCEPT for portal where FL contains
	// the names of parameters and the values -- we'll encode them as a submit

	// If this is a submit, add to the field list the fields to be submitted
	// Compute the form of obj	
	if (this._Post && obj) {
		// Compute the form
		if (obj.form!=undefined) f = obj.form;
		else {
			p = obj;
			while (p!=null && p.tagName!=undefined && p.tagName!=hX_5._FRM) p=p.parentNode;
			f = (p!=null && p.tagName!=undefined && p.tagName==hX_5._FRM)?p:null;
			if (f==null) {
				p = document.getElementsByTagName(hX_5._FRM);
				f = (p && p.length>0) ? p[0] : null;
			}
		}
		// Copy the form elements over to the field list
		if (f) {
			var str1="1", str2="Submit", hitSubmit=false;
			// Remove duplicates -- we will put them back when we call getValuableValue if duplication matters
			for (i=0; i<f.elements.length; i++) {
				if (i==(f.elements.length-1) || f.elements[i].name!=f.elements[i+1].name) {
					p = (fl2[f.elements[i].name]!=null);
					for (j=0; !p && j<fl.length; j++) if (fl[j][0].name==f.elements[i].name) p=true;
					if (!p) {
						v = hX_5.getValuableValue(f.elements[i], true, false);
						if (sName!=null && v==null && sName==f.elements[i].name) {
							v = str2;
							hitSubmit=true;
						}
						if (v!=null) {
							fl2[f.elements[i].name]=str1;
							if (hX_5.isString(v))
								fl1.push([f.elements[i].name,v]);
							else
								for (j=0; j<v.length; j++) fl1.push([f.elements[i].name,v[j]]);
						}
					}
				}
			}
			if (sName!=null && !hitSubmit) {
				// Didn't find the submit button in the form.  Could be due to bug where f.elements (form elements)
				// doesn't list <input type=image" name='foo'> in the list
				var imgs = f.getElementsByTagName(hX_5._INP);
				imgs = (imgs.length==undefined)?[imgs]:imgs;
				for (j=0; j<imgs.length; j++) {
					if (imgs[j].type && imgs[j].type.toLowerCase()==hX_5._lIMG && imgs[j].name && imgs[j].name==sName) {
						fl1.push([sName,str2]);
						break;
					}
				}
			}
			if (fl1.length>0) fl = fl.concat(fl1);
		}
	}
	this.Load(url, null, thisP, null, fl);
	delete fl;
	delete fl1;
	delete fl2;
}

// Private: helper for above
HxG_5.prototype.HTTPBackchannel.prototype.checkCell = function (firstChild, fl, pre, suff) {
	var hit=false, c=firstChild;
	while (!hit && c!=null) {
		if (c.id!=undefined && (c.id.indexOf(pre)==0) && (c.id.lastIndexOf(suff)==(c.id.length-suff.length)))  {
			fl.push([suff,c.value]);
			hit = true;
		} else {
			if (c.firstChild!=null) hit = this.checkCell(c.firstChild, fl, pre, suff);
			if (!hit) c = c.nextSibling;
		}
	}
	return hit;
}

// Public:  Basic call to get data 
// 			Url		The url to make the request on.  Usually has encoded parameters on the end
//			param	(optional) Extra paramater to pass to the callback procedure when the request returns
//			thisp	REQUIRED, the "this" pointer to the object containing the callbacks (error callback and success callback)
//				 	(If not provided, when the callback is called, the "this" pointer in the callback is "window"
//			fieldlist	(optional) Used only if this is a "post", an array of DOM objects whose values
//						shoud be encoded and sent to the server as part of the request
//						To submit all the values of the form, set fieldlist to <form>.elements where form
//						is the form object
//			fieldlistEx If called from loadEx, the fully formed field list
HxG_5.prototype.HTTPBackchannel.prototype.Load = function(Url, param, thisp, fieldlist, fieldlistEx) {
   this._extraParam = param;
   this._this = thisp;
   var retry = false, txt, reqString, i, l, v;

   if (this._UseXmlHttp) {
      // See if the thing is already active
      if (this._XmlHttp != null) {
         if (this._XmlHttp.readyState != 0 && this._XmlHttp.readyState != 4)
            this._XmlHttp.abort(); 
      } else {
         // Try using the native XMLHttpRequest object
	     if (window.XMLHttpRequest) {
	        try {
	           if (this._AllowCrossDomain && netscape && netscape.security.PrivilegeManager.enablePrivilege ) {
	              try {
	                 netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserRead");
	               } catch (e) { /* ignore the error, we'll fail below if we needed the privilege */ }
	            }
	            this._XmlHttp = new XMLHttpRequest();
	         } catch(e) {
// Fixme: we're going to retry, so don't flag error      
//				this._Onerror.call(((this._this!=null)?this._this:this), (e.message == undefined) ? e : e.message);
	            this._XmlHttp = null;
	         }
         // Otherwise, try IE's ActiveX version
         } else if (window.ActiveXObject) {
            try {
               this._XmlHttp = new ActiveXObject("Msxml2.XMLHTTP");
            } catch(e) {
               try {
                  this._XmlHttp = new ActiveXObject("Microsoft.XMLHTTP");
               } catch(e) {
                  this._XmlHttp = null;
               }
            }
         }
      }

      // Try loading the doc
      if (this._XmlHttp == null) retry = true;
      else {
         try {
            this._XmlHttp.open(((this._Post || this._portal)? "POST" : "GET"), Url, this._Async,
                               this._Username, this._Username ? this._Password : null);
            if (this._Async) {
               var localThis = this;
               this._XmlHttp.onreadystatechange =  function() { localThis.Callback(); };
            }
            if (this._SoapAction) {
               reqString = this._SoapRequest.join(hX_5._E); 
               this._XmlHttp.setRequestHeader("SOAPAction", this._SoapAction);
               this._XmlHttp.setRequestHeader("Content-Type", "text/xml; charset=utf-8");
               this._XmlHttp.send(reqString);
            } else {
               reqString = hX_5._E;
               // Build up the post request
               if (this._Post) {
                  // FIXME: should the following include a charset encloding?
                  this._XmlHttp.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
                  if (fieldlist!=null) {
                     l = fieldlist.length;
                  	 for (i=0; i<l; i++) {
                  	    if (fieldlist[i]!=null && fieldlist[i].name!=undefined && fieldlist[i].name.length>0) {
                  	       v = hX_5.getValuableValue(fieldlist[i], true, false);
                  	       if (v) reqString = reqString + ((reqString.length>0)?hX_5._AMP:hX_5._E) + hX_5.encode(fieldlist[i].name) + hX_5._EQ +  hX_5.encode(v);
                  	    }
                  	 }
                  } else if (fieldlistEx!=null) {
                     l = fieldlistEx.length;
                     for (i=0; i<l; i++) if (fieldlistEx[i]!=null) reqString = reqString + ((reqString.length>0)?hX_5._AMP:hX_5._E) + hX_5.encode(fieldlistEx[i][0]) + hX_5._EQ +  hX_5.encode(fieldlistEx[i][1]);
                  }
               // Build up a get request
               } else if (this._portal && fieldlistEx!=null) {
	              this._XmlHttp.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
                  l = fieldlistEx.length;
				  for (i=0; i<l; i++) if (fieldlistEx[i]!=null) reqString = reqString + ((reqString.length>0)?hX_5._AMP:hX_5._E) + hX_5.encode(fieldlistEx[i][0]) + hX_5._EQ +  hX_5.encode(fieldlistEx[i][1]);
               }
               // We'll try to force the no-caching on the call.  We don't want IE in particular to try to re-use a 
               // previously sent request (Mozilla doesn't seem to ever cache requests)
			   this._XmlHttp.setRequestHeader("Pragma","no-cache");
			   this._XmlHttp.setRequestHeader("Cache-control","no-cache");
               this._XmlHttp.send(reqString);
            }
            if (this._Async == false) {
               this.Callback();
            }
         } catch (e) {
			this._Onerror.call(((this._this!=null)?this._this:this), (e.message == undefined) ? e : e.message);
         }
      }
   }
	
   // If we were directed to use an iframe instead of xmlHTTP OR if failover is turned on
   // and we were unable to create an instance of xmlHTTP, then use an iframe
   if (((retry && this._retryXmlHTTP) || (!this._UseXmlHttp))) {
      if (this._Iframe==null && null!=this._IframeName) this._Iframe = hX_5.ifr.create(document, this._IframeName, true, false);
      if (this._Iframe) { if (this._Post) this.PostIframe(Url); else this.GetIframe(Url) };
   }
}

// Set the SOAP data to access Web Services through the XmlHttpRequest
// If the method is null, this resets the SOAP-ness of the XmlHttpRequest.
// Also set the soap params. These are sent in NV pairs, i.e. the args are
// alternating names and values.
HxG_5.prototype.HTTPBackchannel.prototype.setSoapInfo = function() {
   var foo = arguments;
   if (arguments.length < 1) {
      this._SoapRequest = null;
      this._SoapAction = null;
   } else {
      // Count number of args: must be even!
      var nArgs = arguments.length;
      if (nArgs % 2 > 0)
         nArgs--;

      // The first two are the action and the method
      this._SoapAction = arguments[0] + "/" + arguments[1];

      // Concatenate the args  pairs as:
      //      <param>value</param> = <arg[n]>arg[n+1]</arg[n]>
      var params = " xmlns=\"" + arguments[0] + "\">";

      // For security's sake: no markup in the params!
      for (var iX = 2; iX<nArgs; iX+=2) {
         params += hX_5._LT + arguments[iX] + hX_5._GT
                   + arguments[iX+1].toString().replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;")
                   + this._TAGEND + arguments[iX] + hX_5._GT;
      }

      // The call params are set in _SoapRequest[3]
      this._SoapRequest = [this._SOAPSTART, arguments[1], params, this._TAGEND, arguments[1], this._SOAPEND ];

      // SOAP requires a POST
      this.setPost(true);
   }
}

///// PRIVATE METHODS

// Submits a URL through the IFrame (POST)
HxG_5.prototype.HTTPBackchannel.prototype.PostIframe = function(Url) {
// FIXME: This case is not working yet
   var myFrameDoc = hX_5.ifr.getDoc(this._Iframe);
   var ifrmForm   = myFrameDoc.forms[0];
   if (null == ifrmForm) {
      var myForm = myFrameDoc.createElement("form");
      var myBodies = myFrameDoc.getElementsByTagName("body");
      ifrmForm = myBodies[0].appendChild(myForm);
   }

   if (myFrameDoc && ifrmForm) {

      // Remove any existing fields on the form
      // (if any) from the last call
      var kidcnt = ifrmForm.childNodes.length; 
      for (var iX = kidcnt-1; iX >= 0; iX--) {
          ifrmForm.removeChild(ifrmForm.childNodes[iX]);
      }

      // Extract the parameters from the URL
      var urlBase     = Url.split('?'); // Split into URL + params
      ifrmForm.name   = "f";
      ifrmForm.action = urlBase[0];     // The URL itself
      ifrmForm.target = this._Iframe.name;

      var fldNode = null;

      // If there are any params, create fields for them
      if (urlBase.length > 1) {
         var nvPair  = null;
         var urlParams = urlBase[1].split('&'); // Separate params
         // Process each param
         for (iX = 0; iX < urlParams.length; iX++) {
            // Split the Name-Value pair
            nvPair = urlParams[iX].split("=");

            // Create an input text field with the ID of the param name
            fldNode      = myFrameDoc.createElement(hX_5._INP);
            fldNode.type = "text";
            fldNode.name = nvPair[0];
            fldNode.id   = nvPair[0];
            if (nvPair.length > 1) {
               fldNode.value = nvPair[1];
            }
            ifrmForm.appendChild(fldNode);
         }
      }

      // Specify the callback method that has to be called
      fldNode       = myFrameDoc.createElement(hX_5._INP);
      fldNode.type  = "text";
      fldNode.name  = "callback";
      fldNode.value = this._Id+".Callback";
      ifrmForm.appendChild(fldNode);
      
      // Add a submit button
      var fldBtn   = myFrameDoc.createElement(hX_5._INP);
      fldBtn.type  = "submit";
      fldBtn.name  = "submit";
      fldBtn.id    = "submit";
      fldBtn.value = "input";
      ifrmForm.appendChild(fldBtn);

      // Set timer to ensure that the load doesn't just hang
      //this._Ack = 0;
      //setTimeout(this._Id+".CheckFrame(1);", this._TIMEOUT);

      // Submit the generated form, and load the return markup in the IFrame
      try {
        fldBtn.click();
        //myFrameDoc.forms(0).submit();//ifrmForm.submit();
      } catch (e) {
         alert("ERROR callback URL: "+ ((e.message == undefined) ? e : e.message));
      }
   }
}

// Loads a URL through the IFrame (GET)
HxG_5.prototype.HTTPBackchannel.prototype.GetIframe = function(Url) {
   if (this._timer) {
      hX_5.i.killTimerTimed(this._timer);
      this._timer = null;
   }
   try {
      this._Iframe.src = Url;
      this._contentCount = 0;
      this._timer = hX_5.i.startObjTimerTimed(this, 0, 25);
   } catch (e) {
      this._Onerror.call(((this._this!=null)?this._this:this), (e.message == undefined) ? e : e.message);
   }
}

// Wait until the iFrame is loaded
HxG_5.prototype.HTTPBackchannel.prototype.dispatchTimer = function(param) {
	var ack=false, x=0, v = parseInt(param,10)+1, doc = hX_5.ifr.getDoc(this._Iframe), textContent = null;
	this._timer = null;
	if (doc ) {
		// We need to test if the document in the iframe is done loading
		if (doc.readyState!=undefined) {
			// For IE, this is easy -- ask the document if it's done
			if (doc.readyState.toUpperCase()=="COMPLETE") {
				textContent = hX_5.getIHTML(doc.documentElement);		
				if (textContent.length > 0) ack = true;
			}
		} else {
			// Mozilla has no such test, so we'll do a crude approximation
			// If this is the first time we're testing, calcuate a checksum of the # of elements in the page
			if (this._contentCount == 0) {
				if (doc.documentElement!=undefined && doc.documentElement.childNodes!=undefined) x += doc.documentElement.childNodes.length;
				if (doc.body!=undefined && doc.body.childNodes!=undefined && doc.body.childNodes.length>0) {
					x += doc.body.childNodes.length;
					var q = doc.body.firstChild;
					while (q!=null) {
						if (q.childNodes!=undefined) x+=q.childNodes.length;
						q = q.nextSibling;
					}
				} else x = 0;
				this._contentCount = x;
			} else {
				// See if we match the checksum, if we do, assume the doc is all loaded
				if (doc.documentElement!=undefined && doc.documentElement.childNodes!=undefined) x += doc.documentElement.childNodes.length;
				if (doc.body!=undefined && doc.body.childNodes!=undefined && doc.body.childNodes.length>0) {
					x += doc.body.childNodes.length;
					var q = doc.body.firstChild;
					while (q!=null) {
						if (q.childNodes!=undefined) x+=q.childNodes.length;
						q = q.nextSibling;
					}
				} else x = 0;
				if (x==0 || x!=this._contentCount) {
					this._contentCount = x;
				} else {
					textContent = hX_5.getIHTML(doc.documentElement);		
					if (textContent.length > 0) ack = true;
				}
			}
		}
		
	}

	// If we have the iframe content, call the callback	
	if (ack) {
		this._AppCallback.call(((this._this!=null)?this._this:this), null, textContent, this._extraParam);

	// Otherwise, requeue the timer
	} else {
		if (v < 5)
			this._timer = hX_5.i.startObjTimerTimed(this, (v+hX_5._E), 100);
		else
			this._Onerror.call(((this._this!=null)?this._this:this), hX_5.INTERR_bc_timeout);
	}
}

// This function is the callback function. Its purpose is to handle
// success/failure conditions, and forward the response back to the application
// callback supplied.
HxG_5.prototype.HTTPBackchannel.prototype.Callback = function(Success, Data) {
   if (this && this._XmlHttp) {
      if (this._XmlHttp.readyState != 4 && this._XmlHttp.readyState != 0)
         return; // Not yet complete!
      // If XML object has finished retrieving the data
      if (this._XmlHttp.readyState == 4 && (this._XmlHttp.status == 200 || this._XmlHttp.status == 0)) {
         if (this._AllowCrossDomain && netscape && netscape.security.PrivilegeManager.enablePrivilege ) {
            try {
               netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserRead");
            } catch (e) {};
         }
         try {
            this._AppCallback.call(((this._this!=null)?this._this:this),this._XmlHttp.responseXML, this._XmlHttp.responseText, this._extraParam);
         } catch (e) {
			this._Onerror.call(((this._this!=null)?this._this:this), (e.message == undefined) ? e : e.message);
         }
      } else if (this._XmlHttp.status != 0)  {
         this._Onerror.call(((this._this!=null)?this._this:this), this._XmlHttp.statusText);
      } 
   }
}

// Default error handling
HxG_5.prototype.HTTPBackchannel.prototype.defaultOnError = function(msg) {
   window.status = msg;
}

// Private manifest constants
HxG_5.prototype.HTTPBackchannel.prototype._WINDOW  = "window.";
HxG_5.prototype.HTTPBackchannel.prototype._TAGEND  = "</";
HxG_5.prototype.HTTPBackchannel.prototype._SOAPSTART = "<soap:Envelope \
                     xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' \
                     xmlns:xsd='http://www.w3.org/2001/XMLSchema' \
                     xmlns:soap='http://schemas.xmlsoap.org/soap/envelope/'> \
                    <soap:Body><";
HxG_5.prototype.HTTPBackchannel.prototype._SOAPEND = "></soap:Body></soap:Envelope>";
