/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2007 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

//----------------------------------------------------
// DateTime Converter, Validator and Assist
//----------------------------------------------------
//
//  ----------------------------------------------------------------------------------------
//  DateTime Converter
//  ----------------------------------------------------------------------------------------
//
//  Methods (common to all converters)
//
//	constructor		Create a converter with a specified set of attributes
//	setAttribute	Change an attribute of the converter
//  getAttribute	Get an attribute of the converter
//	stringToValue	Convert a string to a JS Date object according to the converter's rules
//  valueToString	Convert a JS Date object to a string according to the converter's rules
//  constantToValue Convert a string in "constant" format to a JS Date Object
//  valueToConstant	Convert a JS Date object to a "constant" format of the form yyyyddmmhhssmmzzz
//	lastError		Return a (localized) string describing the reason the last conversion failed
//	getMaxLength	Return the maximum length of a value
//
//  Constructor attributes:
//
//	Description		Meaning
//  -----------		------------------------------------------------------------------------
//
//	format			Format string describing input/output of date.  See below for description.
//	strict			How strict to be when converting, 0 = liberal, 1 = strict, 2 = very strict
//					E.g., with liberal 12-30-2004 and 2004-30-12 will both be allowed, with
//					strict only one will be allowed (the one that matches the format)
//					0 (liberal):		all "literals" are ignored while parsing
//										"E" is ignored
//										misordered components will be rearranged
//										G, am/pm and timezone can be omitted
//					1 (strict):			all "literals" are ignored while parsing
//										"E" is ignored
//										G, am/pm and timezone can be omitted
//					2 (very strict):	Exact match is required
//	ICU4J			Treat pattern as containing additional ICU4J pattern characters.
//					If this is set, then an "A" for example is not treated as a literal (as it is
//				 	with a non-ICU4J pattern), instead it's a pattern character.
//  locale			If present, the locale identifier (e.g., "en") used to look up the strings used
//					for this converter instance.  Defaults to the page's (portlet's) localized strings.
//					Only needs to be set if this instance of the converter uses a different set of localized
//				 	strings from the ones used in the enclosing page/portlet.
//	digits			If present, the first character (in Unicode) of a run of 10 characters
//					that represent the digits 0 through 9.  stringToValue will parse these digit symbols
//	base-2digit-span	When parsing a two-digit year, this many years before the current date
//						are treated as "last century", e.g., if this year is 2003 and the span is
//						30 than years between "73" and "99" will be treated as "1973 through 1999"
//						Microsoft norm is "30", Java norm is "80".
//	epoch			Which calendar/epoch to use when interpreting years
//					""	= Gregorian Christian (AD/BC)
//					j	= Gregorian Japanese (Year of the Emperor)
//					b	= Gregorian Buddhist (Year of The Buddha = --/BE)
//					h 	= Hebrew Calendar
//					i	= Islamic (Hijiri) Calendar
//	
//	Format strings are based on "Java" format strings.  Supported format characters include:
//
//			GGGG	-	Epoch symbol as a full string, e.g., "Heisei" or "Anno Domini"
//			GGG		-	Epoch symbol as a 3 character string, e.g., "Hei"
//			GG		-   Epoch symbol as a 2 character string, e.g., "AD"
//			G		-	Epoch symbol as a 1 or 2 character string, e.g., "AD"
//						NOTE:  All calendars are "epoch" based, that is the year is relative
//						to a point in time (an epoch).  In the Chrisitan Gregorian calendar, there
//						are two epochs -- Before Christian Era, and Christian Era where
//						"0" (a non-existent date), marks the epoch.  In calendars such as the
//						Islamic and Buddhist calendars, the epoch is equally a religious event
//						= the year of the Hegira (622 AD) for the Islamic calendar and the birth
//						of The Buddha = 543 BC) for the Buddhist calendar.  In both of these calendars,
//						there is no concept of a date "before" the epoch starts (there is no real
//						"Before Hegira or "Before Buddha" in either calendar).
//							AD		Anno Domini		(in the year of our Lord)
//							CE		Christian Era	(AD implies you are a believer)
//							AH		Anno Hegirae	(the year in which Muhammad emigrated from Mecca to Medina)
//							BE		Buddhist Era	(the year of The Buddha's birth)
//							BC		Before Christ	(note that this abbrev is indeed an English abbreviation)
//							BCE		Before Christian Era (goes with CE)
//						Most other epoch systems are defined by the years of a dynasty or
//						a king/emperor, for example the third year of the reign of King Fred.
//						In modern epoch-based calendars, the name of the political dynast(y) is
//						encluded in the year, e.g., "Meiji 15".  In these systems, there are potentially
//						many epochs (e.g., one for each king). Epoch based systems can be apolitical such as the Sothic calendar which
//						uses eras based on when the star Sirius appears at a certain point in
//						the horizon (approximately every 2000 years).
//
//			y			Year as a string, For non-Japanese, same as yy; For Japanese, no leading zeroes (e.g., used in "Heisei 2")
//			yy			Year as a string, For non-Japanese, last two digits of year (with leading zero), treated as current century (modulo 
//						for Japanese, two digit year with leading zero
//			yyy			Year as a string, For non-Japanese, same as yy; For Japanese, year with up to 2 leading zeroes (very rare, probably would only happen in something like the Mayan calendar)
//			yyyy		Year as a string, up to 3 leading zeroes
//			Y			Anywhere y can be used a capital Y means "adjust for use with "w"), that is same as y except at beginning/end of year
//			MMMM	-	Month as a full string, eg January
//			MMM		-	Month as an abreviated string, eg Jan
//			MM		-	Month as a zero lead number, eg 01, 12
//			M		-	Month as a non-zero lead number. eg 1, 12
//
//			EEEE	-	Day of Week as full string, eg Wednesday
//			EEE		-	Day of Week as abbreviated string, eg Wed
//			EE,E	-  	Treated as EEE
//						NOTE: E is ignored on input
//			e		-	Day of weeek as a number, 1-based, reflecting locale's first day of week,
//						e.g., if Monday is the first day of week, then a Tuesday is "2"
//
//			dd		-	Day of Month as zero lead number, eg 01, 12, 31
//			d		-	Day of Month as non-zero lead number, eg 1, 2, 12, 31
//			DDD		-	Day in year, e.g., 189 with 2 lead zeroes
//			DD		-	Day in year, e.g., 189 with 1 lead zero
//			D		-	Day in year, e.g., 189 with no lead zero
//			F		-	Day of week in month, e.g., 2 (meaning second Wednesday in July), with no lead zero
//			g		-  	Julian Date (Julian Dates are day numbers relative to January 1, 4713 BCE)
//
//			ww		-   week in year, e.g., 51 with leading zero
//			w		-	week in year, e.g., 51 no leading zeroes
//			W		-	week in month, e.g., 2
//						NOTE: w and W are ignored on INPUT (but ignoring them can cause a mis-parse of the date)
//						NOTE:  "w" and "W" are output according to Java's definition of "week"
//						This meaning is not the "common use" meaning of "w" and "W" in most businesses
//						That is, in Java, week 1 of the year ends on the first Saturday of the year (so may be
//						less than 7 days in length)
//
//			hh  	-	Hour in 12 hour clock with leading zero, eg 18 is 06, 0 is 12
//			h  		-	Hour in 12 hour clock without leading zero, eg 18 is 6, 0 is 12
//			HH		-	Hour in 24 hour clock with leading zero, eg 18 is 18, 6 is 06, 0 is 00
//			H		-	Hour in 24 hour clock without leading zero, eg 18 is 18, 6 is 6, 0 is 0
//			k       -	Hour in day (1~24), e.g., 24 	(differs from H in that it is 1-based)
//			kk      -	As k but always shows leading zero
//			K       -	Hour in am/pm (0~11), e.g., 0	(differs from h in that it is zero-based) 
//			KK      -	As K but always shows leading zero
//
//			mm		-	Minutes with leading zeroes, eg 08
//			m		-   Minutes without leading zeroes, eg 8
//
//			ss		-	Seconds with leading zero, eg 08
//			s		-	Seconds without leading zeroes, e.g., 8
//			SSS     - 	Milliseconds (all three digits shown)
//			SS		-	Milliseconds (first two digits shown)
//			S		-	Milliseconds (first digit shown)
//						N.B. on input, Java ignores input SSS and always set them to zero
//			AAAAAAAA-   Milliseconds in day (padded to up to 8 digits)
//			...
//			A		-   Milliseconds in day (no padding)
//			a		-   AM/PM marker
//
//			z		- 	Time zone (that is, the time zone of the local machine) as text
//			Z		- 	Time zone (that is, the time zone of the local machine) as RFC 822 format
//
//			'       -	escape for text (delimiter)
//			''      -	single quote (Literal)
//
//						NOTE:  Any character encountered in the format string that is NOT
//						one of the above format characters will be interpretted as "text"
//						that is "ignored" on input and "displayed" on output.  You need
//						to use single quotes around any text that might conflict with
//						a format string character.
//			
//						For example,	"hh o'' clock" will not return "12 o'clock", rather, it
//										will return "12 12th' cl12thck" as "o" is a format character
//
//	Java formatting characters that are currently not supported as the cannot be internationalized
//  correctly:
//
//			o		-	Ordinal of day of month, eg st, nd, rd, th
// 			zzzz	-	"Long form" of the time zone
//			ZZZZ	-   "Long form" of the time zone
//			v		-   No Daylight savings form of the time zone

HxG_5.prototype.DateTimeConverter = function () {
	this.type = hX_5.CVT_datetime;

	this.ICU4J	= false;				// If true, ICU4J parsing is used
	this.lc = hX_5.currentLocale();		// The locale for the values of the converter (default)
	this.plc = this.lc;					// Page level locale for this converter (for error messages, isn't affected by locale attribute)
	this.locale = null;					// The locale provided as an argument for this converter
	this.nStrict = hX_5.DTFMT_DEFAULTSTRICT;	// Strictness of parsing
	this.sFormat = hX_5.DTFMT_DATE0;	// Format pattern (supplied by caller), assumed to be localized
	this.sFormatTokenized = null;		// Format pattern tokenized (e.g., yyyy is now %yyyy, literals marked by ^n
										// and removed to following array
	this.nLiteralCnt=this.nTokenCnt=0;	// Number of literals in tokenized format and Number of tokens in the tokenized format
	this.sLiterals = [];				// Literals removed from tokenized format, array[n] = literal marked by ^n
	this.sTokens = [];					// Ordered array of tokens
	this.bHasDate=this.bHasTime=false;	// Is there a date and/or time component?
	this.FDOW = hX_5.DTFMT_DEFAULTFIRSTDAYWEEK;// First day of week
	// If present, start of non-western diit set, the format characters
	this.digitFirst = this.FMTChars = this.FMTDateChars = this.FMTTimeChars = null;
	this.FMTBaseChars = "aDdEFGHhKkMmSsWwyz";	// JAVA 1.4x Characters interpreted as "format" characters
	this.FMTBaseDateChars = "DdEFGMWwy";	// JAVA 1.4x Characters interpreted as "date format" characters
	this.FMTBaseTimeChars = "aHhKkmSsz";	// JAVA 1.4x Characters interpreted as "time format" characters
	this.FMTICU4JChars = "gAeYZ";			// ICU4J: Additional characters interpreted as "format" characters
	this.FMTICU4JDateChars = "geY";			// ICU4J: Additional characters interpreted as "date format" characters
	this.FMTICU4JTimeChars = "AZ";			// ICU4J: Additional characters interpreted as "time format" characters
	this._8chars = "gA";					// Format characters than can repeat up to 8 times
	this._4chars = "gAyYGME";				// Format characters that can repeat up to 4 times
	this._3chars = "gAyYGMEDS";				// Format characters that can repeat up to 3 times
	this._2chars = "gAyYGMEDSdhHmskKw";		// Format characters that can repeat up to 2 times
	this.FMTSEPChars  = "-_ .,;:|()<>/\\";	// Characters interpreted as "separators"
	this.SEPChars	  = this.FMTSEPChars;
	this.bIsGregorian = true;			// Is this a gregorian calendar? If has epoch, is it Gregorian?
	// Is it Islamic, Is it Hebrew?, Is it Christian Gregorian?, Is it Buddhist Gregorian?	s it Year of Emperor Gregorian?
	this.bIsHijri = this.bIsHebrew = this.bIsChristianEra = this.bIsBuddhistEra = this.bIsJapaneseEra = false;				
	this.sEpoch = hX_5._E;				// Value set in converter constructor
	this.nEpoch = null;					// If epoch, offset into epoch system strings
	// This calendar's epochs as labels, This calendar's epochs as full years, This calendar's epochs as dates
	this.descLbEpoch = this.descYrEpoch = this.descDtEpoch = null;
	this.bShowEpoch = false;			// If true, show epoch either always for this kind of calendar or if G is in pattern
	// Use "short form years" with this epoch, Max length of an abbreviated epoch name, Max length of an full epoch name
	this.isShortYr = this.abbEpochLength = this.fulEpochLength = null;
	this.minEpoch = 1;					// Min # of characters to make a unique epoch
	this.TZSeparator = ":";				// Currently hardcoded, separator character in a GMT style timezeone (e.g., GMT -05:00)
	this.sAM = hX_5.L[this.lc].STR_AM_SUFFIX; // String (case specific) for "AM"
	this.sPM = hX_5.L[this.lc].STR_PM_SUFFIX;// String (case specific) for "PM"
	this.base2DigitSpan = 80;			// See above for description -- used when converting 2 digit years
										// to "full 4 digit" years
	this.DF_MaxLength = null;			// Maximum number of characters allowed in value
	this.xDigits = this.zeroDigit = null;// Digit handling

	// Error handling -- error message associated with last failure
	this.msgFail = this.intMsgFail = hX_5._E;	// Debug messages can be put here

	// Assist handling
	this.maskFormat = this.maskFormatDigits = this.maskFormatLast = this.maskFormatFirst = this.simpleFormat = this.simpleConverter = null;	
	this.isSimple=false;
	this.markers = hX_5._LB + hX_5._QM;
	this.promptChar = hX_5._E;

	// String literals used here
	this._l = "#"; this._ll = "##"; this._lll = "###"; this._llll = "####";
	this._MIN = "-";
	this._PLS = "+";
	this._AD = null;
	hX_5.c.strInit(this.lc);
	
    // Attributes supported on this object
    this.attributeTable = [];
	 	this.attributeTable ['strict']				= ['nStrict',		hX_5.ARG_TO_NUM,	false,	0,	2];
	 	this.attributeTable ['format']				= ['sFormat',		hX_5.ARG_TO_NAS,	false,	null,	null];
	 	this.attributeTable ['base-2digit-span']	= ['base2DigitSpan',hX_5.ARG_TO_NUM,	false,	0,	99];
	 	this.attributeTable ['epoch']				= ['sEpoch',		hX_5.ARG_TO_NAN,	false,	null,	null];
    	this.attributeTable ['icu4j']				= ['ICU4J', 		hX_5.ARG_IS_TRUE,	false,	null,	null];
    	this.attributeTable ['locale']				= ['locale', 		hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['first-day-of-week']	= ['FDOW',			hX_5.ARG_TO_NUM,	false,	0,		6];
		this.attributeTable ['digits']				= ['digitFirst',	hX_5.ARG_TO_NUM,	false,	0,	999999];
		// Following two are obsolete and are only here for backwards compatibility.  Info is now taken from string file
	 	this.attributeTable ['am']					= ['sAM',			hX_5.ARG_TO_NAN,	false,	null,	null];
	 	this.attributeTable ['pm']					= ['sPM',			hX_5.ARG_TO_NAN,	false,	null,	null];
	this.setAttribute(arguments);
}

// Public:  Set an "attribute" on the Converter.  Attributes are passed as strings of the form "name:value"
HxG_5.prototype.DateTimeConverter.prototype.setAttribute = function(a) {
	var i,j,k; 
	hX_5.parseArg (this.attributeTable, this, a);

	// Reset anything that might be affected by the locale changing
	if (this.locale && this.locale != this.lc) {
		this.lc = this.locale;
		hX_5.c.strInit(this.lc);
		this.sAM = hX_5.L[this.lc].STR_AM_SUFFIX; // String (case specific) for "AM"
		this.sPM = hX_5.L[this.lc].STR_PM_SUFFIX; // String (case specific) for "PM"
	}

	// Set up the allowable pattern characters
	this.FMTChars = this.FMTBaseChars + ((this.ICU4J)?this.FMTICU4JChars:hX_5._E);
	this.FMTDateChars = this.FMTBaseDateChars + ((this.ICU4J)?this.FMTICU4JDateChars:hX_5._E);
	this.FMTTimeChars = this.FMTBaseTimeChars + ((this.ICU4J)?this.FMTICU4JTimeChars:hX_5._E);

	// Always rebuild the information related to the format
	this.parseDigits();
	this.parseFormatString();

	// Set up epoch/calendar handling
	this.sEpoch = (this.sEpoch==null)?hX_5._E:this.sEpoch; this.nEpoch = 0;
	for (i=0; i < hX_5.DATE_EPOCH_IDS.length;  i++) {
		if (hX_5.DATE_EPOCH_IDS[i] == this.sEpoch) {
			this.nEpoch = i;
			break;
		}
	}
	this.bIsGregorian = this.bIsHijiri = this.bIsHebrew = this.bIsChristianEra = this.bIsBuddhistEra = this.bIsJapaneseEra = false;
	this.bIsChristianEra = (this.nEpoch <= 0);
	this.bIsHijiri = (this.nEpoch == 1);
	this.bIsHebrew = (this.nEpoch == 2);
	this.bIsBuddhistEra = (this.nEpoch == 3);
	this.bIsJapaneseEra = (this.nEpoch == 4);
	this.bIsGregorian = (this.nEpoch == 0 || this.nEpoch >= 3);
	this.bShowEpoch = (this.bIsJapaneseEra || this.sFormatTokenized.indexOf("%G")>=0);
	this.descLbEpoch = hX_5.L[this.lc].STRA_DATE_EPOCHS[this.nEpoch];	// This calendar's epoch marker strings
	this.descYrEpoch = [];	// This calendar's epochs as full years
	this.descDtEpoch = [];	// This calendar's epochs as full dates
	// The following means that year values (no matter how many digits are displayed) are all
	// treated as being < 99. That is, no year value is > 99 as is the case in the Japanese
	// Gregorian calendar or in any other calendar that is based on a "short political period"
	this.isShortYr = (this.descLbEpoch.length > 2);
	// Calc the longest epoch length (either short or long)
	this.setCalendar();
	this.abbEpochLength = this.fulEpochLength = 0; this.minEpoch = 1;
	this._AD = this.abbEpochLength,this.descLbEpoch[1][1];		
	for (i=0; i < this.descLbEpoch.length; i++) {
		this.fulEpochLength = Math.max(this.fulEpochLength,this.descLbEpoch[i][0].length);
		this.abbEpochLength = Math.max(this.abbEpochLength,this.descLbEpoch[i][1].length);
		this.minEpoch = (i<=0)?this.minEpoch:((this.descLbEpoch[i-1][1].charAt(0)==this.descLbEpoch[i][1].charAt(0))?2:1);
		if (this.bIsChristianEra) {
			this.descYrEpoch.push(0);
			this.descDtEpoch.push(0);
		} else {
			j = this.constantToValue(hX_5.DATE_EPOCH_YEARS[this.nEpoch][i]);
			this.descYrEpoch.push((j!=null)?j.getFullYear():0);
			this.descDtEpoch.push(j);
		}
	}
	// If we have the simple masks set up on this object, regen them
	if (this.simpleFormat) this.genAssistMasks();
	return true;
}

// Private: Look up the epoch index used with the main calendrical systems
HxG_5.prototype.DateTimeConverter.prototype.epochIndex = function () {
	return ((this.bIsHijiri)?1:((this.bIsHebrew)?2:0));
}

// Private: Structure of a format token
HxG_5.prototype.DateTimeConverter.prototype.FormatToken = function (token, typ) {
	this.type = typ; this.token = token;
}

// Private: Parse the digits argument
HxG_5.prototype.DateTimeConverter.prototype.parseDigits = function () {
	this.xDigits = null; this.zeroDigit = hX_5._Z;
	if (this.digitFirst != null) {
		this.xDigits = String.fromCharCode(this.digitFirst, this.digitFirst+1, this.digitFirst+2, this.digitFirst+3, this.digitFirst+4, this.digitFirst+5, this.digitFirst+6, this.digitFirst+7, this.digitFirst+8, this.digitFirst+9);
		this.zeroDigit = this.xDigits.charAt(0);
	}
}

// Private: Parse the format argument
HxG_5.prototype.DateTimeConverter.prototype.parseFormatString = function () {
	var fmt = this.sFormat;
	this.sFormatTokenized = hX_5._E;
	this.nLiteralCnt = this.nTokenCnt = this.DF_MaxLength = 0;

	// Walk the format
	var index, i, xchar, nchar, last=hX_5._E, lastlen=1, q=false, noT=false, len=fmt.length;
	for (index=0; index < len; index++) {
		xchar = fmt.charAt(index);
		// Handle a formatting character
		if (!q && hX_5.isChar(xchar, this.FMTChars)) {
			if (xchar==last) {
				// tidy tokens to only supported lengths
				noT = false;
				if (lastlen >= 8) noT = true;
				else if (lastlen >= 4 && !hX_5.isChar(xchar, this._8chars)) noT = true;
				else if (lastlen >= 3 && !hX_5.isChar(xchar, this._4chars)) noT = true;
				else if (lastlen >= 2 && !hX_5.isChar(xchar, this._3chars)) noT = true;
				else if (lastlen >= 1 && !hX_5.isChar(xchar, this._2chars)) noT = true;
				if (!noT) this.sFormatTokenized = this.sFormatTokenized + xchar;
				lastlen++;
			} else {
				this.sFormatTokenized = this.sFormatTokenized + hX_5._PCT + xchar;
				lastlen = 1;
			}
			last = xchar;
			this.bHasDate = (this.bHasDate) ? this.bHasDate : hX_5.isChar(xchar, this.FMTDateChars);
			this.bHasTime = (this.bHasTime) ? this.bHasTime : hX_5.isChar(xchar, this.FMTTimeChars);
		// Handle an unquoted literal character
		} else {
			if (xchar == hX_5._SQT) {
				nchar = fmt.charAt(index+1);
				// It's just a quote character?
				if (nchar == hX_5._SQT) {
					index++;
				// Closing a quote?
				} else if (q) {
					xchar = hX_5._E;
					q = false;					
				// Starting a quote?
				} else {
					xchar = nchar;
					q = true;
					index++;
				}
			}
			// Starting a new literal?
			if (last != hX_5._UPC) {
				this.sFormatTokenized = this.sFormatTokenized + hX_5._UPC + this.nLiteralCnt;
				this.sLiterals[this.nLiteralCnt] = hX_5._E;
				this.nLiteralCnt++;
			}
			// Add the character to the literal
			this.sLiterals[this.nLiteralCnt-1] = this.sLiterals[this.nLiteralCnt-1] + xchar;
			last = hX_5._UPC;
		}
	}

	// Convert the Tokenized string to an array of tokens
	fmt = this.sFormatTokenized;
	var token, next, nextp, nextl, c, l;
	while (fmt.length > 0) {
		xchar = fmt.charAt(0);
		fmt   = fmt.substr(1);
		nextp = fmt.indexOf(hX_5._PCT);
		nextl = fmt.indexOf(hX_5._UPC);
		next  = (nextp < 0) ? nextl : ((nextl < 0) ? nextp : Math.min(nextp,nextl));
		if (next >= 0) {
			token = fmt.substr(0,next);
			fmt  = fmt.substr(next);
		} else {
			token = fmt;
			fmt = hX_5._E;
		}
		if (xchar==hX_5._PCT) {
			this.sTokens[this.nTokenCnt] = new this.FormatToken(token, 0);
			this.nTokenCnt++;
		} else {
			index = parseInt(token, 10);
			// If the literal looks like a standard separator, mark it as one
			token = this.sLiterals[index];
			for (i = 1; i < this.FMTSEPChars.length; i++)
				token = hX_5.replaceAll(token, this.FMTSEPChars.charAt(i), this.FMTSEPChars.charAt(0));
			token = hX_5.replaceAll(token,  this.FMTSEPChars.charAt(0)+ this.FMTSEPChars.charAt(0),  this.FMTSEPChars.charAt(0));
			this.sTokens[this.nTokenCnt] = new this.FormatToken(this.sLiterals[index], (token == this.FMTSEPChars.charAt(0)) ? 2 : 1);
			this.nTokenCnt++;
		}
	}

	// Given a tokenized format, compute the max length of a field that matches the format
	for (i = 0; i < this.nTokenCnt; i++) {
		// Token is not a literal
		if (this.sTokens[i].type == 0) {
			l = this.sTokens[i].token.length;
			switch (this.sTokens[i].token.charAt(0)) {
				case hX_5._uF:  case hX_5._uW:  case hX_5._lE:
					this.DF_MaxLength += 1;
					break;
				case hX_5._uD: case hX_5._uS:
					this.DF_MaxLength += 3;
					break;
				case hX_5._uZ:
					this.DF_MaxLength += 5;
					break;
				case hX_5._uA: case hX_5._lG:
					this.DF_MaxLength += 8;
					break;
				case hX_5._lZ:
					this.DF_MaxLength += 10;	// This is at best a guess
					break;
				case hX_5._uG:
					this.DF_MaxLength += ((l<=3)?this.abbEpochLength:this.fulEpochLength);
					break;
				case hX_5._lY: case hX_5._uY:
					this.DF_MaxLength += ((this.bIsJapaneseEra)?l:((l==2)?2:4));
					break;
				case hX_5._uE:
					this.DF_MaxLength += ((l > 3) ? hX_5.c.L[this.lc].MAX_STR_DATE_DAYS[this.epochIndex()] : hX_5.c.L[this.lc].MAX_STR_DATE_SHDAYS[this.epochIndex()]);
					break;
				case hX_5._uM:
					this.DF_MaxLength += ((l < 3)?2:((l > 3) ? hX_5.c.L[this.lc].MAX_STR_DATE_MONTHS[this.epochIndex()] : hX_5.c.L[this.lc].MAX_STR_DATE_SHMONTHS[this.epochIndex()]));
					break;
				case hX_5._lA:
					this.DF_MaxLength += Math.max(this.sAM.length, this.sPM.length);
					break;
				default:
					this.DF_MaxLength += 2;
					break;
			}
		} else this.DF_MaxLength += this.sTokens[i].token.length;
	}
}

// Public: Retrieve the value of an attribute
HxG_5.prototype.DateTimeConverter.prototype.getAttribute = function (attribute) {
	return (hX_5.getArg(this.attributeTable, this, attribute));
}

// Public:	Return the reason the last conversion failed as a localized message
HxG_5.prototype.DateTimeConverter.prototype.lastError = function () {
	return ((this.msgFail && this.msgFail.length > 0) ? this.msgFail : null);
}

// Public:	Construct a Javascript Date object from a string
//			Returns a JS Date object if string can be parsed
//			Returns NULL and lastError empty if string is empty
//			Returns NULL and lastError valued if string cannot be parsed
HxG_5.prototype.DateTimeConverter.prototype.stringToValue = function (iDate) {
	var r = this.stringToValueBase(iDate);
	return ((r) ? r.value : null);
}
HxG_5.prototype.DateTimeConverter.prototype.stringToValueBase = function (iDate, tokenStop) {

	// Set up arguments passed to a failure
	this.intMsgFail = this.msgFail = hX_5._E;
	var sDate = iDate, args = [iDate];

	// Handle empty/null string
	if (iDate == null || !hX_5.isString(iDate)) {
		this.msgFail = hX_5.L[this.plc].ERR_empty;
		return null;
	}
	while (sDate.length > 0 && sDate.charAt(sDate.length-1) == hX_5._B)
		sDate = sDate.substr(0, sDate.length-1);
	if (sDate.length == 0)
		return null;

	// Construct default values for the above date/time parts
	this.setCalendar();
	var tok=this.sTokens, tokI=0, curdate=new Date(), o=hX_5.cal.dateToCalendar(curdate);
	var intMonth=o.month+1, intDay = o.date, intYear = o.year, intSvDay = o.date, intJDay=0;
	var intHour = 12, intMin = 0, intSec = 0, intMS = 0, intMSDay = 0, ampm = hX_5._E;
	var ADtoCal = false, iEpoch = null, zISO = false, zOffset = null, bShortYear = false, bShortYearSpec = false, bShortHour = false, bKampm = false, bJulianDate = false, bNormalDate = false;
	var bNumberInShMonth = this.matchDigit(hX_5.L[this.lc].STRA_DATE_SHMONTHS[this.epochIndex()][0]);
	var bNumberInMonth = this.matchDigit(hX_5.L[this.lc].STRA_DATE_MONTHS[this.epochIndex()][0]);
	var monthStrict = (bNumberInMonth || bNumberInShMonth)?2:this.nStrict;

	// Walk across string, matching the tokenized format string until we find a
	// mismatch or succeed.
	var tokenC = ((tokenStop != null) ? tokenStop : this.nTokenCnt), token, pose, i, j, r, n, ch;
	while (this.msgFail.length == 0 && sDate.length > 0 && tokI < tokenC) {
		token = tok[tokI].token;
		this.SEPChars = this.FMTSEPChars;
		if (tokI+1<tokenC && tok[tokI+1].type < 1 && (tok[tokI+1].token==hX_5._lZ || tok[tokI+1].token==hX_5._uZ))
			this.SEPChars = this.FMTSEPChars.substr(1);

		// Skip a token that was removed
		if (tok[tokI].type == hX_5._lX) {
			// no-op
		// Handle matching a literal or delimiter
		} else if (tok[tokI].type >= 1) {
			// If very strict or a literal, then must match exactly
			if (this.nStrict > 1 || tok[tokI].type == 1) {
				if (token == sDate.substr(0,token.length)) {
					sDate = sDate.substr(token.length);
				} else {
					if (this.nStrict <= 1) sDate = this.skipSeparators(sDate);
					else {
						this.msgFail = hX_5.L[this.plc].ERR_datetime_parse_fail;
						this.intMsgFail = hX_5.INTERR_Parse_Literal + token;
						break;
					}
 				}
 			// If not very strict, skip to the next non-literal character
 			} else sDate = this.skipSeparators(sDate);

		// Handle matching a format character
		} else {
	 		switch (token) {

				// Handle two digit 'numeric only' tokens
				case hX_5._lD:  case hX_5._l2D:
				case hX_5._lH:	case hX_5._l2H:
				case hX_5._uH:  case hX_5._u2H:
				case hX_5._lK:  case hX_5._l2K:
				case hX_5._uK:  case hX_5._u2K:
				case hX_5._lM:  case hX_5._l2M:
				case hX_5._lS:  case hX_5._l2S:
				case hX_5._lW:  case hX_5._l2W:
				// Strictly speaking these are 1 character numbers, but we'll take two chars as we're going to ignore them
				case hX_5._uF:  case hX_5._uW:  case hX_5._lE:

					// If not strict, can omit any of these
					if (this.nStrict <= 1)
						if (this.skipIfMissing(sDate, true)) break;

					// Otherwise, parse the next characters as a digit
					n = this.matchDigits (sDate, 2, false);
					if (n.fail) {
						this.msgFail = hX_5.L[this.plc].ERR_datetime_parse_fail;
						this.intMsgFail = hX_5.INTERR_Parse_TwoDig + token;
						break;
					}
					sDate = n.s;
					switch (tok[tokI].token) {
						case hX_5._lD:  case hX_5._l2D:
							intDay = n.n;
							bNormalDate = true;
							break;
						case hX_5._lH:  case hX_5._l2H:
							intHour = n.n;
							bShortHour = true;
							break;
						case hX_5._uH:  case hX_5._u2H:
							intHour = n.n;
							break;
						case hX_5._lK:  case hX_5._l2K:
							intHour = n.n - 1;
							break;
						case hX_5._uK:  case hX_5._u2K:
							intHour = n.n + 1;
							bShortHour = true;
							bKampm = true;
							break;
						case hX_5._lM:  case hX_5._l2M:
							intMin = n.n; break;
						case hX_5._lS:  case hX_5._l2S:
							intSec = n.n; break;
						case hX_5._uF:	// We simply ignore these three on input
						case hX_5._lW:	// (as per Java -- they need to be present,
						case hX_5._uW:	// but they aren't unique, so they are ignored)
						case hX_5._l2W:  case hX_5._lE:
						default:
							break;
					}
				break;

				// Handle three digit 'numeric only' tokens
				case hX_5._uS:  case hX_5._u3S:  case hX_5._u2S:
				case hX_5._uD:  case hX_5._u3D:  case hX_5._u2D:

					// If not strict, can omit any of these
					if (this.nStrict <= 1)
						if (this.skipIfMissing(sDate, true)) break;

					// Parse it as a number
					n = this.matchDigits (sDate, 3, false);
					if (n.fail) {
						this.msgFail = hX_5.L[this.plc].ERR_datetime_parse_fail;
						this.intMsgFail = hX_5.INTERR_Parse_TwoDig + token;
						break;
					}
					sDate = n.s;
					if (token.charAt(0) == hX_5._uD) {
						// It's a Julian date
						bJulianDate = true;
						intJDay = n.n;
					} else {
						// It's milliseconds
						intMS = n.n;
					}
				break;

				// Handle years
				case hX_5._lY:  case hX_5._l2Y:  case hX_5._l3Y:  case hX_5._l4Y:
				case hX_5._uY:  case hX_5._u2Y:  case hX_5._u3Y:  case hX_5._u4Y:
					// If year is in the format, one must be provided
					n = this.matchDigits (sDate, 4, false);
					if (n.fail) {
						this.msgFail = hX_5.L[this.plc].ERR_datetime_parse_fail;
						this.intMsgFail = hX_5.INTERR_Parse_TwoDig + token;
						break;
					}
					if (token.charAt(0) != hX_5._uY) {
						sDate = n.s;
						intYear = n.n;
						if (n.cnt <= 2) bShortYear = true;
						bShortYearSpec = (token==hX_5._lY || token==hX_5._l2Y || token==hX_5._l3Y);
					}
					break;

				// Handle the 8 digit 'numeric only' tokens
				case hX_5._uA:  case hX_5._u2A:  case hX_5._u3A:  case hX_5._u4A:  case hX_5._u5A:  case hX_5._u6A:  case hX_5._u7A:  case hX_5._u8A:
				case hX_5._lG:  case hX_5._l2G:  case hX_5._l3G:  case hX_5._l4G:  case hX_5._l5G:  case hX_5._l6G:  case hX_5._l7G:  case hX_5._l8G:
					// If not strict, can it
					if (this.nStrict <= 1)
						if (this.skipIfMissing(sDate, true)) break;
					// Parse it as a number
					n = this.matchDigits (sDate, 8, false);
					if (n.fail) {
						this.msgFail = hX_5.L[this.plc].ERR_datetime_parse_fail;
						this.intMsgFail = hX_5.INTERR_Parse_TwoDig + token;
						break;
					}
					if (token.charAt(0)==hX_5._uA) {
						sDate = n.s;
						intMSDay = n.n;
					}
					// N.B.  We currently don't convert Julian Dates to dates
				break;

				// Handle epochs
				case hX_5._uG:  case hX_5._u2G:		// treat as GGG
				case hX_5._u3G: case hX_5._u4G:

					// If not strict, can omit the epoch
					if (this.nStrict <= 1)
						if (this.skipIfMissing(sDate)) break;

					// We have something that is alphabetic, so treat it as an epoch
					// N.B. case doesn't matter, just match an epoch
					r = false;
					if (sDate.substr(0,2)==this._AD) ADtoCal = true; // If AD is set, then use it to forceably swap calendar input
					for (i=0; i < this.descLbEpoch.length; i++) {
						pose = ((token.length <= 3) ? this.descLbEpoch[i][1] : this.descLbEpoch[i][0]).toUpperCase();
						if ((sDate.substr(0, pose.length)).toUpperCase()==pose) {
							sDate = sDate.substr(pose.length);
							iEpoch = i;
							r = true;
							break;
						}
						// If not strict, accept some misspellings
						pose = this.descLbEpoch[i][((token.length<=3)?1:0)].toUpperCase();
						if (this.nStrict <= 1 && sDate.charAt(0).toUpperCase()==pose.charAt(0)) {
							// Find the first n characters of the string that match this pose
							n = 1;
							while (n<sDate.length && n<pose.length && sDate.charAt(n).toUpperCase()==pose.charAt(n)) n++;
							// Do these characters uniquely identify an epoch?
							pose = pose.substr(0,n);
							for (j=0; j<this.descLbEpoch.length; j++) {
								if (j!=i && pose==this.descLbEpoch[j][((token.length <= 3)?1:0)].substr(0,n).toUpperCase()) j=99;
								}
							// Appears to be unique, figure out how many chars to skip
							if (j<99) {
								sDate = sDate.substr(n); j=0; ch=sDate.charAt(j);
								while (j < sDate.length && ((ch==hX_5._P) || !this.matchDigit(ch) && !hX_5.isChar(ch,this.SEPChars))) {
									j++; ch = sDate.charAt(j);
								}
								sDate = sDate.substr(j);
								iEpoch = i;
								r = true;
								break;
							}
						}
					}
					if (!r) this.msgFail = hX_5.L[this.plc].ERR_epoch_name;
					break;

				// Handle months
				case hX_5._uM:  case hX_5._u2M:  case hX_5._u3M:  case hX_5._u4M:
					// If we're not strict, you can enter a number or string
					// If we're strict, and this is a short token, a number is required
					// Special handling of parsing is needed for "numeric" month names (notably in Japanese = 12M or 12 M)
					if (token.length <= 2 || this.nStrict <= 1) {
						n = this.matchDigits (sDate, 2, false);
						if (n.fail && this.nStrict > 1) {
							this.msgFail = hX_5.L[this.plc].ERR_datetime_parse_fail;
							this.intMsgFail = hX_5.INTERR_Parse_TwoDig + token;
							break;
						} else if (!n.fail) {
							sDate = n.s;
							intMonth = n.n;
							if (intMonth>0 && ((token.length==3 && bNumberInShMonth) || (token.length>3 && bNumberInMonth))) {
								pose = ((token.length > 3)?hX_5.L[this.lc].STRA_DATE_MONTHS[this.epochIndex()][intMonth-1]:hX_5.L[this.lc].STRA_DATE_SHMONTHS[this.epochIndex()][intMonth-1]).toUpperCase();
								i = pose.length-1;
								if (!this.matchDigit(pose.substr(i,1))) {
									while (i>=0 && !this.matchDigit(pose.substr(i-1,1))) i--;
									while (sDate.length > 0 && i<pose.length && sDate.charAt(0)==pose.charAt(i)) {
										sDate = sDate.substr(1);
										i++;
									}
								}
							}
							break;
						}
					}

					// See if we have match against a month name
					r = false;
					for (i = 0; i < hX_5.cal.monthsInYear(); i++) {
						// See if we have an exact match
						pose = ((token.length > 3 || monthStrict <= 1)?hX_5.L[this.lc].STRA_DATE_MONTHS[this.epochIndex()][i]:hX_5.L[this.lc].STRA_DATE_SHMONTHS[this.epochIndex()][i]).toUpperCase();
						if (sDate.substr(0, pose.length).toUpperCase() == pose) {
							sDate = sDate.substr(pose.length);
							intMonth = i+1;
							r = true;
							break;
						}
						// If not strict, accept some misspellings
						pose = hX_5.L[this.lc].STRA_DATE_MONTHS[this.epochIndex()][i].toUpperCase();
						if (monthStrict <= 1 && sDate.charAt(0).toUpperCase()==pose.charAt(0)) {
							// Find the first n characters of the string that match this month
							n = 1;
							while (n<sDate.length && n<pose.length && sDate.charAt(n).toUpperCase()==pose.charAt(n)) n++;
							// Do these characters uniquely identify a month?
							pose = pose.substr(0,n);
							for (j=0; j<hX_5.cal.monthsInYear(); j++)
								if (j!=i && pose==hX_5.L[this.lc].STRA_DATE_MONTHS[this.epochIndex()][j].substr(0,n).toUpperCase()) j=99;
							// Appears to be unique, figure out how many chars to skip
							if (j<99) {
								sDate = sDate.substr(n); j=0; ch=sDate.charAt(j);
								while (j < sDate.length && (!this.matchDigit(ch) && !hX_5.isChar(ch,this.SEPChars))) {
									j++; ch = sDate.charAt(j);
								}
								sDate = sDate.substr(j);
								intMonth = i+1;
								r = true;
							}
						}
					}
					if (!r)	this.msgFail = hX_5.L[this.plc].ERR_month_name;
					break;

				// Handle day formats
				case hX_5._uE:  case hX_5._u2E:  // treat as EEE
				case hX_5._u3E: case hX_5._u4E:

					// If not strict, skip if missing
					if (this.nStrict <= 1)
						if (this.skipIfMissing(sDate)) break;
					
					// Check for an exact match (ignoring case)
					r = false;
					for (i=0; i<7; i++) {
						pose = ((token.length <= 3)?hX_5.L[this.lc].STRA_DATE_SHDAYS[this.epochIndex()][i]:hX_5.L[this.lc].STRA_DATE_DAYS[this.epochIndex()][i]).toUpperCase();
						if (sDate.substr(0, pose.length).toUpperCase() == pose) {
							sDate = sDate.substr(pose.length);
							r = true;
							break;
						}
					}
					// For strict parsing, a day must be provided (though it can be any day)
					// For non-strict parsing, if we didn't find a match, ignore this "word"
					if (!r && this.nStrict <= 1) {
						sDate = (token.length <= 3)?(this.skipToSeparator(sDate,(this.nStrict==0)?-1:hX_5.c.L[this.lc].MAX_STR_DATE_SHDAYS[this.epochIndex()])):(this.skipToSeparator(sDate,-1));
						// Both Hebrew and Hijiri allow spaces and other separators in the word, so match as far as we can
						// For now we can't handle this case 
						r = true;
					}
					if (!r) this.msgFail = hX_5.L[this.plc].ERR_day_name;
					break;

				// Handle am/pm
				case hX_5._lA:

					// If not strict, can omit the am/pm
					if (this.nStrict <= 1)
						if (this.skipIfMissing(sDate)) break;

					// If not omitted, a full am or pm is required
					if (sDate.length >= 2 && sDate.substr(0,2).toUpperCase() == this.sAM.toUpperCase()) {
						// N.B. Case doesn't matter
						ampm = this.sAM;
						sDate = sDate.substr(2);
					} else if (sDate.length >= 2 && sDate.substr(0,2).toUpperCase() == this.sPM.toUpperCase()) {
						ampm = 	this.sPM;
						sDate = sDate.substr(2);
					} else {
						this.msgFail = hX_5.L[this.plc].ERR_ampm_name;
					}
					break;

				// Handle zone
				case hX_5._lZ: case hX_5._uZ:
					// If not strict, can omit the zone
					this.SEPChars = this.FMTSEPChars.substr(1);
					if (this.nStrict <= 1)
						if (this.skipIfMissing(sDate)) break;

					// If a zone is provided, we need to interpret it
					r = false; 
					zISO = (token.charAt(0)==hX_5._uZ);
					if (!zISO || this.nStrict <= 1) {
						for (i = 0; i < hX_5.L[this.lc].STR_TIMEZONES.length; i++) {
							// N.B. Case doesn't matter
							if (hX_5.L[this.lc].STR_TIMEZONES[i][1] != null && hX_5.L[this.lc].STR_TIMEZONES[i][1].length > 0) {
								pose  = hX_5.L[this.lc].STR_TIMEZONES[i][1].toUpperCase();
								if (sDate.substr(0, pose.length).toUpperCase() == pose) {
									zOffset = this.readTimeZoneOffset(hX_5.L[this.lc].STR_TIMEZONES[i][0]);
									sDate = sDate.substr(pose.length);
									r = true;
									break;
								}
								// If it parses as a Daylight Savings Time, lose an hour
								pose  = hX_5.L[this.lc].STR_TIMEZONES[i][3].toUpperCase();
								if (sDate.substr(0, pose.length).toUpperCase() == pose) {
									zOffset = this.readTimeZoneOffset(hX_5.L[this.lc].STR_TIMEZONES[i][2]);
									sDate = sDate.substr(pose.length);
									r = true;
									break;
								}
							}
						}
					}
					// If this is ISO format and we have no match, accept a digit or sign
					if (zISO && !r) {
						pose = sDate.charAt(0);
						r = zISO = (this.matchDigit(pose) || pose == this._PLS || pose == this._MIN);
					} else zISO = false;
					// If we matched GMT, we can have an explict offset after it
					if (r && (i==0 || zISO)) {
						// Format we understand is GMT+nn:nn where + can be minus or missing
						// ISO format omits the "GMT"
						var sign = +1;
						if (sDate.charAt(0) == this._PLS)
							sDate = sDate.substr(1);
						else if (sDate.charAt(0) == this._MIN) {
							sign = -1;
							sDate = sDate.substr(1);
						}
						n = this.matchDigits (sDate, 2, (this.nStrict>1));
						r = !n.fail;
 						if (r) {
							// At this point we have either a sign and at least one digit
							// OR at least one digit
							sDate = n.s;
							// N.B. Java skips the following check
							if (n.n < 0 || n.n > 13) r = false;
							else zOffset = sign * (n.n * 60);
							if (r) {
								if (sDate.charAt(0) == this.TZSeparator) sDate = sDate.substr(1);
								n = this.matchDigits (sDate, 2, (this.nStrict>1));
								r = !n.fail;
								if (!r && this.nStrict <= 1) {
									r = true;
									n.s = 0;
								}
								if (r) {
									sDate = n.s;
									// N.B. Java skips the following check
									if (n.n < 0 || n.n > 59)
										r = false;
									else
										zOffset = sign * (Math.abs(zOffset) + n.n);
								}
							}
						}
					} 
					if (!r) this.msgFail = hX_5.L[this.plc].ERR_tzone_name;
					break;

				// Could have no token, so just ignore
				default:
					break;

			}
		} // end matching a format character

		// Advance to next token
		tokI++;

	} // end of while loop that walks the string

	// See if there's scrap at the end of the string
	if (this.msgFail.length == 0 && this.nStrict > 1 && (tokI != tokenC || sDate.length > 0)) {
		this.msgFail = hX_5.L[this.plc].ERR_datetime_parse_fail;
		this.intMsgFail = hX_5.INTERR_Parse_end + sDate;
	}

	// Hack used mostly in testing, if calendar is non-Gregorian Christian, allow
	// an "AD" to be entered and then convert the date from AD to non-Christian format
	if (ADtoCal) hX_5.cal.setCalendar(true, false, false);
	
	// Now validate (or correct) the values provided
	
	// If we have an epoch scheme with multiple epochs, make sure we are in bounds
	if (this.msgFail.length == 0 && iEpoch!=null && this.isShortYr && iEpoch<this.descLbEpoch.length-2) {
		if (this.nStrict > 1 && intYear>(1+this.descYrEpoch[iEpoch+1]-this.descYrEpoch[iEpoch]))
			this.msgFail = hX_5.L[this.plc].ERR_epoch_range;
	}

	if (this.msgFail.length == 0) {
		// Verify  that we have a valid year and correct things in Gregorian calendars
		// If this is a short year spec (1 or 2 yy characters) and 2 characters are entered
		// convert it to a full year
		if (bShortYearSpec && !this.isShortYr && bShortYear)
			intYear = hX_5.cal.twoYearToFourYear(intYear, curdate, this.getBaseYear(curdate));
		// Handle "BC"
		if (this.bIsChristianEra)
			intYear = (iEpoch==0)?(-intYear):intYear;
		// Handle adjusting to the epoch base for Gregorian calendars
		else if (this.bIsGregorian) {
			iEpoch = (iEpoch!=null) ? iEpoch : this.descLbEpoch.length-1;
			intYear += this.descYrEpoch[iEpoch]-1;
		}
		if (intYear == 0 && !this.bIsChristianEra)
			this.msgFail = hX_5.L[this.plc].ERR_epoch_range;
		else {
			// intYear is now known to be valid for calendrical year
			// If we have a julian date and no normal date use the Julian
			if (bJulianDate && !bNormalDate) {
				o = hX_5.cal.julianToMonthDay(intJDay, intYear);
				intMonth = o.month+1;
				intDay = o.date;
			}
			// If we have neither day nor Julian Day, we have to use a default day
			// So adjust intDay to the month in use (last day of month)
			if (!bJulianDate && !bNormalDate) {
				intDay = hX_5.cal.daysInMonth(intMonth-1,intYear);
				// If no day is provided and we have a timezone, we have a problem
				// We need the real day to compute the time zone.  We have to guess
				// this.  Use today's day if it's less than the last day of the input month
				if (zOffset) intDay = Math.min(intSvDay,intDay);
			}
			// If we have two numbers which are clearly in the wrong place, swap them
			if (this.nStrict == 0) {
				if( intMonth > hX_5.cal.monthsInYear(intYear) && intDay > 0 && intDay <= hX_5.cal.monthsInYear(intYear) ) {
					i = intMonth;
					intMonth = intDay;
					intDay = i;
				}
			}
		}
	}

	// Check for a valid month
	if (this.msgFail.length == 0 && (intMonth < 1 || intMonth > hX_5.cal.monthsInYear(intYear)))
		this.msgFail = hX_5.L[this.plc].ERR_month_range;

	// Check for a valid day
	if (this.msgFail.length == 0 && (intDay < 1 || intDay > hX_5.cal.minDaysInMonth())) {
		i = hX_5.cal.daysInMonth(intMonth-1, intYear);
		if (intDay < 1 || intDay > i) {
			this.msgFail = hX_5.L[this.plc].ERR_month_rangelimit;
			args[1]= hX_5.L[this.lc].STRA_DATE_MONTHS[this.epochIndex()][intMonth-1];
			args[2]= i;
		}
	}

	// Check for valid hours
	if (this.msgFail.length == 0) {
		// If this is a 24 hour clock, and we're not strict
		// we'll use am/pm markers if pm is set and hour is less
		// than or equal to 12 otherwise we'll ignore ampm (when not strict)
		if (!bShortHour) {
			if (this.nStrict <= 1 && ampm.length > 0) {
				if (ampm == this.sPM && intHour <= 12)
					intHour += 12;
			}
			// Now verify the resulting time
			if (intHour < 0 || intHour > 23) {
				this.msgFail = hX_5.L[this.plc].ERR_hour_range_long;
			}
		// If this is a 12 hour clock, we'll allow 13-24
		// if not strict.  If am/pm not supplied, we'll
		// apply one using the 6am to 6pm times are most likely rule
		} else {
			if (this.nStrict <= 1) {
				if (intHour > 12) {
					ampm = this.sPM;
					intHour -= 12;
				}
				if (ampm.length == 0) 
					ampm = (intHour >= 6 && intHour <= 11) ? this.sAM : this.sPM;
			}
			if (intHour < 1 || intHour > 12)
				this.msgFail = hX_5.L[this.plc].ERR_hour_range_short;
			if (bKampm) {
				if (ampm == this.sPM && intHour != 0) {
					intHour += 12;
				}
				else if (ampm == this.sAM && intHour == 0)
					intHour = 0;
			} else {
				if (ampm == this.sPM && intHour != 12)
					intHour += 12;
				else if (ampm == this.sAM && intHour == 12)
					intHour = 0;
			}
		}
	}

	// Verify minutes and seconds
	if (this.msgFail.length == 0 && (intMin < 0 || intMin > 59))
		this.msgFail = hX_5.L[this.plc].ERR_minute_range;
	if (this.msgFail.length == 0 && (intSec < 0 || intSec > 59))
		this.msgFail = hX_5.L[this.plc].ERR_second_range;

	// If an error was found while parsing, bail out with the error
	if (this.msgFail.length > 0) {
		this.msgFail = hX_5.NLSFormatMsg (this.msgFail, args);
		return null;
	}

	// Penultimately, construct the Javascript date
	r = hX_5.cal.calendarToDate(intYear, intMonth-1, intDay, intHour, intMin, intSec, intMS);
	// Ulimately the user supplied a zoffset (and only if), then adjust the
	// Date we just created (which should have a pretty good timezone offset
	// in it) relative to the offset the user provided
	if (zOffset && (zOffset !=  -(r.getTimezoneOffset()))) {
		intMin = r.getMinutes() - (zOffset + r.getTimezoneOffset());
		r = hX_5.cal.calendarToDate(intYear, intMonth-1, intDay, intHour, intMin, intSec, intMS);
	}
	return {value:r, token:(tokI > 0)?tokI-1:0, pos:(iDate.length-sDate.length)};
}

// Public: Parse a string figuring out what the time token is under the cursor
HxG_5.prototype.DateTimeConverter.prototype.stringToLastToken = function (iDate, pos) {
	var r, token=0, all="hHkKms";
	// If empty or no position, find the first token and return it
	if (iDate.length <= 0 || pos == 0) {
		while (token < this.sTokens.length && !(this.sTokens[token].type == 0 && hX_5.isChar(this.sTokens[token].token.charAt(0),all)))
			token++;
	} else {
		// Find out which token "pos" is on
		r = this.stringToValueBase(iDate.substr(0,pos));
		if (r) {
			token = (r.token < this.sTokens.length) ? r.token : this.sTokens.length-1;
			// Look right if we're not on a token we want
			if (this.sTokens[token].type != 0) {
				while (token < this.sTokens.length && this.sTokens[token].type != 0)
					token++;
			}

			// Look left if we didn't find a token
			token = (token >= this.sTokens.length) ? this.sTokens.length-1:token;
			while (token >= 0 && !(this.sTokens[token].type == 0 && hX_5.isChar(this.sTokens[token].token.charAt(0),all)))
				token--;
		}
	}

	// Return h, m or s depending on what we're on
	if (token < 0 || token >= this.sTokens.length)
		return hX_5._lH;
	else if (this.sTokens[token].token.charAt(0) == hX_5._lM)
		return hX_5._lM;
	else if (this.sTokens[token].token.charAt(0) == hX_5._lS)
		return hX_5._lS;
	else
		return hX_5._lH;
}

// Public:  Return the position of the last character in a token
HxG_5.prototype.DateTimeConverter.prototype.posFromLastToken = function (iDate, token) {

	if (iDate.length==0 || !token || token.length!=1) return 0;

	// Find the position of the token (h, m or s)			
	var r, i, hit=false, test=(token==hX_5._lS) ? hX_5._lS : ((token==hX_5._lM) ? hX_5._lM : "hHkK");
	for (i=0; i < this.sTokens.length && !hit; i++) {
		if (this.sTokens[i].type==0 && hX_5.isChar(this.sTokens[i].token.charAt(0),test))
			hit = true;
	}
	if (!hit) return 0;

	// Go back to the start of the token and find it in the string
	i = (i>0)?i-1:0;
	r = this.stringToValueBase(iDate, i);
	return ((r) ? r.pos : 0);
}

// Public:	Return the maximum length of a value that uses this converter
HxG_5.prototype.DateTimeConverter.prototype.getMaxLength = function () {
	return this.DF_MaxLength;
}

// Private:	Test if a character is a mask character or not
HxG_5.prototype.DateTimeConverter.prototype.isMarker = function (c) {
	return (this.markers.indexOf(c) >= 0);
}

// Given the pattern set on the converter, generate a short-form date
HxG_5.prototype.DateTimeConverter.prototype.genAssistMasks = function(promptC) {
	// For now we only want Month, Day and Year, epoch, ours, mins, secs and the separators after them (plus any initial seps)
	// We generate a bunch of things:
	//	this.simpleFormat	  -- the format we use when in input assismt mode, e.g., MM/dd/yyyy (derived from the original full format such as MMMM/d/yyyy (E))
	// 	this.maskFormat	 	  -- the usual "mask" of commands e.g., ##/##/####
	// 	this.maskFormatDigits -- like maskFormat but the # are replaced with markers indicating what kind of number is parsed in this slot
	//						     markers are y (year), m (two digit month), d (two digit day), G (epoch), h (2 digit hour) A (2 digit minutes) and s (2 digit mins/secs)
	// m = the format we use, p is the mask format (where you can type, what's a literal), d defines what each characters slot in the mask format is
	var m=hX_5._E, p=hX_5._E, d=hX_5._E, sep=true, i,t,x,y,noT;
	if (promptC) this.promptChar = promptC;
	for (i=0; i < this.nTokenCnt; i++) {
		t = this.sTokens[i].token;
		// Token is not a literal
		if (this.sTokens[i].type == 0) {
			sep = true;
	 		switch (t.charAt(0)) {
				case hX_5._lY:
					if (this.isShortYr) {
						m = m + hX_5._l2Y;
						p = p + this._ll;
						d = d + hX_5._l2Y;
					} else {
						m = m + hX_5._l4Y;
						p = p + this._llll;
						d = d + hX_5._l4Y;
					}
					break;
				case hX_5._uG:
					m = m + hX_5._uG;
					d = d + hX_5.padR(hX_5._uG, this.abbEpochLength, hX_5._uG);
					if (this.isShortYr || this.bIsChristianEra)
						p = p + ((this.minEpoch==2)?hX_5.padR(this._l+this._l,this.abbEpochLength,hX_5._B):hX_5.padR(this._l,this.abbEpochLength,hX_5._B));
					else
						p = p + this.descLbEpoch[1][1];
					noT = true;
					break;
				case hX_5._uM:
					m = m + hX_5._u2M;
					p = p + this._ll;
					d = d + hX_5._l2M;
					break;
				case hX_5._lD:
					m = m + hX_5._l2D;
					p = p + this._ll;
					d = d + hX_5._l2D;
					break;
				case hX_5._lH: case hX_5._uH:
					m = m + hX_5._u2H;
					p = p + this._ll;
					d = d + hX_5._l2H;
					break;
				case hX_5._lM:
					m = m + hX_5._l2M;
					p = p + this._ll;
					d = d + hX_5._u2A;
					break;
				case hX_5._lS:
					m = m + hX_5._l2S;
					p = p + this._ll;
					d = d + hX_5._l2S;
					break;
				default:
					sep = false;
					break;
			}
		// Token is a literal
		} else {
			if (t.length > 0) {
				m = m + ((sep) ? (hX_5._SQT+t+hX_5._SQT) : hX_5._E);
				p = p + ((sep) ? t : hX_5._E);
				d = d + ((sep) ? t : hX_5._E);
			}
		}
	}
	// Trim leading and trailing spaces off the mask (and format).  Note that the format may have
	// the lead/trailing spaces inside the quote characters
	this.maskFormat = (noT)?hX_5.trimL(p):hX_5.Trim(p);
	this.maskFormatDigits = hX_5.Trim(d);
	m = hX_5.Trim(m);
	if (m.length > 0) {
		if (m.charAt(0) == hX_5._SQT) 		m = hX_5._SQT + hX_5.Trim(m.substr(1));
		if (m.substr(0,2) == "''")			m = hX_5.Trim(m.substr(2));
		if (m.charAt(m.length-1) == hX_5._SQT)	m = hX_5.Trim(m.substr(0, m.length-1)) + hX_5._SQT;
		if (m.substr(m.length-2,2) == "''")	m = hX_5.Trim(m.substr(0,m.length-2));
	}
	this.simpleFormat = m;
	var args = [];
	args.push("strict:2");
	if (this.simpleFormat)	args.push("format:"+this.simpleFormat);
	if (this.base2DigitSpan)args.push("base-2digit-span:"+this.base2DigitSpan);
	if (this.sEpoch)		args.push("epoch:"+this.sEpoch);
	if (this.ICU4J)			args.push("icu4j:"+this.ICU4J);
	if (this.digitFirst)	args.push("digits:" +this.digitFirst);
	args.push("first-day-of-week:"+this.FDOW);
	if (this.sAM) 			args.push("am:"+this.sAM);
	if (this.sPM) 			args.push("pm:"+this.sPM);
	if (this.locale)		args.push("locale:"+this.locale);
	this.simpleConverter = new hX_5.DateTimeConverter(args[0],args[1],args[2],args[3],args[4],((args.length>5)?args[5]:null),((args.length>6)?args[6]:null),((args.length>7)?args[7]:null),((args.length>8)?args[8]:null),((args.length>9)?args[9]:null),((args.length>10)?args[10]:null));
	this.simpleConverter.isSimple = true;
	this.maskFormatFirst = -1;
	this.maskFormatLast = -1;
	for (i=0; i < this.maskFormat.length; i++) {
		if (this.maskFormat.charAt(i) == hX_5._LB || this.maskFormat.charAt(i) == hX_5._QM) {
			this.maskFormatLast = i;
			this.maskFormatFirst = (this.maskFormatFirst >= 0) ? this.maskFormatFirst : i;
		}
	}
}

// Private:  Compute the base of the range of years we'll consider in the last century
HxG_5.prototype.DateTimeConverter.prototype.getBaseYear = function(today) {
	return (today.getFullYear() - this.base2DigitSpan);
}

// Private:  Skip over leading separators in the string 
HxG_5.prototype.DateTimeConverter.prototype.skipSeparators = function (str) {
	var r = str;
	while (r.length > 0 && hX_5.isChar (r.charAt(0), this.SEPChars)) r = r.substr(1);	
	return r;
}

// Private:  Skip to the next separator in the string (skipping a max number of characters)
HxG_5.prototype.DateTimeConverter.prototype.skipToSeparator = function (str, max) {
	var i = this.countToSeparator(str, max);
	return (str.substr(i));
}

// Private:  As above, but just count the separators 
HxG_5.prototype.DateTimeConverter.prototype.countToSeparator = function (str, max) {
	var r = str, i = 0;
	while (r.length > 0 && (i < max || max < 0) && !hX_5.isChar (r.charAt(0), this.SEPChars)) {
		r = r.substr(1);
		i++;
	}
	return i;
}

// Private:  Check to see if we're already at a separator OR a digit
HxG_5.prototype.DateTimeConverter.prototype.skipIfMissing = function (str, notdigit) {
	var n, r = (hX_5.isChar (str.charAt(0), this.SEPChars));
	if (!r) {
		n = this.matchDigits (str, 1, true);
		r = (notdigit) ? n.fail : (!n.fail);
	}
	return r;
}

// Private:  Check the next char is a digit
//			 Returns true or false
HxG_5.prototype.DateTimeConverter.prototype.matchDigit = function (str) {
	if (str.length > 0) {
		if (hX_5.WDIGITS.indexOf(str.charAt(0)) >= 0) return true;
		if (this.xDigits != null && this.xDigits.indexOf(str.charAt(0)) >= 0) return true;
	}
	return false;
}

// Private:  Check the next chars (at most n or exactly n) to see if they are a number
//			 Returns structure: 
//				fail:	true if not digit(s)
//				s:		if not fail, input string without digits
//				n:		if not fail, integer value of digits
//				cnt:	if not fail, number of digits removed
HxG_5.prototype.DateTimeConverter.prototype.matchDigits = function (str, n, exact) {

	var cnt = 0, s = str, oN = hX_5._E, w, a;
	while (cnt < n && s.length > 0) {
		w = (hX_5.WDIGITS.indexOf(s.charAt(0)) >= 0);
		a = (!w && this.xDigits != null && this.xDigits.indexOf(s.charAt(0)) >= 0);
		if (!a && !w)
			break;
		else {
			oN = oN + this.getDigit(s);
			cnt++; s=s.substr(1);
		}
	}
	return {fail:(cnt==0 || (exact && cnt!=n)), s:s, n:parseInt(oN,10), cnt:cnt};
}

// Private:  Return the first character of the string as a Western digit
//			 Returns the digit character (as a string)
HxG_5.prototype.DateTimeConverter.prototype.getDigit = function (s) {
	if (this.xDigits == null || (hX_5.WDIGITS.indexOf(s.charAt(0)) >= 0) || (this.xDigits.indexOf(s.charAt(0)) < 0)) return (s.charAt(0));
	return (hX_5.WDIGITS.charAt(s.charCodeAt(0) - this.xDigits.charCodeAt(0)));
}

// Private:  Convert a string from Western to non-Western digits
//			 Returns converted string
HxG_5.prototype.DateTimeConverter.prototype.cvtToDigits = function (str) {
	var s=str+hX_5._E, r=hX_5._E;
	if (this.xDigits == null) return s;
	while (s.length > 0) {
		if (hX_5.WDIGITS.indexOf(s.charAt(0)) >= 0) r = r + this.xDigits.charAt(s.charCodeAt(0)-hX_5.WDIGITS.charCodeAt(0));
		else r = r + s.charAt(0);
		s = s.substr(1);
	}
	return r;
}

// Private:  Convert a character code in an event from Western to non-Western digits
//			 Updates the code in the event.
HxG_5.prototype.DateTimeConverter.prototype.digitTransmute = function (ev) {
	if (this.xDigits != null) {
		var c = hX_5.getChrCode(ev);
		if (hX_5.WDIGITS.indexOf(c)>=0) hX_5.setChrCode(ev, this.xDigits.charAt(c.charCodeAt(0)-hX_5.WDIGITS.charCodeAt(0)));
	}
}

// Local:  Return the "zero" digit
HxG_5.prototype.DateTimeConverter.prototype.getZeroDigit = function () {
	return (this.zeroDigit);
}

// Private: Read timezone offset from timezone table
HxG_5.prototype.DateTimeConverter.prototype.readTimeZoneOffset = function(str) {
	var sign = (str.charAt(0) == this._MIN) ? -1 : +1;
	var zoffset = (parseInt(str.substr(1,2), 10)) * 60;
	zoffset += parseInt(str.substr(3,2), 10);
	zoffset = zoffset * sign;
	return zoffset;
}

// Public: Convert a string in "constant" format to a Javscript Date Object
HxG_5.prototype.DateTimeConverter.prototype.constantToValue = function (sDate) {

	var oDate = null, len = sDate.length;
	var i, yr, mo, dy, hr, mn, sc, z;
	yr = mo = dy = mn = sc = 0;
	hr = z = null;

	// Handle constants today, tomorrow and yesterday
	var bRelative = false, t, tstr = sDate.toUpperCase();
	for (i = 0; i < 4; i++) {
		t = hX_5.STR_DATE_TODAY[i].toUpperCase();
		if (tstr.length >= t.length && tstr.substr(0, t.length) == t) {
			bRelative = true;
			break;
		}
	}
	if (bRelative) {
		oDate = new Date();
		oDate.setSeconds(0);
		var daysAdj = 0, dayAdj  = (i==1) ? 1 : ((i==2) ? -1 : 0);
		tstr = tstr.substr(t.length);
		tstr = hX_5.allTrim(tstr);
		if (tstr.charAt(0) == this._PLS) {
			daysAdj = parseInt(tstr.substr(1),10);
		} else if (tstr.charAt(0) == this._MIN) {
			daysAdj = -(parseInt(tstr.substr(1),10));
		}
		if (i==3) { // Now
			oDate = hX_5.cal.adjustDate(oDate,0,0,daysAdj,0,0,0);
		} else {	// today, yesterday, tomorrow
			oDate.setHours(12);
			oDate.setMinutes(0);
			oDate = hX_5.cal.adjustDate(oDate,0,0,(dayAdj+daysAdj),0,0,0);
		}
	} else {
		var iSign=+1;
		if (sDate.charAt(0)==this._MIN) {
			sDate = sDate.substr(1);
			iSign = -1;
		}
		if (len >= 4) yr = (parseInt(sDate.substr(0,4), 10) * iSign) + ((iSign<0)?-1:0);
		if (len >= 6) mo = parseInt(sDate.substr(4,2), 10);
		if (len >= 8) dy = parseInt(sDate.substr(6,2), 10);
		if (len >= 10) hr = parseInt(sDate.substr(8,2), 10);
		if (len >= 12) mn = parseInt(sDate.substr(10,2), 10);
		if (len >= 14) sc = parseInt(sDate.substr(12,2), 10);
		if (len >= 16) z = parseInt(sDate.substr(14,3), 10);

		if (yr == 0 && mo == 0 && dy == 0) {
			oDate = new Date();
			yr = oDate.getFullYear();
			mo = oDate.getMonth()+1;
			dy = oDate.getDate();		
		}
		if (z) {
			oDate = new Date( Date.UTC( yr, mo-1, dy, hr, mn, sc, 0 ) );
			oDate.setTime( oDate.getTime() - (z * 1000) );
		} else {
			oDate = (!hr)?(new Date (yr, mo-1, dy, 12, 0, 0)):(new Date (yr, mo-1, dy, hr, mn, sc));
		}
	}
	return oDate;
}

// Public: Convert a Javascript Date object to a string in "constant" format
HxG_5.prototype.DateTimeConverter.prototype.valueToConstant = function (oDate) {
	var m = hX_5.padL(oDate.getMonth()+1, 2);
	var d = hX_5.padL(oDate.getDate(), 2);
	return (oDate.getFullYear() + m + d);
}

// Private: Helper for following that replaces %n,%nn,%nnn below (pads on left)
HxG_5.prototype.DateTimeConverter.prototype.nReplace = function (fmt, c, cc, p, x, n) {
	var f=fmt, i=f.indexOf(cc), j;
	while (i >= 0) {
		j = i+2;
		while (f.charAt(j)==c) j++;
		f = hX_5.replaceAll(f, f.substring(i,j), hX_5.padL(this.cvtToDigits(x), ((n!=null)?n:(j-i-1)), p));
		i = f.indexOf(cc);
	}
	return f;
}

// Public: Convert a Javascript Date object to a string according to the converter's rules
HxG_5.prototype.DateTimeConverter.prototype.valueToString = function (oDate, ifmt, promptChar) {
	this.msgFail = hX_5._E;
	if (oDate == null) return hX_5._E;
	var fmt = ((ifmt != null) ? ifmt : this.sFormatTokenized), oddampm = (fmt.indexOf("%K") >= 0);
	var o,x,y,z,h,k, pChar=((promptChar)?promptChar:this.getZeroDigit()), bDate = ((ifmt!= null)?true:this.bHasDate), bTime = ((ifmt!= null)?true:this.bHasTime);
	this.setCalendar();
	// These are all format characters that will result in strings (not numbers) so remove them
	if (bDate) {
		fmt = hX_5.replaceAll(fmt, "%MMMM", "@0");
		fmt = hX_5.replaceAll(fmt, "%MMM",  "@1");
		fmt = hX_5.replaceAll(fmt, "%EEEE", "@2");
		fmt = hX_5.replaceAll(fmt, "%EEE",  "@3");
		fmt = hX_5.replaceAll(fmt, "%EE",	"@3");
		fmt = hX_5.replaceAll(fmt, "%E",	"@3");
		fmt = hX_5.replaceAll(fmt, "%GGGG", "@4");
		fmt = hX_5.replaceAll(fmt, "%GGG",  "@5");
		fmt = hX_5.replaceAll(fmt, "%GG",   "@6");
		fmt = hX_5.replaceAll(fmt, "%G",   	"@7");
	}
	if (bTime) {		
		fmt = hX_5.replaceAll(fmt, "%a",    "@8");
		fmt = hX_5.replaceAll(fmt, "%z",    "@9");
		fmt = hX_5.replaceAll(fmt, "%Z",    "@!");
	}

	// Handle date components
	if (bDate) {
		// First do 4 digit years
		o = this.valueToYearEpoch(oDate); y = o.year; z = (y==0)?pChar:this.getZeroDigit();
		fmt = hX_5.replaceAll(fmt, "%yyyy",hX_5.padL(this.cvtToDigits(y), 4, z));
		if (fmt.indexOf("%Y") >= 0) {
			x = (hX_5.cal.getJulianWeek(oDate, this.FDOW)).yr;
			fmt = hX_5.replaceAll(fmt, "%YYYY", hX_5.padL(this.cvtToDigits(x), 4, z));
		}
		// Now 2 digit years
		if (fmt.indexOf("%y") >= 0 || fmt.indexOf("%Y") >= 0) {
			if (this.isShortYr) {
				fmt = this.nReplace(fmt,hX_5._lY,"%y",pChar,y);
				if (fmt.indexOf("%Y") >= 0) fmt = this.nReplace(fmt,hX_5._uY,"%Y",pChar,x);
			} else {
				fmt = this.nReplace(fmt,hX_5._lY,"%y",pChar,(y%100),2);
				if (fmt.indexOf("%Y") >= 0) fmt = this.nReplace(fmt,hX_5._uY,"%Y",pChar,(x%100),2);
			}
		}
		// Replace numeric month format characters with the month
		fmt = this.nReplace(fmt,hX_5._uM,"%M",pChar,(o.month+1));
		// Replace numeric week format characters with the week
		if (fmt.indexOf("%w") >= 0)	fmt = this.nReplace(fmt,hX_5._lW,"%w",pChar,hX_5.cal.getJulianWeek(oDate, this.FDOW).wk);
		if (fmt.indexOf("%W") >= 0) fmt = this.nReplace(fmt,hX_5._uW,"%W",pChar,hX_5.cal.getJulianWeekOfMonth(oDate, this.FDOW));
		// Replace numeric day format characters with the day
		fmt = this.nReplace(fmt,hX_5._lD,"%d",pChar,o.date);
		if (fmt.indexOf("%D") >= 0)	fmt = this.nReplace(fmt,hX_5._uD,"%D",pChar,hX_5.cal.getJulianDay(oDate));
		if (fmt.indexOf("%g") >= 0)	fmt = this.nReplace(fmt,hX_5._lG,"%g",pChar,hX_5.cal.gregorianToJulian(oDate));
		if (fmt.indexOf("%F") >= 0) fmt = this.nReplace(fmt,hX_5._uF,"%F",pChar,hX_5.cal.getDayOfWeekInMonth(oDate));
		if (fmt.indexOf("%e") >= 0) {
			x = (hX_5.cal.getDay(oDate)-this.FDOW)+1;
			x = (x<1)?x+7:x;
			fmt = this.nReplace(fmt,hX_5._lE,"%e",pChar, x);
		}
	}

	// Replace the hour, minute and second format characters with the hour
	if (bTime) {
		h = oDate.getHours();
		k = this.cvtToDigits(oDate.getMilliseconds());
		fmt = this.nReplace(fmt,hX_5._lH,"%h",pChar,(((h%12)==0)?12:(h%12)));
		fmt = this.nReplace(fmt,hX_5._lK,"%k",pChar,(h+1));
		fmt = this.nReplace(fmt,hX_5._uH,"%H",pChar,h);
		fmt = this.nReplace(fmt,hX_5._uK,"%K",pChar,((((h%12)==0)?12:(h%12))-1));
		fmt = this.nReplace(fmt,hX_5._lM,"%m",pChar,oDate.getMinutes());
		fmt = this.nReplace(fmt,hX_5._lS,"%s",pChar,oDate.getSeconds());
		fmt = hX_5.replaceAll(fmt, "%SSS", hX_5.padL(k, 3, pChar));
		fmt = hX_5.replaceAll(fmt, "%SS",  hX_5.padL(k, 3, pChar).substr(0,2));
		fmt = hX_5.replaceAll(fmt, "%S",   hX_5.padL(k, 3, pChar).substr(0,1));
		fmt = this.nReplace(fmt,hX_5._uA,"%A", pChar, oDate.getMilliseconds() + (oDate.getSeconds()*1000) + (oDate.getMinutes()*60*1000) + (oDate.getHours()*60*60*1000));
	}

	// Do the string replaces
	if (bDate) {
		fmt = hX_5.replaceAll(fmt, "@0", this.valueToMonthLabel(oDate));
		fmt = hX_5.replaceAll(fmt, "@1", this.valueToShortMonthLabel(oDate));
		fmt = hX_5.replaceAll(fmt, "@2", this.valueToDayLabel(oDate));
		fmt = hX_5.replaceAll(fmt, "@3", this.valueToShortDayLabel(oDate));
		fmt = hX_5.replaceAll(fmt, "@4", o.name);
		fmt = hX_5.replaceAll(fmt, "@5", o.nameabb);
		fmt = hX_5.replaceAll(fmt, "@6", o.nameabb);
		fmt = hX_5.replaceAll(fmt, "@7", o.nameabb);
	}
	if (bTime) {
		fmt = hX_5.replaceAll(fmt, "@8", this.valueToAmPmLabel(oDate, oddampm));
		fmt = hX_5.replaceAll(fmt, "@9", this.valueToTimeZone(oDate));
		fmt = hX_5.replaceAll(fmt, "@!", this.valueToTimeZone(oDate, true));
	}
	// Clean up the literals
	for (x=0; x < this.nLiteralCnt; x++)
		fmt = hX_5.replaceAll(fmt, hX_5._UPC+x, this.sLiterals[x]);
	return fmt;
}

/*---- Additional specialized functions of the DateTime converter ------*/

// Public: Return am/pm label
HxG_5.prototype.DateTimeConverter.prototype.valueToAmPmLabel = function(oDate, oddampm) {
	var o = oDate.getHours();
	if (oddampm)
		return (o==0) ? this.sPM : ((o <= 12) ? this.sAM : this.sPM);
	else
		return (o < 12) ? this.sAM : this.sPM;
}

// Public: Return timezone label
HxG_5.prototype.DateTimeConverter.prototype.valueToTimeZone = function(oDate, ISO) {
	// Note:  Javascript returns timezone offsets (getTimezoneOffset) with the wrong sign
	// 		  that is, offsets "east" of GMT are returned as negative
	//		  So always invert the result of the call
	var r = hX_5._E, j, zoffset, k, z1, z2, z3, z4, dst=false, znames;
	var z = -(oDate.getTimezoneOffset());
	// Javascript doesn't give us a way of knowing a datetime is in Daylight Savings time or not
	// So we use the following somewhat scuzzy algorithm for guessing.  It seems that the
	// spread of daylight savings time is such that early Feb and mid June are always either
	// in or out of DST no matter where you are, so ...
	var feb = new Date(oDate.getFullYear(), 1, 7);
	var jun = new Date(oDate.getFullYear(), 5, 15);
	var zfeb = -(feb.getTimezoneOffset());
	var zjun = -(jun.getTimezoneOffset());
	if (z == zfeb && z != zjun)	dst = false;
	if (z == zjun && z != zfeb)	dst = true;
	z1 = (dst) ? 2 : 0;
	z2 = (dst) ? 3 : 1;
	znames = hX_5.L[this.lc].STR_TIMEZONES.length;
	for (j = znames-1; !ISO && (j>0); j--) {
		if (hX_5.L[this.lc].STR_TIMEZONES[j][0] == null || hX_5.L[this.lc].STR_TIMEZONES[j][0].length == 0) {
			// noop
		} else {
			zoffset = this.readTimeZoneOffset(hX_5.L[this.lc].STR_TIMEZONES[j][z1]);
			if (zoffset == z) {
				r = hX_5.L[this.lc].STR_TIMEZONES[j][z2];
				for (k=0;k<r.length;k++) {
					if (this.matchDigit(r.charAt(k))) {
						r = r.substr(0,k) + this.cvtToDigits(r.charAt(k)) + r.substr(k+1);
					}
				}
				break;
			}
		}
	}
	if (r.length == 0) {
		r = ((ISO)?hX_5._E:hX_5.L[this.lc].STR_TIMEZONES[0][1]) + ((z < 0) ? this._MIN : this._PLS);
		z = Math.abs(z);
		var hr = Math.floor(z/60);
		var mn = z-(hr*60);
		hr = hX_5.padL(this.cvtToDigits(hr),2);
		mn = hX_5.padL(this.cvtToDigits(mn),2);
		r = r + hr + ((ISO)?hX_5._E:this.TZSeparator) + mn;
	}
	return (r);
}

// Public: Return if format includes date formatting chars
HxG_5.prototype.DateTimeConverter.prototype.isDateFormat = function() {
	return this.bHasDate;
}

// Public: Return if format includes time formatting chars
HxG_5.prototype.DateTimeConverter.prototype.isTimeFormat = function() {
	return this.bHasTime;
}

// Public: Set the calendar type for the current work
HxG_5.prototype.DateTimeConverter.prototype.setCalendar = function() {
	hX_5.cal.setCalendar(this.bIsGregorian, this.bIsHebrew, this.bIsHijiri);
}

// Public: Return (long) Day label
HxG_5.prototype.DateTimeConverter.prototype.valueToDayLabel = function(oDate) {
	return hX_5.L[this.lc].STRA_DATE_DAYS[this.epochIndex()][hX_5.cal.getDay(oDate)];
}

// Public: Return (short) Day label
HxG_5.prototype.DateTimeConverter.prototype.valueToShortDayLabel = function(oDate) {
	return hX_5.L[this.lc].STRA_DATE_SHDAYS[this.epochIndex()][hX_5.cal.getDay(oDate)];
}

// Public: Return (very short) Day label
HxG_5.prototype.DateTimeConverter.prototype.valueToSShortDayLabel = function(i) {
	return hX_5.L[this.lc].STRA_DATE_SHSHDAYS[this.epochIndex()][i];
}

// Public: Return (long) Month label
HxG_5.prototype.DateTimeConverter.prototype.valueToMonthLabel = function(oDate) {
	var o = hX_5.cal.dateToCalendar(oDate);
	return hX_5.L[this.lc].STRA_DATE_MONTHS[this.epochIndex()][o.month];
}

// Public: Return (short) Month label
HxG_5.prototype.DateTimeConverter.prototype.valueToShortMonthLabel = function(oDate) {
	var o = hX_5.cal.dateToCalendar(oDate);
	return hX_5.L[this.lc].STRA_DATE_SHMONTHS[this.epochIndex()][o.month];
}

// Public: Return Year  as a "label" suitable for use in the date picker
HxG_5.prototype.DateTimeConverter.prototype.valueToYearLabel = function (oDate) {
	if (this.bShowEpoch)
		return (this.isShortYr) ? this.valueToString(oDate, hX_5.DTFMT_GEYEAR) : this.valueToString(oDate, hX_5.DTFMT_GCYEAR);
	else
		return this.valueToString(oDate, hX_5.DTFMT_YEAR);
}

// Public: Convert JS date to a calendrical date (including epoch)
HxG_5.prototype.DateTimeConverter.prototype.valueToYearEpoch = function (oDate) {
	var n,sn,i=1,o=hX_5.cal.dateToCalendar(oDate), y=o.year;
	// Simple Epoch system
	if (!this.isShortYr) {
		if (this.bIsGregorian) {
			i = (y < this.descYrEpoch[0])?0:1;
			if (this.bIsChristianEra)
				y = (y<0)?-y:y;
			else {
				y -= this.descYrEpoch[0];
				if (this.bIsBuddhistEra) y++;
			}
		}
	} else {
		// Walk the eras
		for (i=this.descDtEpoch.length-1; i>=0; i--) {
			if (this.descDtEpoch[i]!=null && (oDate.Hx4CompareDate(this.descDtEpoch[i]) >= 0)) {
				y = y - this.descYrEpoch[i] + 1;
				break;
			}
		}
		if (i<0) i = this.descDtEpoch.length-1;
	}
	n = this.descLbEpoch[i][0];
	sn = this.descLbEpoch[i][1];
	// If we're missing the long name, use the short if we have it
	if (n == "--" && sn != "--") n=sn;
	if (this.isSimple) sn = hX_5.padR(sn,this.abbEpochLength,hX_5._B);
	return { name: n, nameabb: sn, year: y, month: o.month, date:o.date };
}

//  ----------------------------------------------------------------------------------------
//  DateTime Validator
//  ----------------------------------------------------------------------------------------
//
//  Methods (common to all validators)
//
//	constructor		Create a validator with a specified set of attributes
//	setAttribute	Change an attribute of the validator
//  getAttribute	Get an attribute of the validator
//	validate		Check if a JS date is valid according to the validator's rules
//  coerce			Force a JS date object to be valid
//	lastError		Return a (localized) string describing the reason the last validation failed
//
//  Constructor attributes:
//
//	Description		Meaning
//  -----------		------------------------------------------------------------------------
//
//	required		True if the values is required.  False if not.
//	min-bound		Minimum value (bound).  Formatted using the "constant" format of yyyymmddhhmmsszzz
//					where trailing components are optional (e.g., 20030201 is ok)
//	max-bound		Maximum value (bound).  Formatted using the "constant" format of yyyymmddhhmmsszzz
//					where trailing components are optional (e.g., 20030201 is ok)
//  constraint-expression	An EL expression that returns true/false if the value is good/bad

HxG_5.prototype.DateTimeValidator = function () {
	this.type = hX_5.CVT_datetime;
	this.lc = hX_5.currentLocale();

	this.required = false;
	this.minimumBound = this.maximumBound = this.minimumDate = this.maximumDate = null;
	this.oMinimum = new Date (100, 0, 1);	// Minimum the picker can handle
	this.oMinimum.setFullYear(100);
	this.ELexpression = null;				// An EL expression specified as a string (e.g., "@testValue > 'a'"
	this.ELexpressionImp = null;			// Compiled (executable) instance of the EL expression
	this.msgFail = hX_5._E;

    // Attributes supported on this object
    this.attributeTable = [];
    	this.attributeTable ['required']			= ['required',		hX_5.ARG_IS_TRUE,	false, null, null];
	 	this.attributeTable ['minimum-bound']		= ['minimumBound',	hX_5.ARG_TO_NAN,	false,	null,	null];
	 	this.attributeTable ['min-bound']			= ['minimumBound',	hX_5.ARG_TO_NAN,	false,	null,	null];
	 	this.attributeTable ['maximum-bound']		= ['maximumBound',	hX_5.ARG_TO_NAN,	false,	null,	null];
	 	this.attributeTable ['max-bound']			= ['maximumBound',	hX_5.ARG_TO_NAN,	false,	null,	null];
	 	this.attributeTable	['constraint-expression']=['ELexpression',	hX_5.ARG_TO_NAN,	false,	null, null];
	this.setAttribute(arguments);
}

// Public:  Set an "attribute" on the Validator.  Attributes are passed as strings of the form "name:value"
HxG_5.prototype.DateTimeValidator.prototype.setAttribute = function(a) {

	var wasmax = this.maximumBound, wasmin = this.minimumBound;
	if (hX_5.c.STDDTP_converter==null) hX_5.c.STDDTP_converter =  new hX_5.DateTimeConverter();
	var DTP = hX_5.c.STDDTP_converter;

	hX_5.parseArg (this.attributeTable, this, a);

	// Construct the minimum and maximum bounds
	if (this.minimumBound && wasmin != this.minimumBound)
		this.minimumDate = DTP.constantToValue (this.minimumBound);
	if (this.maximumBound && wasmax != this.maximumBound)
		this.maximumDate = DTP.constantToValue (this.maximumBound);
	if (this.maximumDate && this.minimumDate) {
		if (1 != this.maximumDate.Hx4CompareDate (this.minimumDate, true)) 
			this.minimumBound = this.maximumBound = this.minimumDate = this.maximumDate = null;
	}

	// And the expression
	if (this.ELexpression != null)
		this.ELexpressionImp = hX_5.regExp.registerELConstraint(this.ELexpression, "date");
	return true;
}

// Public: Retrieve the value of an attribute
HxG_5.prototype.DateTimeValidator.prototype.getAttribute = function (attribute) {
	return (hX_5.getArg(this.attributeTable, this, attribute));
}

// Public:	Retrurn the reason the last validate failed as a localized message
HxG_5.prototype.DateTimeValidator.prototype.lastError = function () {
	return ((this.msgFail && this.msgFail.length > 0) ? this.msgFail : null);
}

// Public: Validate a JS Date object according to the validation rules
HxG_5.prototype.DateTimeValidator.prototype.validate = function (testV, iDTP, treatAsDate, treatAsPick) {

	this.msgFail = hX_5._E;
	var r, e, test = null, DTP = (iDTP != null) ? hX_5.getConverterById(iDTP) : null;
	if (DTP == null || !hX_5.isObject(DTP))
		return false;
	var bTCompare = ((treatAsDate) ? false : DTP.isTimeFormat()), bTOnly =  (DTP.isTimeFormat() && !DTP.isDateFormat());

	// If the testvalue is a string or null, determine if required or not
	if (testV==null || hX_5.isString(testV)) {
		// Verify a value is required or not
		r = !(this.required);
		if (testV==null) {
			this.msgFail = (r) ? hX_5._E : hX_5.L[this.lc].ERR_required;
			return r;
		}
		e = hX_5.allTrim (testV);
		if (e.length == 0) {
			this.msgFail = (r) ? hX_5._E : hX_5.L[this.lc].ERR_required;
			return r;
		}
		// Must be a non-empty string, so try converting it to a date
		test = DTP.stringToValue(e);
	} else
		if (testV.Hx4Clone) test = testV.Hx4Clone(2);

	// If cannot interpret value as a date, validation fails
	if (test==null) {
		this.msgFail = hX_5.L[this.lc].ERR_required;
		return false;
	}

	// By convention, if this is a time-only and it has a min/max bound, we use 1/1/1970 as the date 
	if (bTOnly && (this.minimumDate || this.maximumDate)) {
		if (test.getFullYear() > 1970) {
			test.setYear(1970);
			test.setMonth(0);
			test.setDate(1);
		}
		test.setMilliseconds(0);
	}

	// Check against date picker minimum (100 AD)
	if (treatAsPick && this.oMinimum && !bTOnly) {
		if (-1 == test.Hx4CompareDate(this.oMinimum)) {
			this.msgFail = hX_5.L[this.lc].ERR_minimum;
			return false;
		}
	}
	// Check against supplied minimum
	if (this.minimumDate) {
		if (-1 == test.Hx4CompareDate(this.minimumDate, bTCompare)) { 
			this.msgFail = hX_5.L[this.lc].ERR_minimum;
			return false;
		}
	}
	// Check against supplied maximum
	if (this.maximumDate) {
		if (1 == test.Hx4CompareDate(this.maximumDate, bTCompare)) {
			this.msgFail = hX_5.L[this.lc].ERR_maximum;
			return false;
		}
	}
	// Check against the EL expression 
	if (this.ELexpressionImp != null) {
		if (!hX_5.regExp.testELExp(test, this.ELexpressionImp)) {
			this.msgFail = hX_5.L[this.lc].ERR_constraint;
			return false;
		}
	}
	return true;
}

// Public: Force a JS Date object to be valid
HxG_5.prototype.DateTimeValidator.prototype.coerce = function (test, iDTP, treatAsDate, treatAsPick) {
	this.msgFail = hX_5._E;
	var e, o=null, DTP = (iDTP != null) ? hX_5.getConverterById(iDTP) : null;
	if (DTP == null || !hX_5.isObject(DTP)) {
		this.msgFail = hX_5.L[this.lc].ERR_required;
		return false;
	}
	var bTCompare = ((treatAsDate) ? false : DTP.isTimeFormat()), bTOnly =  (DTP.isTimeFormat() && !DTP.isDateFormat());

	// If the testvalue is a string, convert it to a date
	if (test != null && hX_5.isString(test)) {
		e = hX_5.allTrim (test);
		if (e.length > 0) o = DTP.stringToValue(e);
	} else if (test.Hx4Clone) o = test.Hx4Clone(2);

	// If we don't have something we can make into a date, create a date
	if (o == null) o = new Date ();

	// By convention, if this is a time-only and it has a min/max bound, we use
	// 1/1/1970 as the date 
	if (bTOnly && (this.minimumDate || this.maximumDate)) {
		if (o.getFullYear() > 1970) {
			o.setYear(1970);
			o.setMonth(0);
			o.setDate(1);
		}
		o.setMilliseconds(0);
	}
	// Check against date picker minimum (100 AD)
	if (this.oMinimum && !bTOnly && treatAsPick) {
		if (-1 == o.Hx4CompareDate(this.oMinimum))
			o = this.oMinimum.Hx4Clone(2);
	}
	// Check against the minimum
	if (this.minimumDate) {
		if (-1 == o.Hx4CompareDate(this.minimumDate, bTCompare))
			o = this.minimumDate.Hx4Clone(2);
	}
	// Check against supplied maximum
	if (this.maximumDate) {
		if (1 == o.Hx4CompareDate(this.maximumDate, bTCompare))
			o = this.maximumDate.Hx4Clone(2);
	}
	return o;
}

//  ----------------------------------------------------------------------------------------
//  DateTime Assist
//  ----------------------------------------------------------------------------------------
//
//  NOTE:  Changing the converter associated with an assist (e.g., changing the format)
//		   is BAD.  We currently cannot rebuild the value(s) on display that use the assist object
//
//  Methods (common to all assist object)
//
//	onFocus				On focus: coerce the number to be valid, add in prompt chars, set cursor position
//	onBlur				On blur: Remove prompt chars
//	onClick				On click: Handle a mouse click
//	onCut				On cut: Handle cutting text
//	onPaste				On paste: Handle pasting text
// 	onChangeKeyCode		On key:  Handle a function keystroke, e.g., DEL or Backspace
// 	onChangeChrCode		On key:  Handle an alphanumieric keystroke, e.g., "A"
//	onValidate			Validate the string as it stands and return true/false if it succeeds/fails
//	isFilled			For auto-tab checking:  Is the number "filled" so auto-tab should occur
// 	toAssist			Sub-handler of onFocus -- takes a value and adds prompt-chars
// 	fromAssist			Sub-handler of onBlur -- takes a value and removes prompt-chars

HxG_5.prototype.DateTimeAssist = function (_elem, _state, _converter, _validator, _promptChar) {
	if (!_elem || !_converter || !_state) return;
	this.lc = hX_5.currentLocale();

	// Exported functions
	this.isFilled		= isFilled;			// Public
	this.onFocus		= onFocus;			// Public
	this.onBlur			= onBlur;			// Public
	this.onCut			= onCut;			// Public
	this.onPaste		= onPaste;			// Public
	this.toAssist		= toAssist;			// Public
	this.fromAssist		= fromAssist;		// Public
	this.onClick		= onClick;			// Public
	this.onChangeKeyCode= onChangeKeyCode;	// Public
	this.onChangeChrCode= onChangeChrCode;	// Public
	this.onValidate		= onValidate;		// Public
	this.destroy		= destroy;			// Public

	// Basic information kept by assist
	var elem			= _elem;
	var state			= _state;
	var cvt		 		= _converter;
	var validator 		= _validator;
	var pChar 			= _promptChar;
	var wasP			= null;

	// Actual keyboard handling is done by the mask handler sub-object in "masked string"
	var handler	= null;

	// == Start constructor =====================================================

	// Generate the simple and mask formats from the current format
	// This also generates the special "extra" converter used with the simple format
	if (cvt.simpleFormat == null)
		cvt.genAssistMasks(pChar);

	// Build a mask handler that pre-handles keystrokes for this converter
	handler = new hX_5.MaskAssist.prototype.maskHandler(elem, state, cvt, pChar);

	// == End constructor =======================================================

	// == Destructor ============================================================
	function destroy() {
		if (handler!=null) {
			if (handler.destroy) handler.destroy();
			delete handler;
		}
	}

	// ==========================================================================
	// Private:  Use the "subconverter" to convert to/from a "simple formatted" version of the converter
	// 			 Handle the prompting character while we're at it
	function valueToSimpleString (iDate) {
		// If the date "isn't", then make one using prompt-chars
		// If the date "is", then formatting it will use "zeroes" in lieu of prompt chars
		return (iDate == null)?handler.getEmptyMask():cvt.simpleConverter.valueToString(iDate,null,pChar);
	}

	// ==========================================================================
	// Private:  If we got the whole date string (might not if somebody is doing a cut/copy/paste
	// 			 or a multi-char delete), Then handle any prompt chars that were left in it
	function simpleStringToValue (sDate, ending, pos) {
		var r=false, i, j, test=sDate, out=sDate, rv=null, l=sDate.length, empty=true, epoch=(cvt.sFormatTokenized.indexOf("%G")>=0);
		cvt.setCalendar();
		// If the string is empty, we're done
		if (l <= 0) return {reset:false, value:null};

		// Test to see if the date is "empty" in which case we return an empty string
		for (i=0; empty && ending && i<l; i++) {
			if (cvt.maskFormat.charAt(i) == hX_5._LB && test.charAt(i) != pChar && test.charAt(i) != hX_5._B)
				empty = false;
		}
		if (ending && empty) return {reset:false, value:hX_5._E};

		// Pad the value out with the "empty" mask
		if (l < cvt.maskFormat.length)
			test = test + handler.getEmptyMask().substr(l);
		else if (!ending && l > cvt.maskFormat.length)
			return {reset:false, value:null};

		// Now fix up the digits if we can. We do this in two passes.  First pass, compute month, date, year and epoch
		// Second pass, adjust them and build the date value
		var c1, c2, c3, c4, c5, m, m1, intY=-1, intYL=-1, intYT=-1, intM=-1, intD=-1, intH=-1, intMn=-1, intS=-1, strE=hX_5._E, fixH=false, fixD=false, fixM=false, fixY=false, fixG=false, leapT=false;
		// Pass one compute the components
		for (i=0; i < l; i++) {
			// Is this a spot where someone can type?
			if (cvt.maskFormat.charAt(i) == hX_5._LB) {
				// Markers are y (year), m (two digit month), d (two digit day), G (epoch), h (2 digit hour, s (2 digit mins/secs)
				c1 = test.charAt(i);
				m = cvt.maskFormatDigits.charAt(i);
				// Handle epoch markers
				if (m == hX_5._uG) {
					if (c1==pChar || c1==hX_5._B || c1==hX_5._E) {
						strE = cvt.descLbEpoch[cvt.descLbEpoch.length-1][1];
					} else {
						strE = hX_5._E;
						c1 = c1.toUpperCase();
						for (j=0; j < cvt.descLbEpoch.length; j++) {
							if (c1==cvt.descLbEpoch[j][1].charAt(0).toUpperCase()) {
								if (cvt.minEpoch<=1 || test.charAt(i+1).toUpperCase()==cvt.descLbEpoch[j][1].charAt(1).toUpperCase()) {
									strE = cvt.descLbEpoch[j][1];
									break;
								}
							}
						}
						// If we "complete" the epoch, mark this as fixed
						if (strE != hX_5._E) fixG = r = true;
					}
					i += cvt.abbEpochLength;
				// Handle two digit things = m,d,s,h,A 
				} else if (m != hX_5._lY) {
					c1 = cvt.getDigit(c1);
					c2 = ((i+1)<l) ? cvt.getDigit(test.charAt(i+1)) : hX_5._B;
					// We could have ** or *9 or 9* or **
					if (c1 == pChar && c2 == pChar) {
						c1 = hX_5._Z;
						c2 = (m==hX_5._lH || m==hX_5._lS || m==hX_5._uA) ? hX_5._Z:hX_5._1;
					} else if (c1 == pChar && c2 != pChar) {
						c1 = hX_5._Z;
					} else if (c1 != pChar && c2 == pChar) {
						c2 = (c1==hX_5._Z) ? hX_5._1 : c1;
						c1 = hX_5._Z;
					} else if (c1 == hX_5._Z && c2 == hX_5._Z && m != hX_5._lH) {
						c1 = hX_5._Z;
						c2 = (m==hX_5._lH || m==hX_5._lS || m==hX_5._uA) ? hX_5._Z:hX_5._1;
					}
					if (!cvt.matchDigit(c1) || !cvt.matchDigit(c2))
						return {reset:true, value:null};
					j = parseInt(c1+c2,10);
					if (m==hX_5._lH) intH = j;
					else if (m==hX_5._uA) intMn = j;
					else if (m==hX_5._lS) intS = j;
					else if (m==hX_5._lM) intM = j;
					else intD = j;
					i = i+1;
				// Handle years
				} else {
					j = (cvt.isShortYr)?2:4;
					c2 = ((i+1)<l) ? test.charAt(i+1) : hX_5._B;
					c3 = ((i+2)<l) ? test.charAt(i+2) : hX_5._B;
					c4 = ((i+3)<l) ? test.charAt(i+3) : hX_5._B;
					// If we're in the midst of typing, just null out the prompt chars so the date will validate
					if (!ending) {
						if (cvt.isShortYr) {
							if (c1 == pChar || c1 == hX_5._B) c1 = hX_5._1;
							if (c2 == pChar || c2 == hX_5._B) c2 = hX_5._B;
							c3 = c4 = hX_5._E;
						} else {
							c5 = c4;
							if (c1 == pChar || c1 == hX_5._B) c1 = hX_5._2;
							if (c2 == pChar || c2 == hX_5._B) c2 = hX_5._Z;
							if (c3 == pChar || c3 == hX_5._B) c3 = hX_5._Z;
							if (c4 == pChar || c4 == hX_5._B) c4 = hX_5._Z;
							leapT = (c5!=c4);
						}
					// If we're ending construct a proper date if we can
					} else {
						// If we simply use spaces, we're OK if client-side
						// validation is on.  The year is rather a mess if it isn't
						// (but it should fail server-side validation which is OK)
						var arranged=false, today = new Date(), thisyr = today.getFullYear()+hX_5._E;
						if (c1 == pChar && c2 == pChar && c3 == pChar && c4 == pChar) {
							if (cvt.isShortYr) {
								c1 = hX_5._Z; c2 = hX_5._1;	c3 = c4 = hX_5._E;
							} else {
								if (epoch) { c1 = thisyr.charAt(0); c2 = c3 = c4 = hX_5._Z; }
								else { c1 = thisyr.charAt(0); c2 = thisyr.charAt(1); c3 = thisyr.charAt(2); c4 = thisyr.charAt(3); }
							}
						} else {
							if (cvt.isShortYr) {
								if (c1 == pChar || c1 == hX_5._B) c1 = hX_5._Z;
								if (c2 == pChar || c2 == hX_5._B) c2 = hX_5._B;
								c3 = c4 = hX_5._E;
							} else {
								// Handle someone entering only 1, 2 or 3 digits specially
								if (c1 != pChar && c1 != hX_5._B && c4 == pChar) {
									if (cvt.bShowEpoch) {
									// If epoch is shown and we are "short some characters", append zero
									// to the "front" of the date unless blank in which case use "2000"
										arranged = true;
										if (c2==pChar && c3==pChar) {
											c4 = c1; c1 = hX_5._Z; c2 = hX_5._Z; c3 = hX_5._Z;
										} else if (c3==pChar) {
											c3 = c1; c4 = c2; c1 = hX_5._Z; c2 = hX_5._Z;
										} else {
											c2 = c1; c3 = c2; c4 = c3; c1 = hX_5._Z;
										}
									} else {
										// Only special handle 1 or 2 digits if there's no epoch
										if (c3==pChar) {
											var intYr = parseInt(c1+((c2!=pChar)?c2:hX_5._E),10);
											var o = hX_5.cal.twoYearToFourYear(intYr, today, cvt.getBaseYear(today)) + hX_5._E;
											while (o.length < 4) o = hX_5._Z + o;
											c1 = o.charAt(0); c2 = o.charAt(1); c3 = o.charAt(2); c4 = o.charAt(3);
										}
									}
								}
								if (!arranged) {
									if (c1 == pChar || c1 == hX_5._B) c1 = (epoch)?hX_5._Z:thisyr.charAt(0);
									if (c2 == pChar) c2 = hX_5._Z;
									if (c3 == pChar) c3 = hX_5._Z;
									if (c4 == pChar) c4 = hX_5._Z;
								}
							}
						}
					}
					if (!cvt.matchDigit(c1) || !cvt.matchDigit(c2) || !cvt.matchDigit(c3) || !cvt.matchDigit(c4))
						return {reset:true, value:null};
					c1 = cvt.getDigit(c1)+cvt.getDigit(c2)+cvt.getDigit(c3)+cvt.getDigit(c4);
					intYL = intY = parseInt(c1,10);
					// Create a leap year
					intYL = hX_5.cal.nearestLeapYear(intY);
					i += (j-1);
				}
			}
		}

		// Adjust any components that can be adjusted
		if (!ending) {
			m = cvt.maskFormatDigits.charAt(pos);
			m1 = cvt.maskFormatDigits.charAt(pos+1);
			intYT = (intY>=0)?((leapT) ? intYL : intY):intY;
			// Deal with the "0***" case (should now be OK above)
			if (1==2 && intY==0) {
				intY = 1;
				intYT = intYL = intY;
				fixY = r = true;
			}
			// First off, we have the case where a month is "03" and the user is trying to type "12"
			if (intM >= 0 && m==hX_5._lM && m1==hX_5._lM && intM > hX_5.cal.monthsInYear(intYT) && (Math.floor(intM/10) <= Math.floor(hX_5.cal.monthsInYear()/10))) {
				intM = hX_5.cal.monthsInYear(intYT);
				fixM = r = true;
			}
			// Similar thing can happen with hours
			if (intH >=0 && m==hX_5._lH && m1==hX_5._lH && intH > 23 && (Math.floor(intH/10) <= 2)) {
				intH = 23;
				fixH = r = true;
			}
			// Given a "fixed" month, days can be "wrong" relative to other things
			if (intD>=0 && intD > 0 && intM > 0 && intM <= hX_5.cal.monthsInYear()) {
				j = hX_5.cal.daysInMonth(intM-1,intYT);
				if (intD > j) {
					// Is this the case where a day is "29" and the user is trying to type "30"
					if (m==hX_5._lD && m1==hX_5._lD && (Math.floor(intD/10) <= Math.floor(hX_5.cal.maxDaysInMonth()/10))) {
						intD = hX_5.cal.daysInMonth(intM-1,intYT);
						fixD = r = true;
					// If days are wrong, we can adjust the days or month based on where the user was typing
					// If we're typing anything but a day, adjust the date
					} else if (intD <= hX_5.cal.maxDaysInMonth()) {
						if (m != hX_5._lD) {
							intD = hX_5.cal.daysInMonth(intM-1,intYT);
							fixD = r = true;
						// If we're typing the date, we could adjust the month or rarely the year -- see if we adjust the year (to a leap year) or the month
						} else if (intYL != intY && intD <= hX_5.cal.daysInMonth(intM-1,intYL)) {
							intY = intYL;
							fixY = r = true;
						// Means the month is wrong, find the "next" month where this day is OK
						} else {
							for (j=intM+1; j!=intM; j++) {			
								j = (j <= hX_5.cal.monthsInYear(intYT)) ? j:1;
								if (intD <= hX_5.cal.daysInMonth(j-1,intYT)) {
									intM = j;
									fixM = r = true;
									break;
								}
							}
						}
					}
				}
			}
		}

		// Pass two, re-assemble
		for (i=0; i < l; i++) {
			if (cvt.maskFormat.charAt(i) == hX_5._LB) {
				m = cvt.maskFormatDigits.charAt(i);
				if (m == hX_5._uG) {
					test = test.substr(0,i) + strE + test.substr(i+cvt.abbEpochLength);
					if (fixG) out = out.substr(0,i) + strE + out.substr(i+cvt.abbEpochLength);
					i += cvt.abbEpochLength;
				} else if (m != hX_5._lY) {
					j = (m==hX_5._uA)?intMn:((m==hX_5._lS)?intS:((m==hX_5._lH)?intH:((m==hX_5._lM)?intM:((m==hX_5._lD)?intD:-1))));
					if (j >= 0) {
						c1 = hX_5.padL(j+hX_5._E,2,cvt.getZeroDigit());
						test = test.substr(0,i) + c1 + test.substr(i+2);
						if ((fixD && m==hX_5._lD) || (fixM && m==hX_5._lM) || (fixH && m==hX_5._lH)) {
							if (out.charAt(i)  != c1.charAt(0)) c1 = hX_5.rplOne(c1,0,pChar);
							if (out.charAt(i+1)!= c1.charAt(1)) c1 = hX_5.rplOne(c1,1,pChar);
							out = out.substr(0,i) + cvt.cvtToDigits(c1) + out.substr(i+2);
						}
					}
					i += 1;
				} else {
					j = (cvt.isShortYr)?2:4;
					c1 = hX_5.padL(((leapT&&!fixY)?intYL:intY)+hX_5._E,j,cvt.getZeroDigit());
					test = test.substr(0,i) + c1 + test.substr(i+j);
					if (fixY) {
						if (out.charAt(i)  != c1.charAt(0)) c1 = hX_5.rplOne(c1,0,pChar);
						if (out.charAt(i+1)!= c1.charAt(1)) c1 = hX_5.rplOne(c1,1,pChar);
						if (j>2) {
							if (out.charAt(i+2)!= c1.charAt(0)) c1 = hX_5.rplOne(c1,2,pChar);
							if (out.charAt(i+3)!= c1.charAt(1)) c1 = hX_5.rplOne(c1,3,pChar);
						}
						out = out.substr(0,i) + cvt.cvtToDigits(c1) + out.substr(i+j);
					}
					i += (j-1);
				}
			}
		}
		rv = cvt.simpleConverter.stringToValue(test);
		if (r && !ending && rv) rv = out;
		r = (rv==null)?false:r;
		return {reset:r,value:rv};
	}

	// ==========================================================================
	// Public:  See if the value is "filled" according to the mask
	function isFilled (curpos) {
		var l = elem.value.length;
		if (l < cvt.maskFormat.length)
			return false;
		// Is the last character present?
		var c = elem.value.charAt(cvt.maskFormatLast);
		// And are we past it ?
		return (curpos > cvt.maskFormatLast && c != pChar && c != hX_5._B);
	}

	// ==========================================================================
	// Public: Put a field in "assist" mode
	function onFocus (evt) {
		// To go into assist mode, we have to start with a valid value
		var newValue = toAssist(elem.value);
		// Set the value and the cursor (directly, not into state which isn't set yet)
		elem.value = (newValue.v==null) ? hX_5._E : newValue.v;
		hX_5.sel.setCaretPos(elem, cvt.maskFormatFirst);
		return true;
	}

	// ==========================================================================
	// Public: Add in prompt chars
	function toAssist (v) {
		var newValue = (v != null) ? hX_5.Trim(v) : null;
		newValue = (newValue != null && newValue.length!=undefined && newValue.length > 0) ? cvt.stringToValue(newValue) : null;
		newValue = valueToSimpleString(newValue);
		return {v:newValue, i:null};
	}

	// ==========================================================================
	// Public: Take a field out of "assist" mode
	function onBlur (evt) {
		var v = fromAssist(elem.value);
		if (v != null) {
			elem.value = v;
			return true;
		}
		return false;
	}

	// ==========================================================================
	// Public: Remove prompt-chars from a field
	function fromAssist (v) {
		if (v.length==0) return hX_5._E;
		var newValue = simpleStringToValue(v, true, 0).value;
		return ((newValue == null || (newValue.length!=undefined && newValue.length==0)) ? hX_5._E : cvt.valueToString(newValue));
	}

	//====================================================================================
	// Public: Handle clicking in the field
	function onClick (evt) {
		return handler.onClick(evt);
	}

	//====================================================================================
	// Public: Handle cutting in the field
	function onCut (evt) {
		return handler.onCut(evt);
	}

	//====================================================================================
	// Public: Handle pasting in the field
	function onPaste (evt) {
		return handler.onPaste(evt);
	}

	//====================================================================================
	// Public: Keystroke-level validation of a "function key"
	function onChangeKeyCode (evt) {
		wasP = state.getPos();
		return handler.onChangeKeyCode (evt);
	}

	//====================================================================================
	// Public: Keystroke-level validation of a "character"
	function onChangeChrCode (evt) {
		wasP = state.getPos();
		// transmute western to non-western digits
		cvt.digitTransmute(evt);
		return handler.onChangeChrCode (evt);
	}

	// ==========================================================================
	// Public: Keystroke-level validation of a datetime
	function onValidate () {
		// Round-trip the value through the converter -- this fails when there's
		// a bad char in the string
		var reset, i, v=state.getValue(), nV=simpleStringToValue(v, false, wasP);
		if (nV.value != null) {
			if (nV.reset) {
				if (v!=nV.value)state.setValue(nV.value);
				return true;
			} else {
				nV = valueToSimpleString(nV.value);
				if (nV != null) return true;
			}
		}
		return false;
	}

}
