/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2006 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

/// ------------------------------------------------
// JSF Menu and JSFMenuItem
//
// JSF MenuItems are used to build a "tree" of menus and menu items much like
// a Windows menu structure (that is each item can either be a "node" that performs
// an action or it can be a node that contains other items).
//
// JSF Menu marks where on the page the menuitem tree is visualized and defines
// the look&feel of the menu
//
// Use pattern:
//
//	hX_5.addComponent("menu1", new JSFMenu(args)); <- "menu1" is a <div> on the page
//	hX_5.addSubComponent("menu1", "menu-item-1", new JSFMenuItem(args)); 
//	hX_5.addSubComponent("menu1", "menu-item-2", new JSFMenuItem(args)); 
//	hX_5.addSubComponent("menu1", "menu-item-3", new JSFMenuItem(args)); 
//	hX_5.addSubComponent("menu-item-1", "menu-item-1-1",new JSFMenuItem(args)); 
//	hX_5.addSubComponent("menu-i tem-1", "menu-item-1-2", new JSFMenuItem(args)); 
//	hX_5.addSubComponent("menu-item-1", "menu-item-1-2", new JSFMenuItem(args)); 
//  ...
//  In other words, put a div in the page where the menu bar is to go.  Make one call
//  to add a JSF Component of type JSFMenu that ties the menu to that div.  Then for 
//  each menu item make an addSubComponent call that adds an item either to the "root"
//  (the jsfMenu) or to another (existing) menu item.  The order you add MenuItems
//  determines the order of the things in a menu (e.g., if you add two menu items to
//  menuItem3, then menuItem3 has a child-menu associated with it and that menu has two
//  items in it, in the order they were added.
// 
// Each menu is visualized the same way using a div and table.  For example, if 
// horizontal it's:
//	<div><table><tr>
//		<td>menu-item-1</td>
//		<td>menu-item-2</td>
//		<td>menu-item-3</td>
//	</tr></table></div>
//
//  A vertical menu is the same except it's one column instead of one row.
//
//  The divs are then either put on the page (and hidden/shown as need be)
//  OR they're put in popups (and hidden/shown as need be) depending on the
//  style assigned to the JSFMenu.
//

// ------------------------------------------------
// Public:  The menu object
// ------------------------------------------------

// ------------------------------------------------
// Public: Constructor
HxG_5.prototype.JSFMenu = function () {
	this.type = hX_5.CMP_menu;

	// General display information set by caller

	// Flyopen determines if when you mouse over a menu item that has children
	// (i.e., that has a sub-menu associated with it), the sub-menu is shown.
	// That is show sub-menu on mouse-over.  If not set, the sub-menu is
	// shown when you click on the menu-item.  
	// isFlyOpen controls the top-level-menu's behavior, isBodyFlyOpen
	// all other levels.
	this.flyOpen = true, this.bodyFlyOpen = true;
	// TopVertical determines if the top level menu is displayed vertically or 
	// horizontally.  BodyVertical sets the style of all other levels
	this.topVertical = false, this.bodyVertical = true;
	// For any menus that ARE NOT flyopen, they stay on display 
	// until another menu is clicked open
	this.openSticky = false;
	// If a menu is "tree-style", it's vertical AND the sub-menus are shown
	// "in-line" (i.e., the menu opens up to show the sub-menus as children)
	// instead of displaying the sub-menus in pop-ups
	this.openInPlace = false;
	// Initially selected menu-item (the id of the menu-item)
	this.deepSel = false;
	this.initSel = this.initSelField = null;
	// When a selection is made, update the following target field with the value
	this.target = this.targetObj = null;
	// Explicitly set tab index of the entire menu bar.  Needed cuz Netscape is unhappy
	// if you set the tab index on the div that contains the tree
	this.tabIndex = 0;
	// Function to call 'onchange'
	this.fcn = null;
	// Form that contains the menu bar
	this.form = null;
	// Positioning information -- if relative is set top level is treated as a "popup"
	this.relative = null;
	this.mousePt = this.relativeTo = this.relativeToObj = null;
	this.testRect = new hX_5.HxRect();

	// Accessor functions for some of the above (which can be computed)
	this.isFlyOpen		= function() { return this.flyOpen;	}	
	this.isBodyFlyOpen	= function() { return this.bodyFlyOpen;	}
	this.isFly			= function(l){ return ((l<=0)?this.flyOpen:this.bodyFlyOpen); }
	this.isTopVertical	= function() { return this.topVertical;	}
	this.isBodyVertical	= function() { return this.bodyVertical; }
	this.isVertical		= function(l){ return ((l<=0)?this.topVertical:this.bodyVertical); }
	this.isOpenSticky	= function() { return this.openSticky; }
	this.isOpenInPlace	= function() { return this.openInPlace; }
	this.isRelative		= function() { return (this.relative!=null); }

	// A menu may also be a "menu-item", so create the menu helper class
	this.m = new hX_5.JSFSubMenu();
	
	// State of the menu tree.  At any point in time, one branch of the
	// tree may have the "mouse on" in it (from the root down to the level
	// that is active).  Equally, at any point in time, one branch of the
	// tree may have selection shown in it.  Finally, branch of the tree
	// will be "open" or not.  In a non-sticky menu, that branch will
	// be the "mouse" branch.  In a sticky menu that branch will be the
	// selection branch.  The terminal level may or may not be actually "open"
	// depending on other conditions.
	function st () {			// Object that describes the state of a level
		// Mouse branch, menu-item at this level with mouse over on it
		// Selection branch, menu-item at this level with selection
		// Mouse/Sel branch, menu-item at this level that's "open"
		this.mouseObj = this.selObj =  this.selOpen = null;	
	}
	this.state = [];			// There will be one st element for each level of the tree
	this.focusObj = null;		// One leaf element may or may not have focus (for keyboarding)
	this.setFocusObj = false;	// True if we've just done a set focus
	this.clickRelease = null;	// Let a click thru

	// Description of the classes which define the menu's look
	this.baseClass = null;
	function cl (max) {
		this.div = this.tab = this.sepL = this.sep = this.sepF = this.sepL = null;
		this.cell = new Array(max);
		this.cellAlt = new Array(max);
		this.cellF= new Array(max);
		this.cellL= new Array(max);
		this.item = new Array(max);
		this.itemAlt = new Array(max);
		this.arrow= new Array(max);
		this.arrowImg = new Array(max);
	}
	this.classes = [];

	// Init arrays
	for (var i=0; i < this.MAXLEVELS; i++) {
		this.state.push(new st());
		this.classes.push(new cl(this.MAXSTATES));
	}

	// Scratch object used when dynamically rebuilding menu
	this.scratch = null;

	// Dispatch optimization
	this.lastMouseMove = null;
	this.lastMouseX = this.lastMouseY = -1;

	// Attributes
    this.attributeTable = [];
    	this.attributeTable ['vertical']		= ['topVertical',	hX_5.ARG_IS_TRUE,	false,	null,	null];
    	this.attributeTable ['vertical-body']	= ['bodyVertical',	hX_5.ARG_IS_TRUE,	false,	null,	null];
    	this.attributeTable ['fly-open']		= ['flyOpen',		hX_5.ARG_IS_TRUE,	false,	null,	null];
    	this.attributeTable ['fly-open-body']	= ['bodyFlyOpen',	hX_5.ARG_IS_TRUE,	false,	null,	null];
    	this.attributeTable ['open-sticky']		= ['openSticky',	hX_5.ARG_IS_TRUE,	false,	null,	null];
    	this.attributeTable ['open-in-place']	= ['openInPlace', 	hX_5.ARG_IS_TRUE,	false,	null,	null];
    	this.attributeTable ['initial-selection']=['initSel', 		hX_5.ARG_TO_NAN,	false,	null,	null];
    	this.attributeTable ['select-all']		=['deepSel', 		hX_5.ARG_IS_TRUE,	false,	null,	null];
    	this.attributeTable ['tab-index']		=['tabIndex',		hX_5.ARG_TO_NUM,	false,  -1, 	10000];
    	this.attributeTable ['tabIndex']		=['tabIndex',		hX_5.ARG_TO_NUM,	false,  -1, 	10000];
		this.attributeTable ['onchange']		=['fcn',			hX_5.ARG_TO_FCNS,	false,	null,	null];
		this.attributeTable ['function']		=['fcn',			hX_5.ARG_TO_FCNS,	false,	null,	null];
    	this.attributeTable ['target']			=['target', 		hX_5.ARG_TO_NAN,	false,	null,	null];
    	this.attributeTable ['relative']		=['relative',		hX_5.ARG_TO_NAN,	false,	null,	null];
    	this.attributeTable ['relative-to']		=['relativeTo', 	hX_5.ARG_TO_NAN,	false,	null,	null];
	this.setAttribute(arguments);
}
HxG_5.prototype.J_M = HxG_5.prototype.JSFMenu;

// ==== Literals associated with JSF Menus
HxG_5.prototype.JSFMenu.prototype.NORMAL = 0;
HxG_5.prototype.JSFMenu.prototype.MOUSED = 1;
HxG_5.prototype.JSFMenu.prototype.SELECTED = 2;
HxG_5.prototype.JSFMenu.prototype.SELECTED_MOUSED = 3;
HxG_5.prototype.JSFMenu.prototype.DISABLED = 4;
HxG_5.prototype.JSFMenu.prototype.MAXSTATES = 5;
HxG_5.prototype.JSFMenu.prototype.MAXLEVELS = 5;
HxG_5.prototype.JSFMenu.prototype.ARROW_1 = "&gt;&nbsp;";
HxG_5.prototype.JSFMenu.prototype.ARROW_2 = "&gt;&gt;";
HxG_5.prototype.JSFMenu.prototype.KW_MOUSE = "MOUSE";
HxG_5.prototype.JSFMenu.prototype.KW_BL = "BOTTOM-LEFT";
HxG_5.prototype.JSFMenu.prototype.KW_BR = "BOTTOM-RIGHT";
HxG_5.prototype.JSFMenu.prototype.KW_TR = "TOP-RIGHT";
HxG_5.prototype.JSFMenu.prototype.KW_TL = "TOP-LEFT";

// ==== PUBLIC functions all JSF components support ====

// Public: Page level onLoad handler
HxG_5.prototype.JSFMenu.prototype.onPageLoad = function () {

	// Build the table for the menu if needed
	if (!this.HTMLrendered && this.DOMobj) {

		// Record the form
		this.form = hX_5.g.findContainer(this.DOMobj, hX_5._FRM);
		this.form = (this.form==null) ? document.body : this.form;

		// Make a scratch object
		var sel = null, i, e = document.createElement(hX_5._DIV);
		e.setAttribute(hX_5._ID, this.id+"_scratch");
		e.style.display = hX_5._NO;
		e.style.position = hX_5._ABS;
		e.style.left = e.style.top = hX_5._ZPX;
		this.scratch = this.form.insertBefore (e, (this.form.firstChild));

		// Build the menu (if the menu has no children, hide it instead)
		this.baseClass = hX_5.getFirstClassName(this.DOMobj);
		if (!this.m.itemArray || this.m.itemArray.length <= 0) {
			this.DOMobj.style.display = hX_5._NO;
		} else {
			this.buildNode();

			// Mark the target
			if (this.target) {
				this.targetObj = hX_5.getElementById(this.target);
				if (this.targetObj) if (this.targetObj.tagName != hX_5._INP) this.targetObj = null;
				if (this.targetObj) sel = this.findTarget(this.targetObj.value);
			}
			// Mark the initial selection
			this.initSelField = hX_5.getElementById(this.id + "_selection");
			if (!this.targetObj) {
				if (this.initSelField) sel = hX_5.getSubComponentById(this.initSelField.value);
				if (sel==null) sel = (this.initSel==null) ? null: hX_5.getSubComponentById(this.initSel);
			}
			if (sel) this.setValue(sel);
		}
	}
	// Redraw to makes it all happen 
	this.redraw();
	return true;
}

// Public:  Redraw the image incorporating any changes to the HTML of the image and or attributes
HxG_5.prototype.JSFMenu.prototype.redraw = function (obj) {
	// If nothing passed in, redraw the menu components
	// Otherwise just redraw the object passed in
	if (obj) {
		var c = (obj && obj.id) ? hX_5.getComponentById(obj.id) : null;
		if (c && c.redraw) c.redraw();
	} else
		this.m.redraw();
	return true;
}

// Public:  Release the ui state of the menu bar
HxG_5.prototype.JSFMenu.prototype.uirelease = function (iLevel) {
	var i, v, oM, oS, level = (iLevel == null) ? 0 : iLevel;

	// Release the mouse on all levels (unless one is specified)
	for (i = this.MAXLEVELS-1; i >= level; i--) {
		oM = this.state[i].mouseObj;
		oS = this.state[i].selObj;
		if (oM != null) {
			// Sneak in a release of a commandButton or like hxcomponent
			if (oM.isComponent && oM.isComponent.uirelease) oM.isComponent.uirelease();
			// Release slide-rights (unless sticky)
			v = (oS!= null && oS.id == oM.id && (this.isOpenSticky() || this.deepSel || oS.isNoNode() || !oS.isNode()))?this.SELECTED:this.NORMAL;
			oM.setState(this.NORMAL, false, (this.isOpenSticky())?false:true);
			if (v != this.NORMAL) oS.setColorState(v);
			this.state[i].mouseObj = null;
		}
		oM = this.state[i].selOpen;
		if (!this.isOpenSticky() && oM!=null) {
			v = (oS != null && oS.id == oM.id && (this.isOpenSticky() || this.deepSel || oS.isNoNode() || !oS.isNode()))?this.SELECTED:this.NORMAL;
			oM.setState(this.NORMAL, false, (this.isOpenSticky())?false:true);
			if (v != this.NORMAL) oS.setColorState(v);
			this.state[i].selOpen = null;
		}
	}

	// Release focus and the open submenus (unless sticky)
	if (!this.isOpenSticky()) {
		// If we might have focus in a menu, throw it to the root of the menu
		if (this.focusObj && this.focusObj.level > 0 && this.focusObj.rootNum != null) {
			var test = this.m.itemArray[this.focusObj.rootNum];
			if (test.ITEMobj) hX_5.i.setFocus(test.ITEMobj, false, true);
		}
		this.focusObj = null;
		this.setFocusObj = false;
	}
	return true;
}

// Public:  Release the ui state of the menu bar
HxG_5.prototype.JSFMenu.prototype.destroy = function () {
	var o;
	if (this.m!=null) {
		if (this.m.destroy) this.m.destroy(); // JSFSubMenu
		delete this.m;
	}
	if (this.scratch!=null) {
		this.scratch.parentNode.removeChild(this.scratch);
		delete this.scratch;
	}
	while (this.state.length > 0) {
		o = this.state.pop();
		delete o;
	}
	while (this.classes.length > 0) {
		o = this.classes.pop();
		while (o.cell.length > 0) o.cell.pop();
		while (o.cellAlt.length > 0) o.cellAlt.pop();
		while (o.cellF.length > 0) o.cellF.pop();
		while (o.cellL.length > 0) o.cellL.pop();
		while (o.item.length > 0) o.item.pop();
		while (o.itemAlt.length > 0) o.itemAlt.pop();
		while (o.arrow.length > 0) o.arrow.pop();
		while (o.arrowImg.length > 0) o.arrowImg.pop();
		delete o;
	}
	delete this.state;
	delete this.classes;
	delete this.attributeTable;
	if (this.testRect!=null) delete this.testRect;
	return true;
}

// Public: Retrieve the state (selection)
// Returns: The leafmost object with selection
HxG_5.prototype.JSFMenu.prototype.getValue = function () {
	for (var i = this.MAXLEVELS-1; i >= 0; i--)
		if (this.state[i].selObj != null) return (this.state[i].selObj.id);
	return null;
}

// Public: Retrieve the state (mouse)
// Returns: The leafmost object with selection
HxG_5.prototype.JSFMenu.prototype.getMouseState = function () {
	for (var i = this.MAXLEVELS-1; i >= 0; i--)
		if (this.state[i].mouseObj) return (this.state[i].mouseObj);
	return null;
}

// Public:  Set the state (selection) of the menu tree
// Sets the specified item to "selected"
HxG_5.prototype.JSFMenu.prototype.setValue = function (obj) {
	if (obj && (hX_5.isString(obj) || (obj.type && obj.type == hX_5.CMP_mapitem))) {
		var sel = hX_5.getSubComponentById(obj);
		if (sel) this.setState(sel, this.SELECTED, true, true, true);
	}
}

// Private:  Set the state (selection) of the menu tree
HxG_5.prototype.JSFMenu.prototype.setState = function (obj, iValue, click, force, initial, evt) {
	var d, i, p, v, sel, value = (arguments.length > 1) ? iValue : this.MOUSED;
	// Ignore separators and/or disabled items
	if (!obj || obj.isSeparator)
		return;
	if (obj.isDisabled) {
		obj.setState(this.DISABLED, false, false, false);
		return;		
	}
	// Ignore click actions in the flyover case if non-terminal
	if (!force && click && this.isFly(obj.level) && obj.isNode())
		return;

	// Make sure everything "below" and "above" is is set right
	if (obj && obj.level!=undefined) {
		// Release everything below
		this.uirelease(obj.level+1);
		// Make sure everything "above" is set right.  The mouse is right
		// by definition.  Selection could be off when sticky
		// For the mouse -- we assume the mouse above this level is in the same branch
		// For selection -- if this is a click, we do resync things in the sticky case or when init'ing
		if (click || force) {
			p = obj;
			for (i=p.level-1; i>=0; i--) {
				if (p && p.p && p.p.type==hX_5.CMP_mapitem) {
					p = p.p;
					if (this.state[i].selObj == null || this.state[i].selObj.id != p.id) {
						if (this.state[i].selObj != null) this.state[i].selObj.setColorState(this.NORMAL);
						this.state[i].selObj = p;
						this.state[i].selObj.setState(this.SELECTED, true, true, initial, ((evt)?evt:null));
					}
				}
			}
		}
	}

	// Set this level, start by clearing the mouse if it's elsewhere on this level
	v = value;
	sel = (this.state[obj.level].selObj != null) ? this.state[obj.level].selObj.id : "";
	if (value == this.MOUSED) {
		if (this.state[obj.level].mouseObj != null && this.state[obj.level].mouseObj.id != obj.id) {
			v = (click) ? this.NORMAL : ((sel == this.state[obj.level].mouseObj.id) ? this.SELECTED : this.NORMAL);
			this.state[obj.level].mouseObj.setColorState(v);
			if (this.state[obj.level].mouseObj.isComponent && this.state[obj.level].mouseObj.isComponent.uirelease)
				this.state[obj.level].mouseObj.isComponent.uirelease();
		}
		// If this is fly-over, we clear visual selection as well if sticky
		// This behavior is questionable == we may want to control this with an attribute
		if (this.isFly(obj.level) && this.isOpenSticky() && this.state[obj.level].selObj!=null) {
			this.state[obj.level].selObj.setColorState(this.NORMAL);
			this.state[obj.level].selObj = null;
		}
		// Now set the new mouse
		v = (value == this.MOUSED && (sel != obj.id)) ? value : this.SELECTED_MOUSED;
		this.state[obj.level].mouseObj = obj;
	}
	// Now set the new selection
	if (value == this.SELECTED) {
		// Unset the old
		if (this.state[obj.level].selObj != null && sel != obj.id)
			this.state[obj.level].selObj.setColorState(this.NORMAL);
		v = (click && !force) ? this.SELECTED_MOUSED : this.SELECTED;
		this.setField(obj);
		this.setTarget(obj);
		this.state[obj.level].mouseObj= this.state[obj.level].selObj = obj;
	}
	// Don't set a selection to unselected
	if (value == this.NORMAL)  v = (sel == obj.id) ? this.SELECTED : v;
	
	// Before we go further, make sure we uncolor selection above us if this isn't "deep" selection
	if (!this.deepSel) {
		for (d=0; d < this.MAXLEVELS; d++)
			if (this.state[d].selObj==null) break;
		for (i=0; i < (d-1); i++)
			this.state[i].selObj.setColorState(this.NORMAL);
		if (v == this.SELECTED_MOUSED && obj.level < (d-1))	v = value;
		if (value == this.NORMAL && obj.level < (d-1)) v = this.NORMAL;
	}

	// Now color it and open it as needed
	obj.setState(v, click, force, initial, ((evt)?evt:null));

	// Re-record the current selection (in case we missed it above)
	this.setField(this.getValue());
	this.setTarget(this.getValue());
	// Clear focus -- if we're keyboarding, we'll set it after a setState call
	// If we switched to the mouse, this will revert it to the menu bar itself
	this.focusObj = null;
	this.setFocusObj = false;
}

// Private:  Update the hidden selected field indicator
HxG_5.prototype.JSFMenu.prototype.setField = function (obj) {
	if (obj && this.initSelField) {
		var sel = hX_5.getSubComponentById(obj);
// fixme:  Is this right or is it (isNode && sel.isNoNode)		
		if (sel != null && (sel.isNoNode() || !sel.isNode())) {
			if (sel.id != this.initSelField.value) {
				if (this.initSelField.disabled) this.initSelField.disabled = false;
				this.initSelField.value = sel.id;
			}
		}
	}
}
// Private:  Update the target input field
HxG_5.prototype.JSFMenu.prototype.setTarget = function (obj) {
	if (obj && this.targetObj) {
		var sel = hX_5.getSubComponentById(obj);
// fixme: review the following line as above		
		if (sel != null && (!sel.isNoNode() && !sel.isNode())) {
			var v = sel.findValue();
			if (v != this.targetObj.value) this.targetObj.value = v;
		}
	}
}

// Private:  Find a menu item that corresponds to a target value
HxG_5.prototype.JSFMenu.prototype.findTarget = function (value) {
	var i, item = null;
	// Scan each menu item looking for a match
	for (i = 0; item == null && i < this.m.itemArray.length; i++)
		item = this.m.itemArray[i].findTarget(value);
	return item;
}

// Public:  Set keyboard focus to a menu item.  Optionally set "mouse focus" as well
HxG_5.prototype.JSFMenu.prototype.setFocus = function (obj, mouse) {
	// Set this level
	this.focusObj = null;
	if (obj && obj.ITEMobj) {
		this.focusObj = obj;
		this.setFocusObj = true;
		hX_5.i.setFocus(obj.ITEMobj, false, true);
	}
	if (mouse!=null && mouse)
		this.setState (this.focusObj, this.MOUSED, false, false, false, null);
}

HxG_5.prototype.JSFMenu.prototype.onPreLoad = function (obj, evt) {
	this.showMenu(evt);
	return false;
}
HxG_5.prototype.JSFMenu.prototype.onPreUnLoad = function (obj, evt) {
	this.hideMenu();
	return false;
}

// Public: Show the top-level menu, positioning it if it's relatively positioned
HxG_5.prototype.JSFMenu.prototype.showMenu = function(evt, focus) {
	this.DOMobj.style.display = hX_5._BLK;
	if (this.isRelative()) {
		var T=this.DOMobj.firstChild,X=0,Y=0,popTW,popW=hX_5.g.getRendWidth(this.DOMobj),popTH,popH=hX_5.g.getRendHeight(this.DOMobj),winH=hX_5.g.getWindowClientHeight(),winW=hX_5.g.getWindowClientWidth();
		// The popup width can be screwed up, so compare the table with the div
		while (T!=null && T.tagName == hX_5._INP) T = T.nextSibling;	// Skip over hidden selection field
		if (T) {
			popTW=hX_5.g.getRendWidth(T);
			popTH=hX_5.g.getRendHeight(T);
			if (popW>popTW+20) popW = popTW;
			if (popH>popTH+20) popH = popTH;
		}
		if (evt && this.relative==this.KW_MOUSE) {
			var pt = hX_5.g.toPoint(evt);
			X = pt.X; Y = pt.Y;
			if ((X+popW)>winW) X = X-popW;
			if ((Y+popH)>winH) Y = Y-popH;
		} else if (this.relative!=this.KW_MOUSE && this.relativeToObj!=null) {
			this.testRect.setRect(this.relativeToObj);
			if (this.relative==this.KW_TR) {
				X = this.testRect.X + this.testRect.width;
				Y = this.testRect.Y;
				if ((X+popW)>winW) X = winW-popW;
				if ((Y+popH)>winH) Y = winH-popH;
			} else if (this.relative==this.KW_TL) {
				X = this.testRect.X - popW;
				Y = this.testRect.Y;
				if ((X+popW)>winW) X = winW-popW;
				if ((Y+popH)>winH) Y = winH-popH;
			} else if (this.relative==this.KW_BR) {
				X = this.testRect.X + this.testRect.width - popW;
				Y = this.testRect.Y + this.testRect.height;
				if ((X+popW)>winW) X = winW-popW;
				if ((Y+popH)>winH) Y = this.testRect.Y-popH;
			} else {
				X = this.testRect.X;
				Y = this.testRect.Y + this.testRect.height;
				if ((X+popW)>winW) X = winW-popW;
				if ((Y+popH)>winH) Y = this.testRect.Y-popH;
			}
		}
		if (X<0) X = 0;
		if (Y<0) Y = 0;
		hX_5.g.setP(this.DOMobj, hX_5._POS, hX_5._ABS);
		hX_5.g.setP(this.DOMobj, hX_5._LFT, X+hX_5._PX);
		hX_5.g.setP(this.DOMobj, hX_5._TOP, Y+hX_5._PX);
	}
	this.redraw();
	if (focus!=null && focus) this.setFocus(this.m.firstItem, true);
}

// Public: Hide the top-level menu
HxG_5.prototype.JSFMenu.prototype.hideMenu = function() {
	this.DOMobj.style.display=hX_5._NO;
	this.redraw();
}

// Public: Set an "attribute" on the behavior
HxG_5.prototype.JSFMenu.prototype.setAttribute = function(a) {
	hX_5.parseArg (this.attributeTable, this, a);
	if (this.isOpenInPlace())
		this.openSticky = true;
	if (this.relative!=null) {
		this.relative = this.relative.toUpperCase();
		this.relativeToObj = (this.relativeTo!=null)?hX_5.getElementById(this.relativeTo):null;
	}
	if (this.target!=null && this.targetObj==null) {
		var sel;
		this.targetObj = hX_5.getElementById(this.target);
		if (this.targetObj) if (this.targetObj.tagName != hX_5._INP) this.targetObj = null;
		if (this.targetObj) sel = this.findTarget(this.targetObj.value);
		if (sel) this.setValue(sel);
	}
	// Redraw
	if (this.HTMLrendered) {
		this.buildNode(true);
		this.redraw();
	}
	return true;
}

// Public: Retrieve the value of an attribute
HxG_5.prototype.JSFMenu.prototype.getAttribute = function (attribute) {
	return (hX_5.getArg(this.attributeTable, this, attribute));
}

// Public: Add an item to the array
HxG_5.prototype.JSFMenu.prototype.addSubComponent = function(obj, position) {
	if (this.DOMobj.style.display == hX_5._NO) this.DOMobj.style.display = hX_5._E;
	this.m.addSubComponent (this, obj, position);
	if (this.HTMLrendered) {
		this.buildNode();
		this.redraw(obj);
	}
}

// Public: Remove an item from the array
HxG_5.prototype.JSFMenu.prototype.removeSubComponent = function(objid) {
	this.m.removeSubComponent (this, objid);
	if (this.HTMLrendered) {
		this.buildNode();
		this.redraw(obj);
	}
}

// Public: Given a DOM obj id, return the menu item that contains it
HxG_5.prototype.JSFMenu.prototype.getMenuItemById = function (inid) {
	var v,mi,id;
	id = (inid && hX_5.isObject(inid) && inid.id!=undefined)?inid.id:inid;
	if (id) {
		for (var v in hX_5.SubComponents) {
			mi = hX_5.SubComponents[v];
			if (mi && mi.itemName!=null && mi.itemName==id) return (mi.id);
		}
	}
	return null;
}

// ==== PRIVATE functions ====

// Private: Build all HTML needed for the menu tree
HxG_5.prototype.JSFMenu.prototype.buildNode = function(noNewChildren) {
	// Render the "static" div (top-level menu)
	this.setClasses();
	this.m.POPobj = null;
	this.m.POPdiv = this.DOMobj;
	// Build the menu for the root
	this.m.buildMenu(this, this, 0, noNewChildren);
	// Walk through all menu items and build their nodes
	for (var i=0; i < this.m.itemArray.length; i++)
		this.m.itemArray[i].buildNode(noNewChildren);
}
	
// Private: Set the class names we use
HxG_5.prototype.JSFMenu.prototype.setClasses = function () {
	var i, t, t1, t2, t3, p, k=1;
	for (i=0; i < this.MAXLEVELS; i++) {
		p = this.baseClass + hX_5._U + (i+1) + hX_5._U;
		t1 = p + hX_5._DIV.toLowerCase();
		t2 = p + hX_5._TAB.toLowerCase();
		k = (hX_5.isClassDefined(t1) || hX_5.isClassDefined(t2)) ? (i+1) : k;
		p = this.baseClass + hX_5._U + (k) + hX_5._U;
		t1 = p + hX_5._DIV.toLowerCase();
		t2 = p + hX_5._TAB.toLowerCase();
		// Set the div and table classes
		this.classes[i].div	= t1;
		this.classes[i].tab	= t2;
		// Set the separator classes
		t  = p + hX_5.CLASS_MB_SEPARATOR;
		t1 = p + hX_5.CLASS_MB_SEPARATOR_L;
		t2 = p + hX_5.CLASS_MB_SEPARATOR_R;
		this.classes[i].sep = t;
		this.classes[i].sepF = (hX_5.isClassDefined(t1)) ? t1 : t;
		this.classes[i].sepL = (hX_5.isClassDefined(t2)) ? t2 : t;
		// Set the cell classes
		this.setClass(p + hX_5.CLASS_MB_CELL, this.classes[i].cell);
		this.setClass(p + hX_5.CLASS_MB_ALTCELL, this.classes[i].cellAlt);
		t  = p + hX_5.CLASS_MB_CELLF + hX_5.CLASS_MB_N;
		if (!hX_5.isClassDefined(t))
			this.copyClass(this.classes[i].cell, this.classes[i].cellF);
		else
			this.setClass(p + hX_5.CLASS_MB_CELLF, this.classes[i].cellF);
		t  = p + hX_5.CLASS_MB_CELLL + hX_5.CLASS_MB_N;
		if (!hX_5.isClassDefined(t))
			this.copyClass(this.classes[i].cell, this.classes[i].cellL);
		else
			this.setClass(p + hX_5.CLASS_MB_CELLL, this.classes[i].cellL);
		// Set the item classes
		this.setClass(p + hX_5.CLASS_MB_ITEM, this.classes[i].item);
		this.setClass(p + hX_5.CLASS_MB_ALTITEM, this.classes[i].itemAlt);
		this.setClass(p + hX_5.CLASS_MB_ARROW, this.classes[i].arrow);
		// Set the arrow classes
		this.classes[i].arrowImg[this.NORMAL]	= hX_5.getClassStyle(this.classes[i].arrow[this.NORMAL], hX_5._BIMG);
		this.classes[i].arrowImg[this.MOUSED]	= hX_5.getClassStyle(this.classes[i].arrow[this.MOUSED], hX_5._BIMG);
		this.classes[i].arrowImg[this.SELECTED]	= hX_5.getClassStyle(this.classes[i].arrow[this.SELECTED], hX_5._BIMG);
		this.classes[i].arrowImg[this.SELECTED_MOUSED]	= hX_5.getClassStyle(this.classes[i].arrow[this.SELECTED_MOUSED], hX_5._BIMG);
		this.classes[i].arrowImg[this.DISABLED]	= hX_5.getClassStyle(this.classes[i].arrow[this.DISABLED], hX_5._BIMG);
	}
}

// Private:  Helper for the above
HxG_5.prototype.JSFMenu.prototype.copyClass = function (src, dst) {
	for (var q=this.NORMAL; q<this.MAXSTATES; q++) dst[q] = src[q];
}

// Private:  Helper for the above
HxG_5.prototype.JSFMenu.prototype.setClass = function (name, element) {
	var t=name+hX_5.CLASS_MB_N, t1=name+hX_5.CLASS_MB_M, t2=name+hX_5.CLASS_MB_S, t3=name+hX_5.CLASS_MB_SM, t4=name+hX_5.CLASS_MB_D, isT2, isT3, isT4;
	isT2 = hX_5.isClassDefined(t2);
	isT3 = hX_5.isClassDefined(t3);
	isT4 = hX_5.isClassDefined(t4);
	t1 = (hX_5.isClassDefined(t1)) ? t1 : ((isT2) ? t2 : t);
	element[this.NORMAL] 	= t;
	element[this.MOUSED] 	= t1;
	element[this.SELECTED]	= (isT2) ? t2 : t1;
	element[this.SELECTED_MOUSED]= (isT3) ? t3 : ((isT2)?t2:t1);
	element[this.DISABLED]= (isT4) ? t4 : t;
}

// Private: Skip to next/prev non-separator
HxG_5.prototype.JSFMenu.prototype.skipSep = function (obj, nAdj, bound) {
	var num = obj.num + nAdj, base = ((obj.level==0) ? this.m : obj.p.m), menu = base.itemArray;
	while ((num >= 0) && (num < menu.length) && (menu[num].isSeparator || menu[num].isDisabled))
		num = num + nAdj;
	if (num < 0) return ((bound!= null && bound) ? base.lastItem : null);
	else if (num >= menu.length) return ((bound!= null && bound) ? base.firstItem : null);
	else return menu[num];
}

// Private:  Given an HTML object id, compute the menu item
HxG_5.prototype.JSFMenu.prototype.menuFromId = function (iSrc) {
	var v, p, r = null, base = null, src = iSrc;
	while (src && src.tagName && src.tagName != hX_5._TD) src = src.parentNode;
	if (src && src.tagName == hX_5._TD && (!hX_5.isString(src.name) || src.name != hX_5.CMP_menu)) {
		src = src.parentNode;
		while (src && src.tagName && src.tagName != hX_5._TD && (!hX_5.isString(src.name) || src.name != hX_5.CMP_menu)) src = src.parentNode;
	}
	if (src==null || !src.tagName || src.tagName != hX_5._TD || (!hX_5.isString(src.name) || src.name != hX_5.CMP_menu))
		return null;

	// Figure out the containing table (= the menu that contains this item)
	p = src;
	while (p && p.tagName && p.tagName!=hX_5._TAB) p = p.parentNode;
	if (p && p.id) {
		base = hX_5.getComponentById(p.id.substr(0, p.id.length-6));
		if (base != null) {
			v = this.isVertical(0);
		} else {
			base = hX_5.getSubComponentById(p.id.substr(0, p.id.length-6));
			if (base != null) v = this.isVertical(1);
		}
	}

	// Figure out the location in the table
	p = src.parentNode;
	if (p && p.tagName && p.tagName == hX_5._TR) {
		// N.B. for some odd reason, IE can report a mouseover on the pad/margin/border
		// as being in "cell 1" of "some other row", so make sure we are in the one and only row
		// row of this table
		r = (!v) ? ((src.cellIndex >= 0 && p.rowIndex==0) ? src.cellIndex : null) : ((p.rowIndex >= 0 && src.cellIndex==0) ? p.rowIndex : null);
		if (this.isOpenInPlace()) r = r/2;
	}
	return ((base != null && r != null) ? base.m.itemArray[r] : null);
}

// ==== RESTRICTED functions that handle event dispatch ====

// Restricted:  Handle a DOM event dispatched by an "always on display" item
HxG_5.prototype.JSFMenu.prototype.dispatchComponent = function (evt) {
	if ((typeof hX_5)=="undefined") return true;
	evt = (evt) ? evt : ((event) ? event: null);
	var src = (this.nodeType == hX_5.ELEMENT_NODE || this.nodeType == hX_5.DOCUMENT_NODE) ? this : this.parentNode;
	if ((evt.type==hX_5._evMO || evt.type==hX_5._evMV) && src.tagName!=hX_5._TD) return true;

	var sub, root=null, div=src;
	while (div && div.tagName && div.tagName != hX_5._DIV)
		div = div.parentNode;
	if (div && div.id)
		root = hX_5.getComponentById(div.id);
	if (!root && div && div.id) {
		sub = div.id;
		if (sub.indexOf("_popup_")) sub = sub.substr(sub.indexOf("_popup_")+7);
		root = hX_5.getSubComponentById(sub);
		root = (root) ? root.root : null;
	}
	if (root && (false==root.dispatchPopEvent(src, evt))) {
		var was = root.clickRelease; root.clickRelease = null;
		// We cancel bubbling on mousedown but not the event
		if (evt.type == hX_5._evMD) {
			hX_5.i.cancelAction(evt);
			return true;
		}
		// For now (and this is questionable) always kill key down as we might scroll if we don't
		if (evt.type==hX_5._evKD) {
			hX_5.i.cancelAction(evt);
			return false;
		}
		// Other events cancel if the dispatcher cancelled them and we're internally processing a click
		if (evt.type!=hX_5._evMC || was==null || was != src.id) {
			hX_5.i.cancelAction(evt, true);
			return false;
		}
	}
	return true;
}

// Restricted: Handle a DOM event dispatched by by the popup dispatcher
HxG_5.prototype.JSFMenu.prototype.dispatchPopEvent = function (iSrc, evt) {
	if ((typeof hX_5)=="undefined") return true;
	var src=iSrc, rtn=false, pt, state, bound=false, skip=false, focObj=null, clk=false, key=null, newObj=null, openObj=null, nAdj=0, toRoot=false;
	if (evt && src) {
			
		// The events we get are the keyboard events: keydown, keyup
		// And the mouse events: mousedown, mouseup, movemove, mouseover, mouseout
		// and the focus events: focus

		// On a focus catch what has focus (occurs on top-level only)
		switch (evt.type) {

		case hX_5._evF:
			src = this.menuFromId(src);
			if (hX_5.brw.isNavOrMoz() && src && src.level > 0 && !this.setFocusObj) {
				if (src.root && src.root.m.firstItem) {
					src = src.root.m.firstItem;
					this.setFocus(src);
				} else src = null;
			}
			if (src) this.focusObj = src;
			this.setFocusObj = false;
			rtn = true;
			break;
			
		// Most menu systems execute key actions on down, so we will too
		case hX_5._evKU:
		case hX_5._evKP:
			switch (evt.keyCode) {

			// Space and Enter = execute the focused item
			case hX_5.KEY_SPACEBAR:
			case hX_5.KEY_ENTER:
			case hX_5.KEY_LTARROW: 
			case hX_5.KEY_RTARROW:
			case hX_5.KEY_UPARROW:
			case hX_5.KEY_DNARROW:
			case hX_5.KEY_TAB:
			case hX_5.KEY_ESC:
				rtn = false;
				break;
			default:
				// FIXME:  In Mozilla, if we return "true", we have some odd
				// bubbling that trys to move focus when it shouldn't
				// If there's a keyhandler on the menu item (e.g., an accelerator)
				// this will break it
				// On the other hand, if we don't return true, not all styles are applied
//				rtn = (hX_5.brw.isIE()) ? true : false;
				rtn = true;
				break;
			}
		break;

		// Keydown either moves focus OR executes an action on a node
		case hX_5._evKD: 
	
			// Before we do anything we need to establish which is the current item
			// If an item has "mouse focus" we'll use that one.
			// If no item appears to have focus, we'll use the first/last item in the top level.
			focObj = (this.focusObj) ? this.focusObj : this.getMouseState();
			if (!focObj) {
				if (src.id && this.m && this.m.lastItem && this.m.firstItem)
					focObj = (this.m.lastItem.ITEMobj && src.id == this.m.lastItem.ITEMobj.id) ? this.m.lastItem : this.m.firstItem;
			}
			if (!focObj) break;
			
			// Now handle the keys.  We're computing two things:
			// 1)	Which object (if any) is given the selection
			// 2)	Which object (if any) is given the keyboard focus
			// For example, suppose the user tabs to the first item. We want the item
			// to just show keyboard focus (but not open its menu).  The user then presses
			// down arrow (or spacebar).  The menu opens (the item gets selection) and the
			// first item in the menu is given keyboard focus.  The user presses TAB, the
			// menu closes (selection is removed) and the second top level menu item
			// gets keyboard focus.
			key = evt.keyCode;
			if ((focObj.level==0 && this.isTopVertical() && !this.isRelative()) || 
				(focObj.level>0 && !this.isVertical(focObj.level))) {
				// Swap the keystrokes in some cases to make things uniform
				if (key == hX_5.KEY_RTARROW)
					key = hX_5.KEY_DNARROW;
				else if (key == hX_5.KEY_LTARROW)
					key = hX_5.KEY_UPARROW;
				else if (key == hX_5.KEY_DNARROW)
					key = hX_5.KEY_RTARROW;
				else if (key == hX_5.KEY_UPARROW)
					key = hX_5.KEY_LTARROW;
			}
			switch (key) {

			// Space and Enter = execute the focused item
			case hX_5.KEY_SPACEBAR:
			case hX_5.KEY_ENTER:
				if (focObj.isDisabled!=undefined && focObj.isDisabled==true) return false;
				this.setState(focObj, this.SELECTED, true, false, false, evt);
				if (focObj.isNode() && (!this.state[focObj.level].selOpen)) {
					newObj = focObj.m.firstItem;
					openObj = focObj;
					if (focObj.level>0)	clk = true;
				} else {
					if (focObj.level == 0)	focObj.setColorState(this.SELECTED);
					return false;
				}
				break;

			// Left/right arrow = open/close a submenu
			case hX_5.KEY_LTARROW: // LeftArrow
			case hX_5.KEY_RTARROW: // RightArrow
				// If this isn't the root and we have/should have a menu open ...
				if (focObj.level > 0 && 
					((focObj.isNode() && key == hX_5.KEY_RTARROW) ||
					 (focObj.p.isNode() && key == hX_5.KEY_LTARROW)) ) {
					// Right arrow opens the menu and puts focus in first item
					if (key == hX_5.KEY_RTARROW) {
						newObj = focObj.m.firstItem;
						openObj = focObj;
					// Left arrow closes the menu and puts focus on its parent item
					} else {
						newObj = focObj.p;																
						focObj = null;
					}
				// If this is the root or a leaf node, then
				// treat arrows as if they are tab/shift+tab
				} else {
					nAdj = (key == hX_5.KEY_LTARROW) ? -1 : +1;
					toRoot = true;
					bound = true;
				}
				break;

			// Up/Down arrow = move to next/prev entity
			case hX_5.KEY_UPARROW: // UpArrow
			case hX_5.KEY_DNARROW: // DownArrow
				// Open it if it's the first level
				if (focObj.isNode() && focObj.level==0) {
					openObj = focObj;
					newObj = focObj.m.firstItem;
				// Otherwise go to the next item (wrapping if need be)
				} else {
					newObj = this.skipSep(focObj, (key == hX_5.KEY_UPARROW) ? -1 : +1, true);
				}
				break;

			// TAB = close up the tree and put focus on the next/prev top item
			case hX_5.KEY_TAB: //Tab, Shift+Tab
				nAdj = (evt.shiftKey) ? -1 : +1;
				// fall through
			// ESC = close up the tree and put focus at the parentmost item of what had focus
			case hX_5.KEY_ESC:
				toRoot = true;
				break;

			default:
				return true;
			}

			// Handle reverting to a root node
			if (toRoot) {
				// Adjust to next/prev top level item
				newObj = this.skipSep(this.m.itemArray[focObj.rootNum], nAdj, bound);
				if (!newObj) {
					// No next/prev top level item, so let advance to next field
					this.uirelease();
					// In IE, let the browser handle the key
					if (hX_5.brw.isIE())
						rtn = true;	// Let browser handle the next/prev TAB ...
					else {
						if (nAdj > 0)
							hX_5.gotoNextPrev(this.m.lastItem.ITEMobj, true);
						else
							hX_5.gotoNextPrev(this.m.firstItem.ITEMobj, false);
					}
				} else {
					// Release all -- we're reverting to the root
					this.uirelease();
					// UI release could have cleared focus, so reset it
					this.setFocus(newObj);
				}
				break;
			}
			// Open a sub-menu
			if (openObj != null) {
				if (!(this.isOpenSticky() && this.state[newObj.p.level].selOpen && this.state[openObj.level].selOpen.id == openObj.id)) {
					this.setState (openObj, ((clk)?this.SELECTED:this.MOUSED), clk, true, false, evt);
					this.setFocus(openObj);
				}
			}
			// Set focus to something in the menu
			if (newObj != null) {
				this.setState (newObj, this.MOUSED, false, false, false, evt);
				// There's a case where the item may not be open -- "action-click", check for it 
				// and skip the call in that case
				if (newObj.p!=undefined && newObj.p.isNoNode!=undefined && newObj.p.isNoNode()) {
					if (this.state[newObj.p.level].selOpen == null || (this.state[newObj.p.level].selOpen && this.state[newObj.p.level].selOpen.id != newObj.p.id))
						newObj = null;
				} 
				if (newObj != null) this.setFocus(newObj);
			}
			break;

		// Handle Mouse Events
		case hX_5._evMU:
		case hX_5._evMC:
		case hX_5._evMCC:
			rtn = false;
			break;

		case hX_5._evMV:
		default:
			rtn = true;
			break;

		// Handle Mouse Events					
		case hX_5._evMO:
		case hX_5._evMM:
		case hX_5._evMD:
			if (evt.type!=hX_5._evMD) rtn = true;
			// Optimize out handling extraneous mousemoves
			if (evt.type==hX_5._evMM) {
				skip = (src.tagName==this.lastMouseMove && evt.clientX == this.lastMouseX && evt.clientY == this.lastMouseY);
				this.lastMouseMove = src.tagName;
				this.lastMouseX = evt.clientX;
				this.lastMouseY = evt.clientY;
			} else {
				this.lastMouseMove = this.lastMouseX = this.lastMouseY = null;
				skip = ((evt.type==hX_5._evMO || evt.type==hX_5._evMV) && src.tagName!=hX_5._TD);
				if (!skip && evt.type==hX_5._evMO) {
					// When you mouse over something in a cell, IE says you've moused out of
					// the cell.  This will cause flicker while moving within a cell, so screen
					// out these spurious mouseouts
					pt = hX_5.g.toPoint(evt);
					this.testRect.setRect(src);
					this.testRect.incRect(-2);
					skip = this.testRect.hitTest(pt.X, pt.Y);
				}
			}
			if (!skip) {
				// Figure out the cell
				newObj = this.menuFromId(src);
				if (newObj) {
					if (newObj.isDisabled!=undefined && newObj.isDisabled==true) return false;
					// IE doesn't generate mouseouts in many cases (probably it "unions"
					// divs that are in divs).  Mozilla generates them all over the place
					// Most of the time, we want them, but if you're mousing from the root
					// to a popup, we get an extraneous one (the mouse is owned in this 
					// case, and we shouldn't get it), so screen it out
					if (evt.type==hX_5._evMO)
						skip = (newObj.level==0 && this.state[0].selOpen && this.isFly(0))
					if (!skip) {
						this.mousePt = hX_5.g.toPoint(evt);
						this.setState (newObj, ((evt.type==hX_5._evMO) ? this.NORMAL : ((evt.type==hX_5._evMD)?this.SELECTED:this.MOUSED)), (evt.type==hX_5._evMD), false, false, evt);
					}
				}
			}
			break;
		} // end case
	}
	// Caller is responsible for cancelling actions
	return rtn;
}

// Restricted:  Catch a timer event
HxG_5.prototype.JSFMenu.prototype.dispatchTimer = function (item, action) {
	if ((typeof hX_5)=="undefined") return;
	if (item == "popup")
		if (action == "MouseOut") this.uirelease();
}

// ------------------------------------------------
// Restricted:  Helper object describing a "menu"
// ------------------------------------------------
HxG_5.prototype.JSFSubMenu = function () {
	this.type = hX_5.CMP_submenu;
	// Array of map items that make up the menu
	this.itemArray = [];
	this.firstItem = this.lastItem = null;
	// Popup/Div that contains it
	this.POPobj	= this.POPdiv = null;	
	// Literal
	this._MD = "menu_delete";
}

// Public: Add a subcomponent (a mapitem)
HxG_5.prototype.JSFSubMenu.prototype.addSubComponent = function (root, obj, position) {
	if (obj.type == hX_5.CMP_mapitem) {
		var base, k;
		if (position == null) {
			this.itemArray.push(obj);
			this.appendEnds();
		} else {
			if (position < 0 || position >= this.itemArray.length) return false;
			this.itemArray.splice(position, 0, obj);
			this.resetEnds();
		}
		base = (root.root) ? root.root : root;
		if (base.HTMLrendered) {
			base.buildNode();
			base.redraw(obj);
		}
		return true;
	}
	return false;
}

// Public: Remove a subcomponent (a mapitem)
HxG_5.prototype.JSFSubMenu.prototype.removeSubComponent = function (root, objid) {
	var base, k;
	for (k=0; k < this.itemArray.length; k++) {
		if (this.itemArray[k].id == objid) break;
	}
	if (k>=0 && k<this.itemArray.length) {
		this.itemArray.splice(k, 1);
		this.resetEnds();
		base = (root.root) ? root.root : root;
		if (base.HTMLrendered) {
			base.buildNode();
			base.redraw();
		}
		return true;
	}
	return false;
}

// ------------------------------------------------
HxG_5.prototype.JSFSubMenu.prototype.destroy  = function () {
	var o;
	while (this.itemArray.length > 0) {
		o = this.itemArray.pop();
		if (o.destroy) o.destroy();
		delete o;
	}
	delete this.itemArray;
	if (this.POPobj) hX_5.removePopup(this.POPobj.id);
}

HxG_5.prototype.JSFSubMenu.prototype.appendEnds = function () {
	var added = this.itemArray[this.itemArray.length-1]
	if (added.separator==null && !added.isDisabled) {
		this.firstItem = (this.firstItem==null)?added:this.firstItem;
		this.lastItem = added;
	}
}

HxG_5.prototype.JSFSubMenu.prototype.resetEnds = function () {
	this.firstItem = this.lastItem = null;
	var k;
	for (k=0; (this.firstItem==null) && (k < this.itemArray.length); k++) {
		if (this.itemArray[k].separator == null && !this.itemArray[k].isDisabled)
			this.firstItem = this.itemArray[k];
	}
	for (k = this.itemArray.length-1; (this.lastItem==null) && (k >= 0); k--) {
		if (this.itemArray[k].separator == null && !this.itemArray[k].isDisabled)
			this.lastItem = this.itemArray[k];
	}
}

// Restricted: Redraw any items in the menu that are hxcomponents
HxG_5.prototype.JSFSubMenu.prototype.redraw = function () {
	var i, c;
	for (i=0; i < this.itemArray.length; i++) {
		c = this.itemArray[i];
		if (!c.isSeparator) {
			if (c.item && c.item.id) {
				c = hX_5.getComponentById(c.item.id);
				if (c && c.redraw) c.redraw();
			}
		}
	}				
}

// Restricted: Build the HTML that makes up this menu
HxG_5.prototype.JSFSubMenu.prototype.buildMenu = function (root, owner, level, noNewChildren) {
	if (level >= 0 && root.DOMobj && owner) {
		var ii = 0, i, j, P, T, R, C, mo;

		// Make sure we have a div (usually a popup)
		if (this.POPdiv == null) {
			this.POPobj = hX_5.addPopup (root.DOMobj.id+"_popup_"+owner.id, root.DOMobj.id, root.form, hX_5.POP_STYLE_MANAGED);
			this.POPobj.onPageLoad(true);
			this.POPdiv = this.POPobj.getElem();
		}
		P = this.POPdiv;
		
		// Ensure the div has the right initial size and has the right class to style it
		// For the non-static divs, we make the div "small" and then let the browser is expand it to 
		// fit around the table inside it
		if (P.style.overflow != hX_5._VS ) P.style.overflow = hX_5._VS;
		if (level > 0) {
			if (P.className != root.classes[level].div) P.className = root.classes[level].div;
			hX_5.g.setP(P, hX_5._WD, hX_5._E);
			hX_5.g.setP(P, hX_5._HT, hX_5._E);
		} else
			hX_5.appendClassName(P, root.classes[level].div);

		// Ensure the div has the correct HTML table in it
		// Verify the basic structure = a table with one row per item
		T = P.firstChild;
		while (T!=null && T.tagName == hX_5._INP) T = T.nextSibling;	// Skip over hidden selection field
		if (T == null || T.tagName != hX_5._TAB) {
			T = document.createElement(hX_5._TAB);
			T.setAttribute (hX_5._ID, owner.id + hX_5._U + hX_5._TAB.toLowerCase());
			T.name = hX_5.CMP_menu;	// Mozilla doesn't like setAttribute here
			T.cellPadding = T.cellSpacing = T.border = hX_5._Z;
			P.appendChild(T);
		}
		if (T.className != root.classes[level].tab) 		
			T.className = root.classes[level].tab;
	
		// Make sure we have the appropriate row/cell structure for
		// each menu item and populate the cell with the item
		if (noNewChildren == null || !noNewChildren) {
			for (i = 0; i < this.itemArray.length; i++) {
				ii = (root.isOpenInPlace()) ? i*2 : i;
				R = T.rows[(root.isVertical(level))?ii:0];
				if (R == null) {
					R = T.insertRow(-1);
					R.name = hX_5.CMP_menu;	// Mozilla doesn't like setAttribute here
				}				
				if (R.style.display == hX_5._NO) R.style.display=hX_5._E;					
				C = R.cells[(root.isVertical(level))?0:ii];
				if (C == null) {
					C = R.insertCell(-1);
					C.name = hX_5.CMP_menu; 	// Mozilla doesn't like setAttribute here
					C.vAlign = hX_5._MID; C.align = hX_5._LFT; C.noWrap = hX_5._TRU;
					C.style.position = hX_5._REL;
				}
				if (C.style.display == hX_5._NO) C.style.display=hX_5._E;					
				this.itemArray[i].BOXobj = C;
				// If this cell has things in it, move them to scratch
				j = hX_5.getFirstChildInCell(C);
				while (j != null) {
					if (j.style) j.style.display = hX_5._NO;
					if (j.name && j.name == this._MD)
						C.removeChild(j);
					else
						root.scratch.appendChild(j);
					j = hX_5.getFirstChildInCell(C);
				}
				// (Re)build the menu item's HTML and put it in the cell
				this.itemArray[i].buildItem(T.id, i, C);
				// If this is an open-in-place style
				if (root.isOpenInPlace()) {
					if (root.isVertical(level)) {
						R = T.insertRow(ii+1);
						R.style.display = hX_5._NO;
						R.name = hX_5.CMP_menu;	// Mozilla doesn't like setAttribute here
						C = R.insertCell(-1);
						C.vAlign = hX_5._TOP; C.align = hX_5._LFT; C.noWrap = hX_5._TRU;
					} else {
						C = R.insertCell(ii+1);
						C.name = hX_5.CMP_menu;	// Mozilla doesn't like setAttribute here					
						C.vAlign = hX_5._MID; C.align = hX_5._LFT; C.noWrap = hX_5._TRU;
					}				
					C.style.position = hX_5._REL;
					C.style.display = hX_5._NO;
				} 
			}

			// N.B. at this point scratch may have "unused" items in it (i.e., items
			// we didn't put back in the menu).  Some of these items may be things
			// we created that could be deleted.  Some could be items that were 
			// created intentionally but have now been removed.  In the latter case
			// someone could want to put them back later.  In the former case, we
			// haven't marked these items (primarily "arrows") in any way that
			// can be distinguished from the latter.  So we'll just let it 
			// accumlate gunk for now
	
			// Remove any extraneous rows/cells
			ii = (root.isOpenInPlace()) ? 2 : 1;
			for (i = ((root.isVertical(level)) ? (this.itemArray.length*ii) : 1); i < T.rows.length; i++)
				T.rows[i].style.display=hX_5._NO;
			for (i = ((root.isVertical(level)) ? 1: (this.itemArray.length*ii)); i < T.rows[0].cells.length; i++)
				T.rows[0].cells[i].style.display=hX_5._NO;
		}
		// Add "edge" classes
		if (this.itemArray[0].ITEMobj == null) {
			if (this.itemArray[0].BOXobj.className != root.classes[level].sepF)
				this.itemArray[0].BOXobj.className = root.classes[level].sepF;
		} 
		if (this.itemArray[this.itemArray.length-1].ITEMobj == null) {
			if (this.itemArray[this.itemArray.length-1].BOXobj.className != root.classes[level].sepL)
				this.itemArray[this.itemArray.length-1].BOXobj.className = root.classes[level].sepL;
		} 
		// If this is level 0 or a sticky menu
		if (level == 0 || root.isOpenSticky()) {
			for (i = 0; i < this.itemArray.length; i++) {
				mo = this.itemArray[i].ITEMobj;
				if (mo) {
					hX_5.attachEvent (mo, hX_5._onKD, hX_5._lF, root.dispatchComponent);
					hX_5.attachEvent (mo, hX_5._onKU, hX_5._lF, root.dispatchComponent);
					hX_5.attachEvent (mo, hX_5._onKP, hX_5._lF, root.dispatchComponent);
					hX_5.attachEvent (mo, hX_5._onMU, hX_5._lF, root.dispatchComponent);	
					hX_5.attachEvent (mo, hX_5._onMD, hX_5._lF, root.dispatchComponent);
					hX_5.attachEvent (mo, hX_5._onMM, hX_5._lF, root.dispatchComponent);
					hX_5.attachEvent (mo, hX_5._onMV, hX_5._lF, root.dispatchComponent);
					hX_5.attachEvent (mo, hX_5._onMO, hX_5._lF, root.dispatchComponent);
					hX_5.attachEvent (mo, hX_5._onMC, hX_5._lF, root.dispatchComponent);
					hX_5.attachEvent (mo, hX_5._onMCC,hX_5._lF, root.dispatchComponent);
					if (level==0 || (level > 0 && hX_5.brw.isNavOrMoz()))
						hX_5.attachEvent (mo, hX_5._onF, hX_5._lF, root.dispatchComponent);
				} 
				mo = this.itemArray[i].BOXobj;
				if (mo) {
					hX_5.attachEvent (mo, hX_5._onMU, hX_5._lF, root.dispatchComponent);	
					hX_5.attachEvent (mo, hX_5._onMD, hX_5._lF, root.dispatchComponent);
					hX_5.attachEvent (mo, hX_5._onMM, hX_5._lF, root.dispatchComponent);
					hX_5.attachEvent (mo, hX_5._onMV, hX_5._lF, root.dispatchComponent);
					hX_5.attachEvent (mo, hX_5._onMO, hX_5._lF, root.dispatchComponent);
				}
			}
		}
		return true;
	}
	return false;
}

// ------------------------------------------------
// Public:  The Menu Item object
// ------------------------------------------------
// Public: Constructor
HxG_5.prototype.JSFMenuItem = function () {
	this.type = hX_5.CMP_mapitem;
	// parent, id, root, grandparent's position in root array, level down from root
	this.p = this.id = this.root = this.rootNum = this.level = null;
	// DOM obj, DOM obj if not separator, tag that handles "box" CSS classes, 
	// Summary of current visual state, value of separator (can be null, else string)
	this.item = this.ITEMobj = this.BOXobj = this.visualState = this.separator = null;
	this.isDisabled = false;
	// position in parent's array
	this.num = -1;
	// If true, use alt named classes
	// If true, it's a separator (and ITEMObj = null)
	// Slide-out arrow
	this.altClass = this.isSeparator = this.hasArrowSpan = false;
	this.arrowSpan = null;
	// If true, don't "open" nodes (treat them as links)
	this.noNode = false;
	this.isNoNode = function() { return (this.isNode() && this.noNode); }
	// If not null, this is an embedded hxcomponent
	this.isComponent = null;

	// Test if this is a node or not	
	this.isNode = function () { return (this.m.itemArray.length > 0); }

	// Instantiate a menu helper class
	this.m = new hX_5.JSFSubMenu();
	
	// User-specified information about this node
	// User-supplied HTML item, dupe of this, label, func1()...etc
	this.itemName = this.wasItemName = this.label = this.fcn = null;
	this.actions= [];		// "click/lock"/"unlock"/ ... etc
	this.targets= [];		// action target ID

	// Attributes supported on this object
	this.attributeTable = [];
    	this.attributeTable ['alt-class']	= ['altClass',		hX_5.ARG_IS_TRUE,	false,	null,	null];
		this.attributeTable ['separator']	= ['separator',		hX_5.ARG_TO_NAB,	false,	null,	null];
		this.attributeTable ['sep']			= ['separator',		hX_5.ARG_TO_NAB,	false,	null,	null];
		this.attributeTable ['label']		= ['label',			hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['lab']			= ['label',			hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['action']		= ['actions',		hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['target']		= ['targets',		hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['function']	= ['fcn',			hX_5.ARG_TO_FCNS,	false,	null,	null];
		this.attributeTable ['onchange']	= ['fcn',			hX_5.ARG_TO_FCNS,	false,	null,	null];
		this.attributeTable ['item']		= ['itemName',		hX_5.ARG_TO_NAN,	false,	null,	null];
    	this.attributeTable ['action-click']= ['noNode',		hX_5.ARG_IS_TRUE,	false,	null,	null];
    	this.attributeTable ['disabled']	= ['isDisabled',	hX_5.ARG_IS_TRUE,	false,	null,	null];
	this.setAttribute(arguments);
}
HxG_5.prototype.J_MI = HxG_5.prototype.JSFMenuItem;

// ==== PUBLIC functions all subcomponents should support ====

// Public: Set an "attribute" on the menu item
HxG_5.prototype.JSFMenuItem.prototype.setAttribute = function(a) {
	var was = this.isDisabled;
	hX_5.parseArg (this.attributeTable, this, a);
	// Redraw
	if (was!=this.isDisabled && this.p && this.p.m && this.p.m.resetEnds!=undefined) this.p.m.resetEnds();
	if (this.root && this.p && this.p.m && this.root.HTMLrendered) {
		this.p.buildNode();
		this.p.m.redraw();
	}
	return true;
}

// Public: Retrieve the value of an attribute
HxG_5.prototype.JSFMenuItem.prototype.getAttribute = function (attribute) {
	return (hX_5.getArg(this.attributeTable, this, attribute));
}

// Public: Add a child mapitem (make this a "node")
HxG_5.prototype.JSFMenuItem.prototype.addSubComponent = function(obj, position) {
	if (!this.isSeparator) this.m.addSubComponent (this, obj, position);
}

// Public: Remove a child mapitem 
HxG_5.prototype.JSFMenuItem.prototype.removeSubComponent = function(objid) {
	this.m.removeSubComponent (this, objid);
}

// ==== Restricted/Private functions ====

// Restricted: Remove a child mapitem 
HxG_5.prototype.JSFMenuItem.prototype.destroy = function() {
	// Remove this from the subcomponent list
	hX_5.destroySubComponent(this.id);
	// Remove events on child objects
	if (this.ITEMobj) hX_5.removeAllEvents(this.ITEMobj, this.ITEMobj.id);
	if (this.BOXobj) hX_5.removeAllEvents(this.BOXobj, this.BOXobj.id);
	// Destory child items
	if (this.m!=null) {
		if (this.m.destroy) this.m.destroy(); // JSFSubMenu
		delete this.m;
	}
	// Destroy large memory objects
	var i,d = [this.actions,this.targets];
	for (i=0; i<d.length; i++) {
		if (d[i]) {
			while (d[i].length>0) d[i].pop();
			delete d[i];
		}
	}
	delete d;
	delete this.attributeTable;
	return true;
}

// Restricted: Build this node of the menu tree
HxG_5.prototype.JSFMenuItem.prototype.buildNode = function(noNewChildren) {
	if (this.m.itemArray.length > 0) {
		// Build me
		this.m.buildMenu (this.root, this, (this.level+1), noNewChildren);
		// Build my children
		for (var i = 0; i < this.m.itemArray.length; i++)
			this.m.itemArray[i].buildNode(noNewChildren);
	}
}
 
// Restricted: Build the HTML that displays this item (not this items menu, the item itself)
HxG_5.prototype.JSFMenuItem.prototype.buildItem = function(id_pre, num, parent) {

	// Set the root/level for this item
	this.root = this.p, this.level = 0;
	this.num = this.rootNum = num;
	while (this.root.type != hX_5.CMP_menu) {
		this.rootNum = this.root.num;
		this.root = this.root.p;
		this.level++;
	}
	
	var a, r = this.root, l = this.level, P = parent;
	this.item = null;
	this.isSeparator = (this.separator != null) ? true : false;
	// If this is a supplied item, just use it
	if (this.itemName) {
		this.item = hX_5.getElementById(this.itemName);
		if (!this.item) {
			var err = hX_5._ANC + hX_5._U + this.itemName;
			a = document.createElement(hX_5._ANC);
			a.setAttribute(hX_5._ID, err);
			a.href = hX_5._LB;
			hX_5.setIHTML(a, "Error - " + this.itemName);
			this.item = a;
		} else {
			// Do we think this is a component (in particular a button?)
			this.isComponent = hX_5.getComponentById(this.item.id);
			// If this isn't in the tab order, put it there
			var tagset = [];
			tagset.push(new hX_5.tagListItem(hX_5._INP));
			tagset.push(new hX_5.tagListItem(hX_5._BTN));
			tagset.push(new hX_5.tagListItem(hX_5._SEL));
			tagset.push(new hX_5.tagListItem(hX_5._ANC));
			if (!hX_5.isTag(this.item, tagset)) {
				this.itemName = hX_5._ANC + hX_5._U + this.itemName;
				a = document.createElement(hX_5._ANC);
				a.setAttribute(hX_5._ID, this.itemName);
				a.href = hX_5._LB;
				a.appendChild(this.item);
				this.item = a;
			}
		}
	// Otherwise make it
	} else {
		if (this.isSeparator) {
			if (this.separator.length > 0) {
				a = document.createElement(hX_5._SPN);
				hX_5.setIHTML(a, this.separator);
			} else {
				a = document.createElement(hX_5._IMG);
				a.src = r.r.getURL(hX_5.R_IMG_SPACER);
				a.width = 1; a.height = 1; a.border = 0;
			}
		} else {
			a = document.createElement(hX_5._ANC);
			a.href = hX_5._LB;
		}
		a.setAttribute (hX_5._ID, id_pre + hX_5._U + hX_5._ANC + hX_5._U + num);
		a.name = "menu_delete";
		this.item = a;
	}
	this.item.style.display = hX_5._E;
	P.appendChild(this.item);
	
	// Add an arrow if one is needed
	if ((l>0 || (l==0 && r.isVertical(0))) && this.isNode()) this.buildArrow(P,l);

	// Ensure the parent cell and item have the right HTML classes and stuff
	if (this.isSeparator) {
		this.ITEMobj = null;
		if (P.className != r.classes[l].sep) P.className = r.classes[l].sep;
	} else {
		this.ITEMobj = this.item;
		// Only the top level should be in the tab order
		// NOTE: Can't remove an item in Mozilla, so we let it in the order
		if (hX_5.brw.isIE())
			this.item.tabIndex = (this.level > 0) ? -1: r.tabIndex;
		else
			this.item.tabIndex = (this.level > 0) ? 99999: r.tabIndex;
		if (this.label != null && hX_5.getIHTML(this.item) != this.label) hX_5.setIHTML(this.item, this.label);
		this.visualState = -1;
		this.setColorState((this.isDisabled)?r.DISABLED:r.NORMAL);
	}
}

// Private: Helper to add an arrow to a menu item
HxG_5.prototype.JSFMenuItem.prototype.buildArrow = function (parent, l) {
	// We make both a normal, moused and selected versions as well
	var w, img, a, a1 = document.createElement(hX_5._SPN), a2 = document.createElement(hX_5._SPN), a3 = document.createElement(hX_5._SPN), a4 = document.createElement(hX_5._SPN), a5 = document.createElement(hX_5._SPN);
	this.hasArrowSpan = true;
	this.arrowSpan = new Array(this.root.MAXSTATES);
	this.arrowSpan[this.root.NORMAL] = this.id+"_arrow_"+(l+1)+"_normal";
	this.arrowSpan[this.root.MOUSED] = this.id+"_arrow_"+(l+1)+"_moused";
	this.arrowSpan[this.root.SELECTED] = this.id+"_arrow_"+(l+1)+"_selected";
	this.arrowSpan[this.root.SELECTED_MOUSED] = this.id+"_arrow_"+(l+1)+"_selected_moused";
	this.arrowSpan[this.root.DISABLED] = this.id+"_arrow_"+(l+1)+"_disabled";
	a1.setAttribute(hX_5._ID, this.arrowSpan[this.root.NORMAL]);
	a2.setAttribute(hX_5._ID, this.arrowSpan[this.root.MOUSED]);
	a3.setAttribute(hX_5._ID, this.arrowSpan[this.root.SELECTED]);
	a4.setAttribute(hX_5._ID, this.arrowSpan[this.root.SELECTED_MOUSED]);
	a5.setAttribute(hX_5._ID, this.arrowSpan[this.root.DISABLED]);
	a1.style.position = a2.style.position = a3.style.position = a4.style.position = a5.style.position = hX_5._ABS;
	a1.style.backgroundImage = a2.style.backgroundImage = a3.style.backgroundImage = a4.style.backgroundImage = a5.style.backgroundImage = hX_5._NO;
	a1.className = this.root.classes[l].arrow[this.root.NORMAL];
	a2.className = this.root.classes[l].arrow[this.root.MOUSED];
	a3.className = this.root.classes[l].arrow[this.root.SELECTED];
	a4.className = this.root.classes[l].arrow[this.root.SELECTED_MOUSED];
	a5.className = this.root.classes[l].arrow[this.root.DISABLED];
	a2.style.display = a3.style.display = a4.style.display = a5.style.display = hX_5._NO;
	// If the arrow class says there's "no height" on the normal case, then
	// we'll suppress the arrows
	w = hX_5.getClassStyle(a1.className, hX_5._HT);
	if (w && w.length > 0 && w.substr(0,1)==hX_5._Z) {
		this.hasArrowSpan = false;
		this.arrowSpan = null;
	} else {
		// Netscape uses "margins" a little differently
		// Since we're trying to vertically/horizontally center things we'll adjust them
		if (hX_5.brw.isNavOrMoz()) {
			w = hX_5.getClassStyle(a1.className, hX_5._MGNT);
			if (w && w.length > 0 && w.indexOf(hX_5._PX > 0)) {
				w = (parseInt(w,10)/2) + hX_5._PX;
				a1.style.marginTop = a2.style.marginTop = a3.style.marginTop = a4.style.marginTop = a5.style.marginTop = w;
			}
		}	
		// Make the arrow
		if (this.root.classes[l].arrowImg[this.root.NORMAL]) {
			for (w = this.root.NORMAL; w < this.root.MAXSTATES; w++) {
				// Make the "normal" image
				img = document.createElement(hX_5._IMG);
				img.src = this.root.classes[l].arrowImg[w];
				img.border = hX_5._Z;
				img.style.padding = img.style.margin = hX_5._ZPX;
				a = (w==this.root.NORMAL) ? a1 : (w==this.root.MOUSED?a2:((w==this.root.SELECTED)?a3:((w==this.root.SELECTED_MOUSED)?a4:a5)));
				a.appendChild(img);
			}
		} else {
			hX_5.setIHTML(a1, this.root.ARROW_1);
			hX_5.setIHTML(a2, this.root.ARROW_2);
			hX_5.setIHTML(a3, this.root.ARROW_2);
			hX_5.setIHTML(a4, this.root.ARROW_2);
			hX_5.setIHTML(a5, this.root.ARROW_1);
		}
		// If the menu item is a table, we need to turn "on" text wrap or things will misposition
		if (parent.firstChild.firstChild && parent.firstChild.firstChild.tagName == hX_5._TAB)
			parent.firstChild.firstChild.align = hX_5._LFT;
			
		// Finally add in the arrows
		parent.appendChild(a1); parent.appendChild(a2); parent.appendChild(a3); parent.appendChild(a4);  parent.appendChild(a5);
	}
}

// Scan a menu item's HTML and figure out what the value is
HxG_5.prototype.JSFMenuItem.prototype.findValue = function () {
	return this.findValueTag(this.ITEMobj);
}
HxG_5.prototype.JSFMenuItem.prototype.findValueTag = function (node) {
	var v = hX_5._E;
	if (!this.isSeparator) {
		while (node!=null) {
			if (node.nodeType == hX_5.TEXT_NODE) v = node.nodeValue;
			else if (node.firstChild) v = this.findValueTag(node.firstChild);
			node = node.nextSibling;
		}
	}
	return v;
}
// Scan this node (if it's a leaf) for a target that matches the value
HxG_5.prototype.JSFMenuItem.prototype.findTarget = function (value) {
	var i, v, item = null;
	if (!this.isSeparator) {
		if (this.isNode()) {
			// Walk through all menu items and build their nodes
			for (i = 0; item == null && i < this.m.itemArray.length; i++)
				item = this.m.itemArray[i].findTarget(value);
		} else {
			v = this.findValue();
			if (v != hX_5._E && v==value) item = this;
		}
	}
	return item;
}

// Restricted:  Set the visual state of the item
HxG_5.prototype.JSFMenuItem.prototype.setColorState = function (state) {
	if (this.ITEMobj!=null && state >= 0 && (state < this.root.MAXSTATES || state== this.root.DISABLED) && this.ITEMobj != null) {
		if (this.visualState != state) {
			var t, cl, arr;
			this.visualState = state;
			cl = (this.altClass) ? (this.root.classes[this.level].cellAlt[state]) : ((this.p.m.itemArray[0].num==this.num) ? (this.root.classes[this.level].cellF[state]) : ((this.p.m.itemArray[this.p.m.itemArray.length-1].num==this.num) ? (this.root.classes[this.level].cellL[state]) : (this.root.classes[this.level].cell[state])));
			this.BOXobj.className = cl;
			// For the ITEMS (notably A tag), preserve any "non menu item" classes in the class list
			// Also, we do ALL items in a cell (all top-level items).
			// This includes arrows or any items a user put there
			t = hX_5.getFirstChildInCell(this.BOXobj);
			while (t && t.tagName) {
				arr = this.hasArrowSpan && t.id && (t.id==this.arrowSpan[this.root.NORMAL] || t.id==this.arrowSpan[this.root.MOUSED] || t.id==this.arrowSpan[this.root.SELECTED] || t.id==this.arrowSpan[this.root.SELECTED_MOUSED] || t.id==this.arrowSpan[this.root.DISABLED]);
				cl = (this.altClass) ? this.root.classes[this.level].itemAlt[state] : this.root.classes[this.level].item[state];
				cl = (arr) ? (cl+hX_5._B+this.root.classes[this.level].arrow[state]) : cl;
				hX_5.appendClassName(t, cl, this.root.baseClass);
				if (arr) hX_5.g.setP(t, hX_5._DSP, (t.id == this.arrowSpan[state]) ? hX_5._E : hX_5._NO);
				t = t.nextSibling;
			}
		}
		return true;
	}
	return false;
}

// Restricted:  Set the state of the item
HxG_5.prototype.JSFMenuItem.prototype.setState = function (iState, clickAction, force, initial, evt) {
	if (this.ITEMobj != null && iState >= 0 && iState < this.root.MAXSTATES) {
		var c, state = iState, vis = ((state == this.root.SELECTED) || state == (this.root.SELECTED_MOUSED) || (state == this.root.MOUSED));
		// First do the visual change
		this.setColorState(state);
		// Now open the item if flying or click the item if not
		if (state!=this.root.DISABLED && (force || clickAction || this.root.isFly(this.level))) {
			if (!(initial && !this.root.isOpenSticky())) {
				c = (state==this.root.SELECTED && force) ? false : clickAction;
				this.doNode(vis, this.BOXobj, c, evt);
			}
		}
		return true;
	}
	return false;	
}

// Show/Hide/Execute a node (only call from setState)
HxG_5.prototype.JSFMenuItem.prototype.doNode = function(display, ilocation, clicked, evt) {
	var i, c, d = null, pop, l = this.level, location = ilocation, openItem = this.root.state[l].selOpen;
	// Show a popup
	if (display) {
		if (!(clicked && this.isNoNode())) {
			// Do we have a different node on display?
			if (openItem != null && openItem.id != this.id) {
				pop = openItem.m.POPobj;
				if (pop && pop.getElem()) {
					if (!this.root.isOpenSticky())
						hX_5.mouser.removeHotSpot(pop.getElem());
					pop.sticky = false;
					pop.uirelease(this.root.isOpenInPlace());
					if (this.root.isOpenInPlace()) this.tidyPop(openItem, pop, false);
				}
				openItem = this.root.state[l].selOpen = null;
			}
			if (this.isNode() && (openItem == null || openItem.id != this.id)) {
				// Now make this popup the active one
				pop = this.m.POPobj;
				if (pop && pop.getElem()) {
					openItem = this.root.state[l].selOpen = this;
					// Set the base of the popup(s)
					if (!this.root.isOpenSticky() && l==0)
						hX_5.mouser.setBaseSpot(this.root, location, !(this.root.isFlyOpen()));
					// Make it visible
					if (this.root.isVertical(l))
						d = hX_5.POP_DIR_RIGHT;
					else if (!this.root.isBodyVertical() && !this.root.isTopVertical()) {
						location = null;
						d = hX_5.POP_DIR_CENTER;
					}
					pop.getElem().style.zIndex = hX_5._E+(1000000+l);
					hX_5.uistate.setActiveComponent (this.root);
					pop.sticky = (this.root.isOpenSticky()) ? true: false;
					if (this.root.isOpenInPlace()) pop.uivisible(d, location, this.tidyPop(this, pop, true));
					else pop.uivisible(d, location, null, null);
					// If the popup contains children which are hxclient compoennts
					// making them visible requires a redraw
					this.m.redraw();
					// Once visible, it can be measured, so add it as a hotspot
					if (!this.root.isOpenSticky()) hX_5.mouser.addHotSpot(pop.getElem());
				}
			}
		}
	// Hide a popup
	} else {
		// If this level has an open child, close it
		if (openItem != null) {
			pop = openItem.m.POPobj;
			if (pop && pop.getElem()) {
				if (!this.root.isOpenSticky()) hX_5.mouser.removeHotSpot(pop.getElem());
				pop.sticky = false;
				pop.uirelease(this.root.isOpenInPlace());
				if (this.root.isOpenInPlace()) this.tidyPop(openItem, pop, false);
			}
			openItem = this.root.state[l].selOpen = null;
		}
	}
	
	// If this is not a node and we have a click then do it
	if (clicked && (!this.isNode() || this.isNoNode())) {
		if (l > 0 && !this.root.isOpenSticky()) this.root.uirelease();
		if (evt) evt.objSelected = this.id;
		if ((this.actions.length==0 && !this.fcn)) {
			if (l==0 || this.root.isOpenSticky()) this.root.clickRelease = this.ITEMobj.id; else this.root.clickRelease = null;
			hX_5.action.doClick(this.ITEMobj);
		} else
			hX_5.action.processAction (this.root.DOMobj, (evt)?evt:null, this.fcn, this.actions, true, this.targets, null, false, false, false);
		if (this.root.fcn)
			hX_5.action.processAction (this.root.DOMobj, (evt)?evt:null, this.root.fcn, null, true, null, null, false, false, false);
		if (this.root.isRelative() && !this.root.isOpenSticky()) this.root.hideMenu();
	}
}

// Show/Hide/Execute a node (only call from setState)
HxG_5.prototype.JSFMenuItem.prototype.tidyPop = function(item, pop, display) {
	var n, R = null, C = null, T = item.BOXobj.parentNode;
	while (T && T.tagName && T.tagName != hX_5._TAB) 
		T = T.parentNode;
	if (T && T.tagName && T.tagName == hX_5._TAB) {
		var d = (display) ? hX_5._E : hX_5._NO;
		n = (item.num*2)+1;
		if (this.root.isVertical(item.level)) {
			if (n < T.rows.length) {
				R = T.rows[n];
				R.style.visibility = hX_5._VS;
				R.style.display = d;
				C = (R.cells.length > 0) ? R.cells[0] : null;
			}
		} else {
			if (t.rows.length > 0 && t.rows[0].cells.length > n) C = t.rows[0].cells[n];
		}
		if (C) {
			C.style.visibility = hX_5._VS;
			C.style.display = d;
		}
	}
	return C;
}
