/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2006 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

//------------------------------------------------------------------
// Tab Order Navigation Map
//
// This object attempts to represent the "tab order" of the page so that
// the nextField and previousField actions (nextTab/prevTab) can work.
//
// It is, however, problematic.
//
// 1)  	First, it's very expensive to build the object as it needs to
// 		walk through the entire page (all tags/nodes) and examine them.
//		So we don't build the object until it is first used (and if it's
//		never used, i.e., no one ever does the nextField/prevField actoin
//		it isn't built.
//
// 2)	Many kinds of changes to the page (e.g., inserting a new element,
//		changing an element's properties, etc.) can cause it to become
//		stale (incorrect).  We do not try to track this (going stale)
//		as there are a very large number of things that can cause it
//		to go stale and monitoring all these events would be prohibitively
//		expensive.  It is, however, wired to the redraw call so that if
//		designer adds JS that changes the page, they can explicitly call
//		hX.redraw() which will cause the map to be rebuilt.
//
// 3)	The behavior of Netscape and IE in regards to next/previous are
//		different.  There is no way to equivalence them as IE allows
//		anything to get focus (e.g., a table row, something inside
//		an <a> or <button> tag, ...  So we assume that the designer
//		is using this in a "form" of some sort (literally an old-fashioned
//		form).  We use the Netscape "simple-minded" approach to determining
//		what is focusable and what isn't.  So more advanced IE functions
//		are ignored (to the best we can).
//
//	4)	There is a known bug in IE (with no known workaround) where the focus rectangle
//		does not draw around radio buttons, checkboxes, or links until the keyboard
//		is used to manually set focus to some control on the page (that is, you must
//		tab on the page).
//
//	In general, it is STRONGLY preferred that a designer use the 
//	focus (or select) actions which explicitly go to a field rather
//	than asking us to try to figure out next/prev fields.  Or at least,
//	they need to thoroughly test their use of next/prev actions.
//
//------------------------------------------------------------------

HxG_5.prototype.tabNavigator = function() {
	// Public -- given an object, find the next/previous one in the tab order
	this.getFocus	= function (obj, dir, h, scope, ignVis) { return gFocus(obj,  ((dir > 0)?+1:-1), h, scope, ignVis); }
	this.getTabItems= function () { return tabItems; }
	this.getFirst	= getFirst;
	this.getLast 	= getLast;
	this.firstInScope = this.lastInScope = this.scope = null;

	// Private
	var i, tabItems = [];		// Array of tabbable items
	function elem(obj, s) {		// Element in the array
		this.obj = obj;
		this.name = (obj.name) ? obj.name : hX_5._E;
		this.tabOrder = obj.getAttribute("tabIndex");
		this.tabOrder = (this.tabOrder==null)?0:this.tabOrder;
		this.srcOrder = s;
	}

	// ===== Start =====

	// Collect all tabble elements
	collectTabElement(document.body);
	// Check any "special" tags added by someone
	for (i=0; i < hX_5.HTMLTags.length; i++) collectTabElement(hX_5.HTMLTags[i]);
	// Order them
	tabItems.sort(compareTabIndex);

	// ===== End =====

	// Private:  Walk all tags and add 'tabable' ones to the array
	function collectTabElement(iNode) {
		var x, child, node = iNode;
		if (node.tagName && node.tagName == hX_5._IFRM) {
			try {
				node = (node.contentDocument) ? node.contentDocument : ((node.contentWindow) ? node.contentWindow.document.body : null);
				// IE and Firefox both will throw an exception if the content of the iframe is "from a different site"
				// That is, you can't read/manipulate the DOM in an iframe if it doesn't seem to be "your own".
				// Firefox sometimes lets you get the contentDocument but will throw a security exception if you
				// "touch it".  Make sure we have a "usable" contentDocument by simply "touching" a standard attribute
				x = (node.nodeType);
			}
			catch(e) {
				node = null;
			}
		}
		if (node!=null) {
			if (hX_5.isTabbable(node)) if (notDuplicateElement(node)) tabItems.push(new elem(node, tabItems.length));
			for (child = node.firstChild; child != null; child = child.nextSibling) collectTabElement(child);
		}
	}

	// Private: Check that this tag isn't a "duplicate" of something already in the tab order
	// The notable example is radio buttons -- only one in the group should be put in the tab order
	function notDuplicateElement(node) {
		// Case:  Radio button groups have only the "first" element added
		if (node && node.nodeType!=undefined && node.nodeType == hX_5.ELEMENT_NODE) {
			if (node.tagName && node.tagName == hX_5._INP && node.type && node.type.toLowerCase() == hX_5._RAD) {
				if (node.name && node.name.length > 0) {
					var ix, l=tabItems.length;
					for (ix=0; ix<l; ix++) if (tabItems[ix].name == node.name) return false;
				}
			}
		}
		return true;
	}

	// Private:  Compare function for sorting
	function compareTabIndex(ia, ib) {
		// Order of array is to end up 1 2 3 ... 0 -1 preserving src order if items equal
		var r=0, a=ia.tabOrder, b=ib.tabOrder;
		if ((a == b) || (a < 0 && b < 0))	r = 0;
		else if (a > 0 && b > 0)			r = a-b;
		else if (a >= 0 && b <= 0)			r = -1;
		else if (b >= 0 && a <= 0)			r = +1;
		// else treat both as equal -1
		return ((r != 0)? r : (ia.srcOrder - ib.srcOrder));
	}

	// public: Return the first tabbable item (optionally first item WITHIN the specified scope)
	function getFirst(scope, skipVis) {
		var r = null, sV=(skipVis!=null && skipVis);
		if (tabItems.length==0)	r = null;
		else {
			if (scope!=null && this.scope==scope && this.firstInScope!=null)
				r = this.firstInScope;
			else {
				if ((tabItems[0]).tabOrder>=0 && (sV || hX_5.i.isFocusable(tabItems[0].obj)) && (scope==null || hX_5.isChildTag(tabItems[0].obj, scope)))
					r = (tabItems[0].obj);
				else
					r = gFocus(tabItems[0].obj, 1, false, scope, sV);
				if (scope!=null) {
					if (this.scope!=scope) this.lastInScope = null;
					this.scope = scope; 
					this.firstInScope = r;
				}
			}
		}
		return r;
	} 
	function getLast(scope, skipVis) {
		var l = tabItems.length-1, r = null, sV=(skipVis!=null && skipVis);
		if (tabItems.length==0)	r = null;
		else {
			if (scope!=null && this.scope==scope && this.lastOutScope!=null) r = this.lastInScope;
			else {
				if ((tabItems[l]).tabOrder >= 0 && (sV || hX_5.i.isFocusable(tabItems[l].obj)) && (scope==null || hX_5.isChildTag(tabItems[l].obj, scope)))
					r = (tabItems[l].obj);
				else 
					r = gFocus(tabItems[l].obj, -1, false, scope, sV);
				if (scope!=null) {
					if (this.scope!=scope) this.firstInScope = null;
					this.scope = scope; 
					this.lastInScope = r;
				}
			}
		}
		return r;
	}

	// public
	function gFocus(obj, delta, skipH, inscope, skipVis) {
		var r=null, ix, hit=false;
		// Find the item
		for (ix=0;ix<tabItems.length; ix++) {
			if (tabItems[ix].obj == obj) { hit=true; break; }
		}
		// Advance to the next one, return this obj if we can't advance
		if (hit) {
			r = (tabItems[ix]).obj;
			var obj0=r, ix0=ix+delta, i=0;
			hit = false;
			while (ix!=ix0 && !hit && i<tabItems.length) {
				// Wrap to other end of array?
				if (ix0 < 0) ix0 = tabItems.length-1;
				if (ix0 >= tabItems.length)	ix0 = 0;
				if (!skipH || tabItems[ix0].obj.id==undefined || !(skipH && tabItems[ix0].obj.id.length>10 && tabItems[ix0].obj.id.substr(tabItems[ix0].obj.id.length-10)=="_HXB_HPBTN")) {
					// Ignore item if tabOrder is < 0 (IE rule, Mozilla undefined in this case)
					obj0 = (tabItems[ix0]).obj;
					if ((tabItems[ix0]).tabOrder >= 0)
						// Make sure the element is enabled
						// N.B.  we used to check if <a> tags had content as part of this check.  
						// Empty <a> tags are valid tab stops in IE/Mozilla and are used to force
						// focus into divs and stuff that have no editable content, so we no longer check this
						hit = (inscope==null || (inscope!=null  && hX_5.isChildTag(obj0, inscope))) && (skipVis || hX_5.i.isFocusable(obj0));
				}
				ix0 += delta; i++;
			}
			r = (hit) ? obj0 : r;
		}
		return r;
	}
}
