/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2006 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

//----------------------------------------------------
// Number Converter, Validator and Assist
//----------------------------------------------------
//
//  ----------------------------------------------------------------------------------------
//  Number Converter
//  ----------------------------------------------------------------------------------------
//
//  Methods (common to all converters)
//
//	constructor		Create a converter with a specified set of attributes
//	setAttribute	Change an attribute of the converter
//  getAttribute	Get an attribute of the converter
//	stringToValue	Convert a string to a JS Number object according to the converter's rules
//  valueToString	Convert a JS Number object to a string according to the converter's rules
//  constantToValue Convert a string in "constant" format to a JS Number Object
//  valueToConstant	Convert a JS Number object to a "constant" format of the form ########0.00
//	lastError		Return a (localized) string describing the reason the last conversion failed
//	getMaxLength	Return the maximum length of a value
//
//  Constructor attributes:
//
//  NOTE:  The number converter uses "Java" formatting/parsing for the number.  It assumes
//		   the server sends down a Java pattern pulled out of a NumberFormat.DecimalFormat
//		   instance (which can be constructed either using a supplied pattern and locale
//		   or you can construct it using a "type" such as "integer" and a locale).
//		   In effect, given an instance of DecimalFormat, the pattern handed to Javascript
//		   is the pattern obtained from "DecimalFormat.toPattern" and the "locale" information
//		   is a string built out of the "DecimalFormatSymbols.getDecimalSymbols".
//
//		   If the ICU4J attribute is set, then the converter assumes that the Java ICU4J
//		   extensions to the DecimalFormat class are in play.
//
//	Description		Meaning
//  -----------		------------------------------------------------------------------------
//
//	strict			How strict to be when converting, 0 = liberal, 1 = strict, 2 = very strict
//	pattern			The format pattern.  See below.
//					NOTE: in the pattern passed to JS, the total number of digits (# or 0) in the
//					pattern should be no more than 16. This is pretty much the limit of Javascript's
//					precsion (17/18 sometimes work).
//	ICU4J			Treat pattern as containing additional ICU4J pattern characters and
//				   	treat pattern as using primary and secondary grouping symbols.
//	digits			If present, the first character (in Unicode) of a run of 10 characters
//					that represent the digits 0 through 9.  stringToValue will parse these digit symbols
//					while valueToString will output these digit symbols.
//	locale			The localization symbols as a string with a well known format.
//					These come from the Java "DecimalFormatSymbols" class. 
//					They are strung together as a string of the form:
//					GDMPINC<C..>
//	G				DecimalFormatSymbols.getGroupingSeparator.  (Character) Grouping separator character, that is the "thousands separator".  For example "," as in 1,000.
//  D				DecimalFormatSymbols.getDecimalSeparator.  (Character) Decimal point character, for example, "." in 123.45
//					NOTE:  if the format is a currency, send DecimalFormatSymbols.getMonetaryDecimalSeparator. (Character) Decimal point character used if a currency symbol
//					instead.
//  P				DecimalFormatSymbols.getPercent.  (Character) Percent character
//  I				DecimalFormatSymbols.getPerMill.  (Character) per mille symbol 
//  N				DecimalFormatSymbols.getMinusSign.  (Character) Negative sign character, for example "-" in -3.0
//	C				DecimalFormatSymbols.getCurrencySymbol. (String) Currency symbol, for example, $ (US)
//					NOTE the Java restriction that the symbol only appears in either the left or right prefix
//					but not both.
//			
//					For example " ,%-FR" means use space as the grouping separator,
//					comma as the decimal symbol, percent sign as the percent symbol, 
//					use the mille sign as the mille symbol, use the minus sign as
//					the negative symbol, and FR as the currency symbol.  So if the 
//					format was " #,##0.00", an example number would be "FR 1 23,45"
//
//  Relevant Java DecimalFormatSymbols currently not supported in this locale string.
//	lZ			  	DecimalFormatSymbols.getZeroDigit	
//	lI  			DecimalFormatSymbols.getInfinity
//	lN				DecimalFormatSymbols.getNaN
//
// The "pattern" has the following form: (braced items are optional)
//
// Pattern:
//          PositivePattern {/ Modifier }
//          PositivePattern {; NegativePattern} {/ Modifier }
//  PositivePattern:
//          {Prefix} Number {Suffix}
//  NegativePattern:
//          {Prefix} Number {Suffix}
//  Prefix:
//          any Unicode characters except special characters described below
//  Suffix:
//          any Unicode characters except special characters described below
//  Number:
//          Integer {Exponent}
//          Integer {. {Fraction} {Exponent}}
//  Integer:
//          MinimumInteger
//          #
//          # Integer
//          # , Integer
//  MinimumInteger:
//          0
//          0 MinimumInteger
//          0 , MinimumInteger
//  Fraction:
//          MinimumFraction {OptionalFraction}
//  MinimumFraction:
//          0 MinimumFraction
//  OptionalFraction:
//          # OptionalFraction
//  Exponent:
//          E {+} MinimumExponent
//  MinimumExponent:
//          0 MinimumExponent
//	Modifier:
//			up to three Unicode characters including special charactes described below
//
//	Special Characters (shown in the above description)
//	0 	Number 	Digit 
//	# 	Number 	Digit, zero shows as absent 
//	. 	Number 	Decimal separator or monetary decimal separator 
//	- 	Number 	Minus sign 
//	+ 	Number 	Plus sign 
//	, 	Number 	Grouping separator 
//	E 	Number 	Separates mantissa and exponent in scientific notation. Need not be quoted in prefix or suffix. 
//	; 	Subpattern boundary 	Separates positive and negative subpatterns
//
//	Special Characters that may be used in the Prefix or Suffix
//	% 	Multiply by 100 and show as percentage 
//	 	(\u2030) 	Multiply by 1000 and show as per mille 
//	' 	Used to quote special characters in a prefix or suffix, for example, "'#'#" formats 123 to "#123". To create a single quote itself, use two in a row: "# o''clock". 
//	 	(\u00A4) 	Currency sign, replaced by currency symbol. If doubled, replaced by international currency symbol. If present in a pattern, the monetary decimal separator is used instead of the decimal separator. 
//  *	(asterisk) In ICU4J mode, this is allowed at start/end of the prefix suffix.  The character
//      that follows is used as the pad character.  When the field is formatted, the pad character is inserted
//      at the marked position as many times as needed to pad the string out to it's maximum length.
//
//  Special parsing rules:
//	1)	If a negative pattern is provided, the "Number" is irrelevant.  Only the positive pattern's "Number" is used
//  2)	If a grouping character is provided, the first "group" to the left of the decimal point defines the grouping
//	    pattern used throughout.  For example, #,##,##0.00 defines a grouping of "3" that's used throughout
//		If ICU4J mode is turned on, the "last two" groupings are applied.  For example,  #,##,##0.00 in this case
// 		defines a grouping of 3 preceded by a grouping of 2.  This alternativing pattern is repeated throughout the string.
//	3)  The characters after the modifier define
//		1.  what character is displayed in lieu of leading zeroes, e.g., *
//		2.  what character is displayed in lieu of trailing zeroes, eg., *
//		3.  what character is used to display zero (repeated throughout value)

// Public:  Construct a number converter
//			Takes zero or more (comma separated "attributename:value" strings as arguments
//			The attribute table defines the attribute's that are allowed
HxG_5.prototype.NumberConverter = function () {
	this.type = hX_5.CVT_number;
	this.lc = hX_5.currentLocale();

	// Local variables to hold the attributes supplied
	this.nStrict = hX_5.NFMT_DEFAULTSTRICT;		// 0 = loose, 1 = normal, 2 = strict
	this.pattern = hX_5.NFMT_DECIMAL0;			// Format pattern
	this.locale  = hX_5.NFMT_LOCALE0;			// Locale string
	this.invalidPattern = false;				// Pattern is good/bad
	this.ICU4J	= false;						// If true, ICU4J parsing is used
	this.digitFirst = null;						// If present, start of non-western digit set

	// Pattern parsed out into key information (based on Java DecimalFormat class)
	// the one that's "generated" if type is specified
	this.upattern = null;						// Pattern with modifiers stripped from it
	this.DF_GroupingSize = this.DF_GroupingSecSize = null;	// Can be zero (no grouping), if groping, can have a secondary one too
	this.DF_NegativePrefix = this.DF_NegativeSuffix = this.DF_PositivePrefix = this.DF_PositiveSuffix = null; 	// Positive/Negative prefix/suffix
	this.DF_NegativePattern = null;				// Has a negative pattern of some sort
	this.DF_IsDecimalShown = null; 				// If fraction off, does decimal char show (i.e., for integers)
	this.DF_PaddingChar = hX_5._E;				// ICU4J style padding - character
	// ICU4J padding
	this.DF_PaddingPos = this.DF_PaddingLeft = this.DF_PaddingRight = null;
	// Min/max number of digits (integer/fractional/exponential)
	this.DF_MinIntegerDigits = this.DF_MaxIntegerDigits = this.DF_MinFractionDigits = this.DF_MaxFractionDigits = this.DF_MinExponentDigits = null;
	// Max length without and with padding
	this.DF_MaxLength = this.DF_MaxPadLength = null;
	// Flags about pattern
	this.isExponent = this.isExponentSigned = this.isPercent = this.isMille = false;						// True if this an "mille" pattern
	this.FMTChars = ";0#.,E";					// Special chars (not in prefix/suffix)
	this.PREChars = hX_5.STR_percent_char+hX_5.STR_mille_char+hX_5.STR_icurrency_char;
												// Special chars (in prefix or suffix) (excluding quote char and minus)
	this.hardPlus = "+";
	this.hardMinus = "-";
	this.hardE = "E";
	this.hardS = "*";
	this.xDigits = this.zeroDigit = null;

	// The following represent the locale parsed out
	this.LC_groupingSymbol = this.LC_decimalSymbol = this.LC_percentSymbol = this.LC_milleSymbol = this.LC_minusSymbol = this.LC_currencySymbol = null;
	this.LC_paddingChar = hX_5._Z;				// Default padding character (reset for some languages)

	// Error handling -- error message associated with last failure
	this.msgFail = this.intMsgFail = hX_5._E;

	// Assist handling
	this.signedConverter = this.zeroConverter = null;

	// Attributes supported on this object
	this.attributeTable = [];
		this.attributeTable ['strict']	= ['nStrict',		hX_5.ARG_TO_NUM,	false,	0,	2];
		this.attributeTable ['pattern']	= ['pattern',		hX_5.ARG_TO_NAS,	false,	null,	null];
		this.attributeTable ['locale']	= ['locale',		hX_5.ARG_TO_NAS,	false,	null,	null];
		this.attributeTable ['digits']	= ['digitFirst',	hX_5.ARG_TO_NUM,	false,	0,	999999];
    	this.attributeTable ['icu4j']	= ['ICU4J', 		hX_5.ARG_IS_TRUE,	false,	null,	null];
	this.setAttribute(arguments);	
}

// Public:  Set an "attribute" on the converter.  
//			a 	= a single attribute string or an array of them
HxG_5.prototype.NumberConverter.prototype.setAttribute = function(a) {

	hX_5.parseArg (this.attributeTable, this, a);

	// Parse the digits string
	this.parseDigits();

	// Parse the locale string
	this.parseLocaleString();

	// Parse the pattern string
	this.parsePatternString();

	// If we have a sub-converter for assist, rebuild it
	if (this.signedConverter)
		this.genAssistMasks();
	return true;
}

// Private: Parse the digits string
HxG_5.prototype.NumberConverter.prototype.parseDigits = function () {
	this.xDigits = null;
	this.zeroDigit = hX_5._Z;
	if (this.digitFirst != null) {
		this.xDigits = String.fromCharCode(this.digitFirst, this.digitFirst+1, this.digitFirst+2, this.digitFirst+3, this.digitFirst+4, this.digitFirst+5, this.digitFirst+6, this.digitFirst+7, this.digitFirst+8, this.digitFirst+9);
		this.zeroDigit = this.xDigits.charAt(0);
	}
	this.LC_paddingChar = this.zeroDigit;
}

// Private: Parse the locale string
HxG_5.prototype.NumberConverter.prototype.parseLocaleString = function () {

	this.LC_groupingSymbol = (this.locale.charAt(0) == hX_5._E) ? hX_5.NFMT_LOCALE0.charAt(0) : this.locale.charAt(0);
	this.LC_decimalSymbol  = (this.locale.charAt(1) == hX_5._E) ? hX_5.NFMT_LOCALE0.charAt(1) : this.locale.charAt(1);
	this.LC_percentSymbol  = (this.locale.charAt(2) == hX_5._E) ? hX_5.NFMT_LOCALE0.charAt(2) : this.locale.charAt(2);
	this.LC_milleSymbol    = (this.locale.charAt(3) == hX_5._E) ? hX_5.NFMT_LOCALE0.charAt(3) : this.locale.charAt(3);
	this.LC_minusSymbol    = (this.locale.charAt(4) == hX_5._E) ? hX_5.NFMT_LOCALE0.charAt(4) : this.locale.charAt(4);
	this.LC_currencySymbol = (this.locale.substr(5) == hX_5._E) ? hX_5.NFMT_LOCALE0.substr(5) : this.locale.substr(5);

}

// Private: Parse the format string
HxG_5.prototype.NumberConverter.prototype.parsePatternString = function () {

	// Reset the parse results to null
	this.DF_GroupingSize = this.DF_GroupingSecSize = 0;
	this.DF_NegativePrefix = this.DF_NegativeSuffix = this.DF_PositivePrefix = this.DF_PositiveSuffix = hX_5._E;
	this.DF_NegativePattern = this.DF_IsDecimalShown = false;
	this.DF_MinIntegerDigits = this.DF_MaxIntegerDigits = this.DF_MinFractionDigits = this.DF_MaxFractionDigits = this.DF_MinExponentDigits = 0;
	this.DF_PaddingPos = null;
	this.DF_PaddingChar = this.DF_PaddingLeft = this.DF_PaddingRight = hX_5._E;
	this.isExponent = this.isExponentSigned = this.isPercent = this.isMille = this.invalidPattern = false;
	this.DF_MaxLength = this.DF_MaxPadLength = 0;

	// Walk the format
	this.upattern = this.pattern;
	var fmt = this.pattern, index, i, j=0, xchar, nchar, bInPositivePattern=true, bInPrefix=true;
	var bInNumber=false, bInExponent=false, bInQuote=false, bHaveGroup=0;
	var bInMinInteger=false, bInMaxInteger=false, bInMinFraction=false, bInMaxFraction=false;

	// Strip off the trailing modifier if there is one
	for (i = fmt.length-1; j < 4 && i > 0; i--) {
		xchar = fmt.charAt(i);
		if (xchar==hX_5._SQT || xchar==hX_5._P || xchar==hX_5._SM)
			break;
		if (hX_5._SL == fmt.charAt(i)) {
			xchar = fmt.substr(i+1);
			this.upattern = fmt = fmt.substr(0,i);
			if (xchar.length==1)
				this.DF_PaddingLeft = xchar;
			else if (xchar.length==2) {
				this.DF_PaddingLeft = xchar.substr(0,1);
				this.DF_PaddingRight = xchar.substr(1,1);
			}
			break;
		}
		j--;
	}

	var len = fmt.length;
	for (index = 0; index < len; index++) {
		xchar = fmt.charAt(index);
		// If in quote, continue it
		if (bInQuote) {
			// Are we ending the quote?
			if (xchar == hX_5._SQT) {
				nchar = fmt.charAt(index+1);
				// It's just a quote character?
				if (nchar == hX_5._SQT) {
					this.addToPreSuf (bInPositivePattern, bInPrefix, bInQuote, xchar);
					index++;
				} else {
					bInQuote = false;
				}
			} else {
				this.addToPreSuf (bInPositivePattern, bInPrefix, bInQuote, xchar);
			}
		// If this isn't a special character, then it goes into a prefix/suffix
		// There are "bugs" in Java's parsing that we won't repeat such as treating 0 and #
		// in the suffix as if they precede the suffix.  Also, if in the negative pattern, 
		// Java treats leading/trailing "." and "," as part of the prefix/suffix
		} else if (!hX_5.isChar(xchar, this.FMTChars)) {
			// If we're in a prefix, then we'll just add the character 
			// If we're not in a prefix, end whatever we're in, start a suffix and add it
			if (!bInPrefix) bInNumber = bInMinInteger = bInMaxInteger = bInMinFraction = bInMaxFraction = bInExponent = false;

			// Asterisks are special
			if (this.ICU4J && xchar == this.hardS) {
				if (this.DF_PaddingPos || fmt.charAt(index+1)==null) {
					this.invalidPattern	= true;
					this.intMsgFail = hX_5.INTERR_Format_Padding;
					break;
				}
				if (bInPositivePattern) {
					if (bInPrefix) {
						this.DF_PaddingPos = (this.DF_PositivePrefix.length == 0) ? 1:2;
						bInPrefix = false; bInNumber = true;
					} else
						this.DF_PaddingPos = (this.DF_PositiveSuffix.length == 0) ? 3:4;
					this.DF_PaddingChar = fmt.charAt(index+1);
					this.upattern = this.upattern.substr(0,index)+this.upattern.substr(index+2);
				}
				index++;
			// Quotes are special
			} else if (xchar == hX_5._SQT) {
				nchar = fmt.charAt(index+1);
				// It's just a quote character?
				if (nchar == hX_5._SQT) {
					this.addToPreSuf (bInPositivePattern, bInPrefix, bInQuote, xchar);
					index++;
				} else {
					bInQuote = true;
				}
			// Not a special literal character so add it to pre/suffix
			} else {
				this.addToPreSuf (bInPositivePattern, bInPrefix, bInQuote, xchar);
			}
		// It is a special character
		} else {
			// Start a negative pattern
			if (xchar == hX_5._SM) {
				// Close off anything that's in progress and start the new prefix
				bInPositivePattern = bInNumber = bInMinInteger = bInMaxInteger = bInMinFraction = bInMaxFraction = bInExponent = false;
				this.DF_NegativePattern = true;
				bInPrefix = true;
			// We're in the actual Number
			} else {
				// If we're not in a prefix or a Number, we have a problem
				if (!bInPrefix && !bInNumber) {
					this.invalidPattern	= true;
					this.intMsgFail = hX_5.INTERR_Format_Symbol;
					break;
				}
				bInPrefix = false; bInNumber = true;
				if (!bInPositivePattern) {
					// no-op -- don't care what's here as long as it's a number symbol
					// except if we're treating an escape character (E or *)
					if (xchar == this.hardE && fmt.charAt(index+1)) index++;
					if (xchar == this.hardS) index++;
				} else {
					if (xchar == hX_5._Z) {
						if (bInExponent) {
							this.DF_MinExponentDigits++;
						} else if (bInMinInteger) {
							this.DF_MinIntegerDigits++;
						} else if (bInMinFraction) {
							this.DF_MinFractionDigits++;
						} else if (bInMaxFraction) {
							this.invalidPattern	= true;
							this.intMsgFail = hX_5.INTERR_Format_ZSymbol;
							break;	
						} else {	// starting case must be in MaxInteger or start of number
							if (this.DF_MinIntegerDigits > 0) {
								this.invalidPattern	= true;
								this.intMsgFail = hX_5.INTERR_Format_ZSymbol;
								break;
							}
							this.DF_MinIntegerDigits++;
							bInMaxInteger = false; bInMinInteger = true;
						}
						if (bInMaxInteger || bInMinInteger)	this.DF_GroupingSize++;
					} else if (xchar == hX_5._LB) {
						if (bInMinInteger || bInExponent) {
							this.invalidPattern	= true;
							this.intMsgFail = hX_5.INTERR_Format_LSymbol;
							break;
						} else if (bInMaxInteger) {
							this.DF_MaxIntegerDigits++;
						} else if (bInMaxFraction) {
							this.DF_MaxFractionDigits++;
						} else if (bInMinFraction) {
							this.DF_MaxFractionDigits++;
							bInMinFraction = false; bInMaxFraction = true;
						} else {
							this.DF_MaxIntegerDigits++;
							bInMaxInteger = true;
						}
						if (bInMaxInteger || bInMinInteger)	this.DF_GroupingSize++;
					} else if (xchar == hX_5._C) {
						if (!bInMinInteger && !bInMaxInteger) {
							this.invalidPattern	= true;
							this.intMsgFail = hX_5.INTERR_Format_Group + bInMaxInteger + hX_5._P;
							break;
						}
						if (bInMaxInteger || bInMinInteger)	{
							this.DF_GroupingSecSize = this.DF_GroupingSize;
							this.DF_GroupingSize = 0;
						}
						bHaveGroup++
					} else if (xchar == hX_5._P) {
						if (bInExponent || this.DF_IsDecimalShown) {
							this.invalidPattern	= true;
							this.intMsgFail = hX_5.INTERR_Format_Decimal;
							break;
						}
						bInMinInteger = false; bInMaxInteger = false; bInMinFraction = true;
						this.DF_IsDecimalShown = true;
					} else if (xchar == this.hardE) {
						if (this.isExponent || (!bInMinInteger && !bInMaxInteger && !bInMinFraction && !bInMaxFraction)) {
							this.invalidPattern	= true;
							this.intMsgFail = hX_5.INTERR_Format_Exponent;
							break;
						}
						bInMinInteger = bInMaxInteger = bInMinFraction = bInMaxFraction = false; 
						bInExponent = true;
						this.isExponent = true;
						if (this.ICU4J && fmt.charAt(index+1) == this.hardPlus) {
							this.isExponentSigned = true;
							index++;
						}
					}
				} // End positive pattern
			} // End Number
		} // end handling special character
	} // end for loop walking the string
	
	// Verify that the results are valid
	this.DF_MaxIntegerDigits  += this.DF_MinIntegerDigits;
	this.DF_MaxFractionDigits += this.DF_MinFractionDigits;
	if (bHaveGroup==0 || this.isExponent) this.DF_GroupingSize = this.DF_GroupingSecSize = 0;
	if (this.DF_GroupingSize > 0) {
		if (!this.ICU4J || this.DF_GroupingSecSize==0 || bHaveGroup<=1)
			this.DF_GroupingSecSize = this.DF_GroupingSize;
	}
	if (this.isPercent && this.isMille) {
		this.invalidPattern	= true;
		this.intMsgFail = hX_5.INTERR_Format_Percent;
	}
	if (this.DF_PaddingPos)	this.DF_PaddingLeft = this.DF_PaddingRight = hX_5._E;

	// Compute the maximum number of characters in the field
	if (!this.invalidPattern) {
		var pre, post, dec, exp, com=0;
		if (this.isExponent) {
			// FIXME:  The following calculation is only sort of accurate.  
			pre = Math.max(this.DF_NegativePrefix.length, this.DF_PositivePrefix.length);
			post = Math.max(this.DF_NegativeSuffix.length, this.DF_PositiveSuffix.length);
			dec = (this.DF_IsDecimalShown) ? this.LC_decimalSymbol.length : 0;
			exp = this.DF_MinExponentDigits + ((this.isExponentSigned)?1:0);
			this.DF_MaxLength = pre + this.DF_MaxIntegerDigits + dec + this.DF_MaxFractionDigits + 1 + exp + post;
			this.DF_MaxPadLength = this.DF_PositivePrefix.length + this.DF_MaxIntegerDigits + dec + this.DF_MaxFractionDigits + 1 + exp + this.DF_PositiveSuffix.length;
		} else {
			pre = Math.max(this.DF_NegativePrefix.length, this.DF_PositivePrefix.length);
			post = Math.max(this.DF_NegativeSuffix.length, this.DF_PositiveSuffix.length);
			dec = (this.DF_IsDecimalShown) ? this.LC_decimalSymbol.length : 0;
			if (this.DF_GroupingSize == 0 || this.DF_GroupingSize > this.DF_MaxIntegerDigits)
				com = 0;
			else if (this.DF_GroupingSize == this.DF_GroupingSecSize)
				com = Math.floor((this.DF_MaxIntegerDigits-1)/this.DF_GroupingSize);
			else {
				i = this.DF_MaxIntegerDigits; j = this.DF_GroupingSize;
				while (i > j) {
					com++;
					i -= j;
					j = (j == this.DF_GroupingSize) ? this.DF_GroupingSecSize : this.DF_GroupingSize;
				}
			}
			this.DF_MaxLength = pre + this.DF_MaxIntegerDigits + com + dec + this.DF_MaxFractionDigits + post;
			this.DF_MaxPadLength = this.DF_PositivePrefix.length + this.DF_MaxIntegerDigits + com + dec + this.DF_MaxFractionDigits + this.DF_PositiveSuffix.length;
		}	
		if (!this.DF_NegativePattern) this.DF_MaxLength++;
		if (!this.DF_NegativePattern) this.DF_MaxPadLength++;
	}
}

// Private:  Helper function that adds to the right prefix/suffix
HxG_5.prototype.NumberConverter.prototype.addToPreSuf = function (bInPositivePattern, bInPrefix, bInQuote, c) {

	var xchar = c;
	// if not in a quote, do character substitutions
	if (!bInQuote && hX_5.isChar(xchar, this.PREChars)) {
		if (xchar == hX_5.STR_percent_char) {
			xchar = this.LC_percentSymbol;
			this.isPercent = true;
		// mille symbol
		} else if (xchar == hX_5.STR_mille_char){
			xchar = this.LC_milleSymbol;
			this.isMille = true;
		// international currency symbol
		} else if (xchar == hX_5.STR_icurrency_char) {
			xchar = this.LC_currencySymbol;
		}
	}

	// Add the character(s) to the correct prefix/suffix
	if (bInPositivePattern) {
		if (bInPrefix)
			this.DF_PositivePrefix = this.DF_PositivePrefix + xchar;
		else 
			this.DF_PositiveSuffix = this.DF_PositiveSuffix + xchar;
	} else {
		if (bInPrefix)
			this.DF_NegativePrefix = this.DF_NegativePrefix + xchar;
		else 
			this.DF_NegativeSuffix = this.DF_NegativeSuffix + xchar;
	}
}

// Public: Retrieve the value of an attribute
//		   attribute = the name of the attribute 	
HxG_5.prototype.NumberConverter.prototype.getAttribute = function (/* string*/ attribute) {
	return (hX_5.getArg(this.attributeTable, this, attribute));
}

// Public:	Return the reason the last conversion failed as a localized message
HxG_5.prototype.NumberConverter.prototype.lastError = function () {
	return ((this.msgFail && this.msgFail.length > 0) ? this.msgFail : null);
}

// Public:	Return the maximum length of a value that uses this converter
HxG_5.prototype.NumberConverter.prototype.getMaxLength = function (v) {
	if (!this.DF_NegativePattern && v && this.DF_MaxLength>1)
		return ((v.validate(-1, this) || v.validate(-10, this))?this.DF_MaxLength:this.DF_MaxLength-1);
	return this.DF_MaxLength;
}

// For assist, given the pattern set on the converter:
// 1) Create a second one that has real vs. implied signs in it
// 2) Create a third one that puts a "zero" digit in only one place -- to the left of the decimal point
HxG_5.prototype.NumberConverter.prototype.genAssistMasks = function() {
	// See if the converter already has "signs" in it and NO padding
    var args = [], pop=false;
	if (this.nStrict)	args.push("strict:" +this.nStrict);
	if (this.locale)	args.push("locale:" +this.locale);
	if (this.digitFirst)args.push("digits:" +this.digitFirst);
	if (this.ICU4J)		args.push("icu4j:"  +this.ICU4J);
	if (this.DF_NegativePattern) {
		if  (this.DF_PaddingLeft.length==0 && this.DF_PaddingRight.length==0) {
			this.signedConverter = this;
		} else {
			// Pattern had a modifier in it, we need a signed converter without it
			if (this.upattern)	args.push("pattern:"+this.upattern);
			pop=true;
			this.signedConverter = new hX_5.NumberConverter(args[0],args[1],args[2],((args.length>3)?args[3]:null),((args.length>4)?args[4]:null),((args.length>5)?args[5]:null));
		}
	} else {
	// It doesn't so add a "negative" version of the pattern using a "minus" sign
		if (this.upattern)	args.push("pattern:"+this.upattern + hX_5._SM + this.LC_minusSymbol + this.upattern);
		pop=true;
		this.signedConverter = new hX_5.NumberConverter(args[0],args[1],args[2],((args.length>3)?args[3]:null),((args.length>4)?args[4]:null),((args.length>5)?args[5]:null));
	}
	// Given this "signed" converter, create one that has only a single "0" in the pattern
	var p1 = hX_5._E, p = this.signedConverter.upattern;
	// Replace all zeroes with # chars
	p = hX_5.replaceAll(p, hX_5._Z,hX_5._LB);
	// Split the pattern into pos and neg
	var k = p.indexOf(hX_5._SM);
	if (k >= 0) {
		p1 = p.substr(k);
		p = p.substr(0,k);
	}
	// Pos pattern - replace the # before the decimal point with a zero
	// One could argue that we should leave a # before the decimal point if there
	// was one in the original pattern, but it seems preferable that while inputting
	// values, we use the same base pattern no matter what so all input works the same way
	k = p.indexOf(hX_5._P);
	if (k > 0 && p.charAt(k-1)==hX_5._LB)
		p = p.substr(0,k-1) + hX_5._Z + p.substr(k); 
	else if (k < 0) {
		k = p.length-1;
		while (k >= 0 && p.charAt(k) != hX_5._LB) k--;
		p = p.substr(0,k) + hX_5._Z + p.substr(k+1); 
	}
	// Neg pattern - replace the # before the decimal point with a zero
	if (p1.length > 0) {
		k = p1.indexOf(hX_5._P);
		if (k > 0 && p1.charAt(k-1)==hX_5._LB)
			p1 = p1.substr(0,k-1) + hX_5._Z + p1.substr(k); 
		else if (k < 0) {
			k = p1.length-1;
			while (k >= 0 && p1.charAt(k) != hX_5._LB) k--;
			p1 = p1.substr(0,k) + hX_5._Z + p1.substr(k+1); 
		}
	}
	p = p+p1;
	if (pop) args.pop();
	args.push("pattern:"+p);
	this.zeroConverter = new hX_5.NumberConverter(args[0],args[1],args[2],((args.length>3)?args[3]:null),((args.length>4)?args[4]:null),((args.length>5)?args[5]:null));
}

// Public:  Convert a string to a Javascript Number according the format/locale in play
//			numString	= the string to be converted
//			return		= a JS Number if conversion succeeds
//						  null and lastError of hX_5.ERR_empty if string is empty (null)
//						  null and lastError if string cannot be parsed
HxG_5.prototype.NumberConverter.prototype.stringToValue = function (/*String*/ numString, promptChar) {

	// Set up arguments passed to a failure
	this.msgFail = this.intMsgFail = hX_5._E;
	var argsFail = [numString];

	// Handle empty/null string
	if (numString == null || hX_5.isUnDef(typeof(numString)) || !hX_5.isString(numString)) {
		this.msgFail = hX_5.L[this.lc].ERR_empty;
		return null;
	}
	if (this.invalidPattern) {
		this.msgFail = hX_5.L[this.lc].ERR_number_fail;
		return null;
	}
	// Strip lead trailing spaces (ok even if they are in the suffix/prefix)
	var sNum = hX_5.Trim(numString);
	if (this.DF_PaddingPos == 1) sNum = hX_5.trimL(sNum, this.DF_PaddingChar);
	if (this.DF_PaddingPos == 4) sNum = hX_5.trimR(sNum, this.DF_PaddingChar);
	if (this.DF_PaddingRight.length > 0 && this.DF_PaddingLeft.length > 0 && this.DF_PaddingRight == hX_5._B && this.DF_PaddingLeft == this.DF_PaddingRight)
		sNum = numString;
	if (sNum.length==0) {
		this.msgFail = hX_5.L[this.lc].ERR_empty;
		return null;
	}

	// See if we can strip off the prefix/suffix
	var i, j, sign = +1;

	// See if we have patterns to match, it we don't use "standard" positive and negative patterns
	var preposmatch = false, prenegmatch = false, postposmatch = false, postnegmatch = false;
	var nopos = this.DF_PositivePrefix.length == 0 && this.DF_PositiveSuffix.length == 0;
	var noneg = this.DF_NegativePrefix.length == 0 && this.DF_NegativeSuffix.length == 0;
	if ((!this.DF_NegativePattern) && sNum.charAt(0) == this.LC_minusSymbol) {
		sNum = sNum.substr(1);
		sign = -1;
		prenegmatch = true;
		postnegmatch = true;
	}
	if (sign > 0 && nopos && sNum.charAt(0) == this.hardPlus) {
		// Appears to be a holdover from the Java 1.0 days
		sNum = sNum.substr(1);
		preposmatch = true;
		postposmatch = true;
	}

	var pre, post, match = false;
	// See if we match the negative pattern
	if (!noneg) {
		pre  = hX_5.Trim(this.DF_NegativePrefix.toUpperCase());
		post = hX_5.Trim(this.DF_NegativeSuffix.toUpperCase());
		prenegmatch = !(pre.length > 0);
		postnegmatch = !(post.length > 0);
		if (!prenegmatch && sNum.length >= pre.length && pre == (sNum.substr(0,pre.length)).toUpperCase())
			prenegmatch = true;
		if (!postnegmatch && sNum.length >= post.length && post == (sNum.substr(sNum.length-post.length)).toUpperCase())
			postnegmatch = true;
		if (prenegmatch && postnegmatch) {
			match = true;
			sNum = sNum.substr(pre.length);
			sNum = sNum.substr(0,sNum.length-post.length);
		}
	}
	// See if we match the postive pattern
	if (!nopos && !match) {
		pre  = hX_5.Trim(this.DF_PositivePrefix.toUpperCase());
		post = hX_5.Trim(this.DF_PositiveSuffix.toUpperCase());
		preposmatch = !(pre.length > 0);
		postposmatch = !(post.length > 0);
		if (!preposmatch && sNum.length >= pre.length && pre == (sNum.substr(0,pre.length)).toUpperCase())
			preposmatch = true;
		if (!postposmatch && sNum.length >= post.length && post == (sNum.substr(sNum.length-post.length)).toUpperCase())
			postposmatch = true;
		if (preposmatch && postposmatch) {
			match = true;
			sNum = sNum.substr(pre.length);
			sNum = sNum.substr(0,sNum.length-post.length);
		}
	}
	// Handle oddball case of no visible negative patten
	if (!match && noneg && this.DF_NegativePattern)
		prenegmatch = postnegmatch = match = true;

	// Handle the various match cases
	if (prenegmatch && postnegmatch)
		sign = -1;
	else if (preposmatch && postposmatch)
		sign = +1;
	else if (!noneg || !nopos) {
		// We have either a positive or a negative pattern and we have no matches
		// Then if we're not strict, just assume positive and see if we can parse
		// the thing as digits (done below).
		if (this.nStrict > 1) {
			// If we're strict, we fail if both patterns are present -- we didn't
			// match either.  If no pattern is present, for one, we're OK
			// as we can assume we matched it
			if (!noneg && !nopos)
				this.msgFail = hX_5.L[this.lc].ERR_number_fail;
		}
	}

	// Split the number into left and right		
	if (this.msgFail.length == 0) {
		var left, right;
		if (promptChar)
			sNum = hX_5.replaceAll(sNum, promptChar, hX_5._B);
		sNum = hX_5.Trim(sNum);
		if (this.DF_PaddingPos >= 1 && this.DF_PaddingPos <= 2) sNum = hX_5.trimL(sNum, this.DF_PaddingChar);
		if (this.DF_PaddingPos >= 3 && this.DF_PaddingPos <= 4) sNum = hX_5.trimR(sNum, this.DF_PaddingChar);
		j = sNum.indexOf(this.LC_decimalSymbol);
		if (j >= 0) {
			left = sNum.substr(0,j);
			right= sNum.substr(j+1);
		} else {
			if (this.isExponent) {
				sNum = sNum.toUpperCase();
				j = sNum.indexOf(this.hardE);
				if (j >= 0) {
					left = sNum.substr(0,j);
					right= sNum.substr(j);
				} else {
					left = sNum;
					right = hX_5._E;
				}
			} else {
				left = sNum;
				right = hX_5._E;
			}
		}
		// Remove any remaining pad characters
		if (this.DF_PaddingLeft.length > 0 && this.DF_PaddingLeft != this.LC_paddingChar) 	left = hX_5.trimL(left, this.DF_PaddingLeft);
		if (this.DF_PaddingPos >= 1 && this.DF_PaddingPos <= 2) 							left = hX_5.trimL(left, this.DF_PaddingChar);
		if (this.DF_PaddingRight.length > 0 && this.DF_PaddingRight != this.LC_paddingChar)	right= hX_5.trimR(right, this.DF_PaddingRight);
		if (this.DF_PaddingPos >= 3 && this.DF_PaddingPos <= 4) 							right= hX_5.trimR(right, this.DF_PaddingChar);
		left = (left.length==0) ? hX_5._Z:left;

		// Waltz to the left of the decimal point, cleaning things up
		var leftc = hX_5._E, rightc = hX_5._E;
		while (left.length > 0) {
			if (this.matchDigit(left)) {
				leftc = leftc + this.getDigit(left);
				left = left.substr(1);
			} else if (!this.isExponent && left.charAt(0) == this.LC_groupingSymbol) {
				left = left.substr(1);
			} else {
				argsFail[1] = left.charAt(0);
				this.msgFail = hX_5.L[this.lc].ERR_number_char;
				break;
			}
		}
		// Now the right
		var bE = false, bS = false;
		while ((this.msgFail.length == 0) && (right.length > 0)) {
			if (this.matchDigit(right)) {
				rightc = rightc + this.getDigit(right);
				right = right.substr(1);
			} else if (this.isExponent) {
				if ((!bE && right.charAt(0) == this.hardE) || 
				   ((!bS && (right.charAt(0) == this.LC_minusSymbol || right.charAt(0) == this.hardMinus || right.charAt(0) == this.hardPlus)))) {
					if (right.charAt(0) == this.hardE)
						bE = true;
					else
						bS = true;
					rightc = rightc + right.charAt(0);
					right = right.substr(1);
				} else {
					argsFail[1] = right.charAt(0);
					this.msgFail = hX_5.L[this.lc].ERR_number_char;
					break;
				}
			} else {
				argsFail[1] = right.charAt(0);
				this.msgFail = hX_5.L[this.lc].ERR_number_char;
				break;
			}
		}
		if ((this.msgFail.length == 0) && (leftc.length == 0) && (rightc.length==0))
			this.msgFail = hX_5.L[this.lc].ERR_empty;

		// Handle parsing exponential numbers
		var r, o;
		if (this.msgFail.length == 0 && this.isExponent) {
			rightc = (rightc.length == 0) ? hX_5._Z : rightc;
			sNum = ((sign < 0) ? this.hardMinus:hX_5._E) + leftc + this.LC_decimalSymbol + rightc;
			r = new Number(sNum);
			if (isNaN(r))
				r = null;
			if (r == null)
				this.msgFail = hX_5.L[this.lc].ERR_number_fail;

		// Handle parsing decimal numbers
		} else if (this.msgFail.length == 0) {
		
			// Handle out of bounds
			if (leftc.length == 0)		leftc  = hX_5._Z;
			if (rightc.length == 0)		rightc = hX_5._Z;
			if (isNaN(leftc))			leftc  = hX_5._Z;
			if (isNaN(rightc))			rightc = hX_5._Z;
	
			// Convert it to a number
			// First try to eliminate overflow
			while (leftc.length + rightc.length > 17) {
				if (rightc.length > 0)
					rightc = rightc.substr(0,rightc.length-1);
				else
					leftc = leftc.substr(1);
			}
			o = leftc + hX_5._P + rightc;
			r = new Number(o);
			if (isNaN(r))
				r = null;
			if (r == null) {
				this.msgFail = hX_5.L[this.lc].ERR_number_fail;
			} else {
				if (this.isPercent) r = r/100;
				if (this.isMille)   r = r/1000;
				r = r * sign;
			}
		}
	}

	// If an error was found while parsing, bail out with the error
	if (this.msgFail.length > 0) {
		this.msgFail = hX_5.NLSFormatMsg (this.msgFail, argsFail);
		return null;
	}
	return (r);
}

// Private:  Check the next char is a digit
//			 Returns true or false
HxG_5.prototype.NumberConverter.prototype.matchDigit = function (str) {
	if (str.length > 0) {
		if (hX_5.WDIGITS.indexOf(str.charAt(0)) >= 0) return true;
		if (this.xDigits != null && this.xDigits.indexOf(str.charAt(0)) >= 0) return true;
	}
	return false;
}

// Private:  Return the first character of the string as a Western digit
//			 Returns the digit character (as a string)
HxG_5.prototype.NumberConverter.prototype.getDigit = function (str) {
	if (this.xDigits == null) return (str.charAt(0));
	if (hX_5.WDIGITS.indexOf(str.charAt(0)) >= 0) return (str.charAt(0));
	return (hX_5.WDIGITS.charAt(str.charCodeAt(0) - this.xDigits.charCodeAt(0)));
}

// Private:  Convert a string from Western to non-Western digits
//			 Returns converted string
HxG_5.prototype.NumberConverter.prototype.convertToDigits = function (str) {
	if (this.xDigits == null) return str;
	var r = hX_5._E;
	while (str.length > 0) {
		if (hX_5.WDIGITS.indexOf(str.charAt(0)) >= 0) r = r + this.xDigits.charAt(str.charCodeAt(0)-hX_5.WDIGITS.charCodeAt(0));
		else r = r + str.charAt(0);
		str = str.substr(1);
	}
	return r;
}

// Local:  Return the "zero" digit
HxG_5.prototype.NumberConverter.prototype.getZeroDigit = function () {
	return (this.zeroDigit);
}

// Local:  Is this a zero digit?
HxG_5.prototype.NumberConverter.prototype.isZeroDigit = function (c) {
	return (c==hX_5._Z || c==this.zeroDigit);
}

// Private:  Helper that does a look for a decimal point without regard to what it is
//			 Returns position of decimal
HxG_5.prototype.NumberConverter.prototype.markDecimal = function (str) {
	for (var i = 0; i < str.length; i++) {
		if (!this.matchDigit(str.charAt(i)))
			return i;
		}
	return -1;		
}

// Public:  Convert a JS Number to a string using the format in the converter
//			numValue	-- number to be converted
//			return		-- string  (empty string if value is null)
HxG_5.prototype.NumberConverter.prototype.valueToString = function (/* number */ numValue, promptChar) {

	this.msgFail = hX_5._E;
	if (numValue == null || numValue == 'undefined' || isNaN(numValue) || this.invalidPattern) 
		return hX_5._E;
		
	var strValue = null, i, j, n, g, zero, sign = ((numValue >= 0) ? +1 : -1), abs = Math.abs(numValue);

	// Handle formatting exponential forms
	if (this.isExponent) {

		// Let JS convert number to string, then pull it into pieces
		var nExp = 0, fMant = hX_5._E, dMant = hX_5._E, expnum = new String(abs);
		expnum = expnum.toUpperCase();

		i = expnum.indexOf(this.hardE);
		if (i >= 0) {
			nExp = new Number(expnum.substr(i+1));
			expnum = expnum.substr(0,i);
		}
		i = expnum.indexOf(this.LC_decimalSymbol);
		i = (i < 0) ? expnum.indexOf(hX_5._P) : i;
		dMant = expnum;
		if (i >= 0) {
			dMant = expnum.substr(0,i);
			fMant = expnum.substr(i+1);
		}
		while (dMant.length > 0 && this.isZeroDigit(dMant.charAt(0))) dMant = dMant.substr(1);
		while (fMant.length > 0 && this.isZeroDigit(fMant.charAt(fMant.length-1))) fMant = fMant.substr(0, fMant.length-1);

		// Is this "scientific" notation ?
		var sci = (this.DF_MaxIntegerDigits > this.DF_MinIntegerDigits) && (this.DF_MaxIntegerDigits > 1);
		// Compute how many digits we need to shift from the fractional to decimal portions of the number
		// And adjust the exponent based on the shift
		zero = (dMant.length==0 && fMant.length==0);
		var tmp_DF_MinIntegerDigits = (!sci) ? this.DF_MinIntegerDigits : ((this.DF_MinIntegerDigits > 0) ? 1: 0);
		var e = (zero) ? 0 : tmp_DF_MinIntegerDigits - dMant.length;
		for (i=0; dMant.length==0 && i < fMant.length && this.isZeroDigit(fMant.charAt(i)); i++)	e++;
		nExp -= e;
		//  For sci notation, make sure the exponent is a multiple of MaxIntegerDigits
		if (sci) {
			var eToBe = (nExp<0) ?  ((Math.ceil (Math.abs(nExp)/this.DF_MaxIntegerDigits)) * -this.DF_MaxIntegerDigits) :
									((Math.floor(Math.abs(nExp)/this.DF_MaxIntegerDigits)) *  this.DF_MaxIntegerDigits);
			e = e + (nExp - eToBe);
			nExp = eToBe;
		}
		if (e > 0) {
			fMant = hX_5.padR(fMant,e);
			dMant = dMant + fMant.substr(0,e);
			fMant = fMant.substr(e);
		} else if (e < 0) {
			fMant = dMant.substr(dMant.length+e) + fMant;
			dMant = dMant.substr(0,dMant.length+e);
		}
		
		// Make the exponent string  note that in Java, the "E" in an exponential number is constant across languages
		var exp = (Math.abs(nExp)).toString();
		exp = this.hardE + ((nExp >= 0) ? (this.isExponentSigned?this.hardPlus:hX_5._E):this.LC_minusSymbol) + hX_5.padL(exp, this.DF_MinExponentDigits);

		// Make sure the number has enought significant digits
		// Pad the decimal number to the left and on the right for non-scientific format
		while (dMant.length > tmp_DF_MinIntegerDigits  && this.isZeroDigit(dMant.charAt(0))) dMant = dMant.substr(1);
		var sigDigits = this.DF_MaxIntegerDigits  - dMant.length + 
						((this.DF_MinFractionDigits==0 || 1==1) ? this.DF_MaxFractionDigits : this.DF_MinFractionDigits);
		var fM = hX_5.truncateRoundHalfEven(fMant, sigDigits);
		fMant = fM.value;
		if (fM.overflow) {
			n = new Number(dMant) + 1;
			dMant = ((sign < 0)?this.hardMinus:hX_5._E) + n.toString() + hX_5._P + fMant + exp;
			n = new Number(dMant);
			return (this.valueToString (n));
		} else {
			sigDigits = (zero) ? this.DF_MinFractionDigits : ((!sci) ? this.DF_MinFractionDigits : (this.DF_MinIntegerDigits + this.DF_MinFractionDigits - dMant.length));
			// Note that we don't use the pad characters in exponential numbers
			dMant = hX_5.padL(dMant, tmp_DF_MinIntegerDigits);
			fMant = hX_5.padR(fMant, sigDigits);
			while (fMant.length > sigDigits  && this.isZeroDigit(fMant.charAt(fMant.length-1))) fMant = fMant.substr(0,fMant.length-1);
					
			// Assemble
			strValue = (fMant.length == 0 || !this.DF_IsDecimalShown) ? (dMant + exp) : (dMant + this.LC_decimalSymbol + fMant + exp);
		}
		// done with exponential number
		
	// Handle formatting decimal forms
	} else {
	
		// Split the number into two positive integers representing the
		// left and right of the decimal point. 
		var left = Math.floor(abs), right = abs - left;
		// We run into rounding problems here, so we'll do this "two ways" and
		// if they aren't equal we'll assume the "shorter" way is best
		// Way One:
		var fR, nR, nL, fullstring=abs.toString(), fullleft=fullstring, fullright=hX_5._E, q=this.markDecimal(fullstring);
		if (q >= 0) {
			fullleft = fullstring.substr(0,q);
			fullright = fullstring.substr(q+1);
		}
		// Way Two:
		left = left.toString();
		right = right.toString();
		right = right.substr(this.markDecimal(right)+1);
		
		if (right != fullright && !(right.length==1 && this.isZeroDigit(right))) {
// FIXME:  This is questionable code and should be checked across a lot of numbers
			if (fullright.length < right.length)
				right = fullright;
//			alert ("Rounding Error: left: " + left + " fullleft: " + fullleft + " right: " + right + " fullright: " + fullright);
		}

		// N.B. we don't multiply the number by 100 (or a 1000) to make percents/milles
		// as Javascript seems to love losing precision when you do this 
		if (this.isPercent) {
			right = hX_5.padR(right,2);
			left = left + right.substr(0,2);
			right = right.substr(2);
		} else if (this.isMille) {
			right = hX_5.padR(right,3);
			left = left + right.substr(0,3);
			right = right.substr(3);
		}

		// Convert the left and right into strings
		if (right.length > this.DF_MaxFractionDigits) {
			fR = hX_5.truncateRoundHalfEven(right, this.DF_MaxFractionDigits);
			right = fR.value;
			if (fR.overflow) {
				n = new Number(left);
				n = (n+1);
				left = n.toString();
			}
		}
		
		// Pad out the number with zeroes
		right = hX_5.padR(right, this.DF_MinFractionDigits, this.LC_paddingChar);
		left  = hX_5.padL(left,  this.DF_MinIntegerDigits, this.LC_paddingChar);
		while (left.length  > this.DF_MinIntegerDigits  && left.charAt(0) == this.LC_paddingChar) left = left.substr(1);
		while (right.length > this.DF_MinFractionDigits && right.charAt(right.length-1) == this.LC_paddingChar) right = right.substr(0,right.length-1);
		nR = new Number(right); nL = new Number(left);
		zero = (nR == 0) && (nL == 0);
		var zR=this.LC_paddingChar, zL=zR, zZ=false;
		// Repad the number (now that it's exactly the right size) using the pad character
		if (this.DF_PaddingLeft.length > 0) {
			zL = (this.DF_PaddingLeft.length > 0) ? this.DF_PaddingLeft : this.LC_paddingChar;
			zR = (this.DF_PaddingRight.length > 0) ? this.DF_PaddingRight : this.LC_paddingChar;
			zZ = (this.DF_PaddingLeft.length > 0) && (this.DF_PaddingRight.length > 0) && (this.DF_PaddingLeft == this.DF_PaddingRight);
			if (zL != this.LC_paddingChar) {
				if (zero && zZ) {
					left = hX_5.padR(hX_5._E, this.DF_MinIntegerDigits, zL);
					right = hX_5.padR(hX_5._E, this.DF_MinFractionDigits, zL);
				} else {
					for (i=0; i < left.length; i++) {
						if (left.charAt(i) != this.LC_paddingChar) break;
						left = left.substr(0,i) + zL + left.substr(i+1);
					}
				}
			}
			if (zR != this.LC_paddingChar) {
				for (i=right.length-1; i >= 0; i--) {
					if (right.charAt(i) != this.LC_paddingChar) break;
					right = right.substr(0,i) + zR + right.substr(i+1);
				}
			}
		}

		// If a prompt character is provided, pad out the number on the left/right with it
		if (promptChar) {
			if (left.length > 0 && left.length > this.DF_MaxIntegerDigits)
				left = left.substr(left.length-this.DF_MaxIntegerDigits);
			if (right.length > 0 && right.length > this.DF_MaxFractionDigits)
				right = right.substr(0, this.DF_MaxFractionDigits);
			while (left.length  > this.DF_MinIntegerDigits  && left.charAt(0) == zL) left = left.substr(1);
			while (right.length > this.DF_MinFractionDigits && right.charAt(right.length-1) == zR) right = right.substr(0,right.length-1);
			left  = hX_5.padL(left,  this.DF_MaxIntegerDigits, promptChar);
			right = hX_5.padR(right, this.DF_MaxFractionDigits, promptChar);
		}

		// Starting building the string by converting left/right to non-Western digit representations
		if (this.xDigits != null) {
			left = this.convertToDigits(left);
			right= this.convertToDigits(right);
		}

		// Now add in the left side with grouping
		j = 0;
		strValue = left;
		if (this.DF_GroupingSize > 0) {
			strValue = hX_5._E; g = this.DF_GroupingSize;
			for (i=left.length-1; i >= 0; i--) {
				if (j==g) {
					if (promptChar)
						strValue = ((left.charAt(i)==promptChar) ? hX_5._E : this.LC_groupingSymbol) + strValue;
					else if (zL != this.LC_paddingChar)
						strValue = ((left.charAt(i)==zL) ? zL : this.LC_groupingSymbol) + strValue;
					else
						strValue = this.LC_groupingSymbol + strValue;
					j = 0; g = (g==this.DF_GroupingSize) ? this.DF_GroupingSecSize:this.DF_GroupingSize;
				}
				strValue = left.charAt(i) + strValue;
				j++;
			}
		}
		// Add in the decimal point and the fraction
		if (this.DF_IsDecimalShown) {
			if (zero && zZ && zL != this.LC_paddingChar)
				strValue = strValue + zL + right;
			else
				strValue = strValue + this.LC_decimalSymbol + right;
		}
	}

	// Add in the positive and negative prefixes and suffixes
	if (sign > 0 || zero)
		strValue = this.DF_PositivePrefix + strValue + this.DF_PositiveSuffix;
	else
		strValue = (!this.DF_NegativePattern) ? (this.LC_minusSymbol + this.DF_PositivePrefix + strValue + this.DF_PositiveSuffix) : (this.DF_NegativePrefix + strValue + this.DF_NegativeSuffix);

	// Finally handle the ICU4J padding
	if (this.DF_PaddingPos > 0) {
		if (this.DF_PaddingPos == 1) strValue = hX_5.padL(strValue, this.DF_MaxPadLength, this.DF_PaddingChar);
		if (this.DF_PaddingPos == 4) strValue = hX_5.padR(strValue, this.DF_MaxPadLength, this.DF_PaddingChar);
// FIXME: The case of a pattern with NO positive prefix (or suffix) where there is a negative prefix (or suffix) is ambiguous
// as we don't know where to put the padding in the neg case (before or after the prefix?)  We put it after the prefix
// since that's the "hard" case -- a user can use a "pad" call of their own to padd "outside" the prefix/suffix
		if (this.DF_PaddingPos == 2) {
			if (sign > 0 || zero) {
				if (this.DF_PositivePrefix.length == 0) strValue = hX_5.padL(strValue, this.DF_MaxPadLength, this.DF_PaddingChar);
				else strValue = this.DF_PositivePrefix + hX_5.padL(strValue.substr(this.DF_PositivePrefix.length), this.DF_MaxPadLength-this.DF_PositivePrefix.length, this.DF_PaddingChar);
			} else {
				if (!this.DF_NegativePattern && this.DF_PositivePrefix.length > 0) strValue = hX_5.padL(strValue.substr(this.DF_PositivePrefix.length), this.DF_MaxPadLength-1-this.DF_PositivePrefix.length, this.DF_PaddingChar);
				else if (this.DF_NegativePrefix.length == 0) strValue = hX_5.padL(strValue, this.DF_MaxPadLength, this.DF_PaddingChar);
				else strValue = this.DF_NegativePrefix + hX_5.padL(strValue.substr(this.DF_NegativePrefix.length), this.DF_MaxPadLength-this.DF_NegativePrefix.length, this.DF_PaddingChar);
			}
		}
		if (this.DF_PaddingPos == 3) {
			if (sign > 0 || zero) {
				if (this.DF_PositiveSuffix.length == 0) strValue = hX_5.padR(strValue, this.DF_MaxPadLength, this.DF_PaddingChar);
				else strValue = hX_5.padR(strValue.substr(0,strValue.length-this.DF_PositiveSuffix.length), this.DF_MaxPadLength-this.DF_PositiveSuffix.length, this.DF_PaddingChar) + this.DF_PositiveSuffix;
			} else {
				if (!this.DF_NegativePattern && this.DF_PositiveSuffix.length > 0) strValue = hX_5.padR(strValue.substr(0,strValue.length-this.DF_PositiveSuffix.length), this.DF_MaxPadLength-this.DF_PositiveSuffix.length, this.DF_PaddingChar) + this.DF_PositiveSuffix;
				else if (this.DF_NegativeSuffix.length == 0) strValue = hX_5.padR(strValue, this.DF_MaxPadLength, this.DF_PaddingChar);
				else strValue = hX_5.padR(strValue.substr(0,strValue.length-this.DF_NegativeSuffix.length), this.DF_MaxPadLength-this.DF_NegativeSuffix.length, this.DF_PaddingChar) + this.DF_NegativeSuffix;
			}
		}
	}	
	return strValue;
}

// Public: Convert a string in "constant" format to a JS Number
HxG_5.prototype.NumberConverter.prototype.constantToValue = function (sNum) {

	var o, j, left, right, sign=1, str = sNum;
	if (sNum.charAt(0) == this.hardMinus) {
		sign = -1;
		str = sNum.substr(1);
	}
	j = str.indexOf(hX_5._P);
	if (j >= 0) {
		left = str.substr(0,j);
		right= str.substr(j+1);
	} else {
		left = str;
		right = hX_5._Z;
	}
	if (left.length == 0)	left  = hX_5._Z;
	if (right.length == 0)	right = hX_5._Z;
	o = left + hX_5._P + right;
	return ((new Number(o))*sign);
}

// Public: Convert a string in "constant" format to a JS Number
HxG_5.prototype.NumberConverter.prototype.valueToConstant = function (nNum) {
	// Not implemented -- no one currently needs this routine
	return (null);
}

//  ----------------------------------------------------------------------------------------
//  Number Validator
//  ----------------------------------------------------------------------------------------
//
//  Methods (common to all validators)
//
//	constructor		Create a validator with a specified set of attributes
//	setAttribute	Change an attribute of the validator
//  getAttribute	Get an attribute of the validator
//	validate		Check if a JS Number is valid according to the validator's rules
//  coerce			Force a JS Number object to be valid
//	lastError		Return a (localized) string describing the reason the last validation failed
//
//  Constructor attributes:
//
//	Description		Meaning
//  -----------		------------------------------------------------------------------------
//
//	required		True if the values is required.  False if not.
//	min-bound		Minimum value (bound).  Formatted using the "constant" format of #########0.00
//	max-bound		Maximum value (bound).  Formatted using the "constant" format of #########0.00
//  constraint-expression	An EL expression that returns true/false if the value is good/bad
//	modulus			Type of modulus validation (10 or 11) the field must pass

HxG_5.prototype.NumberValidator = function () {
	this.type = hX_5.CVT_number;
	this.lc = hX_5.currentLocale();

	this.minimumBound = this.maximumBound = this.minimum = this.maximum = this.required = null;
	this.ELexpression = this.ELexpressionImp = this.modulus =null;
	this.msgFail = hX_5._E;
	
	// Attributes supported on this object
	this.attributeTable = [];
		this.attributeTable ['required']			= ['required',		hX_5.ARG_IS_TRUE,	false, null, null];
		this.attributeTable ['modulus']				= ['modulus',		hX_5.ARG_TO_NAN,	false,	null, null];
		this.attributeTable	['minimum-bound']		= ['minimumBound',	hX_5.ARG_TO_BIGNUM,	false,	-999999999999999,999999999999999];
		this.attributeTable	['min-bound']			= ['minimumBound',	hX_5.ARG_TO_BIGNUM,	false,	-999999999999999,999999999999999];
		this.attributeTable	['maximum-bound']		= ['maximumBound',	hX_5.ARG_TO_BIGNUM,	false,	-999999999999999,999999999999999];
		this.attributeTable	['max-bound']			= ['maximumBound',	hX_5.ARG_TO_BIGNUM,	false,	-999999999999999,999999999999999];
	 	this.attributeTable	['constraint-expression']=['ELexpression',	hX_5.ARG_TO_NAN,	false,	null, null];
	this.setAttribute(arguments);	
}

// Public:  Set an "attribute" on the validator.  Attributes are passed as strings of the form "name:value"
HxG_5.prototype.NumberValidator.prototype.setAttribute = function(a) {

	var wasmax = this.maximumBound, wasmin = this.minimumBound;
	hX_5.parseArg (this.attributeTable, this, a);

	// Construct the minimum and maximum bounds
	if (this.minimumBound != null && (wasmin == null || wasmin != this.minimumBound)) this.minimum = new Number(this.minimumBound);
	if (this.maximumBound != null && (wasmax == null || wasmax != this.maximumBound)) this.maximum = new Number(this.maximumBound);
	if (this.maximum && this.minimum)
		if (this.minimum >= this.maximum) this.minimumBound = this.maximumBound = this.minimum = this.maximum = null;

	// And the expression
	if (this.ELexpression != null) this.ELexpressionImp = hX_5.regExp.registerELConstraint(this.ELexpression);
	// And the modulus
	if (this.modulus != null) if (this.modulus != "10" && this.modulus != "11") this.modulus = null;
	return true;
}

// Public: Retrieve the value of an attribute
HxG_5.prototype.NumberValidator.prototype.getAttribute = function (attribute) {
	return (hX_5.getArg(this.attributeTable, this, attribute));
}

// Public:	Return the reason the last validation failed as a localized message
HxG_5.prototype.NumberValidator.prototype.lastError = function () {
	return ((this.msgFail && this.msgFail.length > 0) ? this.msgFail : null);
}

// Public: Validate a number
HxG_5.prototype.NumberValidator.prototype.validate = function (testnum, iNP) {

	this.msgFail = hX_5._E;
	var e, r, test=null, NP = (iNP != null) ? hX_5.getConverterById(iNP) : null;
	if (NP == null || !hX_5.isObject (NP)) {
		this.msgFail = hX_5.L[this.lc].ERR_required;
		return false;
	}		
	
	// If the testvalue is a string or null, determine if required or not
	if (testnum == null || hX_5.isString(testnum)) {
		// Verify a value is required or not
		r = !(this.required);
		if (testnum == null) {
			this.msgFail = (r) ? hX_5._E : hX_5.L[this.lc].ERR_required;
			return r;
		}
		e = hX_5.allTrim (testnum);
		if (e.length == 0) {
			this.msgFail = (r) ? hX_5._E : hX_5.L[this.lc].ERR_required;
			return r;
		}
		// Must be a non-empty string, so try converting it to a number
		test = NP.stringToValue(e);
	} else test = testnum;

	// If cannot interpret value as a number, validation fails
	if (test == null || isNaN (test)) {
		this.msgFail = hX_5.L[this.lc].ERR_required;
		return false;
	}

	// Check against supplied minimum
	if (this.minimum && test < this.minimum) {
		this.msgFail = hX_5.L[this.lc].ERR_minimum;
		return false;
	}
	// Check against supplied maximum
	if (this.maximum && test > this.maximum) {
		this.msgFail = hX_5.L[this.lc].ERR_maximum;
		return false;
	}
	// Check against the EL expression 
	if (this.ELexpressionImp != null) {
		if (!hX_5.regExp.testELExp(test, this.ELexpressionImp)) {
			this.msgFail = hX_5.L[this.lc].ERR_constraint;
			return false;
		}
	}
	// And the modulus check
	if (this.modulus) {
		this.msgFail = hX_5.L[this.lc].ERR_constraint;
		// Convert the number to a string and then the string to a sequence of digits only
		var c, exp, strnum = hX_5._E, strnum1 = NP.valueToString(test);
		exp = hX_5.regExp.getRegExpByName("DigitOnly");
		while (strnum1.length > 0) {
			if (hX_5.regExp.testRegExp(strnum1.charAt(0),exp))
				strnum = strnum + strnum1.charAt(0);
			strnum1 = strnum1.substr(1);
		}
		if (strnum.length < 2) return false;
		var sum = 0, sum10 = 0, chk = parseInt(strnum.charAt(strnum.length-1),10),  bas = strnum.substr(0, strnum.length-1);
		if (this.modulus == "10") {
			var odd = true;
			while (bas.length > 0) {
				c = parseInt(bas.charAt(bas.length-1,10));
				c = (!odd) ? c : ((c<5)?(c*2):(((c-5)*2)+1));
				sum = sum + c;
				odd = !odd;
				bas = bas.substr(0,bas.length-1);
			}
			sum10 = (Math.floor((sum-1)/10)+1)*10;
		} else if (this.modulus == "11") {
			var w = 2;
			while (bas.length > 0) {
				c = parseInt(bas.charAt(bas.length-1,10));
				sum = sum + (c*w);
				w = (w >= 7) ? 2 : w+1;
				bas = bas.substr(0,bas.length-1);
			}
			sum10 = (Math.floor((sum-1)/11)+1)*11;
		}
		if ((sum10-sum) != chk) return false;
		this.msgFail = hX_5._E;
	}
	return true;
}

// Public: Coerce a number to be valid
HxG_5.prototype.NumberValidator.prototype.coerce = function (testnum, iNP) {

	this.msgFail = hX_5._E;
	var e, o=null, NP = (iNP != null) ? hX_5.getConverterById(iNP) : null;
	if (NP == null || !hX_5.isObject(NP)) {
		this.msgFail = hX_5.L[this.lc].ERR_required;
		return false;
	}		
	// If the testvalue is a string, convert it to a number
	if (testnum != null && hX_5.isString(testnum)) {
		e = hX_5.allTrim (testnum);
		if (e.length > 0) o = NP.stringToValue(e);
	} else o = testnum;

	// If we don't have something we can make into a number, create a number
	if (o == null || isNaN (o)) o = new Number (0);

	// Check against the minimum
	if (this.minimum && o < this.minimum) o = this.minimum;
	// Check against supplied maximum
	if (this.maximum && o > this.maximum) o = this.maximum;

	return o;
}

//  ----------------------------------------------------------------------------------------
//  Number Assist
//  ----------------------------------------------------------------------------------------
//
//  NOTE:  Changing the converter associated with an assist (e.g., changing the format)
//		   is BAD.  We currently cannot rebuild the value(s) on display that use the assist object
//
//  Methods (common to all assist object)
//
//	onFocus				On focus: coerce the number to be valid, add in prompt chars, set cursor position
//	onBlur				On blur: Remove prompt chars
//	onClick				On click: Handle a mouse click
//	onCut				On cut: Handle cutting text
//	onCopy				On copy: Handle copying text
//	onPaste				On paste: Handle pasting text
// 	onChangeKeyCode		On key:  Handle a function keystroke, e.g., DEL or Backspace
// 	onChangeChrCode		On key:  Handle an alphanumieric keystroke, e.g., "A"
//	onValidate			Validate the string as it stands and return true/false if it succeeds/fails
//	isFilled			For auto-tab checking:  Is the number "filled" so auto-tab should occur
// 	toAssist			Sub-handler of onFocus -- takes a value and adds prompt-chars
// 	fromAssist			Sub-handler of onBlur -- takes a value and removes prompt-chars

HxG_5.prototype.NumberAssist = function (_elem, _state, _converter, _validator, _promptChar) {
	if (!_elem || !_converter || !_state) return;
	this.lc = hX_5.currentLocale();

	// Exported functions
	this.isFilled		= isFilled;			// Public
	this.onFocus		= onFocus;			// Public
	this.onBlur			= onBlur;			// Public
	this.toAssist		= toAssist;			// Public
	this.fromAssist		= fromAssist;		// Public
	this.onClick		= onClick;			// Public
	this.onCut			= onCut;			// Public
	this.onCopy			= onCopy;			// Public
	this.onPaste		= onPaste;			// Public
	this.onChangeKeyCode= onChangeKeyCode;	// Public
	this.onChangeChrCode= onChangeChrCode;	// Public
	this.onValidate		= onValidate;		// Public

	// Basic information kept by assist
	var elem=_elem, state=_state, base_converter=_converter, cvt=null, outcvt=null, validator=_validator;
	var promptChar=_promptChar, command=hX_5._E, lastCommand=-1, firstCommand=-1, decimalCommand=-1;
	var minusSign=null, plusSign="+";
	var zero			= null;		// Number: zero or if validator exlcludes it, smallest value
	var zeroS			= null;		// String: string version of the above
	var negZeroS		= null;		// String: negative zero (e.g., -0), ignores range
	var posZeroS		= null;		// String: postive zero (e.g., 0), ignores range
	var negNullS		= null;		// String: negative null (e.g., -*), ignores range
	var posNullS		= null;		// String: postive null (e.g., *) ignores range
	var nullOK			= true;

	// Keyboard handling sub-functions
	var getCChar 		= function (i) 	{ return (i < 0 || i >= command.length) ? hX_5._E : command.charAt(i); }

	// == Start constructor =====================================================
	// Generate a version of the converter that has the negative sign in it
	// And one we actually use for inputting values
	if (base_converter.signedConverter == null) base_converter.genAssistMasks();
	outcvt = (base_converter.signedConverter) ? base_converter.signedConverter : base_converter;
	cvt = (base_converter.zeroConverter) ? base_converter.zeroConverter : base_converter;
	// We could handle the ambiguous null case (where the value is zero and the converter is all #'s so we
	// won't display the zero), but the validator will not know about this (and will equivalence the two).
	// That is, if client-side validation is on, it will be confused.  So we'll display nothing for zero
	// so it's (hopefully) clearer why it's failing
	if (validator && (validator.required) != null && validator.required)
		nullOK = (cvt.nStrict != null && cvt.nStrict > 1) ? true : false;
	// Generate the a "zero" and a "null" value
	minusSign = cvt.LC_minusSymbol;
	zero = (validator) ? validator.coerce(0, cvt) : 0; 
	zeroS =  cvt.valueToString(zero, promptChar);
	posZeroS = cvt.valueToString(0, promptChar);
	negZeroS = cvt.valueToString(-1, promptChar);
	zeroS = (nullOK) ? hX_5.replaceAll(posZeroS, cvt.getZeroDigit(), promptChar) : zeroS;
	if (nullOK) {
		negNullS = hX_5.replaceAll(negZeroS, hX_5._1, promptChar);
		posNullS = zeroS;
	}
	negZeroS = hX_5.replaceAll(negZeroS, hX_5._1, cvt.getZeroDigit());
	renewMaskHandler(elem.value);

	// == End constructor =======================================================


	//== Private Functions =============================

	// Private:  Handle a delete or backspace
	function handleDelete (evt) {

		var v = state.getValue(), pos = state.getPos();
		pos = (evt.type==hX_5._evCT) ? (pos) : ((evt.keyCode == hX_5.KEY_BS) ? pos-1 : pos);
		if (pos < 0 && !state.hasSel()) return true;

		// Check if ok delete it (always ok if a selection)
		var cR = getCChar(pos);
		if (cR != hX_5._LB && !state.hasSel()) {
			if (cR==hX_5._EQ && (v.charAt(pos)== cvt.LC_decimalSymbol || v.charAt(pos)==cvt.LC_groupingSymbol) && evt.type!=hX_5._evCT) {
				pos = (evt.keyCode == hX_5.KEY_DEL) ? pos+1 : pos;
				state.setPos(pos);
				return -2;
			}
			return false;
		}

		// If a selection ...
		var isLeft  = ((!cvt.DF_IsDecimalShown) || (pos <= decimalCommand));
		if (state.hasSel()) {
			var rg = hX_5.sel.getSelectionR(elem);
			// Revert to "zero" if we delete it all
			if ((rg.start <= 0) && (rg.end >= v.length)) {
				state.setValue(zeroS);
				state.setPos((cvt.DF_IsDecimalShown) ? decimalCommand : lastCommand+1);
			// Otherwise delete the selection and the validator will fix it up
			} else {
				// Adjust the range so it excludes prefix/suffix
				adjRange(rg);
				// Check if we're deleting the decimal point (add it in if we are)
				if (cvt.DF_IsDecimalShown && ((v.substring(rg.start,rg.end)).indexOf(cvt.LC_decimalSymbol) >= 0)) {
					state.setValue(v.substr(0,rg.start) + cvt.LC_decimalSymbol + v.substr(rg.end));
					state.setPos(decimalCommand);
				} else {
					state.setValue(v.substr(0,rg.start) + v.substr(rg.end));
					state.setPos((!cvt.DF_IsDecimalShown) ? rg.end : ((rg.end <= decimalCommand)?rg.end:rg.start));
				}
			}
		// Del the char
		} else {
			state.setValue (v.substr(0,pos) + v.substr(pos+1));
			state.setPos(pos+1+((isLeft)?(0):(-1)));
		}
		// Sign or grouping can change if we delete it
		state.setPos (holdCursor(state.getPos(), state.getValue(), isLeft));
		return -2;
	}

	// Private:  Handle insert/overstrike
	function handleInsert (v, ch, pos, sel) {
		// We "overstrike" characters unless we're immediately to the left of the
		// decimal point in which case we insert them
		var l = v.length, wasV = v, isLeft  = ((!cvt.DF_IsDecimalShown) || (pos <= decimalCommand));
		// Handle typing into a selection (delete content of selection)
		if (sel) {
			var rg = hX_5.sel.getSelectionR(elem);
			// Revert to "zero" if we delete it all
			if ((rg.start <= 0) && (rg.end >= v.length)) {
				v = zeroS;
				if (cvt.DF_IsDecimalShown) {
					v = (ch.length > 1 && ch.indexOf(cvt.LC_decimalSymbol) >= 0) ? v : (v.substr(0,decimalCommand-1) + ch + v.substr(decimalCommand));
					pos = decimalCommand;
				} else {
					v = v.substr(0,lastCommand) + ch + v.substr(lastCommand+1);
					pos = lastCommand+1;
				}
			// Otherwise delete the selection and the validator will fix it up
			} else {
				adjRange(rg);
				if (cvt.DF_IsDecimalShown && ((v.substring(rg.start,rg.end)).indexOf(cvt.LC_decimalSymbol) >= 0)) {
					v = (ch.length > 1 && ch.indexOf(cvt.LC_decimalSymbol) >=0) ? (v.substr(0,rg.start) + ch + v.substr(rg.end)) : (v.substr(0,rg.start) + ch + cvt.LC_decimalSymbol + v.substr(rg.end));
					pos = decimalCommand;
				} else {
					v = v.substr(0,rg.start) + ch + v.substr(rg.end);
					pos = rg.end;
				}
			}
			state.setPos(holdCursor(pos, v, isLeft));
		} else {
			// If we're typing on a prompt char and there are prompt chars to the left
			// (right if past decimal), fill in the number with zeros
			var insertMode = false, fillLeft = false, fillRight = false, c = v.charAt(pos);
			if (c==promptChar) {
				if (isLeft && (pos < l-1) && (v.charAt(pos+1)==promptChar)) fillRight = true;
				else if (!isLeft && (v.charAt(pos-1)==promptChar)) fillLeft = true;
			}
			// If we're right at the decimal point, then we switch from "overtype" to "insert" mode
			insertMode = (cvt.DF_IsDecimalShown) ? (c == cvt.LC_decimalSymbol) : (pos > lastCommand);
			// Now insert the char or overtype the char (unless we're full)
			if (isFilled(pos)) return -2;
			if (!insertMode) {
				var ip = pos;
				if (ch.length <= 1) {
					v = v.substr(0,pos) + ch + v.substr(pos+1);
					if (pos+1 < lastCommand && getCChar(pos+1) == hX_5._EQ) pos++;
					state.setPos(pos+1);
					if (fillRight) {
						ip++;
						while (v.charAt(ip) == promptChar) {
							v = v.substr(0,ip) + cvt.getZeroDigit() + v.substr(ip+1);
							ip++;
						}
					} else if (fillLeft) {
						ip--;
						while (v.charAt(ip) == promptChar) {
							v = v.substr(0,ip) + cvt.getZeroDigit() + v.substr(ip+1);
							ip--;
						}
					}
				// Inserting > 1 char
				} else {
					while (v.charAt(ip)==promptChar && ip < pos+ch.length) ip++;
					if (pos!=ip) v = v.substr(0,pos)+v.substr(ip);
					v = v.substr(0,pos) + ch + v.substr(pos);
					state.setPos(pos);
					if (fillRight) {
						ip=pos+ch.length;
						while (v.charAt(ip) == promptChar) {
							v = v.substr(0,ip) + cvt.getZeroDigit() + v.substr(ip+1);
							ip++;
						}
					} else if (fillLeft) {
						ip--;
						while (v.charAt(ip) == promptChar) {
							v = v.substr(0,ip) + cvt.getZeroDigit() + v.substr(ip+1);
							ip--;
						}
					}
				}
			} else {
				// See if we're already full left of the decimal
				c = v.charAt(firstCommand);
				if (isLeft && (c==promptChar || cvt.isZeroDigit(c) || c==hX_5._B))
					v = v.substr(0,pos) + ch + v.substr(pos);
				else
					return false;
			}

			// If we've inserted/removed commas, we've got to adjust the pos
			if (v != wasV) state.setPos(holdCursor(state.getPos(), v, isLeft));
		}
		state.setValue(v);
		return -2;
	}

	// Private:  Handle left arrow
	function handleCaretPrev (evt) {
		// We don't handle it here if it's shifted or a selection is in force
		if (evt.shiftKey || state.hasSel())
			return -1;
		else {
			state.setPos(getMovePrevSkipRange(state.getPos()));
			return -2;
		}
	}

	// Private:  Handle right arrow
	function handleCaretNext (evt) {
		// We don't handle it here if it's shifted or a selection is in force
		if (evt.shiftKey || state.hasSel())
			return -1;
		else {
			state.setPos(getMoveNextSkipRange(state.getPos()));
			return -2;
		}
	}

	// Private: Move left
	function getMovePrevSkipRange (pos) {
		var i, c, v = state.getValue(), l = v.length;
		for (i = pos-1; i >= 0; i--) {
			c = getCChar(i);
			if (c == hX_5._QM || c == hX_5._LB || c == hX_5._EQ) {
				i = (i < 0) ? 0 : ((i > l) ? l : i);
				return (i);
			}
		}
		return (0);
	}

	// Private: Move right
	function getMoveNextSkipRange (pos) {
		var i, c, v = state.getValue(), l = v.length;
		for (i = pos+1; i < l; i++) {
			if (i == lastCommand+1)
				return i;
			c = getCChar(i);
			if (c == hX_5._QM || c == hX_5._LB || c == hX_5._EQ) {
				i = (i < 0) ? 0 : ((i > l) ? l : i);
				return (i);
			}
		}
		return (l);
	}

	// Private: Skip to the next command in the mask that's an "=" literal
	function getMoveNextSkipChar (ch, pos) {
		if (ch == cvt.LC_decimalSymbol) {
			if (!cvt.DF_IsDecimalShown)
				return {hit:true, skip:decimalCommand-pos};
			if (pos == decimalCommand)
				return {hit:true, skip:decimalCommand-pos+1};
			else
				return {hit:true, skip:decimalCommand-pos};
		} else if (cvt.DF_GroupingSize > 0 && cvt.LC_groupingSymbol == ch) {
			if (pos >= decimalCommand)
				return {hit:true, skip:0};
			var i, l = command.length, p = (pos >= decimalCommand-cvt.DF_GroupingSize) ? 0 : pos;
			for (i = p; i < l; ++i) {
				if (getCChar(i) == hX_5._EQ)
					return {hit:true, skip:(i - pos + 1)};
			}
		}
		return {hit:false, skip:0};
	}

	// Private: "Center" the cursor just before the (implied or actual) decimal point
	function centerCursor (v, plus) {
		var newpos = -1;
		if (cvt.DF_IsDecimalShown)
			newpos = v.indexOf(cvt.LC_decimalSymbol);
		if (newpos < 0) {
			newpos = v.length;
			if (plus && cvt.DF_PositiveSuffix)
				newpos -= cvt.DF_PositiveSuffix.length;
			else if ((!plus) && cvt.DF_NegativeSuffix)
				newpos -= cvt.DF_NegativeSuffix.length;
			newpos = (newpos < 0) ? 0 : newpos;
		}
		return newpos;
	}

	// Private:  Adjust the range so it excludes prefixes/suffixes
	function adjRange (rg) {
		var pre=0, suff=command.length;
		while (pre < suff && (command.charAt(pre) == hX_5._VB || command.charAt(pre) == hX_5._QM)) pre++;
		if (command.charAt(suff-1) == hX_5._VB || command.charAt(suff-1) == hX_5._QM) {
			suff--;
			while (suff >= 0 && (command.charAt(suff) == hX_5._VB || command.charAt(suff) == hX_5._QM)) suff--;
		}
		rg.start = (rg.start < pre) ? pre : rg.start;
		rg.end = (rg.end > (suff+1)) ? (suff+1) : rg.end;
	}

	// Private:  See if we can "hold" the cursor pos after thousands separators added/removed
	function holdCursor (pos, v, isLeft) {
		var r=pos, plusPrefix=((cvt.DF_PositivePrefix) ? cvt.DF_PositivePrefix : hX_5._E), minusPrefix=(cvt.DF_NegativePrefix) ? cvt.DF_NegativePrefix : hX_5._E;

		// There's an issue when you add/remove a character (or series), the
		// "grouping symbols" may appear disappear or the sign could change.
		// This will cause the cursor to be in the wrong place.  For example:
		// ***100.00
		//       ^  cursor is just before the decimal point
		// If you now type "1", the result is:
		// **1,000.00
		//       ^ cursor is now in front of the zero. 
		// This is because instead of their being six characters before the decimal point, now there
		// are seven.  
		// So we need to adjust the cursor based on whether "grouping symbols" are going to be
		// added or removed OR if a deletion causes the value to go "zero" the sign could change too
		if (cvt.DF_GroupingSize > 0 || (plusPrefix.length != minusPrefix.length)) {
			// How many digits to the left of the decimal point are we?
			var i, oldDigPos = -1, newValue = cvt.stringToValue(v, promptChar);
			if (isLeft) {
				oldDigPos = 0;
				for (i = decimalCommand-1; (i >= pos) && (i >= 0); i--) {
					if (getCChar(i) != hX_5._EQ) oldDigPos++;
				}				
			}
			// Now convert the new value and see what kind of command string we'd get from it
			newValue = cvt.valueToString(newValue, promptChar);
			newValue = (newValue == null || newValue.length == 0) ? zeroS : newValue;
			// Set the value directly as state isn't valid at this point
			if (newValue != null && newValue.length > 0) {
				var wasCommand = command;
				renewMaskHandler(newValue);
				// If we're "right" of the decimal point, then any changes
				// to the length of the command (e.g., new commas) are entirely
				// to the left of the pos
				if (oldDigPos < 0)
					r = pos + (command.length - wasCommand.length);
				else {
					r = decimalCommand;
					while (oldDigPos > 0 && r > 0) {
						r--;
						if (getCChar(r) != hX_5._EQ) oldDigPos--;
					}
				}
			}
		}
		return r;
	}

	// Private: Save a copied/cut text as a number (not string)
	// We're trying to emulate the real Windows clipboard's ability
	// to have a value on the clipboard exist in multiple (source)
	// types so the receiver can pull data in the most appropriate format.
	// Here we want to put the data on the clipboard as both a raw string
	// AND as a "number" (here a number is a string of just digits with
	// at most a decimal point in them).
	function putNumberToClipboard () {
		var v=state.getValue(), rg=hX_5.sel.getSelectionR(elem);
		hX_5.clip.valueAsString = v.substring(rg.start,rg.end);
		adjRange(rg);
		v = v.substring(rg.start,rg.end);
		if (cvt.DF_GroupingSize > 0) v = hX_5.replaceAll(v, cvt.LC_groupingSymbol,hX_5._E);
		hX_5.clip.valueAsNumber = hX_5.replaceAll(v, promptChar,hX_5._E);
	}

	// Private: Rebuild the command mask
	function renewMaskHandler (v) {
		var i, l=v.length, h=null, c=hX_5._E, prefix=hX_5._E, suffix=hX_5._E, iV=cvt.stringToValue(v, promptChar);

		// Figure out the sign of the number and then the size of the prefix/suffix for that sign
		var plus = (iV==null) ? (!(negNullS != null && v == negNullS)) : ((iV == 0) ? (!(negZeroS != null && v == negZeroS)) : (iV > 0));
		if (plus) {
			prefix = (cvt.DF_PositivePrefix) ? cvt.DF_PositivePrefix : hX_5._E;
			suffix = (cvt.DF_PositiveSuffix) ? cvt.DF_PositiveSuffix : hX_5._E;
		} else {
			prefix = (cvt.DF_NegativePrefix) ? cvt.DF_NegativePrefix : hX_5._E;
			suffix = (cvt.DF_NegativeSuffix) ? cvt.DF_NegativeSuffix : hX_5._E;
		}
		prefix = prefix.length;
		suffix = v.length - suffix.length;

		// Now build a mask that matches the value
		lastCommand = 0; firstCommand = -1; decimalCommand = -1;
		for (i=0; i<l; ++i) {
			h = v.charAt(i);
			if (i < prefix)
				c = c + (((!plus) && h == cvt.LC_minusSymbol) ? hX_5._QM : hX_5._VB);
			else if (i >= suffix)
				c += hX_5._VB;
			else {
				if (cvt.DF_GroupingSize > 0 && h == cvt.LC_groupingSymbol)
					c+= hX_5._EQ;
				else if (cvt.DF_IsDecimalShown && h == cvt.LC_decimalSymbol) {
					c += hX_5._EQ;
					decimalCommand = i;
				} else {
					c += hX_5._LB;
					firstCommand = (firstCommand < 0) ? i : firstCommand;
					lastCommand = i;
				}
			}
		}
		decimalCommand = (cvt.DF_IsDecimalShown) ? decimalCommand : suffix;
		command = c;
		return (plus);
	}	

	//====================================================================================
	// Public:  See if the value is "filled" according to the mask
	function isFilled (curpos) {
		renewMaskHandler(elem.value);
		// If the decimal is shown, we're done if on the rightmost char and it's full
		// If the decimal is not shown, then the right side must be filled, that is,
		// the first character must be present
		var c = (cvt.DF_IsDecimalShown && cvt.DF_MaxFractionDigits>0) ? elem.value.charAt(lastCommand) : elem.value.charAt(firstCommand);
		if (c == hX_5._B || c == promptChar)
			return false;

		// We're done if we're beyond the rightmost char
		return (curpos > lastCommand);
	}

	//====================================================================================
	// Public: Put a field in "assist" mode
	function onFocus (evt) {
		// To go into assist mode, we have to start with a valid value
		var newValue = toAssist(elem.value);
		// Compute the position of the cursor (just before the decimal point or at end)
		renewMaskHandler(newValue.v);
		var newpos = centerCursor(newValue.v, (newValue.i>=0));
		// Set the value and the cursor (directly, not into state which isn't set yet)
		elem.value = newValue.v;
		hX_5.sel.setCaretPos(elem, newpos, null, hX_5.brw.isNavOrMoz());
		return true;
	}

	//====================================================================================
	// Public: Add prompt chars to a field
	function toAssist (v) {
		var newValue=null, inum=((v.length==0) ? null : base_converter.stringToValue(v));
		if (inum == null && nullOK) {
			inum = 0;	// Hardcore zero here so sign is "positive"
			newValue = zeroS;
		} else {
			inum = (inum==null) ? zero : inum;
			// Build the number twice, once with the "real" converter and once with 
			// the input converter
			var newValueO = outcvt.valueToString(inum, promptChar);
			newValue = cvt.valueToString(inum, promptChar);
			// Copy over trailing zeroes
			if (cvt.DF_IsDecimalShown) {
				var i = decimalCommand, j = newValueO.indexOf(cvt.LC_decimalSymbol), k = newValue.indexOf(cvt.LC_decimalSymbol);
				if (i >= 0 && j >= 0 && k >= 0) {
					i++; j++; k++;
					while (i < command.length && command.charAt(i) == hX_5._LB && j < newValueO.length && k < newValue.length) {
						if (cvt.isZeroDigit(newValueO.charAt(j)) && newValue.charAt(k) == promptChar)
							newValue = newValue.substr(0,k) + cvt.getZeroDigit() + newValue.substr(k+1);
						i++; j++; k++;
					}
				}
			}
		}
		newValue = (newValue != null) ? newValue : hX_5._E;
		// Return a string and the value.  Value is only used for "sign" testing
		return {v:newValue, i:inum};
	}

	//====================================================================================
	// Public: Take a field out of "assist" mode
	function onBlur (evt) {
		// Remove prompt chars
		var newValue = fromAssist(elem.value);
		// Set the value directly as state isn't valid at this point
		if (newValue != null) {
			elem.value = newValue;
			return true;
		}
		return false;		
	}

	//====================================================================================
	// Public: Remove prompt chars
	function fromAssist (v) {
		if (v.length==0 ) 
			return hX_5._E;
		var newValue = outcvt.stringToValue(v, promptChar);
		return (newValue != null) ? base_converter.valueToString(newValue) : hX_5._E;
	}

	//====================================================================================
	// Public: Handle clicking in the field
	function onClick (evt) {
// FIXME:  We might want to check if we have focus or not so we can know if we 
// are clicking into the field when it doesn't have focus (in which case we could just return true)
		if (state.hasSel())
			return true;
		var v = state.getValue(), pos = state.getPos();
		renewMaskHandler(v);
		if (pos <= 0 || pos > (v.length-1))
			return true;
		// If the user clicked "next to" an editable space, that's good
		var newpos, MoveR = true, cL = getCChar(pos-1), cR = getCChar(pos);
		if (cL == hX_5._LB || cR == hX_5._LB ||
			cL == hX_5._QM || cR == hX_5._QM || 
		    cL == hX_5._EQ || cR == hX_5._EQ)
		    return true;

		// Otherwise move towards the next editable space		    
		if (cvt.DF_IsDecimalShown)
			MoveR = (pos <= v.indexOf(cvt.LC_decimalSymbol));
		newpos = (MoveR) ? getMoveNextSkipRange(pos) : getMovePrevSkipRange(pos);
		state.setPos(newpos);
		return -2;
	}

	//====================================================================================
	// Public: Keystroke-level validation of a "function key"
	function onChangeKeyCode (evt) {

		// We're looking for a small number of keystrokes we need to intercept and handle instead
		// of letting the input handler deal with them
		renewMaskHandler(state.getValue());
		switch (hX_5.getKeyCode(evt)) {
			case hX_5.KEY_BS:  
			case hX_5.KEY_DEL:
				return handleDelete(evt);
			case hX_5.KEY_LTARROW:
				return handleCaretPrev(evt);
			case hX_5.KEY_RTARROW:
				return handleCaretNext(evt);
		}
		return true;
	}

	//====================================================================================
	// Public: On cut -- essentially the same as selecting text and pressing the DEL/BS key
	function onCut (evt) {
		if (hX_5.brw.isIE() && state.hasSel()) {
			// Put the data on the clipboard and on our pseudo-clipboard stack
			var cutText = hX_5.sel.getSelection(elem);
			window.clipboardData.setData("Text", cutText);
			putNumberToClipboard();
			renewMaskHandler(state.getValue());
			return handleDelete(evt);
		}
		return true;
	}

	//====================================================================================
	// Public: On copy -- essentially the same as selecting text and pressing the DEL/BS key
	function onCopy (evt) {
		if (hX_5.brw.isIE() && state.hasSel())
			// Put the data on our pseudo-clipboard stack
			putNumberToClipboard();
		return true;
	}

	//====================================================================================
	// Public: On paste -- essentially the same as typing (either into a selection or not)
	function onPaste (evt) {
		if (hX_5.brw.isIE()) {
			// Put the data on the clipboard
			var d = false, cutText = window.clipboardData.getData("Text"), v = state.getValue(), pos = state.getPos(), sel = state.hasSel();
			// The selection is pure text and may have included any sign, prefix,
			// suffix, grouping and or decimal points.  We have to leave that goop
			// in the number when you do a copy as there's no way to know 
			// See if we think the value on the clipboard is something copied from another number
			if (cutText == hX_5.clip.valueAsString)
				cutText = hX_5.clip.valueAsNumber;
			// Otherwise, cleanup the clipboard value the best we know how
			else {
				if (cvt.DF_GroupingSize > 0) cutText = hX_5.replaceAll(cutText,cvt.LC_groupingSymbol,hX_5._E);
				cutText = hX_5.replaceAll(cutText,promptChar,hX_5._E);
			}
			// If there's a selection and it does include a decimal point, we can
			// paste the text and we'll use the text's decimal point.
			// Otherwise, remove it and treat the string as a long integer
			if (sel) {
				var rg = hX_5.sel.getSelectionR(elem);
				d = (v.substring(rg.start,rg.end).indexOf(cvt.LC_decimalSymbol) >= 0);
			}
			if (!d) {
				d = cutText.indexOf(cvt.LC_decimalSymbol);
				cutText = (d < 0) ? cutText : cutText.substr(0,d)+cutText.substr(d+1);
			}
			// Now insert the text			
			return (handleInsert(v, cutText, pos, sel));
		}
		return true;
	}

	//====================================================================================
	// Public: Keystroke-level validation of a "character"
	function onChangeChrCode (evt) {
		// Return on tab or newline -- these aren't "control" keys
		// so they come in here
		var ch = hX_5.getChrCode(evt);
		if (evt.keyCode == hX_5.KEY_TAB || evt.keyCode == hX_5.KEY_ENTER)
			return +2;

		// If the user types a "sign" character, handle it
		var pos = state.getPos(), v = state.getValue(), sel = state.hasSel()
		renewMaskHandler(v);
		if (ch == minusSign || ch == plusSign) {
			// Figure out the current sign
			var iV = cvt.stringToValue (v, promptChar), posSignLen = 0, negSignLen = 0;
			var isLeft = ((!cvt.DF_IsDecimalShown) || (pos <= decimalCommand));
			var isNeg = ((iV == null && negNullS && v==negNullS) || (iV == 0 && negZeroS && v==negZeroS) || (iV != null && iV != 0 && iV < 0));
			// If there's a selection outstanding, delete it
			if (sel) {
				// If we'e deleting everything, the sign becomes zero so invert isNeg
				var rg = hX_5.sel.getSelectionR(elem);
				isNeg = ((rg.start <= 0) && (rg.end >= v.length)) ? false : isNeg;
				handleDelete(evt);
				pos = state.getPos(); v = state.getValue();
			}
			// Do a first pass at inverting the sign of the numeric value
			iV = cvt.stringToValue (v, promptChar);
			iV = (iV != null) ? (iV * -1) : null;
			// Look for the special case of signing zero or null
			if (iV == null || iV == 0) {
				// We have to treat this specially (not use a converter) because the
				// converter will always force a negative zero/null to be positive (since
				// there is no such thing as "negative null" or "negative zero"
				// If we have a selection, we need to "pretend" to delete it (we can't actually
				// delete it as the value is null or zero).  The point is to find out
				// where the cursor should go if we did the deletion

				// If the position is "beyond the beginning" of the value, then we need
				// to compute how much it will change due to the change in the sign
				// The sign could be on either or both sides of the value
				if (pos > 0) {
					posSignLen = (cvt.DF_PositivePrefix) ? cvt.DF_PositivePrefix.length : 0;
					negSignLen = (cvt.DF_NegativePrefix) ? cvt.DF_NegativePrefix.length : 0;
				}
				// Now invert the sign and note how far the cursor moved
				if ((iV == 0 && isNeg) || (iV == null && isNeg)) {
					v = (iV==0) ? posZeroS : (posNullS) ? posNullS: posZeroS;
					ch = (sel) ? 0 : (posSignLen-negSignLen);
				} else {
					v = (iV==0) ? negZeroS : (negNullS) ? negNullS : negZeroS;
					ch = negSignLen-posSignLen;
				}
				// Given either a negative zero, negative null, positive zero or positive null
				// we can set the value and change the position
				state.setValue(v);
				state.setPos (pos+ch, v, isLeft);
				renewMaskHandler(v);
				return -2;

			// Otherwise, we're signing a non-zero, non-null value
			} else {
				if (pos > 0) {
					posSignLen = (cvt.DF_PositivePrefix) ? cvt.DF_PositivePrefix.length : 0;
					negSignLen = (cvt.DF_NegativePrefix) ? cvt.DF_NegativePrefix.length : 0;
				}
				// Count how much the sign change affects the position
				ch = (iV < 0) ? (negSignLen-posSignLen) : (posSignLen-negSignLen);
				v = cvt.valueToString(iV);
				state.setValue(v);
				state.setPos (pos+ch, v, isLeft);
				renewMaskHandler(v);
				return -2;
			}
		}

		// Advance if the user types a "literal" character in the mask 
		if (!sel) {
			var skip = getMoveNextSkipChar(ch, pos);
			if (skip.hit) {
				state.setPos(pos+skip.skip);
				return -2;
			}
		}
		// We "overstrike" characters unless we're immediately to the left of the
		// decimal point in which case we insert them
		return (handleInsert (v, ch, pos, sel));
	}

	//====================================================================================
	// Public: Validate the value
	function onValidate (evt) {

		// Round-trip the value through the converter -- this will either "fail" because a bad character
		// was entered OR it will succeed and the value will clean itself up
		var v=state.getValue(), newValue=cvt.stringToValue(v, promptChar);
		if (newValue != null)
			newValue = (newValue == 0 && negZeroS != null && v == negZeroS) ? negZeroS: cvt.valueToString(newValue, promptChar);
		else if (negNullS != null && v == negNullS)
			newValue = negNullS;
		if (newValue != null) {
			// We have a little issue here in that the valueToString call will truncate trailing
			// zeroes so let's clean that up by copying some over from the original value
			if (cvt.DF_IsDecimalShown) {
				var i=decimalCommand, j=v.indexOf(cvt.LC_decimalSymbol), k=newValue.indexOf(cvt.LC_decimalSymbol);
				if (i >= 0 && j >= 0 && k >= 0) {
					i++; j++; k++;
					while (i < command.length && command.charAt(i) == hX_5._LB && j < v.length && k < newValue.length) {
						if (cvt.isZeroDigit(v.charAt(j)) && newValue.charAt(k) == promptChar)
							newValue = newValue.substr(0,k) + cvt.getZeroDigit() + newValue.substr(k+1);
						i++; j++; k++;
					}
				}
			}
			state.setValue(newValue);
			return true;
		} else if (cvt.lastError() == hX_5.L[this.lc].ERR_empty) {
			state.setValue(zeroS);
			return true;
		}
		return false;
	}

// End assist	
}