/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2007 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

//----------------------------------------------------
// String Converter, Validator and Assist
//----------------------------------------------------
//
//  ----------------------------------------------------------------------------------------
//  Masked String Converter
//  ----------------------------------------------------------------------------------------
//
//  Methods (common to all converters)
//
//	constructor		Create a converter with a specified set of attributes
//	setAttribute	Change an attribute of the converter
//  getAttribute	Get an attribute of the converter
//	stringToValue	Convert a string to a (masked) string according to the converter's rules
//  valueToString	Convert a (masked) string to a string according to the converter's rules
//  constantToValue (no-op)
//  valueToConstant	(no-op)
//	lastError		Return a (localized) string describing the reason the last conversion failed
//	getMaxLength	Return the maximum length of a value
//
//  Constructor attributes:
//
//	Description		Meaning
//  -----------		------------------------------------------------------------------------
//
//	strict			How strict to be when converting, 0 = liberal, 1 = strict, 2 = very strict
//	mask			The mask pattern.  See below.
//
// The "mask" consists of literals and special characters.
//
//	Literals are any character that are not special characters.
//	Special charactes are:
//		#	0-9 (digit) (or expression you assign)
//		?   a-z and A-Z (basic alphabetic characters) (or expression you assign)
//		!	(only used if you assign it an expression)
//		~	(only used if you assign it an expression)
//
// Public:  Construct a mask converter
//			Takes zero or more (comma separated "attributename:value" strings as arguments
//			The attribute table defines the attribute's that are allowed
HxG_5.prototype.MaskConverter = function () {
	this.type = hX_5.CVT_mask;

	// Local variables to hold the attributes supplied
	this.nStrict = hX_5.MFMT_DEFAULTSTRICT;		// 0 = loose, 1 = normal, 2 = strict
	this.lc = hX_5.currentLocale();
	this.maskFormat = hX_5._E;
	this.markers = this.mask_Protect = this.maskProtect = this.maskNFormat = null;
	this.maskFormatLast = this.maskFormatFirst = this.maskNFormatLast = this.maskNFormatFirst = -1;
	// A regular expression specified as a string (e.g., "^[a-z]" and the compiled version (for all 4 characters)
	this.constraint1Exp = this.constraint1ExpImp = this.constraint2Exp = this.constraint2ExpImp = this.constraint3Exp = this.constraint3ExpImp = this.constraint4Exp = this.constraint4ExpImp = null;

	// Error handling -- error message associated with last failure
	this.msgFail = this.intMsgFail = hX_5._E;	// Debug messages can be put here

	// Attributes supported on this object
	this.attributeTable = [];
	 	this.attributeTable ['strict']				= ['nStrict',	hX_5.ARG_TO_NUM,	false,	0,	2];
	 	this.attributeTable ['mask']				= ['maskFormat',hX_5.ARG_TO_NAS,	false,	null,	null];
	 	this.attributeTable ['protect-mask']		= ['mask_Protect',hX_5.ARG_TO_NAS,	false,	null,	null];
	 	this.attributeTable	['char1-regexpression']	= ['constraint1Exp',hX_5.ARG_TO_NAN,false,	null, null];
	 	this.attributeTable	['char2-regexpression']	= ['constraint2Exp',hX_5.ARG_TO_NAN,false,	null, null];
	 	this.attributeTable	['char3-regexpression']	= ['constraint3Exp',hX_5.ARG_TO_NAN,false,	null, null];
	 	this.attributeTable	['char4-regexpression']	= ['constraint4Exp',hX_5.ARG_TO_NAN,false,	null, null];
	this.setAttribute(arguments);	
}

// Public:  Set an "attribute" on the converter.  
//			a 	= a single attribute string or an array of them
HxG_5.prototype.MaskConverter.prototype.setAttribute = function(a) {
	var i,c;
	hX_5.parseArg (this.attributeTable, this, a);
	// Set up the character sets for the mask characters
	if (this.constraint1Exp) this.constraint1ExpImp = hX_5.regExp.registerConstraint(this.constraint1Exp);
	if (this.constraint2Exp) this.constraint2ExpImp = hX_5.regExp.registerConstraint(this.constraint2Exp);
	if (this.constraint3Exp) this.constraint3ExpImp = hX_5.regExp.registerConstraint(this.constraint3Exp);
	if (this.constraint4Exp) this.constraint4ExpImp = hX_5.regExp.registerConstraint(this.constraint4Exp);
	this.markers = hX_5._LB + hX_5._QM + ((this.constraint3ExpImp)?hX_5._EX:hX_5._E) + ((this.constraint4ExpImp)?hX_5._TL:hX_5._E)
	// Remember the first/last actual chars in the mask
	this.maskFormatFirst = this.maskFormatLast = -1;
	for (i=0; i < this.maskFormat.length; i++) {
		c = this.maskFormat.charAt(i);
		if (this.isMarker(c)) {
			this.maskFormatLast = i;
			this.maskFormatFirst = (this.maskFormatFirst >= 0) ? this.maskFormatFirst : i;
		}
	}
	// Make sure we have a valid protect mask
	this.maskProtect = null;
	if (this.mask_Protect != null && this.maskFormat != null && this.maskFormat.length >= 0 && this.maskFormat.length == this.mask_Protect.length) {
		this.maskProtect = this.mask_Protect;
		for (i=0; i < this.maskProtect.length; i++) {
			c = this.maskFormat.charAt(i);
			if (this.isMarker(c)) {
				c = this.maskProtect.charAt(i);
				if (!this.isMarker(c))
					this.maskProtect = this.maskProtect.substr(0,i) + hX_5._X + this.maskProtect.substr(i+1);
			} else
				this.maskProtect = this.maskProtect.substr(0,i) + c + this.maskProtect.substr(i+1);
		}
		for (i=0; i < this.maskProtect.length; i++) {
			if (this.isMarker(this.maskProtect.charAt(i))) {
				this.maskNFormatLast = i;
				this.maskNFormatFirst = (this.maskNFormatFirst >= 0) ? this.maskNFormatFirst : i;
			}
		}
	}
	return true;
}

// Public: Retrieve the value of an attribute
//		   attribute = the name of the attribute 	
HxG_5.prototype.MaskConverter.prototype.getAttribute = function (/* string*/ attribute) {
	return (hX_5.getArg(this.attributeTable, this, attribute));
}

// Public:	Return the reason the last conversion failed as a localized message
HxG_5.prototype.MaskConverter.prototype.lastError = function () {
	return ((this.msgFail && this.msgFail.length > 0) ? this.msgFail : null);
}

// Public:	Return the maximum length of a value that uses this converter
HxG_5.prototype.MaskConverter.prototype.getMaxLength = function () {
	return this.maskFormat.length;
}

// Private:	Test if a character is a mask character or not
HxG_5.prototype.MaskConverter.prototype.isMarker = function (c) {
	return (this.markers.indexOf(c) >= 0);
}

// Private:	(Re)bulid the protect mask
HxG_5.prototype.MaskConverter.prototype.rebuildProtectMask = function (newvalue) {
	this.maskNFormat = null;
	if (this.maskProtect && newvalue.length==this.maskFormat.length) {
		this.maskNFormat = this.maskFormat;
		for (var i=0; i < this.maskProtect.length; i++) {
			if (!this.isMarker(this.maskProtect.charAt(i)))
				this.maskNFormat = this.maskNFormat.substr(0,i) + newvalue.charAt(i) + this.maskNFormat.substr(i+1);
		}
	}
}

// Public:  Convert a string with mask literals in it to a string stripped of the literals
//			Fails if the literals don't match or if a char is of the wrong type
//			inString	= the string to be converted
//			promptChar	= if provided, string can have promptchars in it
//						  if it's "short", it will be padded to length with promptchars
//			return		= a masked string if conversion succeeds
//						  null and no lastError if string is empty
//						  null and lastError if string cannot be parsed
HxG_5.prototype.MaskConverter.prototype.stringToValue = function (/*String*/ inString, promptChar, initial) {

	// Set up arguments passed to a failure
	this.msgFail = this.intMsgFail = hX_5._E;
	var argsFail = [inString];
	// Handle empty/null string
	if (inString == null || hX_5.isUnDef(typeof(inString)) || !hX_5.isString(inString)) {
		this.msgFail = hX_5.L[this.lc].ERR_empty;
		return null;
	}
	if (inString.length<=0) return hX_5._E;
	var v=inString, l=v.length, l1=this.maskFormat.length;
	if (l > l1 && (!promptChar && l != l1)) {
		this.msgFail = hX_5.L[this.lc].ERR_constraint;
		return null;
	}
	if (l < l1)	v = hX_5.padR(v,l1,((promptChar)?promptChar:hX_5._B));
	l = v.length;	
	var out = hX_5._E, c, i, m, x, x1;
	for (i = 0; i < l; i++) {
		c = null;
		x = v.charAt(i);
		m = this.maskFormat.charAt(i);
		// See if we match a "match" character
		if (this.isMarker(m)) {
			// If it's the promptChar and we allow it here, then OK
			if (promptChar && (x == promptChar))
				c = hX_5._B;
			// If the character is protected, we allow it no matter what
			else if (this.maskProtect && this.maskProtect.charAt(i) == hX_5._X)
				c = x;
			// Else match the character
			else if (m == hX_5._LB) {
				if (this.constraint1ExpImp)
					c = (hX_5.regExp.testRegExp(x, this.constraint1ExpImp)) ? x : null;
				else
					c = ((hX_5.isChar(x, hX_5.WDIGITS)) ? x : null);
			} else if (m == hX_5._QM) {
				if (this.constraint2ExpImp)
					c = (hX_5.regExp.testRegExp(x, this.constraint2ExpImp)) ? x : null;
				else {
					x1 = x.toUpperCase();
					c = ((hX_5.isChar(x1, hX_5.MASK_ALPHA)) ? x : null);
				}
			} else if (m == hX_5._EX) {
				if (this.constraint3ExpImp)
					c = (hX_5.regExp.testRegExp(x, this.constraint3ExpImp)) ? x : null;
			} else {
				if (this.constraint4ExpImp)
					c = (hX_5.regExp.testRegExp(x, this.constraint4ExpImp)) ? x : null;
			}
		} else
			// Handle a literal
			c = ((x == m) ? hX_5._E : null);
		if (c == null) {
			if (initial && promptChar)
				c = promptChar;
			else {
				this.msgFail = hX_5.L[this.lc].ERR_constraint;
				return null;
			}
		}
		out = out + c;
	}
	return out;
}

// Public:  Convert a string to a masked string using the mask in the converter
//			inStr		-- value to be converted
//			promptChar	-- if provided, string will be padded with promptchars
//			return		-- string 
HxG_5.prototype.MaskConverter.prototype.valueToString = function (/* string */ inStr, promptChar) {
	this.msgFail = hX_5._E;
	if (inStr == null || inStr.length<=0) return hX_5._E;
	var c, i, j=0, out=hX_5._E, v=inStr, l=this.maskFormat.length, l1=v.length;
	if (l1 < l)	v = hX_5.padR(v,l,((promptChar)?promptChar:hX_5._B));
	l1 = v.length;	
	for (i = 0; i < l; i++) {
		c = this.maskFormat.charAt(i);
		if (this.isMarker(c)) {
			if (j < l1) {
				c = v.charAt(j);
				c = (this.maskProtect && this.maskProtect.charAt(i) == hX_5._X)?c:((c==hX_5._B && promptChar) ? promptChar:c);
				out = out + c;
			} else if (promptChar)
				out = out + promptChar;
			else {
				this.msgFail = hX_5.L[this.lc].ERR_constraint;
				return null;
			}
			j++;
		} else {
			out = out + c;
		}
	}
	return out;
}

// Public: Convert a string in "constant" format to a JS Number
HxG_5.prototype.MaskConverter.prototype.constantToValue = function (s) {
	// Not implemented -- no one currently needs this routine
	return (null);
}

// Public: Convert a string in "constant" format to a JS Number
HxG_5.prototype.MaskConverter.prototype.valueToConstant = function (s) {
	// Not implemented -- no one currently needs this routine
	return (null);
}

//  ----------------------------------------------------------------------------------------
//  String Validator
//  ----------------------------------------------------------------------------------------
//
//  Methods (common to all validators)
//
//	constructor		Create a validator with a specified set of attributes
//	setAttribute	Change an attribute of the validator
//  getAttribute	Get an attribute of the validator
//	validate		Check if a string is valid according to the validator's rules
//  coerce			Force a string object to be valid
//	lastError		Return a (localized) string describing the reason the last validation failed
//
//  Constructor attributes:
//
//	Description		Meaning
//  -----------		------------------------------------------------------------------------
//
//	required		True if the values is required.  False if not.
//	min-length		Minimum length. 
//	max-length		Maximum length.
//	constraint		The constraint (pre-canned one)
//  constraint-regexpression	The constraint expressed as a JS regular expression
//  constraint-expression		The constraint expressed as an EL regular expression

HxG_5.prototype.StringValidator = function () {
	this.type = hX_5.CVT_string;
	this.lc = hX_5.currentLocale();

	this.minimumLength = this.maximumLength = this.required = null;
	this.constraint = this.constraintImp = null;		// A regular expression identified via a "keyword", e.g., "alphabetic"
	this.constraintExp = this.constraintExpImp =null;	// A regular expression specified as a string (e.g., "^[a-z]"
	this.ELexpression = this.ELexpressionImp = null;	// An EL expression specified as a string (e.g., "@testValue > 'a'"
	this.msgFail = hX_5._E;

	// Attributes supported on this object
	this.attributeTable = [];
		this.attributeTable ['required']				= ['required',		hX_5.ARG_IS_TRUE,	false,	null, null];
	 	this.attributeTable	['minimum-length']			= ['minimumLength',	hX_5.ARG_TO_NUM,	false,	0, 9999999];
	 	this.attributeTable	['min-length']				= ['minimumLength',	hX_5.ARG_TO_NUM,	false,	0, 9999999];
	 	this.attributeTable	['maximum-length']			= ['maximumLength',	hX_5.ARG_TO_NUM,	false,	1, 9999999];
	 	this.attributeTable	['max-length']				= ['maximumLength',	hX_5.ARG_TO_NUM,	false,	1, 9999999];
	 	this.attributeTable	['constraint']				= ['constraint',	hX_5.ARG_TO_NAN,	false,	null, null];
	 	this.attributeTable	['constraint-regexpression']= ['constraintExp',	hX_5.ARG_TO_NAN,	false,	null, null];
	 	this.attributeTable	['constraint-expression']	= ['ELexpression',	hX_5.ARG_TO_NAN,	false,	null, null];
	this.setAttribute(arguments);	
}

// Public:  Set an "attribute" on the validator.  Attributes are passed as strings of the form "name:value"
HxG_5.prototype.StringValidator.prototype.setAttribute = function(a) {

	hX_5.parseArg (this.attributeTable, this, a);

	// Construct the minimum and maximum bounds
	if (this.minimumLength != null && this.minimumLength < 0) this.minimumLength = 0;
	if (this.maximumLength != null && this.maximumLength < 1) this.maximumLength = 1;
	if (this.maximumLength && this.minimumLength && this.minimumLength >= this.maximumLength) this.minimumLength = this.maximumLength = null;

	// Set up the constraints
	if (this.constraint) this.constraintImp = hX_5.regExp.getRegExpByName(this.constraint);
	if (this.constraintExp) this.constraintExpImp = hX_5.regExp.registerConstraint(this.constraintExp);
	if (this.ELexpression != null) this.ELexpressionImp = hX_5.regExp.registerELConstraint(this.ELexpression);
	return true;
}

// Public: Retrieve the value of an attribute
HxG_5.prototype.StringValidator.prototype.getAttribute = function (attribute) {
	return (hX_5.getArg(this.attributeTable, this, attribute));
}

// Public:	Return the reason the last validation failed as a localized message
HxG_5.prototype.StringValidator.prototype.lastError = function () {
	return ((this.msgFail && this.msgFail.length > 0) ? this.msgFail : null);
}

// Public: Validate a string
// Note:  For a masked-string, teststr is the "value", i.e., stripped of literals
HxG_5.prototype.StringValidator.prototype.validate = function (teststr, iC) {

	this.msgFail = hX_5._E;
	var C = (iC != null) ? hX_5.getConverterById(iC) : null;
	
	// If the testvalue is null or an empty string, determine if required or not
	if (teststr != null && !hX_5.isString(teststr)) {
		this.msgFail = hX_5.L[this.lc].ERR_required;
		return false;
	}
	var test = (teststr == null) ? hX_5._E : teststr;
	test = test.replace(/[\r\n]/g, hX_5._E);
	if (test.length == 0) {
		if (this.required) {
			this.msgFail = hX_5.L[this.lc].ERR_required;
			return false;
		// JSF considers the empty string valid if not required
		} else
			return true;
	}
	// Check against supplied minimum
	if (this.minimumLength && test.length < this.minimumLength) {
		this.msgFail = hX_5.L[this.lc].ERR_minimum;
		return false;
	}
	// Check against supplied maximum
	if (this.maximumLength && test.length > this.maximumLength) {
		this.msgFail = hX_5.L[this.lc].ERR_maximum;
		return false;
	}
	// Check against the constraint
	if (this.constraintImp != null) {
		if (!hX_5.regExp.testRegExp(test, this.constraintImp)) {
			this.msgFail = hX_5.L[this.lc].ERR_constraint;
			return false;
		}
	}
	// Check against the constraint expression
	if (this.constraintExpImp != null) {
		if (!hX_5.regExp.testRegExp(test, this.constraintExpImp)) {
			this.msgFail = hX_5.L[this.lc].ERR_constraint;
			return false;
		}
	}
	// Check against the EL expression 
	if (this.ELexpressionImp != null) {
		if (!hX_5.regExp.testELExp(test, this.ELexpressionImp)) {
			this.msgFail = hX_5.L[this.lc].ERR_constraint;
			return false;
		}
	}
	return true;
}

//  ----------------------------------------------------------------------------------------
//  Mask Assist
//  ----------------------------------------------------------------------------------------
//
//  NOTE:  Changing the converter associated with an assist (e.g., changing the format)
//		   is BAD.  We currently cannot rebuild the value(s) on display that use the assist object
//
//  Methods (common to all assist object)
//
//	onFocus				On focus: coerce the string to be valid, add in prompt chars, set cursor position
//	onBlur				On blur: Remove prompt chars
//	onClick				On click: Handle a mouse click
//	onCut				On cut: Handle cutting text
//	onPaste				On paste: Handle pasting text
// 	onChangeKeyCode		On key:  Handle a function keystroke, e.g., DEL or Backspace
// 	onChangeChrCode		On key:  Handle an alphanumieric keystroke, e.g., "A"
//	onValidate			Validate the string as it stands and return true/false if it succeeds/fails
//	isFilled			For auto-tab checking:  Is the number "filled" so auto-tab should occur

HxG_5.prototype.MaskAssist = function (_elem, _state, _converter, _validator, _promptChar) {
	if (!_elem || !_converter || !_state) return;
	this.lc = hX_5.currentLocale();

	// Exported functions
	this.isFilled		= isFilled;			// Public
	this.onFocus		= onFocus;			// Public
	this.onBlur			= onBlur;			// Public
	this.onClick		= onClick;			// Public
	this.onCut			= onCut;			// Public
	this.onPaste		= onPaste;			// Public
	this.onChangeKeyCode= onChangeKeyCode;	// Public
	this.onChangeChrCode= onChangeChrCode;	// Public
	this.onValidate		= onValidate;		// Public
	this.destroy		= destroy;			// Public
	
	// Basic information kept by assist
	var elem			= _elem;
	var state			= _state;
	var cvt		 		= _converter;
	var validator 		= _validator;
	var promptChar 		= _promptChar;

	this.getMask		= function getMask() { return ((cvt.maskNFormat!=null)?cvt.maskNFormat:cvt.maskFormat); }
	this.getMaskLast	= function getMaskLast() { return ((cvt.maskNFormat!=null)?cvt.maskNFormatLast:cvt.maskFormatLast); }
	this.getMaskFirst	= function getMaskFirst() { return ((cvt.maskNFormat!=null)?cvt.maskNFormatFirst:cvt.maskFormatFirst); }

	// Actual keyboard handling is done by the mask handler sub-object in "masked string"
	var handler	= null;

	// == Start constructor =====================================================
	
	// Build a mask handler that pre-handles keystrokes for this converter
	handler = new hX_5.MaskAssist.prototype.maskHandler(elem, state, cvt, promptChar);
	
	// == End constructor =======================================================

	// == Destructor ============================================================
	function destroy() {
		if (handler!=null) {
			if (handler.destroy) handler.destroy();
			delete handler;
		}
	}

	// ==========================================================================
	// Public:  See if the value is "filled" according to the mask
	function isFilled (curpos) {
		var c, v = hX_5.Trim(elem.value), l = v.length;
		if (l < (this.getMask()).length)
			return false;
		// Is the last character present?
		c = v.charAt(this.getMaskLast());
		// And are we past it ?
		return ((curpos > this.getMaskLast()) && (c != promptChar) && (c != hX_5._B));
	}
	
	// ==========================================================================
	// Public: Put a field in "assist" mode
	function onFocus () {
		
		// To go into assist mode, we have to start with a valid value
		// To do this we convert the current string to a value and back
		// If this works, the value is valid (or could be made to be valid)
		// otherwise, we use the "empty string" (the mask with prompt chars in it)
		var newValue = (elem.value) ? hX_5.trimR(elem.value) : null;
		newValue = (newValue && newValue.length > 0) ? cvt.stringToValue(newValue, promptChar, true) : null;
		newValue = (newValue) ? cvt.valueToString(newValue, promptChar) : handler.getEmptyMask();

		// Set the value and the cursor (directly, not into state which isn't set yet)
		elem.value = newValue;
		// If the mask has protect on it, rebuild the mask we use to work
		handler.rebuildMask(newValue);
		// Set selection
		hX_5.sel.setCaretPos(elem, this.getMaskFirst(), null, hX_5.brw.isNavOrMoz());
		return true;
	}

	// Public: Take a field out of "assist" mode
	function onBlur () {
		var v = elem.value;
		if (v.length==0) return true;

		var newValue = cvt.stringToValue(v, promptChar);
		if (newValue != null)
			newValue = (newValue.length==0) ? hX_5._E : cvt.valueToString(newValue);
		if (newValue != null) {
			// If the value is "empty" then turn it back into an empty string 
			var m = handler.getEmptyMask();
			m = hX_5.replaceAll(m,promptChar,hX_5._B);
			if (m!=newValue)
				elem.value = newValue;
			else
				elem.value = hX_5._E;
			return true;
		}
		return false;		
	}

	//====================================================================================
	// Public: Handle clicking in the field
	function onClick (evt) {
		return handler.onClick(evt);
	}

	//====================================================================================
	// Public: Handle cutting in the field
	function onCut (evt) {
		return handler.onCut(evt);
	}

	//====================================================================================
	// Public: Handle pasting in the field
	function onPaste (evt) {
		return handler.onPaste(evt);
	}

	//====================================================================================
	// Public: Keystroke-level validation of a "function key"
	function onChangeKeyCode (evt) {
		return handler.onChangeKeyCode (evt);
	}
	
	//====================================================================================
	// Public: Keystroke-level validation of a "character"
	function onChangeChrCode (evt) {
		return handler.onChangeChrCode (evt);
	}

	// ==========================================================================
	// Public: Keystroke-level validation of a string
	function onValidate () {
		// Round-trip the value through the converter -- this will fail if there's a bad char in the string
		var newValue = cvt.stringToValue(state.getValue(), promptChar);
		if (newValue != null)
			newValue = cvt.valueToString(newValue, promptChar);
		if (newValue != null)
			return true;
		return false;
	}
}

//============MASK HANDLER====================================
// Sub-object used to handle keyboard/mouse events for both masked strings and for datetime
//==================================================================
HxG_5.prototype.MaskAssist.prototype.maskHandler = function(_element, _state, _converter, _promptChar) {

	var elem 		= _element;
	var state		= _state;
	var cvt			= _converter;
	var promptChar	= _promptChar;
	var emptyMask	= null;

	// Public functions
	this.onClick		= onClick;
	this.onCut			= onCut;
	this.onPaste		= onPaste;
	this.onChangeKeyCode= onChangeKeyCode;
	this.onChangeChrCode= onChangeChrCode;
	this.getEmptyMask	= function getEmptyMask() {return emptyMask;}
	this.rebuildMask	= function rebuildMask(v) {if (cvt.rebuildProtectMask) cvt.rebuildProtectMask(v); }

	// Selected imported functions
	var getKeyCode	= hX_5.getKeyCode;
	
	// Keyboard handling sub-functions
	var getMask		= function getMask() { return ((cvt.maskNFormat!=null)?cvt.maskNFormat:cvt.maskFormat); }
	var getCChar 	= function (i) 	{ return (i < 0 || i >= (getMask()).length) ? hX_5._E : (getMask()).charAt(i); }

	//== CONSTRUCTOR
	
	// Make a copy of the mask which has "promptchars" instead of #, ?, ! and ~
	emptyMask = hX_5.replaceAll(cvt.maskFormat, hX_5._LB, promptChar);
	for (var i=1; i<cvt.markers.length; i++)
		emptyMask = hX_5.replaceAll(emptyMask, cvt.markers.charAt(i), promptChar);

	//== END CONSTRUCTOR

	//== Private Functions =============================

	// Private:  Handle a delete or backspace
	function doDelete (evt) {

		var v, s, cR, pos = state.getPos();
		pos = (evt.type==hX_5._evCT) ? (pos) : ((evt.keyCode==hX_5.KEY_BS) ? pos-1 : pos);
		if (pos < 0 && !state.hasSel())
			return true;

		cR = getCChar(pos);
		if (!cvt.isMarker(cR) && cR != hX_5._EQ && !state.hasSel()) {
			if (evt.type!=hX_5._evCT) {
				pos = (evt.keyCode==hX_5.KEY_BS) ? (pos) : (pos+1);
				state.setPos(pos);
				return -2;
			}
			return false;
		}

		v = state.getValue(); s = state.hasSel();
		if (s) {
			var rg = hX_5.sel.getSelectionR(elem);
			state.setValue((rg.start <= 0 && rg.end >= v.length-1)?emptyMask:fillWithMaskPiece(v, rg.start, rg.end));
		} else
			if (pos>=0 && pos<v.length) state.setValue (v.substr(0,pos) + promptChar + v.substr(pos+1));
		pos = (s || (evt.type!=hX_5._evCT && evt.keyCode==hX_5.KEY_BS)) ? pos : pos+1;
		state.setPos(pos);
		return -2;
	}

	// Private:  Handle inserting/overstriking
	function doInsert (v, chi, pos, sel) {
		var ch = (chi==hX_5._B)?promptChar:chi;
		if (sel) {
			var rg = hX_5.sel.getSelectionR(elem);
			v = fillWithMaskPiece(v, rg.start, rg.end);
			var i, l = v.length;
			for (i=rg.start; i < rg.end; i++) {
				if (cvt.isMarker(getCChar(i))) break;
			}
			if (i < rg.end)
				v = v.substr(0,i) + ch + v.substr(i+1);
			state.setPos(rg.start+1);
		} else {
			v = v.substr(0,pos) + ((ch==hX_5._B)?promptChar:ch) + v.substr(pos+1);
			pos = getMoveForward (v, pos+1);
			state.setPos(pos);
		}
		state.setValue(v);
		return -2;
	}

	// Private:  Handle left arrow
	function doCaretPrev (evt) {
		// We don't handle it here if it's shifted or a selection is in force
		if (evt.shiftKey || state.hasSel())
			return -1;
		else {
			var pos = getMovePrevious (state.getValue(), state.getPos()-1);
			state.setPos(pos);
			return -2;
		}
	}

	// Private:  Handle right arrow
	function doCaretNext (evt) {
		// We don't handle it here if it's shifted or a selection is in force
		if (evt.shiftKey || state.hasSel())
			return -1;
		else {
			var pos = getMoveForward (state.getValue(), state.getPos()+1);
			state.setPos(pos);
			return -2;
		}
	}

	// Positioning issues:
	// 1) If you move right, we always "hop" to be on a position where you can type
	//    This does mean you can't position "after" a string of typable spaces, e.g.,
	//	  In ###-##-### you can't position "before" the hyphens (we'll hop to the next #).
	//	  This makes it hard to copy/paste a piece of the value (e.g., the first ###)
	// 2) If you move left (backwards), when we "hop" we'll position the cursor in a non-typable
	//    place (e.g., in ###-###-### if you're after the first hyphen when we go back we hop back
	//    only one position)
	//    This let's you position the cursor in a place where you can do a copy
	// Private: Move left
	function getMovePrevious (v, pos) {
		var i, c, c1, l = v.length;
		pos = (pos < 0) ? 0 : ((pos > l) ? l : pos);
		for (i=pos; i > 0; i--) {
			c = getCChar(i);
			c1 = getCChar(i-1);
			if (cvt.isMarker(c) || cvt.isMarker(c1))
				return (i);
		}
		return (0);
	}

	// Private: Move right
	function getMoveForward(v, pos) {
		var i, l = v.length;
		pos = (pos < 0) ? 0 : ((pos > l) ? l : pos);
		for (i=pos; i <= l; i++ ) {
			if (cvt.isMarker(getCChar(i))) return (i);
		}
		return (l);
	}

	// Private: Fill a "hole" in a value with a chunk of the mask
	function  fillWithMaskPiece (v, start, end) {
		return ( v.substr(0,start) + emptyMask.substr(start, end-start) + v.substr(end) );
	}

	//====================================================================================
	// Public: Handle clicking in the field
	function onClick(evt) {
		if (state.hasSel())
			return true;
		var v = state.getValue(), pos = state.getPos();
		if (pos <= 0 || pos > (v.length-1))
			return true;
		// If the user clicked "next to" an editable space, that's good
		var cL = getCChar(pos-1), cR = getCChar(pos);
		if (cvt.isMarker(cL) || cvt.isMarker(cL))
		    return true;
		    
		// Otherwise, move toward the next editable sapce
		var MoveR = true, posLeft =  getMovePrevious (v, pos), posRight = getMoveForward(v, pos);
		if ((pos-posLeft) < (posRight-pos))
			state.setPos(posLeft);
		else
			state.setPos(posRight);
		return -2;
	}

	//====================================================================================
	// Public: Keystroke-level validation of a "function key"
	function onChangeKeyCode (evt) {
	
		// We're looking for a small number of keystrokes we need to intercept and handle instead
		// of letting the input handler deal with them
		switch (hX_5.getKeyCode(evt)) {
			case hX_5.KEY_BS:  
			case hX_5.KEY_DEL:
				return doDelete(evt);
			case hX_5.KEY_LTARROW:
				return doCaretPrev(evt);
			case hX_5.KEY_RTARROW:
				return doCaretNext(evt);
		}
		return true;
	}

	//====================================================================================
	// Public: On cut -- essentially the same as selecting text and pressing the DEL/BS key
	// NOTE: as with Paste below, this is only called in IE -- event isn't available in Mozilla
	function onCut(evt) {
		if (hX_5.brw.isIE() && state.hasSel()) {
			// Put the data on the clipboard
			var cutText = hX_5.sel.getSelection(elem);
			window.clipboardData.setData("Text", cutText);
			return doDelete(evt);
		}
		return true;
	}

	//====================================================================================
	// Public: On paste -- essentially the same as typing (either into a selection or not)
	// NOTE: as with Cut above, this is only called in IE -- event isn't available in Mozilla
	function onPaste (evt) {
		if (hX_5.brw.isIE()) {
			// Get the data on the clipboard
			var cutText = window.clipboardData.getData("Text");
			// Cut out any old text
			var v = state.getValue(), pos = state.getPos(), sel = state.hasSel();
			// If we have a selection, "clear it", then paste over it (paste could be longer/shorter
			// than what we paste over
			if (sel) {
				var rg = hX_5.sel.getSelectionR(elem);
				v = (rg.start <= 0 && rg.end >= v.length-1)?emptyMask:fillWithMaskPiece(v, rg.start, rg.end);
			} 
			// The normal case is someone is doing a paste WITH format characters
			// (e.g., pasting "191-99-888" with the hyphens).  
			// But they may be pasting an "unformatted value" (e.g., 19199888)
			// So we'll look see if we have format chars in the pasted text
			// and if not, we'll treat it as unformatted
			var mChars = false, c, m, i, r, l = cutText.length;
			// Look to see if we have properly positioned mask chars
			for (i=0; i < l; i++) {
				m = getCChar(i);
				// See if we're pasting a format char on a format char
				if (!cvt.isMarker(m)) {
					mChars = (m == cutText.charAt(i));
					break;
				}
			}
			// Now paste either using the format chars or not
			for (i=0; i < l; i++) {
				m = getCChar(i);
				c = cutText.charAt(i);
				// See if we're "pasting over" a mask character
				if (!cvt.isMarker(m) && m == c) {
				} else {
					c = (c==hX_5._B)?promptChar:c;
					v = v.substr(0,pos) + c + v.substr(pos+1);
				}
				pos = (mChars) ? pos+1 : getMoveForward (v, pos+1);
			}
			state.setValue(v);
			pos = getMoveForward (v, pos);
			state.setPos(pos);
			return -2;
		}
		return true;
	}

	//====================================================================================
	// Public: Keystroke-level validation of a "character"
	function onChangeChrCode (evt) {
		// Return on tab or newline -- these aren't "control" keys
		// so they come in here
		var v = state.getValue(), pos = state.getPos(), sel = state.hasSel(), ch = hX_5.getChrCode(evt);
		if (evt.keyCode == hX_5.KEY_TAB || evt.keyCode == hX_5.KEY_ENTER)
			return +2;
		// We "overstrike" characters
		return (doInsert(v, ch, pos, sel));
	}
}
