/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2006 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

// XMLData Class

/**
 * @class public XMLData
 * @constructor XMLData
 *	The XMLData class is a class for containing and manipulating an XML Document.
 * @param XMLDocument xmlValue
 *	XML element which is represented by the instance of the class.
 * @param boolean cloneNode
 *  Optional parameter which flags whether we store a pointer to the node (false) or a clone of the node (true).
 * @exception XMLDataError
 *	Throws exception if node passed is null or undefined.
**/

// public API

HxG_5.prototype.XMLData = function(node, cloneNode)
{
	if (typeof node == "string")
	{
		var xmlDoc;
		if (hX_5.brw.isIE())
		{
			xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
			xmlDoc.loadXML(node);

		} 
		else if (hX_5.brw.isNavOrMoz())
		{
			//changeReadyState(this, 1);
			var xmlParser = new DOMParser();
			xmlDoc = xmlParser.parseFromString(node, "text/xml");
		}

		this.xmlNode = xmlDoc.documentElement;
	}
	else
	{
		if(!node)
		{
			var args = [];
			args[0] = "node";
			var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_invalid_param, args);
			Log.error("HxG_5.XMLData", Msg);
			throw new hX_5.XMLDataError(Msg);
		}
		else
		{
			this.isRootOfXMLDocument = false;	//default to false
			switch(node.nodeType){
				//at the moment group types except for node of type document
				case this.ELEMENT_NODE:	
				case this.ATTRIBUTE_NODE:
				case this.TEXT_NODE: 
				case this.CDATA_NODE:
				case this.ENTITY_REFERENCE_NODE:
				case this.ENTITY_NODE:
				case this.PROCESSING_INSTRUCTION_NODE:
				case this.COMMENT_NODE:
				case this.DOCUMENT_FRAGMENT_NODE:
				case this.NOTATION_NODE:		
					// for these just store node
					this.xmlNode = (cloneNode) ? node.cloneNode(true) : node; 
					break;
				case this.DOCUMENT_NODE:	
					//for document store root node element
					this.xmlNode = (cloneNode) ? node.documentElement.cloneNode(true) : node.documentElement; // get root Node Element																						
					this.isRootOfXMLDocument = true;
					break;
				default:
					break;
			}
		
			//node should be treated as a protected var of the object, only to be accessed from within the XMLData and XMLDataList class

		}
	}
		
	//for toXMLString method
	this.prettyPrinting = true;
	this.prettyIndent = 2; //default spaces for tabbing
	
}

//node types from W3C:
HxG_5.prototype.XMLData.prototype.ELEMENT_NODE					= 1;
HxG_5.prototype.XMLData.prototype.ATTRIBUTE_NODE				= 2;
HxG_5.prototype.XMLData.prototype.TEXT_NODE						= 3;
HxG_5.prototype.XMLData.prototype.CDATA_NODE					= 4;
HxG_5.prototype.XMLData.prototype.ENTITY_REFERENCE_NODE			= 5;
HxG_5.prototype.XMLData.prototype.ENTITY_NODE					= 6;
HxG_5.prototype.XMLData.prototype.PROCESSING_INSTRUCTION_NODE	= 7;
HxG_5.prototype.XMLData.prototype.COMMENT_NODE					= 8;
HxG_5.prototype.XMLData.prototype.DOCUMENT_NODE					= 9;
HxG_5.prototype.XMLData.prototype.DOCUMENT_TYPE_NODE			= 10;
HxG_5.prototype.XMLData.prototype.DOCUMENT_FRAGMENT_NODE		= 11;
HxG_5.prototype.XMLData.prototype.NOTATION_NODE					= 12;



/**
 * @method public HxG_5.prototype.XMLData.prototype.addNameSpace
 *	Takes one argument which can be a String with a namespace URI, or a Namespace Object, and adds the 
 *	argument to the in-scope namespaces of this XML object. Returns the XML object for which it is called.
 *	xhtml.addNamespace(new Namespace('http://www.w3.org/1999/xhtml'));
 *	xhtml.addNamespace(new Namespace('http://www.w3.org/1999/xlink'));
 * @param String(or Namespace) namespaceToAdd
 *	New Namespace Object or Namespace URI to be added.
 * @return Object result
 *	The value of the requested attribute
 **/
HxG_5.prototype.XMLData.prototype.addNameSpace = function(namespaceToAdd)
{
	//TODO
}

HxG_5.prototype.XMLData.prototype.getNumChildrenOfRoot = function()
{
	return this.xmlNode.childNodes.length;
}


/**
 * @method public HxG_5.prototype.XMLData.prototype.appendChild
 *	This is used to append a new child XMLData object at the end of the XMLData, 
 *	and the changed XMLData object is then returned.
 * @param XMLData childToAppend
 *	new hX_5.XMLData to be Appended at the end of the list of children.
 * @return XMLData result
 *	This Modified XMLData Object
 * @exception XMLDataError
 *	Throws exception if childToAppend is null or undefined.
 *	Throws exception if childToAppend is not of type XMLData or XMLDataList.
 **/
HxG_5.prototype.XMLData.prototype.appendChild = function(childToAppend)
{
	if(!childToAppend)
	{
		var args = [];
		args[0] = "childToAppend";
		var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_invalid_param, args);
		Log.error("HxG_5.XMLData.appendChild", Msg);
		throw new hX_5.XMLDataError(Msg);
	}
	else
	{
		
		if(childToAppend instanceof hX_5.XMLData)
		{
			var newch = childToAppend.copy();
			// single XMLData Object to append
			this.xmlNode.appendChild(newch.xmlNode);
		}
		else if(childToAppend instanceof hX_5.XMLDataList)
		{
			var i, newch = childToAppend.copy();
			for(i=0; i<newch.length(); i++)
			{
				this.appendChild(newch.elementByIndex(i)); //recursive call to appendChild with XMLData param
			} 
		}
		else
		{
			var args = [];
			args[0] = "childToAppend";
			var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_invalid_param_type, args);
			Log.error("HxG_5.XMLData.appendChild", Msg);
			throw new hX_5.XMLDataError(Msg);
		}
	}
}

/**
 * @method public HxG_5.prototype.XMLData.prototype.attribute
 *	This returns the value of the named attribute. For example, 
 *  referring to XML that contains <p id="p1" xml:lang="en" class="style1"> 
 *  using the call obj.attribute('id') returns "p1".
 * @param String attributeName
 *	The name of the requested attribute.
 * @return XMLData value
 *	The XMLData object representing the attribute. Attribute node is not cloned into the new XMLData object
 * @exception XMLDataError
 *	Throws exception if this XMLData object does not represent node of type element.
 **/
HxG_5.prototype.XMLData.prototype.attribute = function(attributeName)
{
	if(this.xmlNode.nodeType!=this.ELEMENT_NODE)
	{
		var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_xmldata_not_element);
		Log.error("HxG_5.XMLData.attribute", Msg);
		throw new hX_5.XMLDataError(Msg);
		return null;
	}
	var attr = this.xmlNode.getAttributeNode(attributeName);	
	return new hX_5.XMLData(attr);
}

/**
 * @method public HxG_5.prototype.XMLData.prototype.attributes
 *	Returns array of all attribute values
 * @return XMLDataList result
 *	XMLDataList containing the attribute nodes
 * @exception XMLDataError
 *	Throws exception if this XMLData object is not of type element.
 **/
HxG_5.prototype.XMLData.prototype.attributes = function()
{

	if(this.xmlNode.nodeType!=this.ELEMENT_NODE)
	{
		var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_xmldata_not_element);
		Log.error("HxG_5.XMLData.attributes", Msg);
		throw new hX_5.XMLDataError(Msg);
		return null;
	}
	var attrs = this.xmlNode.attributes;
	return new hX_5.XMLDataList(attrs);
}

/**
 * @method public HxG_5.prototype.XMLData.prototype.child
 *	Takes one argument with a property name and returns an 
 *	XMLDataList with children matching the property name. 
 *	Thus if the property name is a positive integer (including 0) 
 *	then the child with that index is returned (to be exactly an 
 *	XMLDataList containing that child only), otherwise all children 
 *	matching the property name are returned.
 * @param String childPropertyName
 *	The name of the requested attribute.
 * @return XMLDataList result
 *	The value of the requested attribute. Return null if no matches or index out of bounds.
 * @exception XMLDataError
 *	Throws exception if childPropertyName is integer and out of bounds as index.
 *	Throws exception if childPropertyName is not of type integer or string.
 **/
HxG_5.prototype.XMLData.prototype.child = function(childPropertyName)
{
	if(this.xmlNode.hasChildNodes())
	{
		var children = this.xmlNode.childNodes;
		// check if childPropertyName is index or property name
		if(typeof childPropertyName == "number")
		{
			if (childPropertyName >= 0 && childPropertyName < children.length)
			{
				// Adam: should this return just an XMLData or XMLDataList?
				// unclear about E4X semantics - returning XMLData for now
				var elem = children.item(childPropertyName);
				var arr = [];
				return new hX_5.XMLData(elem);
			}
			else
			{
				var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_invalid_index);
				Log.error("HxG_5.XMLData.child", Msg);
				throw new hX_5.XMLDataError(Msg);
				return null;
			}
		}
		else if (typeof childPropertyName == "string")
		{
			var curr;
			var arr = [];

			for (i=0; i<children.length; i++)
			{
				curr = children.item(i);
				if (curr.nodeName == childPropertyName)
				{
					arr[arr.length] = new hX_5.XMLData(curr);
				}
			}
			var result = new hX_5.XMLDataList(arr);
			return result;
		}
		else
		{
			var args = [];
			args[0] = "childPropertyName";
			var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_invalid_param_type,args);
			Log.error("HxG_5.XMLData.child", Msg);
			throw new hX_5.XMLDataError(Msg);
			return null;
		}
	}
	else
	{
		return new hX_5.XMLDataList();
	}
}


/**
 * @method public HxG_5.prototype.XMLData.prototype.children
 *	This function gets all children of this XMLData object and puts them in an XMLDataList
 * @return XMLDataList result
 *	List of children elements. If none, return null.
 **/
HxG_5.prototype.XMLData.prototype.children = function()
{
	if(this.xmlNode.hasChildNodes())
	{
		var i, 
			children = this.xmlNode.childNodes, 
			child_elems = [];

		for (i=0; i<children.length; i++)
		{
			child_elems[i] = new hX_5.XMLData(children.item(i));
		}

		return new hX_5.XMLDataList(child_elems);
	}
	else
	{
		return new hX_5.XMLDataList();
	}
}

/**
 * @method public HxG_5.prototype.XMLData.prototype.contains
 *	The method contains takes one argument, a value to be compared against this XMLData object. Calling
 *  XMLData.contains(value)
 *  yields the same result as the equality comparsion
 *  XMLData == value
 * @param String value
 *	The value to be compared
 * @return boolean result
 *	The value of the requested attribute
 * @exception XMLDataError
 *	Throws exception if value is null or undefined
 **/
HxG_5.prototype.XMLData.prototype.contains = function(value)
{
	if(value)
	{
		var result = (this == value) ? true : false; 
		return result;
	}
	else
	{
		var args = [];
		args[0] = "value";
		var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_invalid_param, args);
		Log.error("HxG_5.XMLData.contains", Msg);
		throw new hX_5.XMLDataError(Msg);
	}
}


/**
 * @method public HxG_5.prototype.XMLData.prototype.copy
 *	The method copy takes no argument and returns a deep copy of the XMLData object on which it is called
 *  where the internal parent property is set to null.
 * @return XMLData result
 *	The new copy
 **/
HxG_5.prototype.XMLData.prototype.copy = function()
{
	// pass in true to flag deep copy
	return (new hX_5.XMLData(this.xmlNode,true));
}

/**
 * @method public HxG_5.prototype.XMLData.prototype.getDescendants
 *	Has one optional argument, the name of descendants we are looking for. 
 *	The method returns an XMLDataList with the descendants matching the passed name argument or with all 
 *	descendants if no name is specified.
 * @param String name
 *	Name of descendant we're looking for.
 * @return XMLDataList result
 *	If no name is specified, returns all descendants in XMLDataList,
 *	else return descendant(s) matching name.
 **/
HxG_5.prototype.XMLData.prototype.getDescendants = function(name)
{
	
	if (!this.xmlNode.hasChildNodes())
	{
		// this is a leaf. it has already been added. no need to traverse any deeper here
		return null;
	}else{
		var i, ch = this.children(), desc = new hX_5.XMLDataList(new Array());
		for (i=0; i<ch.length(); i++)
		{
			var curr = ch.elementByIndex(i), new_desc;
			//add this to the list of descendants
			if(!name)
			{	
				desc.append(curr);
				new_desc = curr.getDescendants();
			}
			else 
			{
				if (name==curr.name()){
					desc.append(curr);
				}
				//recurse through descendants looking for specified node ("name")
				new_desc = curr.getDescendants(name);
			}

			if(new_desc)
			{
				//add descendants of this node
				desc.append(new_desc);
			}
		}
		return desc;
	}
}


/**
 * @method public HxG_5.prototype.XMLData.prototype.elements
 *	Takes one optional argument, the name of elements we are looking for, and returns an XMLDataList with 
 *  all matching child elements or with all child elements if no name is supplied.
 * @param String name
 *	Name of element we are looking for.
 * @return XMLDataList result
 *	If no name is specified, returns all elements in XMLDataList,
 *	else return element(s) matching name.
 *	returns null if none
 **/
HxG_5.prototype.XMLData.prototype.elements = function(name)
{
	var result = new hX_5.XMLDataList(new Array());
	if(!this.xmlNode.hasChildNodes())
	{
		Log.info("HxG_5.XMLData.elements", "No children found");
		// returning empty XMLDataList
	}
	else
	{
		var ch = this.children(), i;
		if(name)
		{
			Log.info("HxG_5.XMLData.elements", "Looking for elements with name '" + name + "'");
			// name is specified so must do check for matching element names
			for (i=0; i<ch.length(); i++)
			{
				var curr = ch.elementByIndex(i);
				if (curr.name() == name && curr.type() == curr.ELEMENT_NODE)
				{
					result.append(curr);
				}
			}
		}
		else
		{
			Log.info("HxG_5.XMLData.elements", "No name specified so retrieving all elements");
			// get all child elements
			for (i=0; i<ch.length(); i++)
			{
				var curr = ch.elementByIndex(i);
				if (curr.type() == curr.ELEMENT_NODE)
				{
					result.append(curr);
				}
			}
		}
	}
	return result;
}

/**
 * @method public HxG_5.prototype.XMLData.prototype.inScopeNamespaces
 *	Returns an array of Namespace objects representing the namespaces that are in scope for this XML object.
 *	xhtml.addNamespace(new Namespace('http://www.w3.org/1999/xhtml'));
 *	xhtml.addNamespace(new Namespace('http://www.w3.org/1999/xlink'));alert(xhtml.inScopeNamespaces());
 *	// shows 'http://www.w3.org/1999/xhtml,http://www.w3.org/1999/xlink'
 * @return Array result
 *	Array with list of Namespace of objects
 **/
HxG_5.prototype.XMLData.prototype.inScopeNamespaces = function()
{
	// TODO
}

/**
 * @method public HxG_5.prototype.XMLData.prototype.insertChildAfter
 *	Takes two arguments, an existing node after which the new node is to be inserted, and the new child 
 *	node to be inserted. If the first argument is null then the second argument is inserted as
 *	the first child of the XML object the method is called on. Otherwise the second argument is inserted 
 *	directly after the first argument if that exists as a child; otherwise it is not inserted. 
 * @param Node referenceNode
 *	Node after which to insert new child node.
 * @param Node childToInsert
 *	Node to insert.
 * @return XMLData result
 *	If the second argument is inserted successfully, the updated XMLData object on which the
 *	method is called is returned, otherwise 'undefined' is returned.
 **/
HxG_5.prototype.XMLData.prototype.insertChildAfter = function(referenceNode, childToInsert)
{
	// TODO
}

/**
 * @method public HxG_5.prototype.XMLData.prototype.insertChildBefore
 *	This is similar to insertChildAfter, but the new node is inserted before the existing node. If the first
 *	parameter is null then insert the child at the end.
 * @param Node referenceNode
 *	Node before which to insert new child node.
 * @param Node childToInsert
 *	Node to insert.
 * @return XMLData result
 *	If the second argument is inserted successfully, the updated XMLData object on which the
 *	method is called is returned, otherwise 'undefined' is returned.
 **/
HxG_5.prototype.XMLData.prototype.insertChildBefore = function()
{
	// TODO
}

/**
 * @method public HxG_5.prototype.XMLData.prototype.length
 *	 Returns '1' for XMLData
 * @return int result
 *	Return '1'
 **/
HxG_5.prototype.XMLData.prototype.length = function()
{
	var result = 1;
	return result;
}

/**
 * @method public HxG_5.prototype.XMLData.prototype.localName
 *	Returns the local name part if the XMLData object has a name. 
 * @return String result
 *	Return part of name after the namespace e.g. <xhtml:p> has a localname of "p".
 **/
HxG_5.prototype.XMLData.prototype.localName = function()
{
	// TODO
}

/**
 * @method public HxG_5.prototype.XMLData.prototype.name
 *	Returns the qualified name of the XMLData object or null if the object has no name (as for text 
 *	nodes and comment nodes). 
 * @return String result
 *	Return a string representation of the qualified name as the concatenation of the
 *	namespace URI, '::', and the local name e.g. <xhtml:p xmlns:xhtml="http://www.w3.org/1999/xhtml">
 *	has the name "http://www.w3.org.1999/xhtml::p".
 **/
HxG_5.prototype.XMLData.prototype.name = function()
{
	var name = this.xmlNode.nodeName;
	if (name == "#text" ||
		name == "#cdata-section" ||
		name == "#comment" ||
		name == "#document" ||
		name == "#document-fragment")
	{
		return null;
	}
	else
	{
		return this.xmlNode.nodeName;
	}

}

/**
 * @method public HxG_5.prototype.XMLData.prototype.value
 *	Returns value of XML node if it is a node representing text, an attribute, a comment or a pi
 * @return String result
 *	Return a string representation of the qualified name as the concatenation of the
 *	namespace URI, '::', and the local name e.g. <xhtml:p xmlns:xhtml="http://www.w3.org/1999/xhtml">
 *	has the name "http://www.w3.org.1999/xhtml::p".
 **/
HxG_5.prototype.XMLData.prototype.value = function()
{
	var t = this.type();
	if (t == this.TEXT_NODE ||
		t == this.COMMENT_NODE ||
		t == this.PROCESSING_INSTRUCTION_NODE ||
		t == this.ATTRIBUTE_NODE)
	{
		return this.xmlNode.nodeValue;
	}
	else
	{
		return null;
	}
}

/**
 * @method public HxG_5.prototype.XMLData.prototype.namespace
 *	If no argument is passed to the method then it returns the namespace associated with the qualified name 
 *	of this XML object. If a prefix is passed to the method then it looks for a matching namespace in the 
 *	in scope namespace of this XML Object and returns it when found.
 * @param prefix
 *	Optional prefix to find specific namespace
 * @return Namespace result
 *	Returns a matching Namespace Object if found; otherwise 'undefined'.
 **/
HxG_5.prototype.XMLData.prototype.namespace = function(prefix)
{
	//TODO (NTH)
}

/**
 * @method public HxG_5.prototype.XMLData.prototype.namespaceDeclarations
 *	Returns an array containing the namespace declarations associated with the XMLData object in the context of
 *	the parent.
 * @return Array result
 *	Array of namespace objects.
 **/
HxG_5.prototype.XMLData.prototype.namespaceDeclarations = function()
{
	//TODO
}

/**
 * @method public HxG_5.prototype.XMLData.prototype.prependChild
 *	Takes one argument, the value to prepend, inserts that value as the first child of the XMLData object.
 *	Note that when the value passed in is an XMLDataList then each element in the list is prepended.
 * @param XMLData(List) childToPrepend
 *	Child (or list of children) to prepend to this XMLDataObject
 * @exception XMLDataError
 *	Throws exception if childToPrepend is null or undefined
 *	Throws exception if childToPrepend is not of type XMLData or XMLDataList
 **/
HxG_5.prototype.XMLData.prototype.prependChild = function(childToPrepend)
{

	if (!childToPrepend)
	{
		var args = [];
		args[0] = "childToPrepend";
		var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_invalid_param, args);
		Log.error("HxG_5.XMLData.prependChild", Msg);
		throw new hX_5.XMLDataError(Msg);
	}
	else if(childToPrepend instanceof hX_5.XMLData)
	{
		
		// param is single child / XMLData
		// clone new child that has been passed in
		var newch = childToPrepend.copy();

		if(this.xmlNode.firstChild){
			// has children; insert the child node before the first child
			this.xmlNode.insertBefore(newch.xmlNode, this.xmlNode.firstChild);
		}else{
			//this node has no children so append new child as first child
			this.xmlNode.appendChild(newch.xmlNode);
		}
	}
	else if(childToPrepend instanceof hX_5.XMLDataList)
	{
		
		// param is list of children / XMLDataList
		// clone new children that have been passed in
		var newch = childToPrepend.copy();

		var i;
		for(i=0; i<newch.length(); i++)
		{
			this.xmlNode.insertBefore(newch.elementByIndex(childToPrepend.length()-1-i).xmlNode, this.xmlNode.firstChild);
		}
	}
	else
	{
		var args = [];
		args[0] = "childToPrepend";
		var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_invalid_param_type, args);
		Log.error("HxG_5.XMLData.prependChild", Msg);
		throw new hX_5.XMLDataError(Msg);
	}
}

HxG_5.prototype.XMLData.prototype.put = function(P, V)
{
	var c, notXMLType = !( (V instanceof hX_5.XMLData) || (V instanceof hX_5.XMLDataList) );
	
	if ( notXMLType || (V.type() == this.TEXT_NODE) || (V.type() == this.ATTRIBUTE_NODE))
	{
		//if the value is not and XMLData or XMLDataList object
		//or it is an XMLData representing a text or an attribute node
		c = V.toString();
	}
	else
	{
		//clone parameter object
		c = V.copy(true);
	}

	var t = this.type();
	if ( t == this.TEXT_NODE || t == this.COMMENT_NODE || t == this.PROCESSING_INSTRUCTION_NODE || t == this.ATTRIBUTE_NODE)
	{
		//can't have children
		return;
	}

	if(P.charAt(0) == '@')
	{
		//TODO - 'put' for attribute
	}

	var i = null;
	var primitiveAssign = ( notXMLType && (P != "*"));
	var k, len=this.children().length();
	for (k = len-1; k>=0; k--)
	{
		if ((P == "*") || ((this.child(k).type() == this.ELEMENT_NODE) && (this.child(k).name() == P)))
		{
			if (i)
			{
				this.removeByIndex(i);
			}
			i = k;
		}
	}

	if (!i)
	{
		//new child
		i = len;
		if (primitiveAssign == true)
		{
			var newch = new hX_5.XMLData("<"+P+" />");
			this.replace(i,newch);
		}
	}
	if (primitiveAssign == true)
	{
		//primitive value
		this.child(i).remove("*");
		var s = c.toString();
		if (s!="")
		{
			this.child(i).replace(0,s);
		}
	}
	else
	{
		this.replace(i,c);
	}

	return;
}



HxG_5.prototype.XMLData.prototype.replace = function(P,V)
{
	var t = this.type();
	if (t == this.TEXT_NODE || t == this.COMMENT_NODE || t == this.PROCESSING_INSTRUCTION_NODE || t == this.ATTRIBUTE_NODE)
	{
		return;
	} 
	var ch = this.children();
	var len = ch.length();
	if (P >= len)
	{
		//new child
		P = len;
	}
	
	if (V instanceof hX_5.XMLData)
	{
		var t2 = V.type();
		if(t2 == this.ELEMENT_NODE || t2 == this.COMMENT_NODE || t2 == this.PROCESSING_INSTRUCTION_NODE || t2 == this.TEXT_NODE){
			//set parent
			var old = this.xmlNode.childNodes.item(P);
			if(P==len)
			{
				this.appendChild(V);
			}
			else
			{
				this.xmlNode.replaceChild(V.xmlNode,old);
			}
		}
	}
	else if (V instanceof hX_5.XMLDataList)
	{
		this.deleteByIndex(P);
		// TODO Call the [[Insert]] method of x with arguments P and V
	}
	else
	{
		var s = V.toString(), txt;
		var newtext = new hX_5.XMLData(this.xmlNode.ownerDocument.createTextNode(s));

		var old = this.xmlNode.childNodes.item(P);
		if(P==len)
		{
			this.appendChild(newtext);
		}
		else
		{
			this.xmlNode.replaceChild(old,newtext);
		}
	}
}

/**
 * @method public HxG_5.prototype.XMLData.prototype.removeNamespace
 *	Takes one argument which can be a string with a namespace URI, or a Namespace object, and removes the namespace 
 *	from the in-scope namespaces of this XML object. If the namespace is used for the qualified name of the object 
 *	or its attributes it is not removed.
 *	xhtml.addNamespace(new Namespace('http://www.w3.org/1999/xhtml'));
 *	xhtml.addNamespace(new Namespace('http://www.w3.org/1999/xlink'));
 *	xhtml.removeNamespace(new Namespace('http://www.w3.org/1999/xhtml'));
 *	alert(xhtml.inScopeNamespaces());
 *	// shows 'http://www.w3.org/1999/xlink
 * @param Namespace namespace
 *	Namespace Object to be removed
 **/
HxG_5.prototype.XMLData.prototype.removeNamespace = function(namespace)
{
	//TODO
}

/**
 * @method public HxG_5.prototype.XMLData.prototype.remove
 * If passed a String starting in "@", removes attribute whose name matches the rest of the param string. (supports wildcard '*')
 * Otherwise, removes all element nodes whose name matches the param. (supports wildcard '*')
 * @param String P
 *	Name of attr / child node(s) to be removed
 **/
HxG_5.prototype.XMLData.prototype.remove = function(P)
{

	if (!P)
	{
		var args = [];
		args[0] = "P";
		var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_invalid_param, args);
		Log.error("HxG_5.XMLData.remove", Msg);
		throw new hX_5.XMLDataError(Msg);
	}
	else
	{
		if (P.charAt(0) == '@')
		{
			//attribute to be removed
			var attrs = this.attributes(), name = P.substr(1), i;

			for (i=0; i<attrs.length(); i++ )
			{
				if ( (P=="*")
					|| (name == attrs.elementByIndex(i).name()) )
				{
					this.xmlNode.removeAttribute(attrs.elementByIndex(i).name());
				}
			}
		}
		else
		{	
			//element(s) to be removed
			var dp = 0;

			var ch = this.children();
			for (i=0; i<ch.length(); i++)
			{
				var curr = ch.elementByIndex(i);
				if (P == "*" || (curr.type() == this.ELEMENT_NODE && curr.name() == P))
				{
					this.xmlNode.removeChild(curr.xmlNode);
				}
			}
		}
	}
}


/**
 * @method public HxG_5.prototype.XMLData.prototype.removeByIndex
 *	Removes child XMLData at specified index.
 * @param int P
 *	index of child to be removed
 **/
HxG_5.prototype.XMLData.prototype.removeByIndex = function(P)
{
	var ch = this.children();

	if ( P<0 || P >= ch.length() )
	{
		var args = [];
		var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_invalid_index, args);
		Log.error("HxG_5.XMLData.removeByIndex", Msg);
		throw new hX_5.XMLDataError(Msg);
	}
	else
	{
		this.xmlNode.removeChild(this.xmlNode.childNodes.item(P));
	}
}

/**
 * @method public HxG_5.prototype.XMLData.prototype.setChildren
 *	Takes one argument, the replacement value, and replaces all children of the XMLData object with this value, and 
 *	returns the updated XMLData object. The replacement value can be an XMLData  object or an XMLDataList object in 
 *	which case each element becomes a new child of the XMLData object.
 * @param XMLData(List) value
 *	replacement child(ren)
 * @return XMLData result
 *	This updated XMLData Object.
 **/
HxG_5.prototype.XMLData.prototype.setChildren = function(value)
{
	//TODO
	if(this instanceof hX_5.XMLData)
	{
		// value is XMLData

	}
	else if(this instanceof hX_5.XMLDataList)
	{
		// value is XMLDataList
	}
	else
	{
		Log.error("HxG_5.XMLData.setChildren","");
	}
}

/**
 * @method public HxG_5.prototype.XMLData.prototype.setNamespace
 *	Takes one argument, the new namespace, and changes the namespace associated with the name of this XMLData object 
 *	to the new namespace.
 * @param Namespace namespace
 *	The new Namespace Object to be associated with the name of this XMLData Object.
 **/
HxG_5.prototype.XMLData.prototype.setNamespace = function(namespace)
{
	//TODO
}

/**
 * @method public HxG_5.prototype.XMLData.prototype.text
 *	Returns an XMLDataList with all the children of this XMLData object that represent text nodes.
 * @return XMLDataList result
 *	XMLDataList containing only text nodes. Empty if there is none
 **/
///// Adam note:
/*
In Mozilla, each Node Element has at least one child Text Node. This means that each Node Element which 
contains no text will have an empty ("") child Text Node. This is not the case in IE. In IE, a Node Element
is presumed to have no Text Node children unless explicitely needed (text is present). We need to look at whether we should
consolidate this.
*/

HxG_5.prototype.XMLData.prototype.text = function()
{
	var arr = [];
	if (this.xmlNode.hasChildNodes())
	{
		var children = this.children();
		for (i=0; i<children.length(); i++)
		{
			var curr = children.elementByIndex(i).xmlNode;
			if (curr.nodeType == this.TEXT_NODE)
			{
				arr[arr.length] = new hX_5.XMLData(curr);
			}
		}
	}
	var result = new hX_5.XMLDataList(arr);
	return result;
}

/**
 * @method public HxG_5.prototype.XMLData.prototype.toString
 *	Returns a string value of this XMLData object.
 * @return String str
 *	The String representation.
 **/
HxG_5.prototype.XMLData.prototype.toString = function()
{
	var t = this.type();
	if ( ( t == this.ATTRIBUTE_NODE	) || ( t == this.TEXT_NODE ) )
	{
		return this.value();
	}
	if (this.hasSimpleContent())
	{
		var i, ch = this.children(), s = "";
		for (i=0; i<ch.length(); i++)
		{
			var curr = ch.elementByIndex(i);
			var t = curr.type();
			if ( ( t != this.PROCESSING_INSTRUCTION_NODE ) && ( t != this.COMMENT_NODE ) )
			{
				s += ch.elementByIndex(i).toString();
			}
		}
		return s;
	}
	else
	{
		return this.toXMLString();
	}
}
 
 
 /**
 * @method public HxG_5.prototype.XMLData.prototype.toXMLString
 *	Returns serialised XMLString
 * @return String str
 *	The XML String.
 **/

 //TODO: namespacing should be included in params here...
HxG_5.prototype.XMLData.prototype.toXMLString = function(IndentLevel)
{
	var str = "", i;
	if (!IndentLevel)
	{
		IndentLevel = 0;
	}
	if (this.prettyPrinting)
	{
		for (i=0; i<IndentLevel; i++)
		{
			str += " ";
		}
	}
	switch (this.type())
	{
		case this.TEXT_NODE:
			if (this.prettyPrinting)
			{
				var v = this.trimWhitespace(this.value());
				return str + this.EscapeElementValue(v);
			}
			else
			{
				return this.EscapeElementValue(this.value());
			}
			break;
		case this.ATTRIBUTE_NODE:
			return this.value();
			break;
		case this.COMMENT_NODE:
			return (str + "<!--" + this.value() + "-->");
			break;
		case this.PROCESSING_INSTRUCTION_NODE:
			return (str + "<?" +  this.name() + " " + this.value() + "?>");
			break;
	}
	str += "<" + this.name();
	var	attrs = this.attributes();
	for (i=0; i<attrs.length(); i++)
	{
		var curr = attrs.elementByIndex(i);
		str += ' ' + curr.name() + '="' + curr.value() + '"';
	}
	var ch = this.children();
	if (ch.length() == 0)
	{
		return (str + " />");
	}
	else
	{
		str += ">";
	}
	var indentChildren = ((ch.length()>1) || (ch.length() == 1 && ch.elementByIndex(0).type() != this.TEXT_NODE)),
		nextIndentLevel;
	if (this.prettyPrinting && indentChildren)
	{
		nextIndentLevel = IndentLevel + this.prettyIndent;
	}
	else
	{
		nextIndentLevel = 0;
	}
	for (i=0; i<ch.length(); i++)
	{
		if (this.prettyPrinting && indentChildren)
		{
			str += "\n";
		}
		str += ch.elementByIndex(i).toXMLString(nextIndentLevel) 

	}
	if (this.prettyPrinting && indentChildren)
	{
		str += "\n";	// Adam: this isn't in E4X, but it's pretty.
		for (i=0; i<IndentLevel; i++)
		{
			str += " ";	
		}
	}
	str += "</" +  this.name() + ">";
	return str;
}




 
 /** 
 * @method protected HxG_5.prototype.XMLData.prototype.elementNodeToString
 *	Returns a String representing a node of type 'ELEMENT_NODE' to be used by  the XMLData.to*String methods.
 * @return String str
 *	String representing the Element Node
 **/
/*
deprecated. now encapsulated in toXMLString

HxG_5.prototype.XMLData.prototype.elementNodeToString = function()
{
	
	var i, elem = this.xmlNode;

	// process this node
	var str = '<' + elem.tagName;
	var	attrs = elem.attributes;
	
	for (i=0; i<attrs.length; i++)
	{
		str += ' ' + attrs[i].name + '="' + attrs[i].value + '"';
	}

	str += '>';
	
	// process child nodes
	if (elem.hasChildNodes())
	{
		var children = this.children();		
		for (i=0; i<children.length(); i++)
		{
			var curr = children.elementByIndex(i);
			var	n_type = curr.nodeType;
		
			var curr_str = curr.toXMLString();
			if(curr_str != "")
			{
				str += curr_str;
			}
		}
	}

	str += '</' + elem.tagName + '>';
	return str;
}*/

/** 
 * @method protected HxG_5.prototype.XMLData.prototype.type
 *	gets type of Node represented by this XMLData
 * @return int type
 *	Types corresponding to W3C node types (see start of XMLDataList for const var list)
 **/
 HxG_5.prototype.XMLData.prototype.type = function(){
	return this.xmlNode.nodeType;
 }

 /** 
 * @method protected HxG_5.prototype.XMLData.prototype.value
 *	Returns value of node represented by this XMLData object.
 * @return string value
 *	value.
 **/
 HxG_5.prototype.XMLData.prototype.value = function(){
	return this.xmlNode.nodeValue;
 }

 /** 
 * @method public HxG_5.prototype.XMLData.prototype.hasSimpleContent
 *	checks whether this XMLData object contains simple content
 * @return boolean result
 *	Result boolean.
 **/
 HxG_5.prototype.XMLData.prototype.hasSimpleContent = function()
{
	var t = this.xmlNode.nodeType, result;

	if( ( t == this.COMMENT_NODE ) || ( t == this.PROCESSING_INSTRUCTION_NODE ) )
	{
		return false;
	}
	else
	{
		var ch = this.children();
		if (ch)
		{
			var i;
			for (i=0; i<ch.length(); i++)
			{
				if ( ch.elementByIndex(i).type() == this.ELEMENT_NODE )
				{
					return false;
				}
			}
		}
		return true;
	}
 }

/** 
 * @method protected HxG_5.prototype.XMLData.prototype.TrimWhitespace
 *	Escape for element values. used by toXMLString.
 * @param string s
 *	String to be trimmed.
 * @return string str
 *	Escaped String
 **/

//helper function to id whitespace
 HxG_5.prototype.XMLData.prototype.isWhitespaceChar = function(s)
 {
	 return (s==' ' || s=='\n' || s=='\t' || s=='\r');

 }

HxG_5.prototype.XMLData.prototype.trimWhitespace = function(s)
{
	if (!s)
	{
		return "";
	}
	else
	{
		var i=0,j=(s.length-1);

		while(i<s.length&&this.isWhitespaceChar(s.charAt(i)))
		{
			i++;
		}

		while(j>i&&this.isWhitespaceChar(s.charAt(j)))
		{
			j--;
		}

		var str = s.substring(i,j+1);
		return str;
	}
}

/** 
 * @method protected HxG_5.prototype.XMLData.prototype.EscapeElementValue
 *	Escape for element values. used by toXMLString.
 * @param string s
 *	String to be escaped.
 * @return string str
 *	Escaped String
 **/
HxG_5.prototype.XMLData.prototype.EscapeElementValue = function(s)
{
	var str = "", i;
	for (i=0; i<s.length; i++)
	{
		var curr = s.charAt(i);
		switch (curr)
		{
		case '<':
			str += "&lt;";
			break;
		case '>':
			str += "&gt";
			break;
		case '&':
			str +=  "&amp";
			break;
		default:
			str += curr;
			break;
		}
	}

	return str;

}

/** 
 * @method protected HxG_5.prototype.XMLData.prototype.EscapeElementValue
 *	Escape for attribute values. used by toXMLString. 
 * @param string s
 *	String to be escaped.
 * @return string str
 *	Escaped String
 **/
HxG_5.prototype.XMLData.prototype.EscapeAttributeValue = function(s)
{
	var str = "", i;
	for (i=0; i<s.length; i++)
	{
		var curr = s.charAt(i);
		switch (curr)
		{
		case '"':
			str += "&quot;";
			break;
		case '<':
			str += "&lt";
			break;
		case '&':
			str +=  "&amp";
			break;
		case '\u000A':
			str +=  "&#xA;";
			break;
		case '\u000D':
			str +=  "&#xD;";
			break;
		case '\u0009':
			str +=  "&#x9;";
			break;
		default:
			str += curr;
			break;
		}
	}

	return str;

}

 /** 
 * @method public HxG_5.prototype.XMLData.prototype.destroy
 *	Garbage Collect
 **/
 HxG_5.prototype.XMLData.prototype.destroy = function()
 {
	this.xmlNode = null;
 }


// XMLDataList Class

/**
 * @class public XMLDataList(values)
 * @constructor XMLDataList
 *	The XMLDataList class is a class for containing and manipulating a list of XMLData Objects.
 * @param Array values
 *	Array of XMLData objects objects
 * @exception XMLDataError
 *	Throws exception if object in array value is not of type XMLData or a DOM node
 **/

HxG_5.prototype.XMLDataList = function(values)
{
	this.xmlDataArray = [];
	if (values)
	{
		var i;
		// 'values' can be of type Array, NodeList or NamedNodeMap
		// the same syntax should work for each.
		for(i=0; i< values.length; i++)
		{
			if(values[i] instanceof hX_5.XMLData)	// is type hX_5.XMLData
			{
				this.xmlDataArray[i] = values[i];
			}
			else if (values[i].nodeType) // is a Node
			{
				this.xmlDataArray[i] = new hX_5.XMLData(values[i]);
			}
			else
			{
				var args = [];
				args[0] = "values[" + i + "]";
				var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_invalid_param_type, args);	
				Log.error("HxG_5.XMLDataList", Msg);
				throw new hX_5.XMLDataError(Msg);
			}
		}
	}
}

/**
 * @method public HxG_5.prototype.XMLDataList.prototype.attribute(attributeName)
 *	Takes one argument, the attribute name of attributes to look for. It calls the method attribute() 
 *	of each XMLData object in this XMLDataList and returns the results in order in a new Array.
 * @param String attributeName
 *	The name of the requested attribute. 
 * @result XMLDataList attrs
 *	Array of atrributes found
 * @exception XMLDataError
 *	Throws exception if attributeName is null or undefined
 **/
 HxG_5.prototype.XMLDataList.prototype.attribute = function(attributeName)
 {
	if (attributeName)
	{
		var i,j=0, attrs = [];
		for (i=0; i<this.xmlDataArray.length; i++)
		{
			if (this.xmlDataArray[i].type() == this.xmlDataArray[i].ELEMENT_NODE)
			{
				var curr = this.xmlDataArray[i].attribute(attributeName);
				if(curr){
					attrs[j] = curr;
					j++;
				}
				
			}
		}
		return new hX_5.XMLDataList(attrs);
	}
	else
	{
		var args = [];
		args[0] = "attributeName";
		var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_invalid_param, args);
		Log.error("HxG_5.XMLData.attribute", Msg);
		throw new hX_5.XMLDataError(Msg);
		return null;
	}
 }

 /**
 * @method public HxG_5.prototype.XMLDataList.prototype.elementByIndex(index)
 *  method which returns a pointer to the XMLData specified in this XMLDataList by index
 * @result int index
 *	index of requested XMLData object
 * @exception XMLDataError
 *	Throws exception if index is out of bounds.
 **/
 HxG_5.prototype.XMLDataList.prototype.elementByIndex = function(index)
 {	
	if( index < this.xmlDataArray.length && index >= 0)
	 {
		return this.xmlDataArray[index];
	 }
	 else
	 {
		var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_invalid_index);
		Log.error("HxG_5.XMLData.elementByIndex", Msg);
		throw new hX_5.XMLDataError(Msg);
	 }
 }
 
 /**
 * @method public HxG_5.prototype.XMLDataList.prototype.attributes()
 *  This calls the method attributes() of each XMLData object in this XMLDataList and returns a new 
 *	XMLDataList with the results in order.
 * @result XMLDataList attrs
 *	XMLDataList of attributes
 **/
 HxG_5.prototype.XMLDataList.prototype.attributes = function()
 {
	var arr = [], i, result;
	for (i=0; i<this.xmlDataArray.length; i++)
	{
		arr[i] = this.xmlDataArray[i].attributes();
		if(i==0)
		{
			result = arr[i];
		}
		else
		{ //i<0 join up attrs so far
			result.append(arr[i]);
		}
	}

	return result;
 }

 /**
 * @method public HxG_5.prototype.XMLDataList.prototype.append(y)
 *  Utility function for appending an XMLData or XMLDataList object to this XMLDataList object. 
 * @param XMLData(List) propertyName
 *	XMLData(List) to append
 * @exception XMLDataError
 *	Throws exception if y is null or undefined
 *	Throws exception if y is not of type XMLData or XMLDataList
 **/
HxG_5.prototype.XMLDataList.prototype.append = function(y)
{
	if(!y)
	{
		var args = [];
		args[0] = "y";
		var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_invalid_param, args);	
		Log.error("HxG_5.XMLDataList.append", Msg);
		throw new hX_5.XMLDataError(Msg);
	}
	else
	{
		var i;

		//Note: in the following code, y is not cloned so any copying or cloned should be done before calling append

		if(y instanceof hX_5.XMLData) //is type hX_5.XMLData
		{
			this.xmlDataArray[this.xmlDataArray.length] = y;
		}
		else if(y instanceof hX_5.XMLDataList) //is type hX_5.XMLDataList
		{
			for (i=0; i<(y.length()); i++)
			{
				this.xmlDataArray[this.xmlDataArray.length] = y.elementByIndex(i);
			}
		}
		else
		{
			var args = [];
			args[0] = "y";
			var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_invalid_param_type, args);	
			Log.error("HxG_5.XMLDataList.append", Msg);
			throw new hX_5.XMLDataError(Msg);
		}
	}
	
	
}

 /**
 * @method public HxG_5.prototype.XMLDataList.prototype.child(propertyName)
 *  Takes one argument, the child property name to look for, and calls the method child() of each XMLData 
 *	object in this XMLDataList object to return an XMLDataList with the matching children in order.
 * @param String propertyName
 *	The name of the child(ren) we are looking for.
 * @result XMLDataList result
 *	List of children found.
 * @exception XMLDataError
 *	Throws exception if propertyName is null or undefined
 **/
 HxG_5.prototype.XMLDataList.prototype.child = function(propertyName)
 {
	if (propertyName)
	{
		var i, result=new hX_5.XMLDataList();
		for (i=0; i<this.length(); i++)
		{
			var ch = this.elementByIndex(i).child(propertyName); 
			if(ch!=null)
			{
				result.append(ch);
			}
		}
		return result;
	}
	else
	{
		var args = [];
		args[0] = "propertyName";
		var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_invalid_param, args);	
		Log.error("HxG_5.XMLDataList.child", Msg);
		throw new hX_5.XMLDataError(Msg);
		return null;
	}
 }

 /**
 * @method public HxG_5.prototype.XMLDataList.prototype.children()
 *  The method children of XMLDataList objects takes no argument and calls the method children() of each 
 *	XMLData object in this XMLDataList and returns an XMLDataList with the children of all XMLData objects in the XMLDataList. 
 * @result XMLDataList result
 *	List of children found.
 **/
 HxG_5.prototype.XMLDataList.prototype.children = function()
 {
	var i, result = new hX_5.XMLDataList();
	for (i=0; i<this.length(); i++)
	{
		var ch = this.elementByIndex(i).children();
		if(ch!=null)
		{
			result.append(ch);
		}
	}
	return result;
 }

 /**
 * @method public XMLDatalist.prototype.contains(value)
 *	Checks whethere there is an XMLData object in the list which is equal to the specified value.
 * @param XMLData value
 *	XMLData object to be compared against.
 * @return boolean found
 *	Whether an XMLData matching 'value' has been found in this XMLDataList.
 * @exception XMLDataError
 *	Throws exception if value is null or undefined
 **/
 HxG_5.prototype.XMLDataList.prototype.contains = function(value)
 {
	var i;
	if(value)
	 {
		for (i=0; i<this.length(); i++)
		{
			if (value.contains(this.xmlDataArray[i]))
			{
				return true;
			}
		}
		return false;
	 }
	 else
	 {
			var args = [];
			args[0] = "value";
			var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_invalid_param, args);	
			Log.error("HxG_5.XMLDataList.contains", Msg);
			throw new hX_5.XMLDataError(Msg);
	 }
 }

 /**
 * @method public XMLDatalist.prototype.elements([name])
 *	Create an XMLDataList consisting of the children of elements in this XMLDataList which match the 
 *	specified element name if the element name is supplied.
 * @param String name
 *	Optional. Element name to find.
 * @return XMLDataList result
 *	List of matching children. If no parameter is passed, then this is a list of all element children of all XMLData
 *	objects in the original XMLDataList
 */
 HxG_5.prototype.XMLDataList.prototype.elements = function(name)
 {
	var i, result = new hX_5.XMLDataList(), elems;
	for (i=0; i<this.length(); i++)
	{
		if(name)
		{
			elems = this.elementByIndex(i).elements(name);
		}
		else
		{
			elems = this.elementByIndex(i).elements();
		}
		if(elems!=null)
		{
			result.append(elems);
		}
	}
	return result;
 }

 /**
 * @method public XMLDatalist.prototype.length()
 *	Returns the number of elements this XMLDataList contains.
 * @return integer len
 *	Length.
 */
 HxG_5.prototype.XMLDataList.prototype.length = function()
 {
	var len = this.xmlDataArray.length;
	return len;
 }

/**
 * @method public XMLDataList.prototype.remove
 *	
 * @param P
 *	P is:
 *		a) number: an index of an XMLData object to be removed
 *		b) string: iterated through the XMLData objects in the list and call remove(P)
 **/

HxG_5.prototype.XMLDataList.prototype.remove = function(P)
{
	if (typeof P == "number")
	{
		if (P >= this.length())
		{
			var args = [];
			args[0] = "P";
			var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_invalid_index, args);	
			Log.error("HxG_5.XMLDataList.remove", Msg);
			throw new hX_5.XMLDataError(Msg);
		}
		else
		{
			var curr = this.elementByIndex(P);
			if (curr.xmlNode.parentNode)
			{
				var parent = new hX_5.XMLData(this.xmlDataArray[P].xmlNode.parentNode);
				if (curr.type() == curr.ATTRIBUTE_NODE)
				{
					parent.remove("@"+curr.name());
				}
				else
				{
					parent.xmlNode.removeChild(curr.xmlNode);
					var k;
					for (k=P; k<(this.length()-1); k++)
					{
						this.xmlDataArray[k] = this.xmlDataArray[k+1];
					}
					this.xmlDataArray[this.xmlDataArray.length-1] = null;
					this.xmlDataArray.length--;
				}
			}
		}
	}
	else if(typeof P == "string")
	{
		var i;
		for (i=0; i<this.length(); i++)
		{
			var curr = this.elementByIndex(i);
			if (curr.type() == curr.ELEMENT_NODE)
			{
				curr.remove(P);
			}
		}
	}
	else
	{
			var args = [];
			args[0] = "P";
			var Msg = hX_5.NLSFormatMsg(hX_5.INTERR_invalid_param_type, args);	
			Log.error("HxG_5.XMLDataList.remove", Msg);
			throw new hX_5.XMLDataError(Msg);
	}
}

 /**
 * @method public XMLDatalist.prototype.text()
 *	Returns an XMLDataList containing all the text child nodes of all the XML Data objects contained in this 
 *	XMLDataList.
 * @return String str
 *	Concatenation of the result of calling the method text() on each XML Data objects contained in this 
 */
 HxG_5.prototype.XMLDataList.prototype.text = function()
 {
	 var str="", i;

	 for(i=0; i<this.length(); i++)
	 {
		str += this.xmlDataArray[i].text();
	 }

	 return str;
 }

  HxG_5.prototype.XMLDataList.prototype.toString = function()
  {
		if (this.hasSimpleContent())
		{
			var s="", i;
			for (i=0; i<this.length(); i++)
			{
				var curr = this.xmlDataArray[i];
				var t = curr.type();
				if ((t != curr.COMMENT_NODE) && (t != curr.PROCESSING_INSTRUCTION_NODE))
				{
					s += curr.toString();
				}
			}
			return s;
		}
		else
		{
			return this.toXMLString();
		}
  }
 
 /**
 * @method public XMLDatalist.prototype.toXMLString()
 *	Returns a convenient string representation of the XMLDataList.
 * @return String str
 *	The resultant 
 * @exception XMLDataError
 *	Throws exception if a XMLData member of the XMLDataList is not of type text, comment, element or attribute.
 **/
 HxG_5.prototype.XMLDataList.prototype.toXMLString = function()
 {
	 var i, str="";
	
	 for (i=0; i<this.length(); i++)
	 {
		 var curr = this.xmlDataArray[i];
		 if ( (curr.prettyPrinting == true) && ( i!=0 ) )
		 {
			 str += "\n";
		 }
		 str += curr.toXMLString();
	 }

	return str;
 }

/**
*	@method HxG_5.prototype.XMLDataList.prototype.toXMLDataArray()
*	 util for converting to array of XMLData Objects
*	@return Array arr
*	 the array.
**/
 HxG_5.prototype.XMLDataList.prototype.toArrayOfXMLData = function()
 {
	 var arr = [], i;
	 for(i=0; i<this.xmlDataArray[i]; i++){
		 arr[i] = this.xmlDataArray[i].copy(true);
	 }

	 return arr;
 }

 /**
 *	@method HxG_5.prototype.XMLDataList.prototype.hasSimpleContent()
 *	 returns true if this XMLDataList object contains a single XMLData object 
 *	 with simple content, no XMLData objects or contains no elements
 *	@return Boolean result;
 *	 the result boolean.
 */
HxG_5.prototype.XMLDataList.prototype.hasSimpleContent = function()
{
	if (this.length() == 0)
	{
		return true;
	}
	else if (this.length() == 1)
	{
		return this.xmlDataArray[0].hasSimpleContent();
	}
	else
	{
		var i;
		for (i=0; i<this.length(); i++)
		{
			if(this.xmlDataArray[i].type() == this.xmlDataArray[i].ELEMENT_NODE)
			{
				return false;
			}
		}
	}

	return true;
}

/**
 *	@method HxG_5.prototype.XMLDataList.prototype.copy()
 *	 returns a copy of the XMLDataList object
 *	@return XMLDataList result;
 *	 the new copy.
 */
HxG_5.prototype.XMLDataList.prototype.copy = function()
{
	var i, arr = [];
	for (i=0; i<this.length(); i++)
	{
		arr[arr.length] = this.xmlDataArray[i].copy();
	}

	return new XMLDataList(arr);
}

 /** 
 * @method public HxG_5.prototype.XMLDataList.prototype.destroy
 *	Garbage collect
 **/
 HxG_5.prototype.XMLDataList.prototype.destroy = function()
 {
	for (i=0; i<this.length(); i++)
	{
		this.xmlDataArray[i].destroy();
		this.xmlDataArray[i] = null;
	}

	this.xmlDataArray = null;
 }

 /**
 * @class public HxG_5.prototype.XMLDataError
 * @constructor XMLDataError
 * XMLDataError is used to thrown an exception, including an error message.
 *	The constructor takes one parameter.
 * @param String d
 *	The description for the exception.
 **/
HxG_5.prototype.XMLDataError = function(d)
{
	this.description = d;

	/**
	 * @method public toString
	 *	Method to return an error description.
	 *@return String d
	 *	The error description.
	 **/
	this.toString = function () { return d; }
}